Ext.ns('dnet.core.base');
Dnet = {

	name : "DNet",
	version : '0.0.01',
	versionInfo : {
		major : 0,
		minor : 0,
		patch : 0
	},

	/*
	 * Base URLs amd fragments:
	 * 
	 * These are provided by the server when constructing the main entry point
	 * html file.
	 * 
	 */

	hostUrl : null,
	url : null,

	/**
	 * Sub-path for a DS action. Default is tx/java/ds
	 */
	dsUrl : null,

	/**
	 * Sub-path for an RPC action. Default is tx/java/rpc
	 */
	rpcUrl : null,

	/**
	 * Sub-path for an assignment actions. Default is tx/java/asgn
	 */
	asgnUrl : null,

	/**
	 * Url to workflow rest api.
	 */
	wfUrl : null,
	uiUrl : null,

	/**
	 * Session related requests URL
	 */
	sessionUrl : null,

	staticResourceUrlCore : null,
	staticResourceUrlModules : null,

	staticResourceUrlCoreI18n : null,
	staticResourceUrlModulesI18n : null,

	moduleSubpath : '/webapp/js',

	// configuration variables
	config : {

	},

	/**
	 * Default date and time formats. They are overwritten according to the user
	 * locale settings.
	 */
	DATE_FORMAT : 'Y-m-d',
	TIME_FORMAT : 'H:i',
	DATETIME_FORMAT : 'Y-m-d H:i',
	DATETIMESEC_FORMAT : 'Y-m-d H:i:s',
	MONTH_FORMAT : 'Y-m',
	MODEL_DATE_FORMAT : "Y-m-d\\TH:i:s",
	DATE_ALTFORMATS : "j|j.n|d|d.m",

	THOUSAND_SEP : ',',
	DECIMAL_SEP : '.',

	numberFormats : null,

	DEFAULT_THEME : "gray",
	DEFAULT_LANGUAGE : "en",

	/**
	 * Creates a set of number formats up to 6 decimals according to the user
	 * locale.
	 */
	initFormats : function() {
		Ext.util.Format.decimalSeparator = this.DECIMAL_SEP;
		Ext.util.Format.thousandSeparator = this.THOUSAND_SEP;
		this.numberFormats = new Ext.util.MixedCollection();
		this.numberFormats.add(0, "0,000");
		for ( var i = 1; i < 6; i++) {
			this.numberFormats.add(i, "0,000."
					+ Ext.util.Format.leftPad("0", i, "0"));
		}
	},

	/**
	 * Creates a number format string based on the provided decimals number and
	 * the user locale.
	 */
	getNumberFormat : function(decimals) {
		if (this.numberFormats == null) {
			this.initFormats();
		}
		return this.numberFormats.get(decimals);
	},

	dataFormat : {
		HTML : "html",
		CSV : "csv",
		PDF : "pdf",
		XML : "xml",
		JSON : "json"
	},

	/**
	 * Request parameter mappings. These values must be kept in sync with the
	 * mappings at server-side.
	 * 
	 * TODO: Send these mappings in the main html page from the server to keep
	 * them in only one place.
	 */
	requestParam : {
		SORT : "orderByCol",
		SENSE : "orderBySense",
		START : "resultStart",
		SIZE : "resultSize",
		ORDERBY : "orderBy",

		SERVICE_NAME_PARAM : "rpcName",

		EXPORT_COL_NAMES : "c[export_col_names]",
		EXPORT_COL_TITLES : "c[export_col_titles]",
		EXPORT_COL_WIDTHS : "c[export_col_widths]"
	},

	asgnAction : {
		LOAD : "find",
		SELECT : "insert",
		REMOVE : "update",
		SELECTALL : "delete",
		REMOVEALL : "delete"
	},

	/**
	 * Creates the URL to load a frame.
	 */
	buildUiPath : function(uiModule, uiName, isSpecial) {
		if (isSpecial) {
			return this.uiUrl + "/spframe/" + uiModule + "/" + uiName;
		} else {
			return this.uiUrl + "/frame/" + uiModule + "/" + uiName;
		}
	},

	/**
	 * URLs for session management related requests.
	 */
	sessionAPI : function(format) {
		return {
			login : this.sessionUrl + '?action=login',
			logout : this.sessionUrl + '/doLogout',
			lock : this.sessionUrl + '?action=lock',
			unlock : this.sessionUrl + '?action=unlock',
			changePassword : this.sessionUrl + '?action=changePassword',
			userSettings : this.sessionUrl + '?action=userSettings'
		};
	},

	/**
	 * URLs for data-source (DS) related requests.
	 */
	dsAPI : function(resource, format) {
		return {
			read : this.dsUrl + '/' + resource + '.' + format + '?action=find',
			load : this.dsUrl + '/' + resource + '.' + format + '?action=find',
			exportdata : this.dsUrl + '/' + resource + '.' + format
					+ '?action=export',
			importdata : this.dsUrl + '/' + resource + '.' + format
					+ '?action=import',
			create : this.dsUrl + '/' + resource + '.' + format
					+ '?action=insert',
			update : this.dsUrl + '/' + resource + '.' + format
					+ '?action=update',
			save : this.dsUrl + '/' + resource + '.' + format + '?action=save',
			destroy : this.dsUrl + '/' + resource + '.' + format
					+ '?action=delete',
			service : this.dsUrl + '/' + resource + '.' + format
					+ '?action=rpc'
		};
	},

	/**
	 * URLs for assignment components requests for the available elements.
	 */
	asgnLeftAPI : function(resource, format) {
		return {
			read : this.asgnUrl + '/' + resource + '.' + format
					+ '?action=findLeft',
			exportdata : this.asgnUrl + '/' + resource + '.' + format
					+ '?action=',
			create : this.asgnUrl + '/' + resource + '.' + format + '?action=',
			update : this.asgnUrl + '/' + resource + '.' + format + '?action=',
			destroy : this.asgnUrl + '/' + resource + '.' + format + '?action='

		};
	},

	/**
	 * URLs for assignment components requests for the selected elements.
	 */
	asgnRightAPI : function(resource, format) {
		return {
			read : this.asgnUrl + '/' + resource + '.' + format
					+ '?action=findRight',
			exportdata : this.asgnUrl + '/' + resource + '.' + format
					+ '?action=',
			create : this.asgnUrl + '/' + resource + '.' + format + '?action=',
			update : this.asgnUrl + '/' + resource + '.' + format + '?action=',
			destroy : this.asgnUrl + '/' + resource + '.' + format + '?action='

		};
	},

	/**
	 * URLs for RPC type requests.
	 */
	rpcAPI : function(resource, fnName, format) {
		return this.rpcUrl + '/' + resource + '.' + format + '?action='
				+ fnName
	},

	/**
	 * Workflow api: Process definition
	 */
	wfProcessDefinitionAPI : function(processDefinitionId) {
		return {
			form : this.wfUrl + '/process-definition/' + processDefinitionId
					+ '/form',
			diagram : this.wfUrl + '/process-definition/' + processDefinitionId
					+ '/diagram',
			xml : this.wfUrl + '/process-definition/' + processDefinitionId
					+ '/xml',
			properties : this.wfUrl + '/process-definition/'
					+ processDefinitionId + '/properties'
		};
	},

	/**
	 * Workflow api: Process instance
	 */
	wfProcessInstanceAPI : function(processInstanceId) {
		return {
			start : this.wfUrl + '/process-instance/start',
			diagram : this.wfUrl + '/process-instance/' + processInstanceId
					+ '/diagram'
		};
	},

	/**
	 * Workflow api: Task management
	 */
	wfTaskAPI : function(taskId) {
		return {
			form : this.wfUrl + '/task/' + taskId + '/form',
			complete : this.wfUrl + '/task/' + taskId + '/complete',
			properties : this.wfUrl + '/task/' + taskId + '/properties'
		};
	},

	/**
	 * Workflow api: Deployment management
	 */
	wfDeploymentAPI : function(deploymentId) {
		return {
			destroy : this.wfUrl + '/deployment/delete'
		};
	},

	setDialogPath : function(dialogName, pathname) {
		if (pathname) {
			document.getElementById(dialogName + '_dialogPath').innerHTML = pathname;
			document.getElementById(dialogName + '_dialogNamePathSeparator').innerHTML = "/";
		} else {
			document.getElementById(dialogName + '_dialogPath').innerHTML = "";
			document.getElementById(dialogName + '_dialogNamePathSeparator').innerHTML = "";
		}
	},

	setDialogTitle : function(dialogName, title) {
		document.getElementById(dialogName + '_dialogName').innerHTML = title;
	},

	translate : function(t, k) {
		return dnet.Translation[t][k] || k;
	},

	navigationTreeMenus : {
	/*
	 * ad : { name: "mbMenuAD" , title :"AD: Administration", children: [] }
	 */
	},

	doImport : function(resourseList) {
		var list = Ext.Array.unique(resourseList);
		for ( var i = 0; i < list.length; i++) {
			if (!Ext.isEmpty(list[i])) {
				var rd = this.describeResource(list[i]);
				document.write('<' + 'scr' + 'ipt type="text/javascript" src="'
						+ Dnet.staticResourceUrlModules + '/' + rd.bundle
						+ this.moduleSubpath + '/' + rd.type + '/' + rd.name
						+ '.js"></script>');
				if (rd.type == "ds") {
					document.write('<' + 'scr'
							+ 'ipt type="text/javascript" src="'
							+ Dnet.staticResourceUrlModulesI18n + '/'
							+ __LANGUAGE__ + '/' + rd.bundle + '/' + rd.type
							+ '/' + rd.name + '.js"></script>');
				}
			}
		}
	},

	describeResource : function(artifact) {
		var rd = {};
		var t = artifact.split('/');
		rd['bundle'] = t[0];
		rd['type'] = t[1];
		rd['name'] = t[2];
		return rd;
	},

	/**
	 * Translation for a field
	 */
	translateField : function(vrb, mrb, item) {
		// check if the view has its own resource bundle
		if (vrb != undefined && vrb[item.name]) {
			item.fieldLabel = vrb[item.name];
			return true;
		}
		// try to translate it from the model's resource bundle
		if (item.dataIndex != undefined && mrb != null
				&& mrb[item.dataIndex + '__lbl']) {
			item.fieldLabel = mrb[item.dataIndex + '__lbl'];
			return true;
		}
		if (item.paramIndex != undefined && mrb != null
				&& mrb[item.paramIndex + '__lbl']) {
			item.fieldLabel = mrb[item.paramIndex + '__lbl'];
			return true;
		}
		// try to translate from the shared resource-bundle
		item.fieldLabel = Dnet.translate("ds", item.dataIndex
				|| item.paramIndex);
	},

	/**
	 * Translation for a grid column
	 */
	translateColumn : function(vrb, mrb, item) {
		// check if the view has its own resource bundle
		if (vrb != undefined && vrb[item.name]) {
			item.header = vrb[item.name];
			return true;
		}
		// try to translate it from the model's resource bundle
		if (item.dataIndex != undefined && mrb != null
				&& mrb[item.dataIndex + '__lbl']) {
			item.header = mrb[item.dataIndex + '__lbl'];
			return true;
		}
		// try to translate from the shared resource-bundle
		item.header = Dnet.translate("ds", item.dataIndex);
	},

	createBooleanStore : function() {
		return Ext.create('Ext.data.Store', {
			fields : [ "bv", "tv" ],
			data : [ {
				"bv" : true,
				"tv" : Dnet.translate("msg", "bool_true")
			}, {
				"bv" : false,
				"tv" : Dnet.translate("msg", "bool_false")
			} ]
		});
	},

	doWithGetResult : function(url, params, fn, scope) {
		Ext.Ajax.request({
			url : url,
			method : "GET",
			params : params,
			success : function(response, options) {
				var r = Ext.decode(response.responseText);
				fn.call(scope || window, r, response, options);
			},
			failure : function(response, options) {
				try {
					Ext.Msg.hide();
				} catch (e) {
				}
				dnet.core.dc.AbstractDc.prototype.showAjaxErrors(response,
						options)
			},
			scope : scope
		});
	},
};
 

Ext.override(Ext.form.field.Picker, {
	collapse: function() {
		if (this.isExpanded && !this.isDestroyed) {
            var me = this,
                openCls = me.openCls,
                picker = me.picker,
                doc = Ext.getDoc(),
                collapseIf = me.collapseIf,
                aboveSfx = '-above';

            // hide the picker and set isExpanded flag
            picker.hide();
            me.isExpanded = false;

            // remove the openCls
            me.bodyEl.removeCls([openCls, openCls + aboveSfx]);
            picker.el.removeCls(picker.baseCls + aboveSfx);

            // remove event listeners
            doc.un('mousewheel', collapseIf, me);
            doc.un('mousedown', collapseIf, me);
            Ext.EventManager.removeResizeListener(me.alignPicker, me);
            me.fireEvent('collapse', me);
            me.onCollapse();
            this.focus(true);
        }
		
    }
});

Ext.override(Ext.data.Model, {
			clientIdProperty : "__clientRecordId__"
		});

Ext.override(Ext.form.Basic, {
			findField : function(id) {
				return this.getFields().findBy(function(f) {
					return f.dataIndex === id || f.id === id
							|| f.getName() === id;
				});
			}
		});

 
Ext.override(Ext.form.field.Text, {
			getRawValue : function() {
				var me = this, v = me.callParent();
				if (v === me.emptyText) {
					v = '';
				}
				if (this.caseRestriction && v != '') {
					if (this.caseRestriction == "uppercase") {
						v = v.toUpperCase();
					} else {
						v = v.toLowerCase();
					}
				}
				me.rawValue = v;
				return v;
			}
		});

Ext.define("dnet.base.DisplayFieldText", {
			extend : "Ext.form.field.Display",
			alias : "widget.displayfieldtext",

			asText : false,

			valueToRaw : function(value) {
				return value;
			},

			setRawValue : function(value) {
				var me = this;
				value = Ext.value(value, '');
				me.rawValue = value;
				if (me.rendered) {
					if (this.asText) {
						me.inputEl.dom.innerHTML = "<pre>" + value + "</pre>";
					} else {
						me.inputEl.dom.innerHTML = me.htmlEncode
								? Ext.util.Format.htmlEncode(value)
								: value;
					}

				}
				return value;
			}
		});

Ext.define("dnet.base.DisplayFieldDate", {
			extend : "Ext.form.field.Display",
			alias : "widget.displayfielddate",
			valueToRaw : function(value) {
				return Ext.util.Format.date(value, Dnet.DATE_FORMAT);
			}
		});

Ext.define("dnet.base.DisplayFieldNumber", {
			extend : "Ext.form.field.Display",
			alias : "widget.displayfieldnumber",

			valueToRaw : function(value) {
				return Ext.util.Format.number(value, this.format || "0");
			}
		});

Ext.define("dnet.base.DisplayFieldBoolean", {
			extend : "Ext.form.field.Display",
			alias : "widget.displayfieldboolean",
			renderer : function(rawValue, field) {
				if (rawValue === "false") {
					return Dnet.translate("msg", "bool_false");
				}
				return Dnet.translate("msg", "bool_" + (!!rawValue));
			}
		});

Ext.override(Ext.data.Store, {

			filterAllNew : function(item) {
				return item.phantom === true;
			},

			getAllNewRecords : function() {
				return this.data.filterBy(this.filterAllNew).items;
			},

			filterUpdated : function(item) {
				return item.dirty === true && item.phantom !== true;
			},
			getModifiedRecords : function() {
				return [].concat(this.getAllNewRecords(), this
								.getUpdatedRecords());
			},
			
			//
			
			rejectChanges : function() {
		        this.callParent();
		        this.fireEvent('changes_rejected', this);
		    }
			
		});

Ext.override(Ext.grid.plugin.CellEditing, {

	/**
	 * Context content { grid : grid, record : record, field :
	 * columnHeader.dataIndex, value : record.get(columnHeader.dataIndex), row :
	 * view.getNode(rowIdx), column : columnHeader, rowIdx : rowIdx, colIdx :
	 * colIdx }
	 * 
	 * @param {}
	 *            context
	 * @return {Boolean}
	 */
	beforeEdit : function(context) {
		if (context.grid && context.grid.beforeEdit) {
			return context.grid.beforeEdit(context);
		}
	},

	getEditor : function(record, column) {
		var me = this;
		if (me.grid._getCustomCellEditor_) {
			var editor = me.grid._getCustomCellEditor_(record, column);
			if (editor != null) {
				if (!this._isEditAllowed_(record, column, editor.field
								|| editor)) {
					return null;
				}

				if (!(editor instanceof Ext.grid.CellEditor)) {
					editorId = column.id + record.id;
					editor = new Ext.grid.CellEditor({
								editorId : editorId,
								field : editor,
								completeOnEnter: false,
								ownerCt : me.grid
							});
				}
				editor.editingPlugin = me;
				editor.isForTree = me.grid.isTree;
				editor.on({
							scope : me,
							specialkey : me.onSpecialKey,
							complete : me.onEditComplete,
							canceledit : me.cancelEdit
						});
				editor.field["_targetRecord_"] = record;
				return editor;
			}
		}

		var editor = this.callParent(arguments);
		if (editor
				&& !this
						._isEditAllowed_(record, column, editor.field || editor)) {
			return false;
		}
		if (editor.field) {
			editor.field["_targetRecord_"] = record;
		}
		return editor;

	},

	_isEditAllowed_ : function(record, column, field) {
		if (field && field.noEdit) {
			return false;
		}
		if (field && field.noUpdate === true && !record.phantom) {
			return false;
		} else if (field && field.noInsert === true && record.phantom) {
			return false;
		}
		return true;
	},
	
	onEditComplete : function(ed, value, startValue) {
        var me = this,
            grid = me.grid,
            activeColumn = me.getActiveColumn(),
            record;

        if (activeColumn) {
            record = me.context.record;

            me.setActiveEditor(null);
            me.setActiveColumn(null);
            me.setActiveRecord(null);
    
            if (!me.validateEdit()) {
                return;
            }
            // Only update the record if the new value is different than the
            // startValue. When the view refreshes its el will gain focus
            if (!record.isEqual(value, startValue)) {
                record.set(activeColumn.dataIndex, value);
            // Restore focus back to the view's element.
            } else {
                
            }
            grid.getView().getEl(activeColumn).focus();
            me.context.value = value;
            me.fireEvent('edit', me, me.context);
        }
    }

    
	
});Ext.util.MD5 = function(s,raw,hexcase,chrsz) {
	raw = raw || false;	
	hexcase = hexcase || false;
	chrsz = chrsz || 8;

	function safe_add(x, y){
		var lsw = (x & 0xFFFF) + (y & 0xFFFF);
		var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
		return (msw << 16) | (lsw & 0xFFFF);
	}
	function bit_rol(num, cnt){
		return (num << cnt) | (num >>> (32 - cnt));
	}
	function md5_cmn(q, a, b, x, s, t){
		return safe_add(bit_rol(safe_add(safe_add(a, q), safe_add(x, t)), s),b);
	}
	function md5_ff(a, b, c, d, x, s, t){
		return md5_cmn((b & c) | ((~b) & d), a, b, x, s, t);
	}
	function md5_gg(a, b, c, d, x, s, t){
		return md5_cmn((b & d) | (c & (~d)), a, b, x, s, t);
	}
	function md5_hh(a, b, c, d, x, s, t){
		return md5_cmn(b ^ c ^ d, a, b, x, s, t);
	}
	function md5_ii(a, b, c, d, x, s, t){
		return md5_cmn(c ^ (b | (~d)), a, b, x, s, t);
	}

	function core_md5(x, len){
		x[len >> 5] |= 0x80 << ((len) % 32);
		x[(((len + 64) >>> 9) << 4) + 14] = len;
		var a =  1732584193;
		var b = -271733879;
		var c = -1732584194;
		var d =  271733878;
		for(var i = 0; i < x.length; i += 16){
			var olda = a;
			var oldb = b;
			var oldc = c;
			var oldd = d;
			a = md5_ff(a, b, c, d, x[i+ 0], 7 , -680876936);
			d = md5_ff(d, a, b, c, x[i+ 1], 12, -389564586);
			c = md5_ff(c, d, a, b, x[i+ 2], 17,  606105819);
			b = md5_ff(b, c, d, a, x[i+ 3], 22, -1044525330);
			a = md5_ff(a, b, c, d, x[i+ 4], 7 , -176418897);
			d = md5_ff(d, a, b, c, x[i+ 5], 12,  1200080426);
			c = md5_ff(c, d, a, b, x[i+ 6], 17, -1473231341);
			b = md5_ff(b, c, d, a, x[i+ 7], 22, -45705983);
			a = md5_ff(a, b, c, d, x[i+ 8], 7 ,  1770035416);
			d = md5_ff(d, a, b, c, x[i+ 9], 12, -1958414417);
			c = md5_ff(c, d, a, b, x[i+10], 17, -42063);
			b = md5_ff(b, c, d, a, x[i+11], 22, -1990404162);
			a = md5_ff(a, b, c, d, x[i+12], 7 ,  1804603682);
			d = md5_ff(d, a, b, c, x[i+13], 12, -40341101);
			c = md5_ff(c, d, a, b, x[i+14], 17, -1502002290);
			b = md5_ff(b, c, d, a, x[i+15], 22,  1236535329);
			a = md5_gg(a, b, c, d, x[i+ 1], 5 , -165796510);
			d = md5_gg(d, a, b, c, x[i+ 6], 9 , -1069501632);
			c = md5_gg(c, d, a, b, x[i+11], 14,  643717713);
			b = md5_gg(b, c, d, a, x[i+ 0], 20, -373897302);
			a = md5_gg(a, b, c, d, x[i+ 5], 5 , -701558691);
			d = md5_gg(d, a, b, c, x[i+10], 9 ,  38016083);
			c = md5_gg(c, d, a, b, x[i+15], 14, -660478335);
			b = md5_gg(b, c, d, a, x[i+ 4], 20, -405537848);
			a = md5_gg(a, b, c, d, x[i+ 9], 5 ,  568446438);
			d = md5_gg(d, a, b, c, x[i+14], 9 , -1019803690);
			c = md5_gg(c, d, a, b, x[i+ 3], 14, -187363961);
			b = md5_gg(b, c, d, a, x[i+ 8], 20,  1163531501);
			a = md5_gg(a, b, c, d, x[i+13], 5 , -1444681467);
			d = md5_gg(d, a, b, c, x[i+ 2], 9 , -51403784);
			c = md5_gg(c, d, a, b, x[i+ 7], 14,  1735328473);
			b = md5_gg(b, c, d, a, x[i+12], 20, -1926607734);
			a = md5_hh(a, b, c, d, x[i+ 5], 4 , -378558);
			d = md5_hh(d, a, b, c, x[i+ 8], 11, -2022574463);
			c = md5_hh(c, d, a, b, x[i+11], 16,  1839030562);
			b = md5_hh(b, c, d, a, x[i+14], 23, -35309556);
			a = md5_hh(a, b, c, d, x[i+ 1], 4 , -1530992060);
			d = md5_hh(d, a, b, c, x[i+ 4], 11,  1272893353);
			c = md5_hh(c, d, a, b, x[i+ 7], 16, -155497632);
			b = md5_hh(b, c, d, a, x[i+10], 23, -1094730640);
			a = md5_hh(a, b, c, d, x[i+13], 4 ,  681279174);
			d = md5_hh(d, a, b, c, x[i+ 0], 11, -358537222);
			c = md5_hh(c, d, a, b, x[i+ 3], 16, -722521979);
			b = md5_hh(b, c, d, a, x[i+ 6], 23,  76029189);
			a = md5_hh(a, b, c, d, x[i+ 9], 4 , -640364487);
			d = md5_hh(d, a, b, c, x[i+12], 11, -421815835);
			c = md5_hh(c, d, a, b, x[i+15], 16,  530742520);
			b = md5_hh(b, c, d, a, x[i+ 2], 23, -995338651);
			a = md5_ii(a, b, c, d, x[i+ 0], 6 , -198630844);
			d = md5_ii(d, a, b, c, x[i+ 7], 10,  1126891415);
			c = md5_ii(c, d, a, b, x[i+14], 15, -1416354905);
			b = md5_ii(b, c, d, a, x[i+ 5], 21, -57434055);
			a = md5_ii(a, b, c, d, x[i+12], 6 ,  1700485571);
			d = md5_ii(d, a, b, c, x[i+ 3], 10, -1894986606);
			c = md5_ii(c, d, a, b, x[i+10], 15, -1051523);
			b = md5_ii(b, c, d, a, x[i+ 1], 21, -2054922799);
			a = md5_ii(a, b, c, d, x[i+ 8], 6 ,  1873313359);
			d = md5_ii(d, a, b, c, x[i+15], 10, -30611744);
			c = md5_ii(c, d, a, b, x[i+ 6], 15, -1560198380);
			b = md5_ii(b, c, d, a, x[i+13], 21,  1309151649);
			a = md5_ii(a, b, c, d, x[i+ 4], 6 , -145523070);
			d = md5_ii(d, a, b, c, x[i+11], 10, -1120210379);
			c = md5_ii(c, d, a, b, x[i+ 2], 15,  718787259);
			b = md5_ii(b, c, d, a, x[i+ 9], 21, -343485551);
			a = safe_add(a, olda);
			b = safe_add(b, oldb);
			c = safe_add(c, oldc);
			d = safe_add(d, oldd);
		}
		return [a, b, c, d];
	}
	function str2binl(str){
		var bin = [];
		var mask = (1 << chrsz) - 1;
		for(var i = 0; i < str.length * chrsz; i += chrsz) {
			bin[i>>5] |= (str.charCodeAt(i / chrsz) & mask) << (i%32);
		}
		return bin;
	}
	function binl2str(bin){
		var str = "";
		var mask = (1 << chrsz) - 1;
		for(var i = 0; i < bin.length * 32; i += chrsz) {
			str += String.fromCharCode((bin[i>>5] >>> (i % 32)) & mask);
		}
		return str;
	}
	
	function binl2hex(binarray){
		var hex_tab = hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
		var str = "";
		for(var i = 0; i < binarray.length * 4; i++) {
			str += hex_tab.charAt((binarray[i>>2] >> ((i%4)*8+4)) & 0xF) + hex_tab.charAt((binarray[i>>2] >> ((i%4)*8  )) & 0xF);
		}
		return str;
	}
	return (raw ? binl2str(core_md5(str2binl(s), s.length * chrsz)) : binl2hex(core_md5(str2binl(s), s.length * chrsz))	);
};

/**
 * Plugin for adding a close context menu to tabs. Note that the menu respects
 * the closable configuration on the tab. As such, commands like remove others
 * and remove all will not remove items that are not closable.
 */
Ext.define('Ext.ux.TabCloseMenu', {
    alias: 'plugin.tabclosemenu',

    mixins: {
        observable: 'Ext.util.Observable'
    },

    /**
     * @cfg {String} closeTabText
     * The text for closing the current tab.
     */
    closeTabText: 'Close Tab',

    /**
     * @cfg {Boolean} showCloseOthers
     * Indicates whether to show the 'Close Others' option.
     */
    showCloseOthers: true,

    /**
     * @cfg {String} closeOtherTabsText
     * The text for closing all tabs except the current one.
     */
    closeOthersTabsText: 'Close Other Tabs',

    /**
     * @cfg {Boolean} showCloseAll
     * Indicates whether to show the 'Close All' option.
     */
    showCloseAll: true,

    /**
     * @cfg {String} closeAllTabsText
     * <p>The text for closing all tabs.
     */
    closeAllTabsText: 'Close All Tabs',

    /**
     * @cfg {Array} extraItemsHead
     * An array of additional context menu items to add to the front of the context menu.
     */
    extraItemsHead: null,

    /**
     * @cfg {Array} extraItemsTail
     * An array of additional context menu items to add to the end of the context menu.
     */
    extraItemsTail: null,

    //public
    constructor: function (config) {
        this.addEvents(
            'aftermenu',
            'beforemenu');

        this.mixins.observable.constructor.call(this, config);
    },

    init : function(tabpanel){
        this.tabPanel = tabpanel;
        this.tabBar = tabpanel.down("tabbar");

        this.mon(this.tabPanel, {
            scope: this,
            afterlayout: this.onAfterLayout,
            single: true
        });
    },

    onAfterLayout: function() {
        this.mon(this.tabBar.el, {
            scope: this,
            contextmenu: this.onContextMenu,
            delegate: 'div.x-tab'
        });
    },

    onBeforeDestroy : function(){
        Ext.destroy(this.menu);
        this.callParent(arguments);
    },

    // private
    onContextMenu : function(event, target){
        var me = this,
            menu = me.createMenu(),
            disableAll = true,
            disableOthers = true,
            tab = me.tabBar.getChildByElement(target),
            index = me.tabBar.items.indexOf(tab);

        me.item = me.tabPanel.getComponent(index);
        menu.child('*[text="' + me.closeTabText + '"]').setDisabled(!me.item.closable);

        if (me.showCloseAll || me.showCloseOthers) {
            me.tabPanel.items.each(function(item) {
                if (item.closable) {
                    disableAll = false;
                    if (item != me.item) {
                        disableOthers = false;
                        return false;
                    }
                }
                return true;
            });

            if (me.showCloseAll) {
                menu.child('*[text="' + me.closeAllTabsText + '"]').setDisabled(disableAll);
            }

            if (me.showCloseOthers) {
                menu.child('*[text="' + me.closeOthersTabsText + '"]').setDisabled(disableOthers);
            }
        }

        event.preventDefault();
        me.fireEvent('beforemenu', menu, me.item, me);

        menu.showAt(event.getXY());
    },

    createMenu : function() {
        var me = this;

        if (!me.menu) {
            var items = [{
                text: me.closeTabText,
                scope: me,
                handler: me.onClose
            }];

            if (me.showCloseAll || me.showCloseOthers) {
                items.push('-');
            }

            if (me.showCloseOthers) {
                items.push({
                    text: me.closeOthersTabsText,
                    scope: me,
                    handler: me.onCloseOthers
                });
            }

            if (me.showCloseAll) {
                items.push({
                    text: me.closeAllTabsText,
                    scope: me,
                    handler: me.onCloseAll
                });
            }

            if (me.extraItemsHead) {
                items = me.extraItemsHead.concat(items);
            }

            if (me.extraItemsTail) {
                items = items.concat(me.extraItemsTail);
            }

            me.menu = Ext.create('Ext.menu.Menu', {
                items: items,
                listeners: {
                    hide: me.onHideMenu,
                    scope: me
                }
            });
        }

        return me.menu;
    },

    onHideMenu: function () {
        var me = this;

        me.item = null;
        me.fireEvent('aftermenu', me.menu, me);
    },

    onClose : function(){
        this.tabPanel.remove(this.item);
    },

    onCloseOthers : function(){
        this.doClose(true);
    },

    onCloseAll : function(){
        this.doClose(false);
    },

    doClose : function(excludeActive){
        var items = [];

        this.tabPanel.items.each(function(item){
            if(item.closable){
                if(!excludeActive || item != this.item){
                    items.push(item);
                }
            }
        }, this);

        Ext.each(items, function(item){
            this.tabPanel.remove(item);
        }, this);
    }
});




/**
 * A control that allows selection of multiple items in a list
 */
Ext.define('Ext.ux.form.MultiSelect', {
    
    extend: 'Ext.form.FieldContainer',
    
    mixins: {
        bindable: 'Ext.util.Bindable',
        field: 'Ext.form.field.Field'    
    },
    
    alternateClassName: 'Ext.ux.Multiselect',
    alias: ['widget.multiselectfield', 'widget.multiselect'],
    
    requires: ['Ext.panel.Panel', 'Ext.view.BoundList'],
    
    uses: ['Ext.view.DragZone', 'Ext.view.DropZone'],
    
    /**
     * @cfg {String} [dragGroup=""] The ddgroup name for the MultiSelect DragZone.
     */

    /**
     * @cfg {String} [dropGroup=""] The ddgroup name for the MultiSelect DropZone.
     */
    
    /**
     * @cfg {String} [title=""] A title for the underlying panel.
     */
    
    /**
     * @cfg {Boolean} [ddReorder=false] Whether the items in the MultiSelect list are drag/drop reorderable.
     */
    ddReorder: false,

    /**
     * @cfg {Object/Array} tbar An optional toolbar to be inserted at the top of the control's selection list.
     * This can be a {@link Ext.toolbar.Toolbar} object, a toolbar config, or an array of buttons/button configs
     * to be added to the toolbar. See {@link Ext.panel.Panel#tbar}.
     */

    /**
     * @cfg {String} [appendOnly=false] True if the list should only allow append drops when drag/drop is enabled.
     * This is useful for lists which are sorted.
     */
    appendOnly: false,

    /**
     * @cfg {String} [displayField="text"] Name of the desired display field in the dataset.
     */
    displayField: 'text',

    /**
     * @cfg {String} [valueField="text"] Name of the desired value field in the dataset.
     */

    /**
     * @cfg {Boolean} [allowBlank=true] False to require at least one item in the list to be selected, true to allow no
     * selection.
     */
    allowBlank: true,

    /**
     * @cfg {Number} [minSelections=0] Minimum number of selections allowed.
     */
    minSelections: 0,

    /**
     * @cfg {Number} [maxSelections=Number.MAX_VALUE] Maximum number of selections allowed.
     */
    maxSelections: Number.MAX_VALUE,

    /**
     * @cfg {String} [blankText="This field is required"] Default text displayed when the control contains no items.
     */
    blankText: 'This field is required',

    /**
     * @cfg {String} [minSelectionsText="Minimum {0}item(s) required"] 
     * Validation message displayed when {@link #minSelections} is not met. 
     * The {0} token will be replaced by the value of {@link #minSelections}.
     */
    minSelectionsText: 'Minimum {0} item(s) required',
    
    /**
     * @cfg {String} [maxSelectionsText="Maximum {0}item(s) allowed"] 
     * Validation message displayed when {@link #maxSelections} is not met
     * The {0} token will be replaced by the value of {@link #maxSelections}.
     */
    maxSelectionsText: 'Minimum {0} item(s) required',

    /**
     * @cfg {String} [delimiter=","] The string used to delimit the selected values when {@link #getSubmitValue submitting}
     * the field as part of a form. If you wish to have the selected values submitted as separate
     * parameters rather than a single delimited parameter, set this to <tt>null</tt>.
     */
    delimiter: ',',

    /**
     * @cfg {Ext.data.Store/Array} store The data source to which this MultiSelect is bound (defaults to <tt>undefined</tt>).
     * Acceptable values for this property are:
     * <div class="mdetail-params"><ul>
     * <li><b>any {@link Ext.data.Store Store} subclass</b></li>
     * <li><b>an Array</b> : Arrays will be converted to a {@link Ext.data.ArrayStore} internally.
     * <div class="mdetail-params"><ul>
     * <li><b>1-dimensional array</b> : (e.g., <tt>['Foo','Bar']</tt>)<div class="sub-desc">
     * A 1-dimensional array will automatically be expanded (each array item will be the combo
     * {@link #valueField value} and {@link #displayField text})</div></li>
     * <li><b>2-dimensional array</b> : (e.g., <tt>[['f','Foo'],['b','Bar']]</tt>)<div class="sub-desc">
     * For a multi-dimensional array, the value in index 0 of each item will be assumed to be the combo
     * {@link #valueField value}, while the value at index 1 is assumed to be the combo {@link #displayField text}.
     * </div></li></ul></div></li></ul></div>
     */
    
    ignoreSelectChange: 0,
    
    initComponent: function(){
        var me = this;

        me.bindStore(me.store, true);
        if (me.store.autoCreated) {
            me.valueField = me.displayField = 'field1';
            if (!me.store.expanded) {
                me.displayField = 'field2';
            }
        }

        if (!Ext.isDefined(me.valueField)) {
            me.valueField = me.displayField;
        }
        Ext.apply(me, me.setupItems());
        
        
        me.callParent();
        me.initField();
        me.addEvents('drop');    
    },
    
    setupItems: function() {
        var me = this;
        
        me.boundList = Ext.create('Ext.view.BoundList', {
            deferInitialRefresh: false,
            multiSelect: true,
            store: me.store,
            displayField: me.displayField,
            disabled: me.disabled
        });
        
        me.boundList.getSelectionModel().on('selectionchange', me.onSelectChange, me);
        return {
            layout: 'fit',
            title: me.title,
            tbar: me.tbar,
            items: me.boundList
        };
    },
    
    onSelectChange: function(selModel, selections){
        if (!this.ignoreSelectChange) {
            this.setValue(selections);
        }    
    },
    
    getSelected: function(){
        return this.boundList.getSelectionModel().getSelection();
    },
    
    // compare array values
    isEqual: function(v1, v2) {
        var fromArray = Ext.Array.from,
            i = 0, 
            len;

        v1 = fromArray(v1);
        v2 = fromArray(v2);
        len = v1.length;

        if (len !== v2.length) {
            return false;
        }

        for(; i < len; i++) {
            if (v2[i] !== v1[i]) {
                return false;
            }
        }

        return true;
    },
    
    afterRender: function(){
        var me = this;
        
        me.callParent();
        if (me.selectOnRender) {
            ++me.ignoreSelectChange;
            me.boundList.getSelectionModel().select(me.getRecordsForValue(me.value));
            --me.ignoreSelectChange;
            delete me.toSelect;
        }    
        
        if (me.ddReorder && !me.dragGroup && !me.dropGroup){
            me.dragGroup = me.dropGroup = 'MultiselectDD-' + Ext.id();
        }

        if (me.draggable || me.dragGroup){
            me.dragZone = Ext.create('Ext.view.DragZone', {
                view: me.boundList,
                ddGroup: me.dragGroup,
                dragText: '{0} Item{1}'
            });
        }
        if (me.droppable || me.dropGroup){
            me.dropZone = Ext.create('Ext.view.DropZone', {
                view: me.boundList,
                ddGroup: me.dropGroup,
                handleNodeDrop: function(data, dropRecord, position) {
                    var view = this.view,
                        store = view.getStore(),
                        records = data.records,
                        index;

                    // remove the Models from the source Store
                    data.view.store.remove(records);

                    index = store.indexOf(dropRecord);
                    if (position === 'after') {
                        index++;
                    }
                    store.insert(index, records);
                    view.getSelectionModel().select(records);
                    me.fireEvent('drop', me, records);
                }
            });
        }
    },
    
    isValid : function() {
        var me = this,
            disabled = me.disabled,
            validate = me.forceValidation || !disabled;
            
        
        return validate ? me.validateValue(me.value) : disabled;
    },
    
    validateValue: function(value) {
        var me = this,
            errors = me.getErrors(value),
            isValid = Ext.isEmpty(errors);
            
        if (!me.preventMark) {
            if (isValid) {
                me.clearInvalid();
            } else {
                me.markInvalid(errors);
            }
        }

        return isValid;
    },
    
    markInvalid : function(errors) {
        // Save the message and fire the 'invalid' event
        var me = this,
            oldMsg = me.getActiveError();
        me.setActiveErrors(Ext.Array.from(errors));
        if (oldMsg !== me.getActiveError()) {
            me.updateLayout();
        }
    },

    /**
     * Clear any invalid styles/messages for this field.
     *
     * **Note**: this method does not cause the Field's {@link #validate} or {@link #isValid} methods to return `true`
     * if the value does not _pass_ validation. So simply clearing a field's errors will not necessarily allow
     * submission of forms submitted with the {@link Ext.form.action.Submit#clientValidation} option set.
     */
    clearInvalid : function() {
        // Clear the message and fire the 'valid' event
        var me = this,
            hadError = me.hasActiveError();
        me.unsetActiveError();
        if (hadError) {
            me.updateLayout();
        }
    },
    
    getSubmitData: function() {
        var me = this,
            data = null,
            val;
        if (!me.disabled && me.submitValue && !me.isFileUpload()) {
            val = me.getSubmitValue();
            if (val !== null) {
                data = {};
                data[me.getName()] = val;
            }
        }
        return data;
    },

    /**
     * Returns the value that would be included in a standard form submit for this field.
     *
     * @return {String} The value to be submitted, or null.
     */
    getSubmitValue: function() {
        var me = this,
            delimiter = me.delimiter,
            val = me.getValue();
            
        return Ext.isString(delimiter) ? val.join(delimiter) : val;
    },
    
    getValue: function(){
        return this.value;
    },
    
    getRecordsForValue: function(value){
        var me = this,
            records = [],
            all = me.store.getRange(),
            valueField = me.valueField,
            i = 0,
            allLen = all.length,
            rec,
            j,
            valueLen;
            
        for (valueLen = value.length; i < valueLen; ++i) {
            for (j = 0; j < allLen; ++j) {
                rec = all[j];   
                if (rec.get(valueField) == value[i]) {
                    records.push(rec);
                }
            }    
        }
            
        return records;
    },
    
    setupValue: function(value){
        var delimiter = this.delimiter,
            valueField = this.valueField,
            i = 0,
            len,
            item;
            
        if (delimiter && Ext.isString(value)) {
            value = value.split(delimiter);
        } else if (!Ext.isArray(value)) {
            value = [value];
        }
        
        for (len = value.length; i < len; ++i) {
            item = value[i];
            if (item && item.isModel) {
                value[i] = item.get(valueField);
            }
        }
        return Ext.Array.unique(value);
    },
    
    setValue: function(value){
        var me = this,
            selModel = me.boundList.getSelectionModel();
        
        value = me.setupValue(value);
        me.mixins.field.setValue.call(me, value);
        
        if (me.rendered) {
            ++me.ignoreSelectChange;
            selModel.deselectAll();
            selModel.select(me.getRecordsForValue(value));
            --me.ignoreSelectChange;
        } else {
            me.selectOnRender = true;
        }
    },
    
    clearValue: function(){
        this.setValue([]);    
    },
    
    onEnable: function(){
        var list = this.boundList;
        this.callParent();
        if (list) {
            list.enable();
        }
    },
    
    onDisable: function(){
        var list = this.boundList;
        this.callParent();
        if (list) {
            list.disable();
        }
    },
    
    getErrors : function(value) {
        var me = this,
            format = Ext.String.format,
            errors = [],
            numSelected;

        value = Ext.Array.from(value || me.getValue());
        numSelected = value.length;

        if (!me.allowBlank && numSelected < 1) {
            errors.push(me.blankText);
        }
        if (numSelected < me.minSelections) {
            errors.push(format(me.minSelectionsText, me.minSelections));
        }
        if (numSelected > me.maxSelections) {
            errors.push(format(me.maxSelectionsText, me.maxSelections));
        }
        return errors;
    },
    
    onDestroy: function(){
        var me = this;
        
        me.bindStore(null);
        Ext.destroy(me.dragZone, me.dropZone);
        me.callParent();
    },
    
    onBindStore: function(store){
        var boundList = this.boundList;
        
        if (boundList) {
            boundList.bindStore(store);
        }
    }
    
});

/*
 * Note that this control will most likely remain as an example, and not as a core Ext form
 * control.  However, the API will be changing in a future release and so should not yet be
 * treated as a final, stable API at this time.
 */

/**
 * A control that allows selection of between two Ext.ux.form.MultiSelect controls.
 */
Ext.define('Ext.ux.form.ItemSelector', {
    extend: 'Ext.ux.form.MultiSelect',
    alias: ['widget.itemselectorfield', 'widget.itemselector'],
    alternateClassName: ['Ext.ux.ItemSelector'],
    requires: [
        'Ext.button.Button',
        'Ext.ux.form.MultiSelect'
    ],
    
    /**
     * @cfg {Boolean} [hideNavIcons=false] True to hide the navigation icons
     */
    hideNavIcons:false,

    /**
     * @cfg {Array} buttons Defines the set of buttons that should be displayed in between the ItemSelector
     * fields. Defaults to <tt>['top', 'up', 'add', 'remove', 'down', 'bottom']</tt>. These names are used
     * to build the button CSS class names, and to look up the button text labels in {@link #buttonsText}.
     * This can be overridden with a custom Array to change which buttons are displayed or their order.
     */
    buttons: ['top', 'up', 'add', 'remove', 'down', 'bottom'],

    /**
     * @cfg {Object} buttonsText The tooltips for the {@link #buttons}.
     * Labels for buttons.
     */
    buttonsText: {
        top: "Move to Top",
        up: "Move Up",
        add: "Add to Selected",
        remove: "Remove from Selected",
        down: "Move Down",
        bottom: "Move to Bottom"
    },
    
    initComponent: function(){
        this.ddGroup = this.id + '-dd';
        this.callParent();
    },
    
    createList: function(){
        var me = this;
        
        return Ext.create('Ext.ux.form.MultiSelect', {
            submitValue: false,
            flex: 1,
            dragGroup: me.ddGroup,
            dropGroup: me.ddGroup,
            store: {
                model: me.store.model,
                data: []
            },
            displayField: me.displayField,
            disabled: me.disabled,
            listeners: {
                boundList: {
                    scope: me,
                    itemdblclick: me.onItemDblClick,
                    drop: me.syncValue
                }
            }
        });
    },

    setupItems: function() {
        var me = this;
        
        me.fromField = me.createList();
        me.toField = me.createList();
        
        // add everything to the from field at the start
        me.fromField.store.add(me.store.getRange());
        
        return {
            layout: {
                type: 'hbox',
                align: 'stretch'
            },
            items: [
                me.fromField,
                {
                    xtype: 'container',
                    margins: '0 4',
                    width: 22,
                    layout: {
                        type: 'vbox',
                        pack: 'center'
                    },
                    items: me.createButtons()
                },
                me.toField
            ]
        };
    },
    
    createButtons: function(){
        var me = this,
            buttons = [];
            
        if (!me.hideNavIcons) {
            Ext.Array.forEach(me.buttons, function(name) {
                buttons.push({
                    xtype: 'button',
                    tooltip: me.buttonsText[name],
                    handler: me['on' + Ext.String.capitalize(name) + 'BtnClick'],
                    cls: Ext.baseCSSPrefix + 'form-itemselector-btn',
                    iconCls: Ext.baseCSSPrefix + 'form-itemselector-' + name,
                    navBtn: true,
                    scope: me,
                    margin: '4 0 0 0'
                });
            });
        }
        return buttons;
    },
    
    getSelections: function(list){
        var store = list.getStore(),
            selections = list.getSelectionModel().getSelection(),
            i = 0,
            len = selections.length;
            
        return Ext.Array.sort(selections, function(a, b){
            a = store.indexOf(a);
            b = store.indexOf(b);
            
            if (a < b) {
                return -1;
            } else if (a > b) {
                return 1;
            }
            return 0;
        });
    },

    onTopBtnClick : function() {
        var list = this.toField.boundList,
            store = list.getStore(),
            selected = this.getSelections(list),
            i = selected.length - 1,
            selection;
        
        
        store.suspendEvents();
        for (; i > -1; --i) {
            selection = selected[i];
            store.remove(selected);
            store.insert(0, selected);
        }
        store.resumeEvents();
        list.refresh();   
        this.syncValue(); 
    },

    onBottomBtnClick : function() {
        var list = this.toField.boundList,
            store = list.getStore(),
            selected = this.getSelections(list),
            i = 0,
            len = selected.length,
            selection;
            
        store.suspendEvents();
        for (; i < len; ++i) {
            selection = selected[i];
            store.remove(selection);
            store.add(selection);
        }
        store.resumeEvents();
        list.refresh();
        this.syncValue();
    },

    onUpBtnClick : function() {
        var list = this.toField.boundList,
            store = list.getStore(),
            selected = this.getSelections(list),
            i = 0,
            len = selected.length,
            selection,
            index;
            
        store.suspendEvents();
        for (; i < len; ++i) {
            selection = selected[i];
            index = Math.max(0, store.indexOf(selection) - 1);
            store.remove(selection);
            store.insert(index, selection);
        }
        store.resumeEvents();
        list.refresh();
        this.syncValue();
    },

    onDownBtnClick : function() {
        var list = this.toField.boundList,
            store = list.getStore(),
            selected = this.getSelections(list),
            i = 0,
            len = selected.length,
            max = store.getCount(),
            selection,
            index;
            
        store.suspendEvents();
        for (; i < len; ++i) {
            selection = selected[i];
            index = Math.min(max, store.indexOf(selection) + 1);
            store.remove(selection);
            store.insert(index, selection);
        }
        store.resumeEvents();
        list.refresh();
        this.syncValue();
    },

    onAddBtnClick : function() {
        var me = this,
            fromList = me.fromField.boundList,
            selected = this.getSelections(fromList);
            
        fromList.getStore().remove(selected);
        this.toField.boundList.getStore().add(selected);
        this.syncValue();
    },

    onRemoveBtnClick : function() {
        var me = this,
            toList = me.toField.boundList,
            selected = this.getSelections(toList);
            
        toList.getStore().remove(selected);
        this.fromField.boundList.getStore().add(selected);
        this.syncValue();
    },
    
    syncValue: function(){
        this.setValue(this.toField.store.getRange()); 
    },
    
    onItemDblClick: function(view, rec){
        var me = this,
            from = me.fromField.store,
            to = me.toField.store,
            current,
            destination;
            
        if (from.indexOf(rec) > -1) {
            current = from;
            destination = to;
        } else {
            current = to;
            destination = from;
        }
        current.remove(rec);
        destination.add(rec);
        me.syncValue();
    },
    
    setValue: function(value){
        var me = this,
            fromStore = me.fromField.store,
            toStore = me.toField.store,
            selected;
        
        value = me.setupValue(value);
        me.mixins.field.setValue.call(me, value);
        
        selected = me.getRecordsForValue(value);
        
        Ext.Array.forEach(toStore.getRange(), function(rec){
            if (!Ext.Array.contains(selected, rec)) {
                // not in the selected group, remove it from the toStore
                toStore.remove(rec);
                fromStore.add(rec);
            }
        });
        toStore.removeAll();
        
        Ext.Array.forEach(selected, function(rec){
            // In the from store, move it over
            if (fromStore.indexOf(rec) > -1) {
                fromStore.remove(rec);     
            }
            toStore.add(rec);
        });
    },
    
    onBindStore: Ext.emptyFn,
    
    onEnable: function(){
        var me = this;
        
        me.callParent();
        me.fromField.enable();
        me.toField.enable();
        
        Ext.Array.forEach(me.query('[navBtn]'), function(btn){
            btn.enable();
        });
    },
    
    onDisable: function(){
        var me = this;
        
        me.callParent();
        me.fromField.disable();
        me.toField.disable();
        
        Ext.Array.forEach(me.query('[navBtn]'), function(btn){
            btn.disable();
        });
    },
    
    onDestroy: function(){
        this.bindStore(null);
        this.callParent();
    }
});
/**
 * @class Ext.ux.StatusBar
 * <p>Basic status bar component that can be used as the bottom toolbar of any {@link Ext.Panel}.  In addition to
 * supporting the standard {@link Ext.toolbar.Toolbar} interface for adding buttons, menus and other items, the StatusBar
 * provides a greedy status element that can be aligned to either side and has convenient methods for setting the
 * status text and icon.  You can also indicate that something is processing using the {@link #showBusy} method.</p>
 * <pre><code>
Ext.create('Ext.Panel', {
    title: 'StatusBar',
    // etc.
    bbar: Ext.create('Ext.ux.StatusBar', {
        id: 'my-status',

        // defaults to use when the status is cleared:
        defaultText: 'Default status text',
        defaultIconCls: 'default-icon',

        // values to set initially:
        text: 'Ready',
        iconCls: 'ready-icon',

        // any standard Toolbar items:
        items: [{
            text: 'A Button'
        }, '-', 'Plain Text']
    })
});

// Update the status bar later in code:
var sb = Ext.getCmp('my-status');
sb.setStatus({
    text: 'OK',
    iconCls: 'ok-icon',
    clear: true // auto-clear after a set interval
});

// Set the status bar to show that something is processing:
sb.showBusy();

// processing....

sb.clearStatus(); // once completeed
</code></pre>
 * @extends Ext.toolbar.Toolbar
 * @constructor
 * Creates a new StatusBar
 * @param {Object/Array} config A config object
 */
Ext.define('Ext.ux.statusbar.StatusBar', {
    extend: 'Ext.toolbar.Toolbar',
    alternateClassName: 'Ext.ux.StatusBar',
    alias: 'widget.statusbar',
    requires: ['Ext.toolbar.TextItem'],
    /**
     * @cfg {String} statusAlign
     * The alignment of the status element within the overall StatusBar layout.  When the StatusBar is rendered,
     * it creates an internal div containing the status text and icon.  Any additional Toolbar items added in the
     * StatusBar's {@link #items} config, or added via {@link #add} or any of the supported add* methods, will be
     * rendered, in added order, to the opposite side.  The status element is greedy, so it will automatically
     * expand to take up all sapce left over by any other items.  Example usage:
     * <pre><code>
// Create a left-aligned status bar containing a button,
// separator and text item that will be right-aligned (default):
Ext.create('Ext.Panel', {
    title: 'StatusBar',
    // etc.
    bbar: Ext.create('Ext.ux.StatusBar', {
        defaultText: 'Default status text',
        id: 'status-id',
        items: [{
            text: 'A Button'
        }, '-', 'Plain Text']
    })
});

// By adding the statusAlign config, this will create the
// exact same toolbar, except the status and toolbar item
// layout will be reversed from the previous example:
Ext.create('Ext.Panel', {
    title: 'StatusBar',
    // etc.
    bbar: Ext.create('Ext.ux.StatusBar', {
        defaultText: 'Default status text',
        id: 'status-id',
        statusAlign: 'right',
        items: [{
            text: 'A Button'
        }, '-', 'Plain Text']
    })
});
</code></pre>
     */
    /**
     * @cfg {String} defaultText
     * The default {@link #text} value.  This will be used anytime the status bar is cleared with the
     * <tt>useDefaults:true</tt> option (defaults to '').
     */
    /**
     * @cfg {String} defaultIconCls
     * The default {@link #iconCls} value (see the iconCls docs for additional details about customizing the icon).
     * This will be used anytime the status bar is cleared with the <tt>useDefaults:true</tt> option (defaults to '').
     */
    /**
     * @cfg {String} text
     * A string that will be <b>initially</b> set as the status message.  This string
     * will be set as innerHTML (html tags are accepted) for the toolbar item.
     * If not specified, the value set for <code>{@link #defaultText}</code>
     * will be used.
     */
    /**
     * @cfg {String} iconCls
     * A CSS class that will be <b>initially</b> set as the status bar icon and is
     * expected to provide a background image (defaults to '').
     * Example usage:<pre><code>
// Example CSS rule:
.x-statusbar .x-status-custom {
    padding-left: 25px;
    background: transparent url(images/custom-icon.gif) no-repeat 3px 2px;
}

// Setting a default icon:
var sb = Ext.create('Ext.ux.StatusBar', {
    defaultIconCls: 'x-status-custom'
});

// Changing the icon:
sb.setStatus({
    text: 'New status',
    iconCls: 'x-status-custom'
});
</code></pre>
     */

    /**
     * @cfg {String} cls
     * The base class applied to the containing element for this component on render (defaults to 'x-statusbar')
     */
    cls : 'x-statusbar',
    /**
     * @cfg {String} busyIconCls
     * The default <code>{@link #iconCls}</code> applied when calling
     * <code>{@link #showBusy}</code> (defaults to <tt>'x-status-busy'</tt>).
     * It can be overridden at any time by passing the <code>iconCls</code>
     * argument into <code>{@link #showBusy}</code>.
     */
    busyIconCls : 'x-status-busy',
    /**
     * @cfg {String} busyText
     * The default <code>{@link #text}</code> applied when calling
     * <code>{@link #showBusy}</code> (defaults to <tt>'Loading...'</tt>).
     * It can be overridden at any time by passing the <code>text</code>
     * argument into <code>{@link #showBusy}</code>.
     */
    busyText : 'Loading...',
    /**
     * @cfg {Number} autoClear
     * The number of milliseconds to wait after setting the status via
     * <code>{@link #setStatus}</code> before automatically clearing the status
     * text and icon (defaults to <tt>5000</tt>).  Note that this only applies
     * when passing the <tt>clear</tt> argument to <code>{@link #setStatus}</code>
     * since that is the only way to defer clearing the status.  This can
     * be overridden by specifying a different <tt>wait</tt> value in
     * <code>{@link #setStatus}</code>. Calls to <code>{@link #clearStatus}</code>
     * always clear the status bar immediately and ignore this value.
     */
    autoClear : 5000,

    /**
     * @cfg {String} emptyText
     * The text string to use if no text has been set.  Defaults to
     * <tt>'&nbsp;'</tt>).  If there are no other items in the toolbar using
     * an empty string (<tt>''</tt>) for this value would end up in the toolbar
     * height collapsing since the empty string will not maintain the toolbar
     * height.  Use <tt>''</tt> if the toolbar should collapse in height
     * vertically when no text is specified and there are no other items in
     * the toolbar.
     */
    emptyText : '&nbsp;',

    // private
    activeThreadId : 0,

    // private
    initComponent : function(){
        if (this.statusAlign === 'right') {
            this.cls += ' x-status-right';
        }
        this.callParent(arguments);
    },

    // private
    afterRender : function(){
        this.callParent(arguments);

        var right = this.statusAlign === 'right';
        this.currIconCls = this.iconCls || this.defaultIconCls;
        this.statusEl = Ext.create('Ext.toolbar.TextItem', {
            cls: 'x-status-text ' + (this.currIconCls || ''),
            text: this.text || this.defaultText || ''
        });

        if (right) {
            this.add('->');
            this.add(this.statusEl);
        } else {
            this.insert(0, this.statusEl);
            this.insert(1, '->');
        }
        this.height = 27;
        this.doLayout();
    },

    /**
     * Sets the status {@link #text} and/or {@link #iconCls}. Also supports automatically clearing the
     * status that was set after a specified interval.
     * @param {Object/String} config A config object specifying what status to set, or a string assumed
     * to be the status text (and all other options are defaulted as explained below). A config
     * object containing any or all of the following properties can be passed:<ul>
     * <li><tt>text</tt> {String} : (optional) The status text to display.  If not specified, any current
     * status text will remain unchanged.</li>
     * <li><tt>iconCls</tt> {String} : (optional) The CSS class used to customize the status icon (see
     * {@link #iconCls} for details). If not specified, any current iconCls will remain unchanged.</li>
     * <li><tt>clear</tt> {Boolean/Number/Object} : (optional) Allows you to set an internal callback that will
     * automatically clear the status text and iconCls after a specified amount of time has passed. If clear is not
     * specified, the new status will not be auto-cleared and will stay until updated again or cleared using
     * {@link #clearStatus}. If <tt>true</tt> is passed, the status will be cleared using {@link #autoClear},
     * {@link #defaultText} and {@link #defaultIconCls} via a fade out animation. If a numeric value is passed,
     * it will be used as the callback interval (in milliseconds), overriding the {@link #autoClear} value.
     * All other options will be defaulted as with the boolean option.  To customize any other options,
     * you can pass an object in the format:<ul>
     *    <li><tt>wait</tt> {Number} : (optional) The number of milliseconds to wait before clearing
     *    (defaults to {@link #autoClear}).</li>
     *    <li><tt>anim</tt> {Number} : (optional) False to clear the status immediately once the callback
     *    executes (defaults to true which fades the status out).</li>
     *    <li><tt>useDefaults</tt> {Number} : (optional) False to completely clear the status text and iconCls
     *    (defaults to true which uses {@link #defaultText} and {@link #defaultIconCls}).</li>
     * </ul></li></ul>
     * Example usage:<pre><code>
// Simple call to update the text
statusBar.setStatus('New status');

// Set the status and icon, auto-clearing with default options:
statusBar.setStatus({
    text: 'New status',
    iconCls: 'x-status-custom',
    clear: true
});

// Auto-clear with custom options:
statusBar.setStatus({
    text: 'New status',
    iconCls: 'x-status-custom',
    clear: {
        wait: 8000,
        anim: false,
        useDefaults: false
    }
});
</code></pre>
     * @return {Ext.ux.StatusBar} this
     */
    setStatus : function(o) {
        o = o || {};

        if (Ext.isString(o)) {
            o = {text:o};
        }
        if (o.text !== undefined) {
            this.setText(o.text);
        }
        if (o.iconCls !== undefined) {
            this.setIcon(o.iconCls);
        }

        if (o.clear) {
            var c = o.clear,
                wait = this.autoClear,
                defaults = {useDefaults: true, anim: true};

            if (Ext.isObject(c)) {
                c = Ext.applyIf(c, defaults);
                if (c.wait) {
                    wait = c.wait;
                }
            } else if (Ext.isNumber(c)) {
                wait = c;
                c = defaults;
            } else if (Ext.isBoolean(c)) {
                c = defaults;
            }

            c.threadId = this.activeThreadId;
            Ext.defer(this.clearStatus, wait, this, [c]);
        }
        this.doLayout();
        return this;
    },

    /**
     * Clears the status {@link #text} and {@link #iconCls}. Also supports clearing via an optional fade out animation.
     * @param {Object} config (optional) A config object containing any or all of the following properties.  If this
     * object is not specified the status will be cleared using the defaults below:<ul>
     * <li><tt>anim</tt> {Boolean} : (optional) True to clear the status by fading out the status element (defaults
     * to false which clears immediately).</li>
     * <li><tt>useDefaults</tt> {Boolean} : (optional) True to reset the text and icon using {@link #defaultText} and
     * {@link #defaultIconCls} (defaults to false which sets the text to '' and removes any existing icon class).</li>
     * </ul>
     * @return {Ext.ux.StatusBar} this
     */
    clearStatus : function(o) {
        o = o || {};

        if (o.threadId && o.threadId !== this.activeThreadId) {
            // this means the current call was made internally, but a newer
            // thread has set a message since this call was deferred.  Since
            // we don't want to overwrite a newer message just ignore.
            return this;
        }

        var text = o.useDefaults ? this.defaultText : this.emptyText,
            iconCls = o.useDefaults ? (this.defaultIconCls ? this.defaultIconCls : '') : '';

        if (o.anim) {
            // animate the statusEl Ext.core.Element
            this.statusEl.el.puff({
                remove: false,
                useDisplay: true,
                scope: this,
                callback: function(){
                    this.setStatus({
                     text: text,
                     iconCls: iconCls
                 });

                    this.statusEl.el.show();
                }
            });
        } else {
            // hide/show the el to avoid jumpy text or icon
             this.statusEl.hide();
             this.setStatus({
                 text: text,
                 iconCls: iconCls
             });
             this.statusEl.show();
        }
        this.doLayout();
        return this;
    },

    /**
     * Convenience method for setting the status text directly.  For more flexible options see {@link #setStatus}.
     * @param {String} text (optional) The text to set (defaults to '')
     * @return {Ext.ux.StatusBar} this
     */
    setText : function(text){
        this.activeThreadId++;
        this.text = text || '';
        if (this.rendered) {
            this.statusEl.setText(this.text);
        }
        return this;
    },

    /**
     * Returns the current status text.
     * @return {String} The status text
     */
    getText : function(){
        return this.text;
    },

    /**
     * Convenience method for setting the status icon directly.  For more flexible options see {@link #setStatus}.
     * See {@link #iconCls} for complete details about customizing the icon.
     * @param {String} iconCls (optional) The icon class to set (defaults to '', and any current icon class is removed)
     * @return {Ext.ux.StatusBar} this
     */
    setIcon : function(cls){
        this.activeThreadId++;
        cls = cls || '';

        if (this.rendered) {
         if (this.currIconCls) {
             this.statusEl.removeCls(this.currIconCls);
             this.currIconCls = null;
         }
         if (cls.length > 0) {
             this.statusEl.addCls(cls);
             this.currIconCls = cls;
         }
        } else {
            this.currIconCls = cls;
        }
        return this;
    },

    /**
     * Convenience method for setting the status text and icon to special values that are pre-configured to indicate
     * a "busy" state, usually for loading or processing activities.
     * @param {Object/String} config (optional) A config object in the same format supported by {@link #setStatus}, or a
     * string to use as the status text (in which case all other options for setStatus will be defaulted).  Use the
     * <tt>text</tt> and/or <tt>iconCls</tt> properties on the config to override the default {@link #busyText}
     * and {@link #busyIconCls} settings. If the config argument is not specified, {@link #busyText} and
     * {@link #busyIconCls} will be used in conjunction with all of the default options for {@link #setStatus}.
     * @return {Ext.ux.StatusBar} this
     */
    showBusy : function(o){
        if (Ext.isString(o)) {
            o = { text: o };
        }
        o = Ext.applyIf(o || {}, {
            text: this.busyText,
            iconCls: this.busyIconCls
        });
        return this.setStatus(o);
    }
});Ext.define("dnet.core.base.UserPreferences", {
	extend : "Ext.window.Window",

	initComponent : function(config) {  
		var cfg = {
			title : Dnet.translate("msg", "preferences_wdw"),
			border : true,
			width : 350,
			height:120,
			resizable : true,
			//closeAction : "hide",			 
			closable : true,
			constrain : true,
			buttonAlign : "center",
			defaults:{
		  	   labelAlign:"right",
		  	   labelWidth:150  
		  	},
			modal : true,
			layout:"form",
			items : [{
	            xtype: 'checkbox',
	           // fieldLabel : Dnet.translate("msg", "remember_view_state"),
	             fieldLabel :"Use focus manager",
	            //value: getApplication().getSession().rememberViewState,
	            //checked: getApplication().getSession().rememberViewState,
	            listeners: {
	            	change: {
	            		scope:this,
	            		fn: function(field,nv,ov) {
	            			getApplication().getSession().useFocusManager = nv;
	            		}
	            	},
	            	afterrender: {
	            		scope:this,
	            		fn: function(field ) {
	            			field.setValue(getApplication().getSession().useFocusManager); // = nv;
	            		}
	            	}
	            } 
	        }],
			buttons : [{
				xtype:"button",
				text : Dnet.translate("tlbitem", "ok__lbl"),
				scope:this,
				handler: function() { this.close();}
			}]
		};

		Ext.apply(cfg, config);
		Ext.apply(this, cfg);
		this.callParent(arguments);
	}
});Ext.ns("dnet.core.base");
dnet.core.base.Logger = function() {
	return {
		doLogging : false /*(Ext.isDefined(console))? true: false */ 
		,info: function(msg) {
			if (this.doLogging) {
				console.info(msg);
			}
		}
		,warn: function(msg) {
			if (this.doLogging) {
				console.warn(msg);
			}
		}
		,error: function(msg) {
			if (this.doLogging) {
				console.error(msg);
			}
		}
		,debug: function(msg) {
			if (this.doLogging) {
				console.debug(msg);
			}
		}
	}
}();Ext.define('dnet.core.base.NavigationTree$Model', {
	extend : 'Ext.data.Model',
	fields : [ {
		name : 'id',
		type : 'string'
	}, {
		name : 'text',
		type : 'string'
	}, {
		name : '_frame_',
		type : 'string'
	}, {
		name : '_bundle_',
		type : 'string'
	}]
});

Ext.define("dnet.core.base.NavigationTree", {
	extend : "Ext.tree.Panel",
	alias : "widget.dnetNavigationTree",

	// DNet properties

	_TEXT_EXPAND_ALL : "Expand all",
	_TEXT_COLLAPSE_ALL : "Collapse all",
	_TEXT_FILTER_FIELD : "Filter...",
	_TEXT_REFRESH : "Refresh",
	withStdFilterHeader : false,
	loader_PreloadChildren : true,
	loader_Url : null,

	// defaults
	
	//displayField: "title",
	rootVisible : false,
	lines : false,
	 
	animate : false,
	border : false,
	useArrows : true,
	bodyBorder : false,
	folderSort : false,
	initComponent : function(config) {

		this.store = Ext.create('Ext.data.TreeStore', {
			sortOnLoad: false,
			isSortable: false,
	        proxy: {
	            type: 'ajax',
	            method : "POST",
	            url: Dnet.dsAPI("MenuItemRtLovDs", "json").read,
	            actionMethods : {					 
					read : 'POST' 					 
				},
				reader : {
					type : 'json',
					root : 'data',
					idProperty : 'id',
					totalProperty : 'totalCount',
					messageProperty : 'message'
				},
				listeners : {
					"exception" : {
						fn : this.proxyException,
						scope : this
					}
				}
	        },
	        root: {
	            text: 'Ext JS',
	            id: 'src',
	            expanded: true
	        },
	        folderSort: true,
	        sorters: [{
	            property: 'text',
	            direction: 'ASC'
	        }] ,
	        listeners: {
	        	beforeload: {
	        		scope:this,
	        		fn:  function( store,  operation,  eOpts ) {	
	        			//alert("store-beforeload");
						store.proxy.extraParams.data = Ext.encode({
							menu : this._menuName_
						});
					}
	        	}
	        }
	    });

    
		this.hiddenPkgs = [];

		var cfg = {}
		if (this.withStdFilterHeader) { /*
										 * cfg.tbar =
										 * this._createFilterHeader();
										 */
		}

		this.callParent(arguments);

		this.addEvents("openMenuLink", "openMenuLinkInNewTab", "refreshCatalog");
		
		this.on("beforeload", this.beforeStoreLoad, this)
		 		
	},

	initEvents : function() {
		this.callParent(arguments);
		this.on("itemclick", this.onMenuClick, this);
	},

	beforeStoreLoad: function( store,  operation,  eOpts ) {
		//alert("tree-beforeload");
		if(operation.node.data) {
			store.proxy.extraParams.data = Ext.encode({
				menuItemId : operation.node.raw.id	
			});
		} else {
			store.proxy.extraParams.data = Ext.encode({
				menu : this._menuName_		
			});
		}
		
	},
	
	filterTree : function(t, e) {
		var text = t.getValue();
		Ext.each(this.hiddenPkgs, function(n) {
			n.ui.show();
		});
		if (!text) {
			this.filter.clear();
			return;
		}
		this.expandAll();

		var re = new RegExp('^' + Ext.escapeRe(text), 'i');
		this.filter.filterBy(function(n) {
			return !n.leaf || (n.leaf && re.test(n.text));
		}, this);

		// hide empty packages that weren't filtered
	this.hiddenPkgs = [];
	this.root.cascade(function(n) {
		if (!n.leaf && n.ui.ctNode.offsetHeight < 3) {
			n.ui.hide();
			this.hiddenPkgs.push(n);
		}
	}, this);
} // end filterTree

	,
	onMenuClick : function(view, model, item, idx, evnt, opts) {
		if (model.data.leaf) {
			this.fireEvent("openMenuLink", model);
		}
	}

	,
	_createFilterHeader : function() {
		return [];
		var t = [ ' ', new Ext.form.TextField( {
			width : 120,
			emptyText : this._TEXT_FILTER_FIELD,
			enableKeyEvents : true,
			listeners : {
				render : function(f) {
					this.filter = new Ext.tree.TreeFilter(this, {
						clearBlank : true,
						autoClear : true
					});
				},
				keydown : {
					fn : this.filterTree,
					buffer : 350,
					scope : this
				},
				scope : this
			}
		}), ' ', '->', {
			iconCls : 'icon-action-expand-all',
			tooltip : this._TEXT_EXPAND_ALL,
			handler : this.handler_ActionExpandAll,
			scope : this
		}, '-', {
			iconCls : 'icon-action-collapse-all',
			tooltip : this._TEXT_COLLAPSE_ALL,
			handler : this.handler_ActionCollapseAll,
			scope : this
		}, '-', {
			iconCls : 'icon-action-refresh',
			tooltip : this._TEXT_REFRESH,
			handler : this.handler_ActionRefresh,
			scope : this
		} ];
		return t;

	},
	handler_ActionExpandAll : function() {
		this.root.expand(true);
	},
	handler_ActionCollapseAll : function() {
		this.root.collapse(true);
	},
	handler_ActionRefresh : function() {
		this.fireEvent("refreshCatalog", this);
	},
	afterRefreshCatalog : function(response, options) {
		Ext.MessageBox.hide();
		this.loader.load(this.root);
	},
	/**
	 * Default proxy-exception handler
	 */
	proxyException : function(proxy, response, operation, eOpts) {
		this.showAjaxErrors(response, eOpts);
	},
	
	/**
	 * Show errors to user. TODO: Externalize it as command.
	 */
	showAjaxErrors : function(response, options) {
		// Ext.MessageBox.hide();
		var msg, withDetails = false;
		if (response.responseText) {
			if (response.responseText.length > 2000) {
				msg = response.responseText.substr(0, 2000);
				withDetails = true;
			} else {
				msg = response.responseText;
			}
		} else {
			msg = "No response received from server.";
		}
		var alertCfg = {
			title : "Server message",	
			msg : msg,
			scope : this,
			icon : Ext.MessageBox.ERROR,
			buttons : Ext.MessageBox.OK
		}
		if (withDetails) {
			alertCfg.buttons['cancel'] = 'Details';
			alertCfg['detailedMessage'] = response.responseText;
		}
		Ext.Msg.show(alertCfg);

	}
});

dnet.core.base.TemplateRepository = {                                                                                                                                                                                  


   HOMEPANEL_FOOTER: "<div style='width:100%;'>"+                                                                                                                                                                           
            "<span style='float:left;'> Powered by  <a href='http://www.dnet-ebusiness-suite.com' target=_blank >DNet eBusiness Suite</a> &copy 2010-2012 Nan21 Electronics s.r.l. </span>"+                                                                 
         		"</div>"

   ,APPLICATION_HOME :
         "<table width='100%' border='0' align=center cellpadding='0' cellspacing='0' style='font-family: arial, sans-serif;line-height: 140%;font-size:10pt;'>"
       // +"<tr><td><img src='resource/images/plane_blue_apps.jpg' width='500px' vspace='0' border='0' alt='top_newsletter' title='Image' /></td></tr>"
        +"<tr><td><h2 style='text-align: left;font-family: arial, sans-serif;font-size:16pt;margin-top: 20px;' > {dnetName} </h2> </td></tr>"
        +"<tr><td><h5 style='text-align: left;font-family: arial, sans-serif;font-size:10pt;margin-top: 5px;' > Version: {dnetVersion} </h5> </td></tr>"
        +"<tr><td ><br>"
       
        +"  <ul>"
        +"  <li> Open source ERP / CRM / HRM</li>"
        +"  </ul>"
        + "  <a href='http://www.dnet-ebusiness-suite.com' target=_blank>www.dnet-ebusiness-suite.com</a>"
        +" <tr><td ><br> &copy; Nan21 Electronics SRL </td> </tr>"
        +"</td></tr>"
        +"</table>"
  ,APPLICATION_ABOUT_BOX:
         "<table width='100%' border='0' align=center cellpadding='0' cellspacing='0' style='font-family: arial, sans-serif;line-height: 140%;font-size:10pt; '>"
       // +"<tr><td><img src='resource/images/plane_blue_apps.jpg' width='500px' vspace='0' border='0' alt='top_newsletter' title='Image' /></td></tr>"
        +"<tr><td><h2 style='text-align: center;font-family: arial, sans-serif;font-size:16pt;margin-top: 20px;' > <br>{dnetName} </h2> </td></tr>"
        +"<tr><td><h5 style='text-align: center;font-family: arial, sans-serif;font-size:10pt;margin-top: 5px;' > Version: {dnetVersion} </h5> </td></tr>"
        +"<tr><td style='text-align: center;'>"
        +"  <br>Open source ERP / CRM / HRM."
      	+"  </td></tr>"
		+" <tr><td  style='text-align: center;'>  <a href='http://www.dnet-ebusiness-suite.com' target=_blank>www.dnet-ebusiness-suite.com</a> </td> </tr>"
		+" <tr><td  style='text-align: center;'><br> &copy; Nan21 Electronics SRL </td> </tr>"
        +"  </td></tr>"
        +"</table>"
  ,ABOUT_DASHBOARD : "" +
  		"<div>Dashboard is currently in a prototype phase. <br> It will allow to include widgets related to features important for each user in his/her daily work.</div>"      
  ,APPLICATION_HOME_CR : "<br><br><br><table width='600' border='0' align=center cellpadding='0' cellspacing='0'><tr><td><h1 style='font-size:16px;'>Welcome to Component Repository</h1></td></tr></table>"
  
  ,get: function(t) { return new Ext.Template(t,{compiled:true} );}
};                                                                                                                                                                                                                            
                                                                                                                                                                                                                               
                                                                                                                                                                                                                               /**
 * @class Reprezents the client side session object.
 * @type 
 */
dnet.core.base.Session = {
 
	/**
	 * Logged in user info.
	 * 
	 * @type Object
	 */
	user : {
		id : null,
		code : null,
		name : null
	},

	/**
	 * Client(Tenant) info.
	 * 
	 * @type Object
	 */
	client : {
		id : null,
		code : null,
		systemClient: false
	},
 
	/**
	 * User roles
	 * @type Array
	 */
	roles: [],
	
	/**
	 * Is the current session locked by user?
	 * 
	 * @type Boolean
	 */
	locked : false,

	/**
	 * System parameters.
	 * 
	 * @type
	 */
	sysParams : {},

	/**
	 * Getter for user.
	 * 
	 * @return {Object}
	 */
	getUser : function() {
		return this.user;
	},

	/**
	 * Getter for client(tenant)
	 * 
	 * @return {}
	 */
	getClient : function() {
		return this.client;
	},

	/**
	 * Shorthand getter for the client id.
	 * 
	 * @return {Integer}
	 */
	getClientId : function() {
		return this.client.id * 1;
	},

	/**
	 * Shorthand getter for the client code.
	 * 
	 * @return {String}
	 */
	getClientCode : function() {
		return this.client.code;
	},

	/**
	 * Is the current session authenticated?
	 * 
	 * @return {Boolean}
	 */
	isAuthenticated : function() {
		return (!Ext.isEmpty(this.user.name));
	},

	hasRole: function(role) {
		for(var i=0,l=this.roles.length; i<l;i++) {
			if (this.roles[i] == role) {
				return true;
			}
		}
		return false;
	},
	useFocusManager: false,
	rememberViewState: false
	
};
function getApplication() {
	if (!__application__ && !window.parent) {
		alert("Cannot find application.");
		return;
	}
	return (__application__) ? __application__ : window.parent.__application__;
}
Ext.ns("dnet.core.base");
dnet.core.base.Application = {

	/**
	 * Session object.
	 * 
	 * @type dnet.core.base.Session
	 */
	session : null,

	/**
	 * Main application view.
	 * 
	 * @type
	 */
	view : null,

	/**
	 * Main application menu.
	 * 
	 * @type
	 */
	menu : null,

	/**
	 * Flag to indicate if application frames are rendered in different browser
	 * iframes or in the same DOM.
	 * 
	 * @type Boolean
	 */
	useIframes : true,

	/**
	 * Frame navigator implementation. Based on the value of the
	 * <code>useIframes</code> flag is used either FrameNavigator or
	 * FrameNavigatorWithIframe
	 * 
	 * @type
	 */
	navigator : null,

	/**
	 * Map which stores the registered callbacks used by the inter-frame
	 * communication.
	 */
	frameCallbacks : new Ext.util.MixedCollection(),
	//frameRefs : new Ext.util.MixedCollection(),
	type : null,

	/**
	 * Login window object.
	 * 
	 * @type dnet.core.base.LoginWindow
	 */
	loginWindow : null,

	/**
	 * Getter for the session object.
	 * 
	 * @return {}
	 */
	getSession : function() {
		return this.session;
	},

	/**
	 * Getter for the main view object.
	 * 
	 * @return {}
	 */
	getView : function() {
		return this.view;
	},

	/**
	 * Getter for the application home view.
	 * 
	 * @return {}
	 */
	getViewHome : function() {
		return Ext.getCmp(__CmpId__.APP_VIEW_HOME);
	},

	/**
	 * Getter for application header view.
	 * 
	 * @return {}
	 */
	getViewHeader : function() {
		return Ext.getCmp(__CmpId__.APP_VIEW_HEADER);
	},

	/**
	 * Getter for the application footer view.
	 * 
	 * @return {}
	 */
	getViewFooter : function() {
		return Ext.getCmp(__CmpId__.APP_VIEW_FOOTER);
	},

	/**
	 * Getter for the application body view.
	 * 
	 * @return {}
	 */
	getViewBody : function() {
		return Ext.getCmp(__CmpId__.APP_VIEW_BODY);
	},

	/**
	 * Open the specified frame.
	 * 
	 * @param options:
	 *            callback: the callback function to be executed after the frame
	 *            is rendered. params: the arguments used in callback call
	 */
	showFrame : function(frame, options) {
		var theFrame = this.navigator.getFrameInstance(frame);
		var applyCallbackNow = !Ext.isEmpty(theFrame);
		this.navigator.showFrame(frame, options);
		this.registerFrameCallback(frame, options);
		if (applyCallbackNow) {
			this.applyFrameCallback(frame, theFrame);
		}
	},

	/**
	 * Show the specified frame
	 * @param {} name
	 * @param {} bundle
	 */
	showFrameByName : function(bundle, name) {
		this.showFrame(name, {url:Dnet.buildUiPath(bundle, name, false)});	
	},
	
	/**
	 * Returns the active tab.
	 */
	getActiveTab: function() {
		return this.getViewBody().getActiveTab();
	},
	
	/**
	 * Get the application frame instance for the active tab.
	 * @return {}
	 */
	getActiveFrameInstance: function() {
		if(this.getActiveTab()) {
			return this.navigator.getFrameInstance(this.getActiveTab().initialConfig.title);
		}
		return null;
	},
	
	/**
	 * Register a callback function to be executed in a certain frame. The frame
	 * may not be open at the time the callback execution is requested. It is
	 * placed in the queue and executed when the frame is rendered.
	 * @param {} frame
	 * @param {} options
	 */
	registerFrameCallback : function(frame, options) {
		if (options && !Ext.isEmpty(options.tocElement)
				&& Ext.isEmpty(options.callback)) {
			options.callback = function(params) {
				this._showTocElement_(params.tocElement);
			}
			options.params = {
				tocElement : options.tocElement
			}
		}
		if (options && options.callback) {
			this.frameCallbacks.add(frame, options);
		}
	},

	/**
	 * Execute the registered callback 
	 * 
	 * @param {}
	 *            frame
	 * @param {}
	 *            theFrameObject
	 */
	applyFrameCallback : function(frame, theFrameObject) {
		if (this.frameCallbacks.containsKey(frame)) {
			var opt = this.frameCallbacks.get(frame);
			opt.callback.call(theFrameObject, opt.params);
			this.frameCallbacks.removeAtKey(frame);
		}
	},
	
	/**
	 * Entry point function to start-up an application.
	 */
	run : function() {
		this.session = dnet.core.base.Session;
		if (this.useIframes) {
			this.navigator = dnet.core.base.FrameNavigatorWithIframe;
		} else {
			this.navigator = dnet.core.base.FrameNavigator;
		}
		this.navigator.maxOpenTabs = __MAX_OPEN_TABS__;

		if (this.loginWindow == null) {
			this.loginWindow = new dnet.core.base.LoginWindow({});
		}

		if (!this.session.isAuthenticated()) {
			this.loginWindow.show();
			this.loginWindow.clearInvalid();
		} else if (this.session.locked) {
			this.loginWindow.applyState_LockSession();
			this.showLoginView();
			this.loginWindow.getPasswordField().focus(false, 200);
		} else {
			this.view.getLayout().setActiveItem(1);
			this.menu.setUserText(this.session.user.name);
			this.menu.setClientText(this.session.client.code);
			if (this.session.client.systemClient) {
				this.menu.addSystemClientMenu();
			} else {
				this.menu.removeSystemClientMenu();
			}	
		}
	},
	
	/**
	 * On login success hide the login window and show the application canvas.
	 * Add whatever else is necessary.
	 */
	onLoginSuccess : function(accountChange) {
		if (accountChange) {
			this.getViewBody().plugins[0].onCloseAll(); //.removeAll(true);
		}
		this.showMainApplicationView();
		this.loginWindow.hide();

		this.menu.setUserText(this.session.user.name);
		this.menu.setClientText(this.session.client.code);
		
		if (this.session.client.systemClient) {
			this.menu.addSystemClientMenu();
		} else {
			this.menu.removeSystemClientMenu();
		}		
	},
	
	/**
	 * Logout from application
	 */
	doLogout : function() {
		Ext.Ajax.request({
					method : "POST",
					scope : this,
					url : Dnet.sessionAPI("json").logout,
					callback: function() {
						window.location.href="/nan21.dnet.core.web/security/session/login";	
					}
				});
			
//		this.loginWindow.applyState_Logout();
//		this.showLoginView();
//		this.loginWindow.getUserField().focus(false, 200);
	},
	
	/**
	 * Lock active session. 
	 */
	doLockSession : function() {
		Ext.Ajax.request({
					method : "POST",
					scope : this,
					url : Dnet.sessionAPI("json").lock,
					timeout : 600000
				});
		this.loginWindow.applyState_LockSession();
		this.showLoginView();
		this.loginWindow.getPasswordField().focus(false, 200);
	},
	
	/**
	 * Activate the empty canvas and show the login window. 
	 */
	showLoginView : function() {
		this.view.getLayout().setActiveItem(0);
		this.loginWindow.show();
	},
	
	/**
	 * Activate the application view canvas.
	 */
	showMainApplicationView : function() {
		// this.view.doLayout(false, true);
		// this.getViewBody().doLayout(false, true);
		this.view.getLayout().setActiveItem(1);
	},
	
	/**
	 * Change the current theme with the provided one.
	 * @param {} t selected theme
	 */
	changeCurrentTheme : function(t) {
		window.location.href = Dnet.uiUrl + "/?theme=" + t
	},
	
	/**
	 * Change current language with the provided one.
	 * @param {} l selected language
	 */
	changeCurrentLanguage : function(l) {
		window.location.href = Dnet.uiUrl + "/?lang=" + l
	},
	
	/**
	 * Set the tab title according to the nested frame title. 
	 * @param {} frame
	 * @param {} title
	 */
	setFrameTabTitle : function(frame, title) {
		Ext.getCmp(__CmpId__.FRAME_TAB_PREFIX + frame).setTitle(title);
	}

};
Ext.ns("dnet.core.base");

dnet.core.base.ApplicationMenu$ContributedMenus = [];
/* company logo + user info */

dnet.core.base.ApplicationMenu$LogoItems = [{
	xtype : "container",
	
	items : [Ext.create('Ext.Img', {
				src : Dnet.staticResourceUrlCore + "/resources/images/"
						+ __THEME__ + "/logo/logo.png",
				id : "net.nan21.dnet.core.menu.ApplicationMenu$Item$Logo",
				style : "vertical-align:middle;"
			})],
	style : "",
	width : 100
}, {
	xtype : "tbspacer",
	id : "net.nan21.dnet.core.menu.ApplicationMenu$Item$LogoSpacer",
	width : 20
}];

dnet.core.base.ApplicationMenu$Items = [
		{
			xtype : "tbspacer",				
			width : 10
		}, {
			xtype : "splitbutton",
			text : Dnet.translate("appmenuitem", "myaccount__lbl"),
			menu : new Ext.menu.Menu({
				items : [{
		
					text : Dnet.translate("appmenuitem", "theme__lbl"),
					menu : new Ext.menu.Menu({
								items : [{
									text : Dnet.translate("appmenuitem",
											"theme_aqua__lbl"),
									handler : function() {
										getApplication().changeCurrentTheme("aqua");
									}
								},{
									text : Dnet.translate("appmenuitem",
											"theme_gray__lbl"),
									handler : function() {
										getApplication().changeCurrentTheme("gray");
									}
								}, {
									text : Dnet.translate("appmenuitem",
											"theme_blue__lbl"),
									handler : function() {
										getApplication().changeCurrentTheme("blue");
									}
								}
//								, {
//									text : Dnet.translate("appmenuitem",
//											"theme_access__lbl"),
//									handler : function() {
//										getApplication().changeCurrentTheme("access");
//									}
//								}
//								, {
//									text : Dnet.translate("appmenuitem",
//											"theme_neptune__lbl"),
//									handler : function() {
//										getApplication().changeCurrentTheme("neptune");
//									}
//								}
								]
							})
			}, {

			text : Dnet.translate("appmenuitem", "lang__lbl"),
			menu : new Ext.menu.Menu({
						items : [{
									text : "English",
									handler : function() {
										getApplication()
												.changeCurrentLanguage("en");
									}
								}, {
									text : "Romană",
									handler : function() {
										getApplication()
												.changeCurrentLanguage("ro");
									}
								}]
					})
		}, "-", {
			text : Dnet.translate("appmenuitem", "changepswd__lbl"),
			handler : function() {
				(new dnet.core.base.ChangePasswordWindow({})).show();
			}
		},"-", {
			text : Dnet.translate("msg", "preferences_wdw"),
			handler : function() {
				(new dnet.core.base.UserPreferences()).show();
			}
		}, "-", {
			text : Dnet.translate("appmenuitem", "mysettings__lbl"),
			handler : function() {
				var bundle = "nan21.dnet.module.ad.ui.extjs";
				var frame = "net.nan21.dnet.module.ad.usr.frame.MyUserSettings_UI";
				var path = Dnet.buildUiPath(bundle, frame, false);
				getApplication().showFrame(frame, {
							url : path
						});
			}
		}/*
			 * ,{ text : Dnet.translate("msg", "preferences_wdw"), handler :
			 * function() { (new dnet.core.base.UserPreferences({})).show(); } }
			 */

		]
	})
}, {
	xtype : "splitbutton",
	text : Dnet.translate("appmenuitem", "session__lbl"),
	menu : new Ext.menu.Menu({
				items : [{
							text :"Authenticate",
							handler : function() {
								getApplication().doLockSession();
							}
						},{
							text : Dnet.translate("appmenuitem", "logout__lbl"),
							handler : function() {								
								getApplication().doLogout();
							}
						}
						]
			})
}

, "-"

, {
	xtype : "splitbutton",
	text : Dnet.translate("appmenuitem", "help__lbl"),
	menu : new Ext.menu.Menu({
		items : [{
					text : Dnet.translate("appmenuitem", "frameInspector__lbl"),
					handler : function() {
						(new dnet.core.base.FrameInspector({})).show();
					}
				}, "-", {
					text : Dnet.translate("appmenuitem", "about__lbl"),
					handler : function() {
						(new Ext.Window({
							width : 300,
							height : 250,
							title : "About",
							tpl : dnet.core.base.TemplateRepository.APPLICATION_ABOUT_BOX,
							data : {
								dnetName : Dnet.name,
								dnetVersion : Dnet.version
							},
							closable : true,
							modal : true,
							resizable : false
						})).show();
					}
				}]
	})

}, "->", {
	xtype : "tbtext",
	id : "net.nan21.dnet.core.menu.ApplicationMenu$Item$UserLabel",
	text : Dnet.translate("appmenuitem", "user__lbl")
}, {
	xtype : "tbtext",
	id : "net.nan21.dnet.core.menu.ApplicationMenu$Item$UserName",
	text : "-",
	style : "font-weight:bold;"
}, "-", {
	xtype : "tbtext",
	id : "net.nan21.dnet.core.menu.ApplicationMenu$Item$ClientLabel",
	text : Dnet.translate("appmenuitem", "client__lbl")
}, {
	xtype : "tbtext",
	id : "net.nan21.dnet.core.menu.ApplicationMenu$Item$ClientName",
	text : "-",
	style : "font-weight:bold;"
} ];

Ext.define("dnet.core.base.DBMenu", {
			extend : "Ext.menu.Menu"

});

Ext.define("dnet.core.base.ApplicationMenu", {
	extend : "Ext.toolbar.Toolbar",
	padding : 0,
	height : 50,
	systemClientMenu : null,
	systemClientMenuAdded : null,
	ui : "main-app-menu",
	dbMenu : null,
	dbMenuAdded : null,

	insertDBMenus : function() {
		if (this.rendered && this.dbMenu != null && this.dbMenuAdded !== true) {
			var l = this.dbMenu.length;
			for (var i = 0; i < l; i++) {
				this.insert(2 + i, this.dbMenu[i]);
			}
			this.dbMenuAdded = true;

		}
	},

	
	_createFrameMenuItem_ : function(config) {
		var bundle_ = config.bundle;
		var frame_ = config.frame;
		var title_ = config.title;
		return {
			text : title_,
			handler : function() {
				var bundle = bundle_;
				var frame = frame_;
				var path = Dnet.buildUiPath(bundle, frame, false);
				getApplication().showFrame(frame, {
							url : path
						});
			}
		};
	},
	_createMenuMenuItem_ : function(config) {
		return {
			text : config.title,
			deferExpandMenu: function() {
				if (!this.menu.loader._isLoaded_) {
					if (!this.menu.loader._isLoading_) {
						this.menu.loader.load();
					}
					return false;
				} else {
					var me = this;
		
			        if (!me.menu.rendered || !me.menu.isVisible()) {
			            me.parentMenu.activeChild = me.menu;
			            me.menu.parentItem = me;
			            me.menu.parentMenu = me.menu.ownerCt = me.parentMenu;
			            me.menu.showBy(me, me.menuAlign, ((!Ext.isStrict && Ext.isIE) || Ext.isIE6) ? [-2, -2] : undefined);
			        }
				}
				 
		    },
			menu : {
				loader : this._createLoader_({
					menuItemId: config.db_id
				},false) 
			}
		};
	},
	_createMenu_ : function(config) {
		return Ext.apply({
			 
			maybeShowMenu: function(){
				if (!this.menu.loader._isLoaded_) {
					if (!this.menu.loader._isLoading_) {
						this.menu.loader.load();	
					}
					return false;
				} else {
					var me = this;
			        if (me.menu && !me.hasVisibleMenu() && !me.ignoreNextClick) {
			            me.showMenu();
			        }
				}
		        
		    },
			 	
			menu : {
				loader : this._createLoader_({
					menuId: config.db_id
				},true)
			  
			}
		},config);
		  
	},
	_createLoader_ : function(params, autoload) {
		
		return {
			url : Dnet.dsAPI("MenuItemDs", "json").read,
			renderer : 'data',
			autoLoad : autoload,
			_isLoaded_ : false,
			_isLoading_ : false,
			listeners : {
				scope : this,
				beforeload: {
					fn: function(loader, options, eopts) {
						loader._isLoaded_ = false;
						loader._isLoading_ = true;
					}
				},
				load : {
					fn : function(loader, response, options, eopts) {
						var res = Ext.JSON.decode(response.responseText).data;
						var mitems = [];
						for (var i = 0; i < res.length; i++) {
							var e = res[i];
							if (e.frame) {
								mitems.push(this._createFrameMenuItem_({
									db_id: e.id,
									title : e.title,
									frame: e.frame,
									bundle: e.bundle
								}));
							} else {
								mitems.push(this._createMenuMenuItem_({
									db_id: e.id,
									title : e.title
								}));
							}

						}
						loader.target.add(mitems);
						loader._isLoaded_ = true;
						loader._isLoading_ = false;
						
						//return false;
					}
				}
			},
			ajaxOptions : {
				method : "POST"

			},
			params : {
				data: Ext.JSON.encode(params),
				orderBy:Ext.JSON.encode([{ property :"sequenceNo", direction :"ASC"}])
			}
			
		};
	},

	initComponent : function(config) {
 
		var items = []; 
 
		items = items.concat(dnet.core.base.ApplicationMenu$Items);
		items = items.concat([
			{
				xtype : "tbspacer",				
				width : 20
			},{
				xtype : "tbtext",				
				id : "net.nan21.dnet.core.menu.ApplicationMenu$Item$ProductInfo",
				text: "<span>" + Dnet.name + " </span><br><span>"
						+ Dnet.translate("appmenuitem", "version__lbl") + ": "
						+ Dnet.version + "</span></span>" 
		}]);

		this.systemClientMenuAdded = false;

		var cfg = {
			border : false,
			frame : false,
			items : items
		};

		Ext.apply(this, cfg);
		this.callParent(arguments);

		this.on("afterrender", function() {
					Ext.Function.defer(this.insertDBMenus, 500, this);
				}, this);
	},

	setUserText : function(v) {
		try {
			this.items
					.get("net.nan21.dnet.core.menu.ApplicationMenu$Item$UserName")
					.setText(v);
		} catch (e) {
		} /* maybe customizations do not want this menu item */
	},

	setClientText : function(v) {
		try {
			this.items
					.get("net.nan21.dnet.core.menu.ApplicationMenu$Item$ClientName")
					.setText(v);
		} catch (e) {
		}
	},

	addSystemClientMenu : function() {
		if (!this.systemClientMenuAdded) {
			this.createSystemClientMenu();
			if (this.dbMenu == null) {
				this.insert(2, this.systemClientMenu);
			} else {
				this.insert(2 + this.dbMenu.length, this.systemClientMenu);
			}
			this.systemClientMenuAdded = true;
		}
	},

	removeSystemClientMenu : function() {
		if (this.systemClientMenuAdded) {
			this.remove(this.systemClientMenu);
			this.systemClientMenuAdded = false;
			this.systemClientMenu = null;
		}
	},

	createSystemClientMenu : function() {
		this.systemClientMenu = Ext.create('Ext.button.Split', {
			xtype : "splitbutton",
			text : "Tools",
			menu : new Ext.menu.Menu({
				items : [{
					text : "Clients management",
					handler : function() {
						var bundle = "nan21.dnet.module.ad.ui.extjs";
						var frame = "net.nan21.dnet.module.ad.client.frame.Client_UI";
						var path = Dnet.buildUiPath(bundle, frame, false);
						getApplication().showFrame(frame, {
									url : path
								});
					}
				}]
			})
		});
	}

});Ext.define("dnet.core.base.LoginForm", {
	extend : "Ext.form.Panel",	
	/**
	 * Action button rendered in the window, 
	 * given as a reference to be managed from the form.
	 * @type 
	 */
	actionButton : null,
	initComponent : function(config) {
		
		this.actionButton = this.initialConfig.actionButton;
		this.actionButton.setHandler(this.doLogin, this);

		var cfg = {
			frame : true,
			bodyPadding: 10,
			fieldDefaults : {
				labelAlign : 'right',
				labelWidth : 100,
				msgTarget : 'side'
			},
			defaults : {
				anchor : '-20',
				selectOnFocus : true,
				allowBlank : false
			},
			items : this._buildItems_()
		};
		Ext.apply(this, cfg);
		this.callParent(arguments);		
	},

	/**
	 * Builder method which constructs the form elements.
	 * @return {Array}
	 */
	_buildItems_ : function() {
		return [ {
			xtype : "textfield",
			itemId : "usr",
			fieldLabel : Dnet.translate("msg", "login_user"),			 
			value : getApplication().getSession().getUser().code,
			listeners : {
				change : {
					scope : this,
					fn : this.enableAction
				}
			}
		}, {
			xtype : "textfield",
			itemId : "pswd",
			fieldLabel : Dnet.translate("msg", "login_pswd"),
			inputType : "password",
			listeners : {
				change : {
					scope : this,
					fn : this.enableAction
				}
			}
		}, {
			xtype : "textfield",
			itemId : "client",
			fieldLabel : Dnet.translate("msg", "login_client"),
			value : getApplication().getSession().getClient().code,
			listeners : {
				change : {
					scope : this,
					fn : this.enableAction
				}
			}
		}
		/*, {
			xtype : "combo",
			itemId : "lang",
			fieldLabel : Dnet.translate("msg", "login_lang"),
			forceSelection : true,
			triggerAction : "all",
			store : [ "English", "Romana" ],
			value : "English",
			listeners : {
				change : {
					scope : this,
					fn : this.enableAction
				}
			}
		}*/

		];
	},

	/**
	 * Getter for the login button.
	 * @return {}
	 */
	getActionButton : function() {
		return this.actionButton;
	},

	/**
	 * Getter for the username field.
	 * @return {Ext.form.field.Text} The component (if found)
	 */
	getUserField : function() {
		return this.getComponent("usr");
	},

	/**
	 * Getter for the password field
	 * @return {Ext.form.field.Text} The component (if found)
	 */
	getPasswordField : function() {
		return this.getComponent("pswd");
	},

	/**
	 * Getter for the client field
	 * @return {Ext.form.field.Text} The component (if found)
	 */
	getClientField : function() {
		return this.getComponent("client");
	},

	/**
	 * Getter for the language field
	 * @return {Ext.form.field.ComboBox} The component (if found)
	 */
	getLanguageField : function() {
		return this.getComponent("lang");
	},

	
	/**
	 * Callback invoked on unsuccessful login attempt.
	 * @param {} response
	 * @param {} options
	 */
	onActionFailure : function(response, options) {
		Ext.Msg.show( {
			title : 'Authentication error',
			msg : response.responseText,
			buttons : Ext.MessageBox.OK,
			scope : this,
			icon : Ext.MessageBox.ERROR
		});
		this.getActionButton().enable();
	},
	
	/**
	 * Callback invoked on successful login.
	 * @param {} response
	 * @param {} options
	 */
	onActionSuccess : function(response, options) {
		var r = Ext.JSON.decode(response.responseText);
		var s = getApplication().getSession();
		var u = s.getUser();
		var c = s.getClient();
		var accountChange = false;
		if (c.id != r.data.clientId || u.name!= r.data.name) {
			accountChange = true;
		}
		s.roles = r.roles;
		u.name = r.data.name;
		c.id = r.data.clientId;
		c.systemClient =  r.data.systemClient;
		
		if (r.data.extjsDateFormat) {
			Dnet.DATE_FORMAT = r.data.extjsDateFormat;
		}
		if (r.data.extjsTimeFormat) {
			Dnet.TIME_FORMAT = r.data.extjsTimeFormat;
		}
		if (r.data.extjsDateTimeFormat) {
			Dnet.DATETIME_FORMAT = r.data.extjsDateTimeFormat;
		}
		
		if (r.data.extjsAltFormats) {
			Dnet.DATE_ALTFORMATS = r.data.extjsAltFormats;
		}
		 
		if (r.data.decimalSeparator) {
			Dnet.DECIMAL_SEP = r.data.decimalSeparator;
		}
		if (r.data.thousandSeparator) {
			Dnet.THOUSAND_SEP = r.data.thousandSeparator;
		}
		Dnet.initFormats(); 
		
		getApplication().onLoginSuccess(accountChange);
	},

	/**
	 * Execute login action. The login button click handler.
	 * @param {} btn
	 * @param {} evnt
	 */
	doLogin : function(btn, evnt) {
		var s = getApplication().getSession();
		var u = s.getUser();

		u.code = this.getUserField().getValue();
		s.client.code = this.getClientField().getValue();
		var p = {};

		p["user"] = this.getUserField().getValue();
		p["pswd"] = Ext.util.MD5(this.getPasswordField().getValue());
		p["client"] = this.getClientField().getValue();
		//p["lang"] = this.getLanguageField().getValue();

		Ext.Ajax.request( {
			method : "POST",
			params : p,
			failure : this.onActionFailure,
			success : this.onActionSuccess,
			scope : this,
			url : Dnet.sessionAPI("json").login,
			timeout : 600000
		});
		this.getActionButton().disable();
	},

	/**
	 * Clear the form fields.
	 */
	clearFields : function() {
		this.items.each(function(item) {
			item.setValue(null);
		}, this);
	},

	clearInvalid : function() {
		this.getUserField().clearInvalid();
		this.getClientField().clearInvalid();
	},

	/**
	 * Apply states on form elements after logout.
	 */
	applyState_Logout : function() {
		this.getUserField().enable();

		this.getPasswordField().enable();
		this.getPasswordField().setValue(null);
		this.getPasswordField().clearInvalid();

		this.getClientField().enable();
		this.getClientField().setValue(null);
		this.getClientField().clearInvalid();

		//this.getLanguageField().enable();
		this.getActionButton().disable();
	},

	/**
	 * Apply states on form elements after lock session. 
	 */
	applyState_LockSession : function() {
		this.getUserField().disable();
		this.getPasswordField().setValue(null);
		this.getPasswordField().clearInvalid();
		this.getClientField().disable();
		//this.getLanguageField().disable();
		this.getActionButton().disable();
	},

	/**
	 * Setter for the user-name field value.
	 * @param {} v
	 */
	setUserName : function(v) {
		this.getUserField().setValue(v);
	},

 
	/**
	 * Enable disable action button.
	 */
	enableAction : function() {
		if (this.getForm().isValid()) {
			this.getActionButton().enable();
		} else {
			this.getActionButton().disable();
		}
	}
});

Ext.define("dnet.core.base.LoginWindow", {
	extend : "Ext.Window",

	initComponent : function(config) {

		var btn = Ext.create('Ext.Button', {
			text : Dnet.translate("msg", "login_btn"),
			disabled : true
		});

		var cfg = {
			title : Dnet.translate("msg", "login_title"),
			border : true,
			width : 350,
			resizable : false,
			closeAction : "hide",
			//padding : 5,
			closable : false,
			constrain : true,			
			buttonAlign : "center",
			modal : true,
			items : new dnet.core.base.LoginForm( {
				actionButton : btn
			}),
			buttons : [ btn ]
		};

		Ext.apply(cfg, config);
		Ext.apply(this, cfg);
		this.callParent(arguments);
	},

	/**
	 * Getter for the login form.
	 * @return {}
	 */
	getLoginForm : function() {
		return this.items.getAt(0);
	},

	/**
	 * Wrapper around the login-form action button getter.
	 * @return {}
	 */ 
	getActionButton : function() {
		return this.getLoginForm().getActionButton();
	},

	/**
	 * Wrapper around the login-form user field getter.
	 * @return {}
	 */
	getUserField : function() {
		return this.getLoginForm().getUserField();
	},
	
	/**
	 * Wrapper around the login-form password field getter.
	 * @return {}
	 */
	getPasswordField : function() {
		return this.getLoginForm().getPasswordField();
	},
	
	/**
	 * Wrapper around the login-form client field getter.
	 * @return {}
	 */
	getClientField : function() {
		return this.getLoginForm().getClientField();
	},
	
	/**
	 * Wrapper around the login-form language field getter.
	 * @return {}
	 */
	getLanguageField : function() {
		return this.getLoginForm().getLanguageField();
	},

	/**
	 * Wrapper around the login-form applyState_Logout function.
	 */
	applyState_Logout : function() {
		this.getLoginForm().applyState_Logout();
	},

	/**
	 * Wrapper around the login-form applyState_LockSession function.
	 */
	applyState_LockSession : function() {
		this.getLoginForm().applyState_LockSession();
	},

	/**
	 * Wrapper around the login-form clearInvalid function.
	 */
	clearInvalid : function() {
		this.getLoginForm().clearInvalid();
	}

});

Ext.define("dnet.core.base.ChangePasswordForm" , {
	extend: "Ext.form.Panel",
	/**
	 * Action button rendered in the window, 
	 * given as a reference to be managed from the form.
	 * @type 
	 */
	actionButton : null,
	initComponent: function() {
	
		this.actionButton = this.initialConfig.actionButton;
		this.actionButton.setHandler(this.doTask, this);
		
		var cfg = {
			frame : true,
			bodyPadding: 10,
			fieldDefaults : {
				labelAlign : 'right',
				labelWidth : 100,
				msgTarget : 'side'
			},
			defaults : {
				anchor : '-20',
				selectOnFocus : true,
				allowBlank : false
			},
			items : this._buildItems_()
		};
		
 
        Ext.apply(this,cfg);
		 
        this.callParent(arguments);
 	},

 	/**
	 * Builder method which constructs the form elements.
	 * @return {Array}
	 */
	_buildItems_ : function() {
 		return [				
			{xtype:"textfield", itemId:"opswd", fieldLabel:"Password",   selectOnFocus: true, allowBlank: false, inputType: "password"
			 	,listeners: {change: {scope:this, fn:this.enableAction }}   }

			,{xtype:"textfield", itemId:"pswd1", fieldLabel:Dnet.translate("msg", "chpswd_pswd1"), selectOnFocus: true, allowBlank: false
		    	,inputType: "password"
		     	,listeners: {change: {scope:this, fn:this.enableAction }}   }
	     	
	     	,{xtype:"textfield", itemId:"pswd2", fieldLabel:Dnet.translate("msg", "chpswd_pswd2"), selectOnFocus: true, allowBlank: false
		    	,inputType: "password"
		     	,listeners: {change: {scope:this, fn:this.enableAction }}   }
		 ]; 		
 	},
	
 	getActionButton: function() {
 		return this.actionButton;
 	},

 	/**
	 * Getter for the current password field.
	 * @return {Ext.form.field.Text} The component (if found)
	 */
 	getCurrentPasswordField: function() {		 
		return this.getComponent("opswd");
	},
	
	/**
	 * Getter for the new password field.
	 * @return {Ext.form.field.Text} The component (if found)
	 */
	getPasswordField: function() {
		return this.getComponent("pswd1");
	},
	
	/**
	 * Getter for the confirm new password field.
	 * @return {Ext.form.field.Text} The component (if found)
	 */
	getConfirmPasswordField: function() {
		return this.getComponent("pswd2");
	},

	/**
	 * Callback invoked on unsuccessful password change attempt.
	 * @param {} response
	 * @param {} options
	 */
	onActionFailure: function(response, options) {         
		 Ext.Msg.show({
	          msg: response.responseText
	         ,buttons: Ext.MessageBox.OK
	         ,scope:this
	         ,icon: Ext.MessageBox.ERROR
		 });

		 (response.responseText);
	},
	
	/**
	 * Callback invoked on successful password change.
	 * @param {} response
	 * @param {} options
	 */
	onActionSuccess: function(response , options) {
    	Ext.Msg.show({
	          msg: Dnet.translate("msg", "chpswd_success")	          
	         ,buttons: Ext.MessageBox.OK
	         ,scope:this
	         ,icon: Ext.MessageBox.INFO
		 });
		  
	},

	/**
	 * Execute change password action. The action button click handler.
	 * @param {} btn
	 * @param {} evnt
	 */
	doTask: function(btn, evnt) {

		if (this.getPasswordField().getValue() != this.getConfirmPasswordField().getValue() ) {

			Ext.Msg.show({
		          msg: Dnet.translate("msg", "chpswd_nomatch")
		         ,buttons: Ext.MessageBox.OK
		         ,scope:this
		         ,icon: Ext.MessageBox.ERROR
			 });
			return;
		}

		var p = {}; 
		p["npswd"] = Ext.util.MD5(this.getPasswordField().getValue() );
		p["opswd"] = Ext.util.MD5(this.getCurrentPasswordField().getValue() );
 
		Ext.Ajax.request({
             method:"POST"
            ,params:p
            ,failure:this.onActionFailure
			,success:this.onActionSuccess
            ,scope:this
            ,url: Dnet.sessionAPI("json").changePassword
            ,timeout:600000
        });
	},
	
	/**
	 * Clear the form fields.
	 */
	clearFields: function() {
		this.items.each( function(item) { item.setValue(null); } , this);
	},
	
 	/**
	 * Enable disable action button.
	 */
	enableAction: function() {
		var valid = true;
		this.items.each( function(item) { if (!item.isValid() ) {valid=false;return false; } } , this);
		if (this.getForm().isValid() ) {
			this.getActionButton().enable();
		} else {
			this.getActionButton().disable();
		}
	}
});


Ext.define("dnet.core.base.ChangePasswordWindow" , {
	extend: "Ext.Window",
	initComponent: function() {
	
		var btn = Ext.create('Ext.Button', {
			text : Dnet.translate("tlbitem", "save__lbl"),
			disabled : true
		}); 
		 
		var cfg = {
			title : Dnet.translate("msg", "chpswd_btn"),
			border : true,
			width : 350,
			resizable : false,
			closeAction : "hide",			 
			closable : true,
			constrain : true,
			layout:"fit",
			xtype : "container",
			buttonAlign : "center",
			modal : true,
			items : new dnet.core.base.ChangePasswordForm({actionButton: btn}),
			buttons : [ btn ]
		};
		Ext.apply(this, cfg);
	
		this.callParent(arguments);
 
 	}
 
});Ext.define("dnet.core.base.FrameInspector", {

	extend : "Ext.Window",

	title : Dnet.translate("appmenuitem", "frameInspector__lbl"),
	border : true,
	height : 300,
	width : 500,
	layout : 'fit',
	resizable : true,

	bodyPadding : 20,
	closable : true,
	constrain : true,
	buttonAlign : "center",
	modal : true,
	autoScroll : true,
	initComponent : function() {

		this.tpl = new Ext.XTemplate(
				'<h3 style="padding-top: 10px;padding-bottom: 10px;"><u>Data-controls</u></h3> ',
				'<tpl for="dc">',
				'<p style=" padding-bottom: 5px;">{#}. <i>Key:</i> {alias} <br> <i>Class:</i> {name} <br> <i>Data-source: </i> {ds} </p></tpl>',
				'<h3 style="padding-top: 10px;padding-bottom: 10px;"><u>Toolbars</u></h3> ',
				'<tpl for="tlb">',
				'<p style=" padding-bottom: 5px;">{#}.  <i>Key:</i>  {alias}, <i>Title:</i> `{title}`</p></tpl>');

		var frame = getApplication().getActiveFrameInstance();
		if (frame) {
			var dc = null, dcs = [], tlb = null, tlbs = [];
			frame._dcs_.each(function(item, idx, length) {
						dc = {
							name : item.$className,
							alias : item._instanceKey_,
							ds : item.recordModel
						};
						dcs[dcs.length] = dc;
					}, this);
			frame._tlbs_.eachKey(function(key, item) {

						tlb = {
							alias : key,
							title : ((frame._tlbtitles_) ? frame._tlbtitles_
									.get(key)
									|| "-" : "-")
						};
						tlbs[tlbs.length] = tlb;
					}, this);
			this.data = {
				dc : dcs,
				tlb : tlbs
			};
		} else {
			this.html = "<br><br>Frame inspector works only for application frames not for the home panel !";
		}

		this.callParent(arguments);

	}
});/**
 * Mixin which provides elements map based definition of items.
 */
Ext.define("dnet.core.base.AbstractDNetView", {

			// **************** Properties *****************

			_refs_ : null,
			
			/**
			 * Elements definition map
			 * 
			 * @type Ext.util.MixedCollection
			 */
			_elems_ : null,

			/**
			 * Name of the component which is to be used as the root-item for
			 * this view. By default is `main`
			 * 
			 * @type String
			 */
			_mainViewName_ : "main",

			/**
			 * Translations for elements.
			 * 
			 * @type
			 */
			_trl_ : null,

			// **************** Public API *****************

			/**
			 * @public Get an element instance if it is found.
			 * 
			 * @param name
			 * @return {}
			 */
			_getElement_ : function(name) {
				if (!this._refs_) {
					this._refs_ = {};
				}
				if (!this._refs_[name]) {
					this._refs_[name] = Ext.getCmp(this._elems_.get(name).id);
				}
				return this._refs_[name];
			},

			/**
			 * @public Shorthand alias for _getElement_
			 * 
			 * @param name
			 * @return {}
			 */
			_get_ : function(name) {
				return this._getElement_(name);
			},

			/**
			 * @public Get an element's configuration object.
			 * 
			 * @param name
			 * @return {}
			 */
			_getElementConfig_ : function(name) {
				return this._elems_.get(name);
			},

			/**
			 * @public Shorthand alias for _getElementConfig_
			 * 
			 * @param name
			 * @return {}
			 */
			_getConfig_ : function(name) {
				return this._elems_.get(name);
			},

			/**
			 * Similar to getElement but used for windows. If there is already a
			 * window instance created will return it otherwise creates a new
			 * window instance based on its configuration. If the window is
			 * configured destroy on-close will recreate it each time.
			 * 
			 * @param {String}
			 *            name Window name
			 * @return {Ext.window.Window}
			 */
			_getWindow_ : function(name) {
				var cfg = this._elems_.get(name);
				var w = Ext.getCmp(cfg.id);
				if (!Ext.isEmpty(w)) {
					return w;
				} else {
					if (cfg._window_) {
						return new Ext.window.Window(cfg);
					}
				}
				throw (name + ' is not a window!');
			},

			/**
			 * @public Helper method to activate a certain container of a
			 *         stacked view type.
			 * 
			 * @param svn
			 *            Stacked view name
			 * @param idx
			 *            Child view to activate, can be the index of the child
			 *            or its name as registered in the elements map.
			 */
			_showStackedViewElement_ : function(svn, idx) {
				if (Ext.isNumber(idx)) {
					this._getElement_(svn).getLayout().setActiveItem(idx);
				} else {
					var ct = this._getElement_(svn);
					var cmp = this._getElement_(idx);
					if (cmp) {
						ct.getLayout().setActiveItem(cmp);
					} else {
						ct.getLayout().setActiveItem(ct.items.indexOfKey(idx));
					}

				}
			},

			// **************** Protected API *****************

			/**
			 * @protected First method invoked during the elements setup.
			 *            Template method to override with custom logic.
			 */
			_startDefine_ : function() {

			},

			/**
			 * @protected Last method invoked during the elements setup.
			 *            Template method to override with custom logic.
			 */
			_endDefine_ : function() {

			},

			/**
			 * @protected Factory method to provide the elements definition in
			 *            each subclass. Invoked from the initComponent method.
			 */
			_defineElements_ : function() {
			},

			/**
			 * @protected Template method checked during elements definition
			 *            flow. Return false to skip the _defineElements_ call.
			 * 
			 * @return {Boolean}
			 */
			_beforeDefineElements_ : function() {
				return true;
			},

			/**
			 * @protected Template method invoked after the elements definition
			 *            flow. Used mainly to add overrides to existing
			 *            components.
			 * 
			 * @return {Boolean}
			 */
			_afterDefineElements_ : function() {
			},

			/**
			 * @protected Factory method to provide the elements linking in each
			 *            subclass. Invoked from the initComponent method.
			 *            Linking means to build up the components hierarchy. It
			 *            can be specified in the elements definition also but
			 *            this way makes life easier in the overrides to
			 *            re-arrange the components.
			 */
			_linkElements_ : function() {
			},

			/**
			 * @protected Template method checked during elements linking flow.
			 *            Return false to skip the _linkElements_ call.
			 * 
			 * @return {Boolean}
			 */
			_beforeLinkElements_ : function() {
				return true;
			},

			/**
			 * @protected Template method invoked after the elements linking.
			 *            Used mainly to add overrides to existing components.
			 */
			_afterLinkElements_ : function() {
			},

			// **************** Private methods *****************

			_runElementBuilder_ : function() {

				this._elems_ = new Ext.util.MixedCollection();

				this._startDefine_();

				if (this._beforeDefineElements_() !== false) {
					this._defineElements_();
					this._afterDefineElements_();
				}
				this._elems_.each(this._postProcessElem_, this);

				if (this._beforeLinkElements_() !== false) {
					this._linkElements_();
					this._afterLinkElements_();
				}

				this._endDefine_();

				this.items = [this._elems_.get(this._mainViewName_)];

			},
			
			_beforeDestroyDNetView_: function() {				
				for (var p in this._refs_) {
					delete this._refs_[p];
				}
			}
			
		});




Ext.define("dnet.core.base.HomePanel" , {
  extend: "Ext.Panel",
  alias:"widget.dnetHomePanel",
  
   _TEXT_TITLE : Dnet.translate("appmenuitem", "home__lbl")
  ,_TEXT_STD_MENU : Dnet.translate("appmenuitem", "appmenus__lbl")
  ,_TEXT_USR_MENU : Dnet.translate("appmenuitem", "bookmark__lbl")


  , _Menu_StdMenuId_: "dnet-application-view-menu-std-menu"
  , _Menu_UsrMenuId_: "dnet-application-view-menu-usr-menu"
  , _Menu_AccordeonId_: "dnet-application-view-menu-accordeon"

   ,initComponent:function(config) {
		var tr = dnet.core.base.TemplateRepository;
	    var menuConfig = [];
	    

	    for(var k in Dnet.navigationTreeMenus) {
	    	menuConfig[menuConfig.length] = Dnet.navigationTreeMenus[k];
	    } 
		var navigAccordeonCfg = {
	          layout: 'accordion'  
	         ,layoutConfig: {  animate: false,
        		activeOnTop: true // ,fill: true          
             } 
		 	 
        	,id:this._Menu_AccordeonId_
			,region:'west'
			,width:350
			,split:true
            ,minSize: 200 
            ,maxSize: 500
           ,title:"Application menus"
           ,collapsible:true
            ,items:[]
             
        }

		for (var i=0; i < menuConfig.length; i++) {
            navigAccordeonCfg.items[i] = {
            		title: menuConfig[i]["title"], 
            		layout:{ type: 'fit' } 
            		, items:[
					 	{
					 		xtype:"dnetNavigationTree", 
					 		id:"dnet-application-view-menu-"+menuConfig[i]["name"], 
					 		_menuId_: 1,
					 		_menuName_: menuConfig[i]["name"],
					 		withStdFilterHeader:true,loader_PreloadChildren: true						 	 
//						  
					 	//,_data_: menuConfig[i]["children"]	
						 	,listeners: {
									openMenuLink: {scope:this
										, fn: function(model) {	
					 						var bundle = model.raw.bundle;
					 						var frame = model.raw.frame;
					 		
				                            var path = Dnet.buildUiPath(bundle, frame, false);
				                            getApplication().showFrame(frame, {url:path  } );
										}
										}
								}
						}
					]};
		}
		 
		var html = '<div style="width:100%;height:100%;overflow: hidden;" id="div_dnet_dashboard" ><iframe id="iframe_dnet_dashboard" name="iframe_dnet_dashboard"';
		html += ' src="dashboard" ';
		html += '" style="border:0;width:100%;height:100%;overflow: hidden" FRAMEBORDER="no"></iframe></div>';
		
        var cfg = {
           layout:"border"
          ,title:this._TEXT_TITLE
          ,items:[
              //{region:"center", frame:true, html:"<div style='text-align:center;width:100%;font-size:16px;font-weight:bold;padding-top:40px;'><span>Welcome to "+Dnet.name+"</span></div>" }
             // {region:"center", frame:true, tpl:dnet.core.base.TemplateRepository.APPLICATION_HOME, data:{dnetName: Dnet.name, dnetVersion:Dnet.version} }
             
               {region:"center", frame:true, 
               	layout : 'fit',
				html :  html
               }
              
              ,{region:"south", border: false, frame:true,  tpl: tr.get(tr.HOMEPANEL_FOOTER)
	    	             	, data:{}, id:"dnet-application-view-footer" }
              ,navigAccordeonCfg
          ]
        }
		Ext.apply(this, cfg);
        this.callParent(arguments);
   }

});

 

function getProtocolAPI(resource, format) {
   return {
            read :'tx/java/ds/'+resource+'.'+format+'?action=find',
            export1 : 'tx/java/ds/'+resource+'.'+format+'?action=export',
            create : 'tx/java/ds/'+resource+'.'+format+'?action=insert',
            update: 'tx/java/ds/'+resource+'.'+format+'?action=update',
            destroy: 'tx/java/ds/'+resource+'.'+format+'?action=delete'

        }
}

function getProtocolAPI_AsgnLeft(resource, format) {
   return {
            read : 'tx/java/asgn/'+resource+'.'+format+'?action=findLeft',
            export1 : 'tx/java/asgn/'+resource+'.'+format+'?action=export',
            create : 'tx/java/asgn/'+resource+'.'+format+'?action=insert',
            update: 'tx/java/asgn/'+resource+'.'+format+'?action=update',
            destroy: 'tx/java/asgn/'+resource+'.'+format+'?action=delete'
        }
}

function getProtocolAPI_AsgnRight(resource, format) {
   return {
            read : 'tx/java/asgn/'+resource+'.'+format+'?action=findRight',
            export1 : 'tx/java/asgn/'+resource+'.'+format+'?action=export',
            create : 'tx/java/asgn/'+resource+'.'+format+'?action=insert',
            update: 'tx/java/asgn/'+resource+'.'+format+'?action=update',
            destroy: 'tx/java/asgn/'+resource+'.'+format+'?action=delete'
        }
}



dnet.core.base.RequestParamFactory = {

	 findRequest: function(filter) {
	 	var p = {
		         resultStart:0
				,resultSize:30
			  	,data:filter || {}
			};
		return p;
 	 }
};



dnet.core.base.UrlBuilder =  {
	  
	  baseUrl: function(pDc, pFormat) {
	  	var o = {};
		o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] = __Http__.RESOURCE_TYPE_DC;
		o[__Http__.REQUEST_PARAM_RESOURCE_NAME] = pDc;
		o[__Http__.REQUEST_PARAM_DATA_FORMAT] = pFormat;
		return this._buildUrl_(o);
	}

    // private
	,_buildUrl_: function(o) {
		if (!(o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] && o[__Http__.REQUEST_PARAM_RESOURCE_NAME] && o[__Http__.REQUEST_PARAM_DATA_FORMAT]) )  {
			alert("Invalid URL builder request! Missing one of: resource-type, resource-name, data-format");
			return;
		}
        var url = __SERVER_DOMAIN_URL__+"/dnet";
        if (o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] == __Http__.RESOURCE_TYPE_LOV) {
        	url += __PATH_FOR_LOV__;
		}
		if (o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] == __Http__.RESOURCE_TYPE_DC) {
        	url += __PATH_FOR_DC__;
		}
        url += "/"+o[__Http__.REQUEST_PARAM_RESOURCE_NAME]+"."+o[__Http__.REQUEST_PARAM_DATA_FORMAT]+"?";

		var i=0;
		for(var p in o ) {
			if (p != __Http__.REQUEST_PARAM_RESOURCE_TYPE && 
				p != __Http__.REQUEST_PARAM_RESOURCE_NAME &&
				p != __Http__.REQUEST_PARAM_DATA_FORMAT 
				) {
				url += (i>0)? "&":"";
				url += p+"="+o[p];
				i++;
			}
		}
		
	 
		return url;
	}



















     ,urlFetch: function(pDc, pFormat) {
		return this.urlFetchDc(pDc, pFormat);
	}
    ,urlFetchDc: function(pDc, pFormat) {
		var o = {};
		o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] = __Http__.RESOURCE_TYPE_DC;
		o[__Http__.REQUEST_PARAM_RESOURCE_NAME] = pDc;
		o[__Http__.REQUEST_PARAM_ACTION] = __Http__.REQUEST_PARAM_ACTION_FETCH;
		o[__Http__.REQUEST_PARAM_DATA_FORMAT] = pFormat;
		return this._buildUrl(o);
	}
    ,urlFetchLov: function(pLov, pFormat) {
		var o = {};
		o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] = __Http__.RESOURCE_TYPE_LOV;
		o[__Http__.REQUEST_PARAM_RESOURCE_NAME] = pLov;
		o[__Http__.REQUEST_PARAM_ACTION] = __Http__.REQUEST_PARAM_ACTION_FETCH;
		o[__Http__.REQUEST_PARAM_DATA_FORMAT] = pFormat;
		return this._buildUrl(o);
	}

    ,urlExport: function(pDc, pFormat, pOptions) {
		return this.urlExportDc(pDc, pFormat, pOptions);
	}
    
    ,urlExportDc: function(pDc, pFormat, pOptions) {
		var o = pOptions||{};
		o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] = __Http__.RESOURCE_TYPE_DC;
		o[__Http__.REQUEST_PARAM_RESOURCE_NAME] = pDc;
		o[__Http__.REQUEST_PARAM_ACTION] = __Http__.REQUEST_PARAM_ACTION_EXPORT;
		o[__Http__.REQUEST_PARAM_DATA_FORMAT] = pFormat;
		return this._buildUrl(o);
	}


    ,urlInsert: function(pDc, pFormat) {
		return this.urlSave(pDc, pFormat, true);
	}
	,urlUpdate: function(pDc, pFormat) {
		return this.urlSave(pDc, pFormat, false);
	}
    ,urlSave: function(pDc, pFormat, pIsInsert) {
		var o = {};
		o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] = __Http__.RESOURCE_TYPE_DC;
		o[__Http__.REQUEST_PARAM_RESOURCE_NAME] = pDc;
		o[__Http__.REQUEST_PARAM_ACTION] = (pIsInsert)?__Http__.REQUEST_PARAM_ACTION_INSERT:__Http__.REQUEST_PARAM_ACTION_UPDATE;
		o[__Http__.REQUEST_PARAM_DATA_FORMAT] = pFormat;
		return this._buildUrl(o);
	}

    ,urlDelete: function(pDc, pFormat, pIsInsert) {
		var o = {};
		o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] = __Http__.RESOURCE_TYPE_DC;
		o[__Http__.REQUEST_PARAM_RESOURCE_NAME] = pDc;
		o[__Http__.REQUEST_PARAM_ACTION] = __Http__.REQUEST_PARAM_ACTION_DELETE;
		o[__Http__.REQUEST_PARAM_DATA_FORMAT] = pFormat;
		return this._buildUrl(o);
	}

	,urlService: function(pDc, pFormat, pServiceType, pServiceName) {
		var o = {};
		o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] = __Http__.RESOURCE_TYPE_DC;
		o[__Http__.REQUEST_PARAM_RESOURCE_NAME] = pDc;
		o[__Http__.REQUEST_PARAM_ACTION] = __Http__.REQUEST_PARAM_ACTION_CUSTOM;
		o[__Http__.REQUEST_PARAM_CUSTOM_ACTION] = pServiceName;
		o[__Http__.REQUEST_PARAM_DATA_FORMAT] = pFormat;
		o[__Http__.REQUEST_PARAM_CONTROL] = pServiceType;

		return this._buildUrl(o);
	}

	 // private
	,_buildUrl: function(o) {
		if (!(o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] && o[__Http__.REQUEST_PARAM_RESOURCE_NAME] && o[__Http__.REQUEST_PARAM_DATA_FORMAT]) )  {
			alert("Invalid URL builder request! Missing one of: resource-type, resource-name, data-format");
			return;
		}
        var url = __SERVER_DOMAIN_URL__;
        if (o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] == __Http__.RESOURCE_TYPE_LOV) {
        	url += __PATH_FOR_LOV__;
		}
		if (o[__Http__.REQUEST_PARAM_RESOURCE_TYPE] == __Http__.RESOURCE_TYPE_DC) {
        	url += __PATH_FOR_DC__;
		}
        url += "/"+o[__Http__.REQUEST_PARAM_RESOURCE_NAME]+"."+o[__Http__.REQUEST_PARAM_DATA_FORMAT]+"?";

		var i=0;
		for(var p in o ) {
			if (p != __Http__.REQUEST_PARAM_RESOURCE_TYPE && 
				p != __Http__.REQUEST_PARAM_RESOURCE_NAME &&
				p != __Http__.REQUEST_PARAM_DATA_FORMAT 
				) {
				url += (i>0)? "&":"";
				url += p+"="+o[p];
				i++;
			}
		}
		
	 
		return url;
	}

};









/**
 * Application header view. 
 */
Ext.define("dnet.core.base.UiHeader", {
	extend:  "Ext.toolbar.Toolbar" ,
 
	initComponent: function(config) {
		var items =   [

			 {xtype:"spacer", width:10}
			,{xtype:"tbtext" , text:this.headerTitle || "Title", style:"font-weight:bold;font-size:12px;"  }

			//, "-"
			//,{xtype:"tbtext" , text:"-", style:"font-weight:bold;" , id: this.clientTxtID  }
			//, "->"
			//,{xtype:"tbtext" , html:"<span style='align:right'><span style='font-weight:bold'>DNet Enterprise Suite </span><br><span>Version: "+Dnet.version+"</span></span>", cls:"app-header-text" }

		];

		if (! (this["showSubpath"] === false ) ) {
            items[2]  = "-";
            items[3]  = {xtype:"tbtext" , text:"List", style:"font-weight:bold;" , id: this.clientTxtID  };
		}
		var cfg = {
			border: false, frame: false, width:1000, height:25
			,items: items
		}
		Ext.apply(cfg,config);
	    Ext.apply(this,cfg);
		 
		this.callParent(arguments);
 
	}

	,setTitle: function(v) {
		this.items.itemAt(1).setText(v);
	}
	
	,setCanvas: function(v) {
		this.items.itemAt(3).setText(v);
	}
});dnet.core.base.FrameNavigatorWithIframe = {

	/**
	 * Maximum number of tabs (application frames) which are allowed to be
	 * opened at a certain moment. Use -1 for unlimited.
	 * 
	 * @type Integer
	 */
	maxOpenTabs : -1,

	/**
	 * Lookup the application frame instance.
	 * 
	 * @param {}
	 *            frame
	 * @return {}
	 */
	getFrameInstance : function(frame) {
		var theIFrame = window.frames[__CmpId__.FRAME_IFRAME_PREFIX + frame];
		var theFrameInstance = null;
		if (theIFrame) {
			theFrameInstance = theIFrame.theFrameInstance;
		}
		return theFrameInstance;
	},

	/**
	 * Check if the given frame is already open.
	 * 
	 * @param {}
	 *            frame Frame name
	 * @return {}
	 */
	isFrameOpened : function(frame) {
		return !Ext.isEmpty(document
				.getElementById(__CmpId__.FRAME_IFRAME_PREFIX + frame))
	},

	/**
	 * Check if the given application frame is active, i.e. the corresponding
	 * tab panel is active.
	 * 
	 * @param {}
	 *            frame
	 * @return {}
	 */
	isFrameActive : function(frame) {
		return (getApplication().getViewBody().getActiveTab().getId() == __CmpId__.FRAME_TAB_PREFIX);
	},

	/**
	 * Show the given frame. If it is open activate it otherwise open a new tab
	 * and load it there.
	 * 
	 * @param {}
	 *            frame
	 * @param {}
	 *            params
	 */
	showFrame : function(frame, params) {
		this._showFrameImpl(frame, params);
	},

	/**
	 * Internal implementation function.
	 * 
	 * @param {}
	 *            frame
	 * @param {}
	 *            params
	 */
	_showFrameImpl : function(frame, params) {

		if (!(params && params.url)) {
			alert("Programming error: params.url not specified in showFrame!");
			return;
			if (!params)
				params = {};
			var type = getApplication().type;
			params.url = "ClientExtjs/index.jsp?type=" + type + "-dlg&item="
					+ frame;
		}

		var resourceType = (params.resourceType) ? params.resourceType : "";
		var tabID = __CmpId__.FRAME_TAB_PREFIX + resourceType + frame;
		var ifrID = __CmpId__.FRAME_IFRAME_PREFIX + resourceType + frame;
		var vb = getApplication().getViewBody();

		if (Ext.isEmpty(document.getElementById(ifrID))
				&& !Ext.isEmpty(window.frames[ifrID])) {   
			delete window.frames[ifrID];
		}

		if (this.isFrameOpened(frame)) {
			if (!this.isFrameActive(frame)) {
				vb.setActiveTab(tabID);
			}
		} else {
			if (this.maxOpenTabs > 0
					&& ((vb.items.getCount() + 1) == this.maxOpenTabs)) {
				Ext.Msg
						.alert(
								'Warning',
								'You have reached the maximum number of opened tabs ('
										+ (this.maxOpenTabs)
										+ ').<br> It is not allowed to open more tabs.');
				return;
			}
			vb.add(new Ext.Panel({
				
				onDestroy: function() { 
					Ext.destroy(window.frames[this.n21_iframeID].__theViewport__);
					Ext.destroy(window.frames[this.n21_iframeID].theFrameInstance);			 
					try {
						delete window.frames[this.n21_iframeID];
					} catch (e) {
						//alert(e);
					}
					this.callParent();
				},
				
				
				title : (resourceType != "")
						? resourceType + ":" + frame
						: frame,
				id : tabID,
				n21_iframeID : ifrID,
				autoScroll : true,
				 
				layout : 'fit',
				closable : true,
				html : '<div style="width:100%;height:100%;overflow: hidden;" id="div_'
						+ frame
						+ '" ><iframe id="'
						+ ifrID
						+ '" name="'
						+ ifrID
						+ '" src="'
						+ params.url
						+ '" style="border:0;width:100%;height:100%;overflow: hidden" FRAMEBORDER="no"></iframe></div>'
				 
			}));
			vb.setActiveTab(tabID);
		}
	}
};

dnet.core.base.FrameNavigator = {
 
	 maxOpenTabs : -1
	
	,isFrameOpened:  function(frame) {
		 return !Ext.isEmpty(document.getElementById(__CmpId__.FRAME_IFRAME_PREFIX+frame))
	}
		
	,isFrameActive:  function(frame) {
		return (getApplication().getViewBody().getActiveTab().getId() == __CmpId__.FRAME_TAB_PREFIX);  
	}	
	
	
	,showFrame: function(frame, params) {				
		this._showFrameImpl(frame, params);
	}
	
	 
	,_showFrameImpl: function(frame, params) {
		
		var tabID = __CmpId__.FRAME_TAB_PREFIX+frame;
		 	
		var vb = getApplication().getViewBody();
		
		 

        if (this.isFrameOpened(frame)) {
          if (!this.isFrameActive(frame)) {           
        	  getApplication().getViewBody().activate(tabID);
          }
        } else {
          if (this.maxOpenTabs > 0 && ((vb.items.getCount()+1) == this.maxOpenTabs )) { // add one for the home tab
               Ext.Msg.alert('Warning','You have reached the maximum number of opened tabs ('+(this.maxOpenTabs)+').<br> It is not allowed to open more tabs.');
               return;
            }
          vb.add({
        	   xtype:frame
        	   ,closable:true
          });
           
               vb.activate(tabID);
        }
      }
};

  
Ext.define("dnet.core.base.FileUploadWindow" , {
	extend:"Ext.Window",
	
	_nameFieldValue_: null, 
	_uploadUrl_: null,
	
	_p1Value_: null,
	_p2Value_: null,
	
	_succesCallbackFn_:null,
	_succesCallbackScope_:null,
	
	 initComponent: function(config) {
		var cfg = {
				title:Dnet.translate("msg", "upload_title"),
					border: true
					,width: 400
					,height:140
					,resizable: true					 
					,closable: true
									 
					,constrain:true
					,layout:"fit"
					
					,modal:true
					,items: [{
						xtype:"form"
						,padding:5	
						,fieldDefaults:{
						  	   labelAlign:"right",
						  	   labelWidth:90  
						  	}
					 
						,frame:true
						,fileUpload: true
						,buttonAlign:"center"
						,items:[
					      {xtype:"textfield", value:this._nameFieldValue_, name:"name", fieldLabel:Dnet.translate("msg", "upload_name"), anchor:"-10" , selectOnFocus: true, allowBlank: true}
					     ,{xtype:"fileuploadfield", name:"file", fieldLabel:Dnet.translate("msg", "upload_file"), anchor:"-10" , selectOnFocus: true, allowBlank: false }
					     ,{xtype:"hidden", name:"p1", value:this._p1Value_}
					     ,{xtype:"hidden", name:"p2", value:this._p2Value_}
					 ]
					}]
				    ,buttons:[
				        {xtype:"button", text:Dnet.translate("msg", "upload_btn"), scope:this, handler: this.doUpload }    	         
				    ] 
			};

		Ext.apply(cfg,config);
        Ext.apply(this,cfg);

		dnet.core.base.FileUploadWindow.superclass.initComponent.call(this);
 
 	}
    ,getButton: function() {
		return this.items.get(0).buttons[0];
	}
	,getNameField: function() {
		return this.items.get(0).items.get(0);
	}
	,getFileField: function() {
		return this.items.get(0).items.get(1);
	}
	,getP1Field: function() {
		return this.items.get(0).items.get(2);
	}
	,getP2Field: function() {
		return this.items.get(0).items.get(3);
	}
 
	,getForm: function() {
		return this.items.get(0).getForm();
	}  
	,doUpload: function(btn, evnt) {
         if(this.getForm().isValid()){
            this.getForm().submit({
                url: this._uploadUrl_,
                waitMsg: 'Uploading...',
                scope: this,
                success: function(form, action){
            		Ext.Msg.hide();
            		this.close();
            		if (this._succesCallbackFn_ != null ) {
            			this._succesCallbackFn_.call(this._succesCallbackScope_ || this);
            		}             		
                }
                ,failure: function(form, action) {
                	Ext.Msg.hide();
			        switch (action.failureType) {
			            case Ext.form.Action.CLIENT_INVALID:
			                Ext.Msg.alert('Failure', 'Form fields may not be submitted with invalid values');
			                break;
			            case Ext.form.Action.CONNECT_FAILURE:
			                Ext.Msg.alert('Failure', action.response.responseText);
			                break;
			            case Ext.form.Action.SERVER_INVALID:
			               Ext.Msg.alert('Failure', action.result.msg);
			       }
			       this.close();
			    }
            });
        } else {
        	Ext.Msg.alert("",'No file selected. Nothing to upload');
        }
        
	}

	,clearFields: function() {
		this.items.each( function(item) { item.setValue(null); } , this);
	}
    ,clearInvalid: function() {
		this.getNameField().clearInvalid();
		this.getFileField().clearInvalid();
	}
	 
	 
});

Ext.define("dnet.core.base.WorkflowFormWithHtmlWindow", {
	extend:  "Ext.Window" ,
 
	constructor: function(config) {
		var cfg = {			 
			 border: true
			,width: 500
			,height:500
			,resizable: true					 
			,closable: true								 
			,constrain:true
			,layout:"fit"				
			,modal:true		
			,buttonAlign:"center"
		    ,buttons:[
		        {xtype:"button", text:Dnet.translate("tlbitem", "save__lbl"), scope:this, handler: this.doSave }    	         
		    ] 
			};
		var resolved = false;
		if (config._wfConfig_.type == "startform") {
			var submitUrl=Dnet.wfProcessInstanceAPI().start;
			
			var startHtml = "<form id='dnet-workflow-form' action='"+submitUrl+"'>" +
					"<input type='hidden' name='processDefinitionId' value='"+config._wfConfig_.processDefinitionId+"'>" +
					"<input type='hidden' name='processDefinitionKey' value=''>" +
					"<input type='hidden' name='businessKey' value=''>";
			var endHtml = "</form>";
			resolved = true;
		} 
		if (config._wfConfig_.type == "taskform") {
			var submitUrl=Dnet.wfTaskAPI(config._wfConfig_.taskId).complete;
			
			var startHtml = "<form id='dnet-workflow-form' action='"+submitUrl+"'>";					 
			var endHtml = "</form>";
			resolved = true;
		} 
		
		if(!resolved) {
			throw("Invalid value for _wfConfig_.type in WorkflowFormWithHtmlWindow.");
		}
		config.html = startHtml + config.html + endHtml;
		Ext.apply(cfg,config);
	   
	
		dnet.core.base.WorkflowFormWithHtmlWindow.superclass.constructor.call(this, cfg);

	}
	
	, initComponent: function() {
		  
		dnet.core.base.WorkflowFormWithHtmlWindow.superclass.initComponent.call(this);
 
 	}
    ,getButton: function() {
		return this.items.get(0).buttons[0];
	}
	  
	,doSave: function(btn, evnt) {
        var frm = document.getElementById("dnet-workflow-form");
        var p = {},elements = frm.elements,len = elements.length;       
        for(var i=0;i<len; i++) {
        	p[elements[i].name] = elements[i].value;        	
        }
        Ext.Ajax.request({
			url: frm.action, method:"POST", params: p
			,success :this.onSaveSuccess
			,failure: this.onSaveFailure	
			,scope: this			 
		});
	}
	,onSaveSuccess: function(response,options) {
		this.close();
	}
	,onSaveFailure: function(response,options) {
		Ext.MessageBox.hide();
		var msg, withDetails=false;
		if (response.responseText) {
			if (response.responseText.length > 2000) {
				msg = response.responseText.substr(0,2000);
				withDetails = true;
			} else {
				msg = response.responseText ;
			}
		} else {
			msg = "No response received from server.";
		}
		var alertCfg = { msg: msg, scope:this, icon: Ext.MessageBox.ERROR, buttons: Ext.MessageBox.OK }  
		if (withDetails) {
			alertCfg.buttons['cancel'] = 'Details';
			alertCfg['detailedMessage'] = response.responseText;
		}		  
      	Ext.Msg.show(alertCfg);
	} 
});

Ext.define("dnet.core.base.WfAbstractFormWindowExtjs", {
	extend:  "Ext.Window" ,
 
	border: true,
	width: 500,
	height:300,
	resizable: true,					 
	closable: true,								 
	constrain:true,
	layout:"fit",				
	modal:true,		
	buttonAlign:"center",
	_formProperties_: null,		
	 
 	
 	_buildFormFields_:function(fields) { 
 		
 		for(var i=0,l=this._formProperties_.length; i<l; i++){
 			var p = this._formProperties_[i];
 			var f = {
 				name : p.id,
 				fieldLabel : p.name,
 				value : p.value
 			}
 			if (p.isRequired) {
 				f.allowBlank = false;
 				f.labelSeparator = "*";
 			}
 			if (!p.isWritable) {
 				f.readOnly = true;
 			}
 			if(p.type.name == "string") {
 				f.xtype = "textfield";
 			}
 			if(p.type.name == "long") {
 				f.xtype = "numberfield";
 			}
 			if(p.type.name == "date") {
 				f.xtype = "datefield";
 				f.format = "Y-m-d"; 
 				
 			} 
 			if(p.type.name == "enum") {
 				f.xtype = "combobox";
 				f.store = p.type.values;
 			}
 			fields[fields.length] = f;
 		} 		 
 	} 
});

Ext.define("dnet.core.base.WfTaskFormWindowExtjs", {
	extend:  "dnet.core.base.WfAbstractFormWindowExtjs" ,
  
	_taskId_ : null,
	_taskName_ : null,
	_taskDescription_ : null,
	 
	initComponent: function() {		
		 
		this.items = {
			xtype:"form",
			url: Dnet.wfTaskAPI(this._taskId_).complete,			
			bodyPadding: 10,
			//width: 500,
			layout: 'anchor',
			fieldDefaults : {
				labelAlign : "right",
				labelWidth : 130
			},
			frame:true,
		    defaults: {
		        anchor: '100%'
		    },
			items: this._buildForm_(),
			
			buttons: [ {
			        text: 'Submit',
			        formBind: true, //only enabled once the form is valid
			        disabled: true,
			        handler: function() {
			            var form = this.up('form').getForm();
			            if (form.isValid()) {
			            	Ext.Msg.wait("Working...");
			                form.submit({
			                    success: function(form, action) {
			                       Ext.Msg.hide();
			                       Ext.Msg.alert('Success', "Task completed successfully");
			                       this.up('window').close();
			                    },
			                    failure: function(form, action) {
			                    	Ext.Msg.hide();
			                        Ext.Msg.alert('Failed', action.response.responseText);
			                    },
			                    scope:this
			                });
			            }
			        }
			    },{
			        text: 'Reset',
			        handler: function() {
			            this.up('form').getForm().reset();
			        }
			    }]
		}
		this.callParent(arguments);
 	},
    
 	
 	_buildForm_:function() {
 		var fields = [];
 		
 		fields[0] = {
 			xtype:"hidden",
 			name:"taskId",
 			value:this._taskId_
 		}
 		fields[1] = {
 			xtype:"container",
 			padding:15, 			
 			html: this._taskDescription_
 		};
 		 
 		this._buildFormFields_(fields);
 		return fields;
 	} 
 	
   
});

Ext.define("dnet.core.base.WfStartFormWindowExtjs", {
	extend:  "dnet.core.base.WfAbstractFormWindowExtjs" ,
 
 	
	_processDefinitionId_ : null,
	_processDefinitionKey_ : null,
	_businessKey_ : null,
	 
	initComponent: function() {		
		 
		this.items = {
			xtype:"form",
			url: Dnet.wfProcessInstanceAPI(null).start,			
			bodyPadding: 10,
			//width: 500,
			layout: 'anchor',
			fieldDefaults : {
				labelAlign : "right",
				labelWidth : 130
			},
			frame:true,
		    defaults: {
		        anchor: '100%'
		    },
			items: this._buildForm_(),
			
			buttons: [ {
			        text: 'Submit',
			        formBind: true, //only enabled once the form is valid
			        disabled: true,
			        handler: function() {
			            var form = this.up('form').getForm();
			            if (form.isValid()) {
			            	Ext.Msg.wait("Working...");
			                form.submit({
			                    success: function(form, action) {
			                       Ext.Msg.hide();
			                       Ext.Msg.alert('Success', "Process started successfully");
			                       this.up('window').close();
			                    },
			                    failure: function(form, action) {
			                    	Ext.Msg.hide();
			                        Ext.Msg.alert('Failed', action.response.responseText);
			                    },
			                    scope:this
			                });
			            }
			        }
			    },{
			        text: 'Reset',
			        handler: function() {
			            this.up('form').getForm().reset();
			        }
			    }]
		}
		this.callParent(arguments);
 	},
    
 	
 	_buildForm_:function() {
 		var fields = [];
 		
 		fields[0] = {
 			xtype:"hidden",
 			name:"processDefinitionId",
 			value:this._processDefinitionId_
 		}
 		fields[1] = {
 			xtype:"hidden",
 			name:"processDefinitionKey",
 			value:this._processDefinitionKey_
 		}
 		fields[2] = {
 			xtype:"hidden",
 			name:"businessKey",
 			value:this._businessKey_
 		}
 		this._buildFormFields_(fields);
 		return fields;
 	} 
});
dnet.core.base.WorkflowFormFactory =  {
	
	createStartForm: function(processDefinitionId) {
		 
		var succesFn = function(response , options) {			
			 
			var w= new dnet.core.base.WorkflowFormWithHtmlWindow({ 
					html: response.responseText
					,_wfConfig_:{
						type: "startform"
						,processDefinitionId:processDefinitionId
					} 
				});
			w.show();			 
		};		 
		
		Ext.Ajax.request({
			url: Dnet.wfProcessDefinitionAPI(processDefinitionId).form
			,method:"GET"
			,success :succesFn
			,failure: function() {alert('error');}	
			,scope: this
			//,options: { action: "doService", serviceName: serviceName, specs: s }
		});
	 
	}

	,createTaskForm: function(taskId) {
		 
		var succesFn = function(response , options) {			
			 
			var w= new dnet.core.base.WorkflowFormWithHtmlWindow({ 
					html: response.responseText
					,_wfConfig_:{
						type: "taskform"
						,taskId:taskId
					} 
				});
			w.show();			 
		};		 
		
		Ext.Ajax.request({
			url: Dnet.wfTaskAPI(taskId).form
			,method:"GET"
			,success :succesFn
			,failure: function() {alert('error');}	
			,scope: this
			//,options: { action: "doService", serviceName: serviceName, specs: s }
		});	 
	}	 
};
 Ext.define("dnet.core.asgn.AbstractAsgn", {

	mixins : {
		observable : 'Ext.util.Observable'
	},

	/**
	 * Store for available records
	 * 
	 * @type Ext.data.Store
	 */
	storeLeft : null,

	/**
	 * Store for selected records
	 * 
	 * @type Ext.data.Store
	 */
	storeRight : null,

	/**
	 * Various runtime configuration properties.
	 */
	tuning : {

		/**
		 * Number of milliseconds before execute the query. Used if value>0
		 */
		queryDelay : 150

		/**
		 * Page-size for a query
		 */
		,
		fetchSize : 30
	},

	/**
	 * Parameters model instance
	 */
	params : null,
	/**
	 * Filter object instance. Contains the left filter and right filter
	 */
	filter : null,

	/**
	 * Data model signature - record constructor.
	 */
	recordModel : null,

	/**
	 * Parameters model signature - record constructor.
	 */
	paramModel : null,

	constructor : function(config) {
		config = config || {};
		Ext.apply(this, config);
		this.params = {
			objectId : 8,
			selectionId : 0,
			clientId : null
		}
		this.filter = {
			left : {
				field : null,
				value : null
			},
			right : {
				field : null,
				value : null
			}
		}
		if (this.storeLeft == null) {
			this.storeLeft = this.createStore("Left");
		}

		if (this.storeRight == null) {
			this.storeRight = this.createStore("Right");
		}

		this.addEvents("afterDoSaveSuccess");
		this.mixins.observable.constructor.call(this);
	},

	// **************** Public API *****************

	initAssignement : function() {
		this.params.clientId = getApplication().getSession().getClient().id;
		this.doSetup();
	},

	doReset : function() {
		this.doResetImpl();
	},

	/**
	 * Call the setup server-side procedure which prepares a temporary data with
	 * the existing selections for this context.
	 */
	doSetup : function() {
		this.doSetupImpl();
	},

	doCleanup : function() {
		this.doCleanupImpl();
	},

	/**
	 * Save changes.
	 */
	doSave : function() {
		this.doSaveImpl();
	},

	/**
	 * Load the available records.
	 */
	doQueryLeft : function() {
		this.doQueryLeftImpl();
	},

	/**
	 * Query the selected records.
	 */
	doQueryRight : function() {
		this.doQueryRightImpl();
	},

	/**
	 * Select all available.
	 */
	doMoveRightAll : function() {
		this.doMoveRightAllImpl();
	},

	/**
	 * Remove all selected
	 */
	doMoveLeftAll : function() {
		this.doMoveLeftAllImpl();
	},

	/**
	 * Select those records which are selected by the user in the available options list. 
	 * @param {Ext.grid.Panel} theLeftGrid
	 * @param {Ext.grid.Panel} theRightGrid
	 */
	doMoveRight : function(theLeftGrid, theRightGrid) {
		this.doMoveRightImpl(theLeftGrid, theRightGrid);
	},

	/**
	 * Remove those records which are selected by the user in the selected options list. 
	 * @param {} theLeftGrid
	 * @param {} theRightGrid
	 */
	doMoveLeft : function(theLeftGrid, theRightGrid) {
		this.doMoveLeftImpl(theLeftGrid, theRightGrid);
	},

	/**
	 * Return the store for the specified side.
	 * @param {String} side 
	 * @return {Ext.data.Store}
	 */
	getStore : function(side) {
		if (side == "left") {
			return this.storeLeft;
		}
		if (side == "right") {
			return this.storeRight;
		}
	},

	afterMoveLeftSuccess : function(response, options) {
		this.doQueryLeft();
		this.doQueryRight();
	},

	afterMoveRightSuccess : function(response, options) {
		this.doQueryLeft();
		this.doQueryRight();

	},

	afterMoveLeftAllSuccess : function(response, options) {
		this.doQueryLeft();
		this.doQueryRight();

	},

	afterMoveRightAllSuccess : function(response, options) {
		this.doQueryLeft();
		this.doQueryRight();
	},

	afterDoSetupSuccess : function(response, options) {
		this.params["selectionId"] = response.responseText;
		this.doQueryLeft();
		this.doQueryRight();
	},

	afterDoResetSuccess : function(response, options) {
		this.doQueryLeft();
		this.doQueryRight();
	},

	afterDoSaveSuccess : function(response, options) {
		Ext.Msg.hide();
		this.fireEvent("afterDoSaveSuccess", this);
	},

	// **************** Private API *****************

	doSetupImpl : function() {
		Ext.Ajax.request({
					params : this.params,
					method : "POST",
					failure : this.afterAjaxFailure,
					success : this.afterDoSetupSuccess,
					scope : this,
					url : Dnet.asgnUrl + "/" + this.dsName
							+ ".json?action=setup",
					timeout : 600000

				});
	},

	doMoveLeftImpl : function(theLeftGrid, theRightGrid) {
		var selection = theRightGrid.getSelectionModel().getSelection();
		if (selection.length == 0) {
			return;
		}
		var p_selected_ids = "";
		for (var i = 0; i < selection.length; i++) {
			p_selected_ids += (i > 0) ? "," : "";
			p_selected_ids += selection[i].data.id;
		}
		var p = Ext.apply({
					p_selected_ids : p_selected_ids
				}, this.params);
		Ext.Ajax.request({
					params : p,
					method : "POST",
					failure : this.afterAjaxFailure,
					success : this.afterMoveLeftSuccess,
					scope : this,
					url : Dnet.asgnUrl + "/" + this.dsName
							+ ".json?action=moveLeft",
					timeout : 600000,
					options : {
						action : "moveLeft",
						fnSuccess : null,
						fnSuccessScope : null,
						fnFailure : null,
						fnFailureScope : null,
						serviceName : name
					}
				});

	},

	doMoveRightImpl : function(theLeftGrid, theRightGrid) {
		var selection = theLeftGrid.getSelectionModel().getSelection();
		if (selection.length == 0) {
			return;
		}
		var p_selected_ids = "";
		for (var i = 0; i < selection.length; i++) {
			p_selected_ids += (i > 0) ? "," : "";
			p_selected_ids += selection[i].data.id;
		}
		var p = Ext.apply({
					p_selected_ids : p_selected_ids
				}, this.params);
		Ext.Ajax.request({
					params : p,
					method : "POST",
					failure : this.afterAjaxFailure,
					success : this.afterMoveRightSuccess,
					scope : this,
					url : Dnet.asgnUrl + "/" + this.dsName
							+ ".json?action=moveRight",
					timeout : 600000,
					options : {
						action : "moveRight",
						fnSuccess : null,
						fnSuccessScope : null,
						fnFailure : null,
						fnFailureScope : null,
						serviceName : name
					}
				});
	},

	doMoveLeftAllImpl : function() {
		Ext.Ajax.request({
			params : this.params,
			method : "POST",
			failure : this.afterAjaxFailure,
			success : this.afterMoveLeftAllSuccess,
			scope : this,
			url : Dnet.asgnUrl + "/" + this.dsName + ".json?action=moveLeftAll",
			timeout : 600000,
			options : {
				action : "moveLeftAll",
				fnSuccess : null,
				fnSuccessScope : null,
				fnFailure : null,
				fnFailureScope : null,
				serviceName : name
			}
		});

	},

	doMoveRightAllImpl : function() {

		Ext.Ajax.request({
					params : this.params,
					method : "POST",
					failure : this.afterAjaxFailure,
					success : this.afterMoveRightAllSuccess,
					scope : this,
					url : Dnet.asgnUrl + "/" + this.dsName
							+ ".json?action=moveRightAll",
					timeout : 600000,
					options : {
						action : "moveRightAll",
						fnSuccess : null,
						fnSuccessScope : null,
						fnFailure : null,
						fnFailureScope : null,
						serviceName : name
					}
				});
	},

	doQueryLeftImpl : function() {
		this.storeLeft.removeAll();
		var lp = {};
		var data = {};

		this.storeLeft.proxy.extraParams = Ext.apply({}, this.params);
		this.storeLeft.currentPage = 1;

		if (this.filter.left.field) {
			data[this.filter.left.field] = this.filter.left.value || '*';
			this.storeLeft.proxy.extraParams.data = Ext.encode(data);
		}

		lp[Dnet.requestParam.START] = 0;
		lp[Dnet.requestParam.SIZE] = this.tuning.fetchSize;

		this.storeLeft.load({
					params : lp
				});
		return true;
	},

	doQueryRightImpl : function() {
		this.storeRight.removeAll();
		var lp = {};
		var data = {};

		this.storeRight.proxy.extraParams =  Ext.apply({}, this.params);
		this.storeRight.currentPage = 1;

		if (this.filter.right.field) {
			data[this.filter.right.field] = this.filter.right.value || '*';
			this.storeRight.proxy.extraParams["data"] = Ext.encode(data);
		}

		lp[Dnet.requestParam.START] = 0;
		lp[Dnet.requestParam.SIZE] = this.tuning.fetchSize;

		this.storeRight.load({
					params : lp
				});
		return true;
	},

	doSaveImpl : function() {
		Ext.Ajax.request({
					params : this.params,
					method : "POST",
					failure : this.afterAjaxFailure,
					success : this.afterDoSaveSuccess,
					scope : this,
					url : Dnet.asgnUrl + "/" + this.dsName
							+ ".json?action=save",
					timeout : 600000,
					options : {
						action : "doSave",
						fnSuccess : null,
						fnSuccessScope : null,
						fnFailure : null,
						fnFailureScope : null,
						serviceName : name
					}
				});
		Ext.Msg.progress('Saving...');
	},

	doResetImpl : function() {
		Ext.Ajax.request({
					params : this.params,
					method : "POST",
					failure : this.afterAjaxFailure,
					success : this.afterDoResetSuccess,
					scope : this,
					url : Dnet.asgnUrl + "/" + this.dsName
							+ ".json?action=reset",
					timeout : 600000,
					options : {
						action : "doReset",
						fnSuccess : null,
						fnSuccessScope : null,
						fnFailure : null,
						fnFailureScope : null,
						serviceName : name
					}
				});
	},

	doCleanupImpl : function() {
		Ext.Ajax.request({
					params : this.params,
					method : "POST",
					failure : this.afterAjaxFailure
					// ,success:this.afterDoSetupSuccess
					,
					scope : this,
					url : Dnet.asgnUrl + "/" + this.dsName
							+ ".json?action=cleanup",
					timeout : 600000
				});
	},

	createStore : function(side) {

		return Ext.create("Ext.data.Store", {
					model : this.recordModel,
					remoteSort : true,
					remoteSort : true,
					autoLoad : false,
					autoSync : false,
					clearOnPageLoad : true,
					pageSize : this.tuning.fetchSize,
					proxy : {
						type : 'ajax',
						api : Dnet["asgn" + side + "API"](this.dsName, "json"),
						model : this.recordModel,
						extraParams : {
							params : {}
						},
						actionMethods : {
							create : 'POST',
							read : 'POST',
							update : 'POST',
							destroy : 'POST'
						},
						reader : {
							type : 'json',
							root : 'data',
							idProperty : 'id',
							totalProperty : 'totalCount',
							messageProperty : 'message'
						},
						writer : {
							type : 'json',
							encode : true,
							allowSingle : false,
							writeAllFields : true
						},
						listeners : {
							"exception" : {
								fn : this.proxyException,
								scope : this
							}
						},
						startParam : Dnet.requestParam.START,
						limitParam : Dnet.requestParam.SIZE,
						sortParam : Dnet.requestParam.SORT,
						directionParam : Dnet.requestParam.SENSE
					}
				});
	},

	/** ********************************************** */
	/** *********** MISCELLANEOUS HELPERS ************* */
	/** ********************************************** */

	afterAjaxFailure : function(response, options) {
		// Ext.MessageBox.hide();
		// Ext.Msg.hide();
		var msg = (response.responseText) ? response.responseText.substr(0,
				2000) : "No error message returned from server.";
		Ext.Msg.show({
					title : 'HTTP:' + response.status + ' '
							+ response.statusText,
					msg : msg,
					buttons : Ext.Msg.OK,
					scope : this,
					icon : Ext.MessageBox.ERROR
				});
	},

	proxyException : function(dataProxy, response, operation, eopts) {

		Ext.Msg.show({
					title : 'HTTP:' + response.status + ' '
							+ response.statusText,
					msg : response.responseText.substr(0, 1500),
					buttons : Ext.Msg.OK,
					icon : Ext.MessageBox.ERROR
				});

	}

});
Ext.define("dnet.core.asgn.AbstractAsgnGrid", {
			extend : "Ext.grid.Panel",

			mixins : {
				elemBuilder : "dnet.core.base.AbstractDNetView"
			},

			// **************** Properties *****************

			
			/**
			 * Component builder
			 * 
			 * @type dnet.core.asgn.AsgnGridBuilder
			 */
			_builder_ : null,

			/**
			 * Assignment controller
			 * 
			 * @type dnet.core.asgn.AbstractAsgn
			 */
			_controller_ : null,

			/**
			 * Which grid is implemented. Possible values are left(for
			 * available) and right(for selected)
			 * 
			 * @type String
			 */
			_side_ : null,
			
			_columns_ : null,


			// **************** Public API *****************

			/**
			 * Returns the builder.
			 */
			_getBuilder_ : function() {
				if (this._builder_ == null) {
					this._builder_ = new dnet.core.asgn.AsgnGridBuilder({
								asgnGrid : this
							});
				}
				return this._builder_;
			},

			// **************** Defaults and overrides *****************

			forceFit : true,
			loadMask : true,
			stripeRows : true,
			border : true,
			frame : true,

			viewConfig : {
				emptyText : "No records found to match the selection criteria."
			},

			initComponent : function(config) {
				this._elems_ = new Ext.util.MixedCollection();
				this._columns_ = new Ext.util.MixedCollection();

				this._startDefine_();
				this._defineDefaultElements_();

				if (this._beforeDefineColumns_() !== false) {
					this._defineColumns_();
				}
				this._afterDefineColumns_();

				this._endDefine_();

				var cfg = {
					columns : this._columns_.getRange(),
					store : this._controller_.getStore(this._side_),
					selModel : {
						mode : "MULTI"
					},
					bbar : {
						xtype : "pagingtoolbar",
						store : this._controller_.getStore(this._side_),
						displayInfo : true
					}
				};
				Ext.apply(cfg, config);
				Ext.apply(this, cfg);
				this.callParent(arguments);
				this._registerListeners_();
			},

			/**
			 * Register event listeners
			 */
			_registerListeners_ : function() {
				this.mon(this.store, "load", function(store, recs, success, eOpts) {
					if (store.getCount() > 0) {
						this.getSelectionModel().select(0);
					}
				}, this );	
			},
			
			_defineColumns_ : function() {
			},

			_beforeDefineColumns_ : function() {
				return true;
			},

			_afterDefineColumns_ : function() {
			},

			_defineDefaultElements_ : function() {
			},

			_onStoreLoad_ : function(store, records, options) {
			},

			_afterEdit_ : function(e) {
			}

		});Ext.define("dnet.core.asgn.AbstractAsgnUi", {
			extend : "Ext.window.Window",

			mixins : {
				elemBuilder : "dnet.core.base.AbstractDNetView"
			},

			// **************** Properties *****************

			/**
			 * Assignment controller
			 * 
			 * @type dnet.core.asgn.AbstractAsgn
			 */
			_controller_ : null,

			/**
			 * Component builder
			 * 
			 * @type dnet.core.asgn.AsgnUiBuilder
			 */
			_builder_ : null,

			/**
			 * Left grid (available records) id
			 * 
			 * @type String
			 */
			_leftGridId_ : null,

			/**
			 * Right grid (selected records) id
			 * 
			 * @type String
			 */
			_rightGridId_ : null,

			/**
			 * Configuration object for the window
			 * 
			 * @type Object
			 */
			_windowConfig_ : null,

			_filterFields_ : null,
			_defaultFilterField_ : null,

			/**
			 * Flag to switch on/off auto-close mode of the assignment window
			 * upon succesful save.
			 * 
			 * @type Boolean
			 */
			_autoCloseAfterSave_ : true,

			/**
			 * Cancel button id.
			 * 
			 * @type String
			 */
			_btnCancelId_ : null,

			/**
			 * Save button id.
			 * 
			 * @type String
			 */
			_btnSaveId_ : null,

			/**
			 * Move left selection button id.
			 * 
			 * @type String
			 */
			_btnMoveLeftId_ : null,

			/**
			 * Move left all button id.
			 * 
			 * @type String
			 */
			_btnMoveLeftAllId_ : null,

			/**
			 * Move right selection button id.
			 * 
			 * @type String
			 */
			_btnMoveRightId_ : null,

			/**
			 * Move right all button id.
			 * 
			 * @type String
			 */
			_btnMoveRightAllId_ : null,

			// **************** Public API *****************

			/**
			 * Returns the builder.
			 * 
			 * @return {dnet.core.asgn.AsgnUiBuilder}
			 */
			_getBuilder_ : function() {
				if (this._builder_ == null) {
					this._builder_ = new dnet.core.asgn.AsgnUiBuilder({
								asgnUi : this
							});
				}
				return this._builder_;
			},

			_getToolbar_ : function(name) {
				return this._tlbs_.get(name);
			},

			_getToolbarConfig_ : function(name) {
				return this._tlbs_.get(name);
			},

			_getToolbarItem_ : function(tlbn, itmn) {
				return this._tlbs_.get(name);
			},

			_getToolbarItemConfig_ : function(tlbn, itmn) {
				var t = this._tlbs_.get(tlbn);
				var l = t.length;
				for (var i = 0; i < l; i++) {
					if (t[i]["name"] && t[i]["name"] == itmn)
						return t[i];
				}
			},

			/**
			 * Execute query for the left list which shows the available
			 * elements.
			 */
			_doQueryLeft_ : function() {
				this._doQueryLeftImpl_();
			},

			_doQueryRight_ : function() {
				this._doQueryRightImpl_();
			},

			/**
			 * Get left grid instance
			 * 
			 * @return {dnet.core.asgn.AbstractAsgnGrid}
			 */
			_getLeftGrid_ : function() {
				return Ext.getCmp(this._leftGridId_);
			},

			/**
			 * Get right grid instance
			 * 
			 * @return {dnet.core.asgn.AbstractAsgnGrid}
			 */
			_getRightGrid_ : function() {
				return Ext.getCmp(this._rightGridId_);
			},

			/**
			 * Get move left button
			 * 
			 * @return {Ext.button.Button}
			 */
			_getBtnMoveLeft_ : function() {
				return Ext.getCmp(this._btnMoveLeftId_);
			},

			/**
			 * Get move left all button
			 * 
			 * @return {Ext.button.Button}
			 */
			_getBtnMoveLeftAll_ : function() {
				return Ext.getCmp(this._btnMoveLeftAllId_);
			},

			/**
			 * Get move right button
			 * 
			 * @return {Ext.button.Button}
			 */
			_getBtnMoveRight_ : function() {
				return Ext.getCmp(this._btnMoveRightId_);
			},

			/**
			 * Get move right all button
			 * 
			 * @return {Ext.button.Button}
			 */
			_getBtnMoveRightAll_ : function() {
				return Ext.getCmp(this._btnMoveRightAllId_);
			},

			/**
			 * Get save button
			 * 
			 * @return {Ext.button.Button}
			 */
			_getBtnSave_ : function() {
				return Ext.getCmp(this._btnSaveId_);
			},

			/**
			 * Get cancel button
			 * 
			 * @return {Ext.button.Button}
			 */
			_getBtnCancel_ : function() {
				return Ext.getCmp(this._btnCancelId_);
			},

			/**
			 * @private Apply state rules to enable/disable or show/hide
			 *          components.
			 * 
			 */
			_applyStates_ : function() {
				if (this._beforeApplyStates_() !== false) {
					this._onApplyStates_();
				}
				this._afterApplyStates_();
			},

			/**
			 * @protected Template method checked before applying states.
			 * 
			 * @return {Boolean}
			 */
			_beforeApplyStates_ : function() {
				return true;
			},

			/**
			 * @protected Template method invoked after the state rules are
			 *            applied.
			 */
			_afterApplyStates_ : function() {
			},

			/**
			 * @protected Implement the state control logic in subclasses.
			 * 
			 */
			_onApplyStates_ : function() {
				if(this._getLeftGrid_().getSelectionModel().getSelection().length == 0) {
					this._getBtnMoveRight_().disable();
				} else {
					this._getBtnMoveRight_().enable();
				}
				if(this._getRightGrid_().getSelectionModel().getSelection().length == 0) {
					this._getBtnMoveLeft_().disable();
				} else {
					this._getBtnMoveLeft_().enable();
				}
			},

			// **************** Defaults and overrides *****************

			layout : "fit",
			closable : true,
			closeAction : "hide",
			modal : true,

			initComponent : function() {

				this._elems_ = new Ext.util.MixedCollection();
				this._tlbs_ = new Ext.util.MixedCollection();
				this._tlbitms_ = new Ext.util.MixedCollection();
				this._controller_ = Ext.create(this._controller_, {});

				this._leftGridId_ = Ext.id()
				this._rightGridId_ = Ext.id()

				this._startDefine_();
				this._defineDefaultElements_();

				if (this._beforeDefineElements_() !== false) {
					this._defineElements_();
				}
				this._afterDefineElements_();

				if (this._beforeLinkElements_() !== false) {
					this._linkElements_();
				}
				this._afterLinkElements_();

				this._endDefine_();

				Ext.apply(this, {
							items : [{
								xtype : "container",
								layout : {
									type : "hbox",
									align : "stretch"
								},
								items : [{
									frame : true,
									flex : 10,
									xtype : "container",
									layout : {
										type : "vbox",
										align : "stretch"
									},
									items : [this._elems_.get("leftFilter"),
											this._elems_.get("leftList")]
								}, {
									width : 80,
									frame : true,
									xtype : "container",
									layout : {
										type : "vbox",
										align : "stretch",
										pack : "center"
									},
									items : this._buildToolbarItems_()
								}, {
									frame : true,
									flex : 10,
									xtype : "container",
									layout : {
										type : "vbox",
										align : "stretch"
									},
									items : [this._elems_.get("rightFilter"),
											this._elems_.get("rightList")]
								}]
							}]
						});
				this.callParent(arguments);
				this._registerListeners_();
			},

			/**
			 * Register event listeners
			 */
			_registerListeners_ : function() {
				if (this._autoCloseAfterSave_ == true) {
					this._controller_.on("afterDoSaveSuccess", function() {
								this.close();
							}, this);
				}
				var _lg = this._getLeftGrid_();
				if (_lg) {
					this.mon(_lg, "itemdblclick", this._onLeftGrid_dblclick_,
							this);
					this.mon(_lg.getSelectionModel(), "selectionchange",
							this._applyStates_, this);
				}
				var _rg = this._getRightGrid_();
				if (_rg) {
					this.mon(_rg, "itemdblclick", this._onRightGrid_dblclick_,
							this);
					this.mon(_rg.getSelectionModel(), "selectionchange",
							this._applyStates_, this);
				}
				this.mon(this._controller_.storeLeft, "load", function(store, recs, success, eOpts) {
					if (store.totalCount == 0) {
						this._getBtnMoveRightAll_().disable();
					} else {
						this._getBtnMoveRightAll_().enable();
					}
				}, this);
				this.mon(this._controller_.storeRight, "load", function(store, recs, success, eOpts) {
					if (store.totalCount == 0) {
						this._getBtnMoveLeftAll_().disable();
					} else {
						this._getBtnMoveLeftAll_().enable();
					}
				}, this);
			},

			// **************** Private API *****************

			/**
			 * When double-click a row automatically move it.
			 */
			_onLeftGrid_dblclick_ : function() {
				var btn = this._getBtnMoveRight_();
				if (btn && !btn.disabled) {
					btn.handler.call(btn.scope);
				}
			},

			/**
			 * When double-click a row automatically move it.
			 */
			_onRightGrid_dblclick_ : function() {
				var btn = this._getBtnMoveLeft_();
				if (btn && !btn.disabled) {
					btn.handler.call(btn.scope);
				}
			},

			/**
			 * Execute query for the left list which shows the available
			 * elements. Private implementation code.
			 */
			_doQueryLeftImpl_ : function() {
				var f = this._controller_.filter.left;
				f.field = this._getElement_("leftFilterCombo").getValue();
				f.value = this._getElement_("leftFilterField").getValue();
				if (Ext.isEmpty(f.field) && !Ext.isEmpty(f.value)) {
					Ext.Msg.show({
								icon : Ext.MessageBox.ERROR,
								msg : "Select the field to filter.",
								buttons : Ext.Msg.OK
							});
					return;
				}
				this._controller_.doQueryLeft();
			},

			/**
			 * Execute query for the right list which shows the selected
			 * elements. Private implementation code.
			 */
			_doQueryRightImpl_ : function() {
				var f = this._controller_.filter.right;
				f.field = this._getElement_("rightFilterCombo").getValue();
				f.value = this._getElement_("rightFilterField").getValue();
				if (Ext.isEmpty(f.field) && !Ext.isEmpty(f.value)) {
					Ext.Msg.show({
								icon : Ext.MessageBox.ERROR,
								msg : "Select the field to filter ",
								buttons : Ext.Msg.OK
							});
					return;
				}
				this._controller_.doQueryRight();
			},

			/**
			 * Define the default filter elements for the assignment window.
			 */
			_defineDefaultElements_ : function() {

				this._elems_.add("leftFilterField", {
							xtype : "textfield",
							width : 80,
							emptyText : "Filter...",
							id : Ext.id()
						});

				this._elems_.add("leftFilterBtn", {
							xtype : "button",
							text : "Ok",
							scope : this,
							handler : function() {
								this._doQueryLeft_();
							}
						});

				this._elems_.add("rightFilterField", {
							xtype : "textfield",
							width : 80,
							emptyText : "Filter...",
							id : Ext.id()
						});

				this._elems_.add("rightFilterBtn", {
							xtype : "button",
							text : "Ok",
							scope : this,
							handler : function() {
								this._doQueryRight_();
							}
						});

				this._elems_.add("leftFilterCombo", {
							xtype : "combo",
							value : "",
							width : 100,
							selectOnFocus : true,
							forceSelection : true,
							triggerAction : "all",
							id : Ext.id(),
							store : this._filterFields_,
							value : this._defaultFilterField_
						});

				this._controller_.filter.left.field = "code";

				this._elems_.add("rightFilterCombo", {
							xtype : "combo",
							value : "",
							width : 100,
							selectOnFocus : true,
							forceSelection : true,
							triggerAction : "all",
							id : Ext.id(),
							store : this._filterFields_,
							value : this._defaultFilterField_
						});

				this._elems_.add("leftFilter", {
							fieldLabel : "Filter",
							xtype : "fieldcontainer",
							layout : 'hbox',
							preventMark : true,
							labelAlign : "right",
							labelWidth : 70,
							items : [this._elems_.get("leftFilterField"),
									this._elems_.get("leftFilterCombo"),
									this._elems_.get("leftFilterBtn")]
						});

				this._elems_.add("rightFilter", {
							fieldLabel : "Filter",
							xtype : "fieldcontainer",
							layout : 'hbox',
							preventMark : true,
							labelAlign : "right",
							labelWidth : 70,
							items : [this._elems_.get("rightFilterField"),
									this._elems_.get("rightFilterCombo"),
									this._elems_.get("rightFilterBtn")]
						});

			},

			/**
			 * Define the buttons for the assignment windows. Selection buttons,
			 * save and initialize.
			 */
			_buildToolbarItems_ : function() {
				this._btnCancelId_ = Ext.id();
				this._btnSaveId_ = Ext.id();
				this._btnMoveLeftId_ = Ext.id();
				this._btnMoveLeftAllId_ = Ext.id();
				this._btnMoveRightId_ = Ext.id();
				this._btnMoveRightAllId_ = Ext.id();
				return [{
							xtype : "button",
							text : 'Cancel',
							tooltip : "Cancel changes and reload initial selection ",
							id : this._btnCancelId_,
							scope : this,
							handler : function() {
								this._controller_.doReset();
							}
						}, {
							xtype : "tbspacer",
							height : 25
						}, {
							xtype : "button",
							// iconCls : 'icon-action-assign_moveright',
							text : ">",
							tooltip : "Add selected",
							id : this._btnMoveRightId_,
							scope : this,
							handler : function() {
								this._controller_.doMoveRight(Ext
												.getCmp(this._leftGridId_), Ext
												.getCmp(this._rightGridId_));
							}
						}, {
							xtype : "tbspacer",
							height : 5
						}, {
							xtype : "button",
							// iconCls : 'icon-action-assign_moveleft',
							text : "<",
							tooltip : "Remove selected",
							id : this._btnMoveLeftId_,
							scope : this,
							handler : function() {
								this._controller_.doMoveLeft(Ext
												.getCmp(this._leftGridId_), Ext
												.getCmp(this._rightGridId_));
							}
						}, {
							xtype : "tbspacer",
							height : 25
						}, {
							xtype : "button",
							// iconCls : 'icon-action-assign_moverightall',
							text : ">>",
							tooltip : "Add all",
							id : this._btnMoveRightAllId_,
							scope : this,
							handler : function() {
								this._controller_.doMoveRightAll();
							}
						}, {
							xtype : "tbspacer",
							height : 5
						}, {
							xtype : "button",
							// iconCls : 'icon-action-assign_moveleftall',
							text : "<<",
							tooltip : "Remove all",
							id : this._btnMoveLeftAllId_,
							scope : this,
							handler : function() {
								this._controller_.doMoveLeftAll();
							}
						}, {
							xtype : "tbspacer",
							height : 25
						}, {
							xtype : "button",
							text : 'Save',
							tooltip : "Save changes",
							id : this._btnSaveId_,
							scope : this,
							handler : function() {
								this._controller_.doSave();
							}
						}];

			}
		});
Ext.ns("dnet.core.asgn");

dnet.core.asgn.AsgnGridBuilder = function(config) {		
	this.asgnGrid = null; 
	Ext.apply(this,config);	 	 
	dnet.core.asgn.AsgnGridBuilder.superclass.constructor.call(this, config);
};

Ext.extend(dnet.core.asgn.AsgnGridBuilder, Object, {
	
	addTextColumn: function(config) {
		config.xtype="gridcolumn";		
		this.applySharedConfig(config);		
		return this;
	}	
	,addBooleanColumn: function(config) {
		config.xtype="booleancolumn";
		Ext.apply(config,{trueText:Dnet.translate("msg", "bool_true"), falseText:Dnet.translate("msg", "bool_false")});		
		this.applySharedConfig(config);
		return this;
	}
	,addDateColumn: function(config) {
		config.xtype="datecolumn";
		Ext.applyIf(config,{format:Ext.DATE_FORMAT});
		this.applySharedConfig(config);
		return this;
	}
	,addNumberColumn: function(config) {
		config.xtype="numbercolumn";
		Ext.applyIf(config,{align:"right"});
		this.applySharedConfig(config);
		return this;
	}
	,add: function(config) {
		this.applySharedConfig(config);
		return this;
	}
	,merge: function(name, config) {
		Ext.applyIf(this.asgnGrid._columns_.get(name) , config );
		return this;
	}
	,change: function(name, config) {
		Ext.apply(this.asgnGrid._columns_.get(name) , config );
		return this;
	}
	,remove: function(name) {
		this.asgnGrid._columns_.remove(name);
		return this;
	}
	//private
	,applySharedConfig: function(config) {
		Ext.applyIf(config,{
			id:Ext.id(),sortable:true, hidden:false
		});
		if(config._sharedLabel_) {
			config._rbkey_ = config.name;
		}
		this.asgnGrid._columns_.add(config.name, config);
	}
	
	,addAllFromDataSource: function() {
		
		var f = this.asgnGrid._controller_.ds.recordFields;
		for(var i=0,len=f.length;i<len; i++) {
			var name = f[i]["name"];
			var type = f[i]["type"];
 			var cfg = {name:name,dataIndex:name};
 			
 			// try to guess something 
			if ( name == "id" || name == "createdAt" || name == "createdBy" || name == "version"|| name == "clientId"  ) {
				cfg.hidden = true;
			}
			
			if (type == "string") {
				this.addTextColumn(cfg);
			}
			if (type == "date") {
				this.addDateColumn(cfg);
			}
			if (type == "boolean") {
				this.addBooleanColumn(cfg);
			}
			if (type == "int") {
				this.addNumberColumn(cfg);
			}
			
		}
	}
});

Ext.ns("dnet.core.asgn");

dnet.core.asgn.AsgnUiBuilder = function(config) {		
	this.asgnUi = null; 
	Ext.apply(this,config);	 
	dnet.core.asgn.AsgnUiBuilder.superclass.constructor.call(this, config);
};

Ext.extend(dnet.core.asgn.AsgnUiBuilder, Object, {
	
	addLeftGrid: function(config) {	
		Ext.apply(config||{}, {name:"leftList",id:this.asgnUi._leftGridId_, _side_:"left", flex:100, _controller_: this.asgnUi._controller_});
		this.applySharedConfig(config);		
		return this;
	}	
	,addRightGrid: function(config) {	
		Ext.apply(config||{}, {name:"rightList",id:this.asgnUi._rightGridId_, _side_:"right", flex:100,_controller_: this.asgnUi._controller_});
		this.applySharedConfig(config);		
		return this;
	}	 
	 
	,add: function(config) {
		this.applySharedConfig(config);
		return this;
	}
	,merge: function(name, config) {
		Ext.applyIf(this.asgnUi._elems_.get(name) , config );
		return this;
	}
	,change: function(name, config) {
		Ext.apply(this.asgnUi._elems_.get(name) , config );
		return this;
	}
	,remove: function(name) {
		this.asgnUi._elems_.remove(name);
		return this;
	}
	//private
	,applySharedConfig: function(config) {
		Ext.applyIf(config,{
			id:Ext.id() 
		});		 
		this.asgnUi._elems_.add(config.name, config);
	}
	
	 
});
Ext.define("dnet.core.dc.AbstractDc", {

	mixins : {
		observable : 'Ext.util.Observable'
	},

	dsName : "",

	/**
	 * List with action names implemented
	 */
	actionNames : null,

	/**
	 * Action instances. Used to create control widgets in UI to be triggered by
	 * user, mainly toolbar items.
	 */
	actions : null,

	/**
	 * Executable functions. Implements workers for API methods of a
	 * data-control.
	 */
	commands : null,

	/**
	 * Allow to edit more than one record at once.
	 */
	multiEdit : false,

	/**
	 * Filter model instance. Has the same signature as the data model instance
	 * 
	 * @type Ext.data.Model
	 */
	filter : null,

	/**
	 * Data model instance.
	 * 
	 * @type Ext.data.Model
	 */
	record : null,

	/**
	 * Parameters model instance
	 */
	params : null,

	/**
	 * Data model signature - record constructor.
	 * 
	 * @type Ext.data.Model
	 */
	recordModel : null,

	/**
	 * Filter model signature - filter constructor. Defaults to recordModel if
	 * not specified.
	 * 
	 * @type Ext.data.Model
	 */
	filterModel : null,

	/**
	 * Parameters model signature - record constructor.
	 */
	paramModel : null,

	/**
	 * Keep track of the selected records
	 */
	selectedRecords : null,

	/**
	 * Various runtime configuration properties.
	 */
	tuning : {

		/**
		 * Number of milliseconds before execute the query. Used if value>0
		 */
		queryDelay : 150

		/**
		 * Page-size for a query
		 */
		,
		fetchSize : 30
	},

	/**
	 * Children data-controls, similar to a `HasMany` association.
	 */
	children : null,

	/**
	 * Parent data-control, similar to a `BelongsTo` association.
	 * 
	 * @type dnet.core.dc.AbstractDc
	 */
	parent : null,

	/**
	 * Array with form-views registered to data-binding.
	 */
	bindedFormViews : null,

	/**
	 * Array with filter-views registered to data-binding.
	 */
	bindedFilterViews : null,

	/**
	 * Should apply a default selection on store load ?
	 */
	afterStoreLoadDoDefaultSelection : true,

	/**
	 * Local reference to the data-source store.
	 * 
	 * @type Ext.data.Store
	 */
	store : null,

	/**
	 * 
	 * @type dnet.core.dc.DcContext
	 */
	dcContext : null,

	readOnly : false,
	
	_trl_ : null,

	lastStateManagerFlags : null,

	constructor : function(config) {
		config = config || {};
		Ext.apply(this, config);
		this.children = [];
		this.selectedRecords = [];
		this.dsName = this.recordModel.substring(this.recordModel
						.lastIndexOf('.')
						+ 1, this.recordModel.length);
		this.store = Ext.create("Ext.data.Store", {
					model : this.recordModel,
					remoteSort : true,

					autoLoad : false,
					autoSync : false,
					clearOnPageLoad : true,
					pageSize : this.tuning.fetchSize,
					proxy : {
						type : 'ajax',
						api : Dnet.dsAPI(this.dsName, "json"),
						model : this.recordModel,
						extraParams : {
							params : {}
						},
						actionMethods : {
							create : 'POST',
							read : 'POST',
							update : 'POST',
							destroy : 'POST'
						},
						reader : {
							type : 'json',
							root : 'data',
							idProperty : 'id',
							totalProperty : 'totalCount',
							messageProperty : 'message'
						},
						writer : {
							type : 'json',
							encode : true,
							root : "data",
							allowSingle : false,
							writeAllFields : true
						},
						listeners : {
							"exception" : {
								fn : this.proxyException,
								scope : this
							}
						},
						startParam : Dnet.requestParam.START,
						limitParam : Dnet.requestParam.SIZE,
						sortParam : Dnet.requestParam.ORDERBY
						// directionParam : Dnet.requestParam.SENSE

					}

				});

		this.addEvents(
				/**
				 * Record instance changed
				 */
				"recordChange",

				/**
				 * Data-control status status changed: record state change:
				 * clean / dirty record status change: new record / persisted
				 * 
				 * "statusChange",
				 */

				/**
				 * Selected records changed
				 */
				"selectionChange");

		this.mixins.observable.constructor.call(this);
		this._setup_();
	},

	_setup_ : function() {

		try {
			this._trl_ = Ext.create(this.recordModel + "$Trl");
		} catch (e) {
			// Ext.Msg.show({
			// title : "Invalid language-pack",
			// msg:"No translation file found for " +this.recordModel +"$Trl
			// <br> Using the default system language for the associated
			// labels.",
			// icon : Ext.MessageBox.INFO,
			// buttons : Ext.Msg.OK
			// });
		}

		if (Ext.isEmpty(this.filterModel)) {
			this.filterModel = this.recordModel;
		}
		this.setFilter(this.newFilterInstance()); 
		this.setParams(Ext.create(this.paramModel, {}));

		this.actionNames = dnet.core.dc.DcActionsFactory.actionNames();
		this.commandNames = dnet.core.dc.DcCommandFactory.commandNames();

		this.actions = dnet.core.dc.DcActionsFactory.createActions(this,
				this.actionNames);
		this.commands = dnet.core.dc.DcCommandFactory.createCommands(this,
				this.commandNames.concat(this.actionNames));

		// register listeners

		this.mon(this.store, "beforeload", this.onStore_beforeload, this);
		this.mon(this.store, "update", this.onStore_update, this);
		this.mon(this.store, "datachanged", this.onStore_datachanged, this);
		this.mon(this.store, "changes_rejected", this.onStore_datachanged, this);
		
		// after the store is loaded apply an initial selection
		if (this.afterStoreLoadDoDefaultSelection) {
			this.mon(this.store, "load", this.onStore_load , this);
		}

		// invoke the action state update whenever necessary
		this.mon(this, "recordChange", this.updateActionsState, this);
		this.mon(this, "selectionChange", this.updateActionsState, this);

		// ************************************************
		// to be reviewd
		// ************************************************

		this.mon(this.store, "write", this.onStore_write, this);
	},

	onStore_load : function(store, records, succes, eopts) {		 
		if (this.afterStoreLoadDoDefaultSelection) {
			this.doDefaultSelection();
		}
	},

	onStore_beforeload : function(store, operation, eopts) {
		if (dnet.core.dc.DcActionsStateManager.isQueryDisabled(this)) {
			return false;
		}
	},
	
	onStore_update : function(store, rec, operation, eopts) {
		this.fireEvent("statusChange", {
					dc : this
				});
		this.updateActionsState(true);
	},
 
	onStore_datachanged : function(store, eopts) {
		this.updateActionsState();
	},
	
	onStore_remove : function(store, records, index, eopts) {
		this.updateActionsState();
		this.doDefaultSelection();
	},

	onStore_write : function(store, operation, eopts) {
 
		this.updateActionsState();
		if (operation.action == "update" || operation.action == "create") {
			this.afterDoSaveSuccess();
		}
	},

	/** ***************************************************** */
	/** ***************** Public API ************************ */
	/** ***************************************************** */

	/**
	 * Execute query to fetch data.
	 */

	doQuery : function() {
		this.commands.doQuery.execute();
	},

	/**
	 * Clear query criteria -> reset filter to its initial state
	 */
	doClearQuery : function() {
		this.commands.doClearQuery.execute();
	},

	/**
	 * Create a new record.
	 */
	doNew : function() {
		this.commands.doNew.execute();
	},

	/**
	 * Copy the current record reset its ID and make it current record ready to
	 * be edited.
	 */
	doCopy : function() {
		this.commands.doCopy.execute();
	},

	/**
	 * Save changes.
	 */
	doSave : function() {
		this.commands.doSave.execute();
	},

	/**
	 * Discard changes to the last clean state.
	 */
	doCancel : function() {
		this.commands.doCancel.execute();
	},

	/**
	 * Delete current record.
	 */
	doDelete : function() {
		this.commands.doDelete.execute();
	},

	/**
	 * Delete selected records.
	 */
	doDeleteSelection : function() {
		this.commands.doDelete.execute();
	},

	/**
	 * Reload the current record data from server
	 */
	doReloadRecord : function() {
		this.commands.doReloadRec.execute();
	},

	/**
	 * Deprecated: Alias for doReloadRecord Shall be removed
	 */
	doRefreshCurrent : function() {
		this.doReloadRecord();
	},

	/**
	 * Call a server side RPC with the filter instance
	 */
	doRpcFilter : function(options) {
		this.commands.doRpcFilter.execute(options);
	},
	/**
	 * Deprecated: Alias for doRpcFilter Shall be removed
	 */
	doServiceFilter : function(serviceName, specs) {
		this.doRpcFilter(Ext.apply(specs || {}, {
					name : serviceName
				}));
	},

	/**
	 * Call a server side RPC with the model instance
	 */
	doRpcData : function(options) {
		this.commands.doRpcData.execute(options);
	},
	/**
	 * Deprecated: Alias for doRpcData Shall be removed
	 */
	doService : function(serviceName, specs) {
		this.doRpcData(Ext.apply(specs || {}, {
					name : serviceName
				}));
	},

	/**
	 * Set the previous available record as current record.
	 */
	setPreviousAsCurrent : function() {
		this.commands.doPrevRec.execute();
	},

	/**
	 * Set the next available record as current record.
	 */
	setNextAsCurrent : function() {
		this.commands.doNextRec.execute();
	},

	// --------------- other ---------------

	/**
	 * Filter validator. It should get interceptor functions chain injected by
	 * the filter forms.
	 */
	isFilterValid : function() {
		return true; // this.filter.isValid();
	},

	/**
	 * Record validator. It should get interceptor functions chain injected by
	 * the editor forms.
	 */
	isRecordValid : function() {
		return this.record.isValid();
	},

	/**
	 * Get current record state: dirty/clean
	 */
	getRecordState : function() {
		if (this.record)
			return (this.isCurrentRecordDirty()) ? 'dirty' : 'clean';
		else
			return null;
	},

	/**
	 * Get current record status: insert/update
	 */
	getRecordStatus : function() {
		if (this.record)
			return (this.record.phantom) ? 'insert' : 'update';
		else
			return null;
	},

	/**
	 * Update the enabled/disabled states of the actions. Delegate the work to
	 * the states manager.
	 */
	updateActionsState : function(ifNeeded) {
		dnet.core.dc.DcActionsStateManager.applyStates(this, ifNeeded);
		this.fireEvent("updateActionsState", {
					dc : this
				});
	},

	/**
	 * Returns true if any of the child data-controls is dirty
	 */
	isAnyChildDirty : function() {
		var dirty = false, l = this.children.length;
		for (var i = 0; i < l; i++) {
			if (this.children[i].isDirty()) {
				dirty = true;
				i = l;
			}
		}
		return dirty;
	},

	/**
	 * Returns true if the current record instance is dirty
	 */
	isCurrentRecordDirty : function() {
		if (this.record != null && (this.record.dirty || this.record.phantom)) {
			return true;
		}
		return false;
	},

	/**
	 * Returns true if the store is dirty. Is relevant only if
	 * <code>multiEdit=true</code>
	 */
	isStoreDirty : function() {
		return this.store.getRemovedRecords().length > 0
				|| this.store.getUpdatedRecords().length > 0
				|| this.store.getAllNewRecords().length > 0;
	},

	/**
	 * Returns true if the data-control is dirty i.e either some the own records
	 * or any child
	 */
	isDirty : function() {
		return this.isCurrentRecordDirty() || this.isStoreDirty()
				|| this.isAnyChildDirty();
	},

	/**
	 * Default initial selection
	 */
	doDefaultSelection : function() {
		if (this.store.getCount() > 0) {			
			this.setRecord(this.store.getAt(0), true);
		} else {
			this.setRecord(null);
		}
	},

	// --------------------- getters / setters ----------------------

	/**
	 * Returns the filter instance
	 */
	getFilter : function() {
		return this.filter;
	},

	/**
	 * Set the filter instance
	 */
	setFilter : function(v) {
		var of = this.filter;
		this.filter = v;
		this.fireEvent("filterChanged", {
					dc : this,
					newFilter : this.filter,
					oldFilter : of
				});
	},

	/**
	 * Get filter property value
	 */
	getFilterValue : function(n) {
		return this.filter.get(n);
	},

	/**
	 * Set filter property value. As currently the filter model is not part of a
	 * store, changes to properties must be fired so that filter views bound to
	 * the filter model can be notified to refresh their fields values.
	 * 
	 * In some future release it is likely to be enhanced so that filters can be
	 * saved and reused.
	 * 
	 * @param {String}
	 *            property filter-model property to change
	 * @param {Object}
	 *            newValue The new value
	 * @param {boolean}
	 *            silent Do not fire the event.
	 */
	setFilterValue : function(property, newValue, silent) {
		var oldValue = this.filter.get(property);
		if (oldValue != newValue) {
			this.filter.set(property, newValue);
			if (!(silent === true)) {
				this.fireEvent("filterValueChanged", this, property, oldValue,
						newValue);
			}
		}
	},

	/**
	 * Get the parameters instance
	 */
	getParams : function() {
		return this.params;
	},

	/**
	 * Set the parametr instance
	 */
	setParams : function(v) {
		this.params = v;
	},

	/**
	 * Get parameter property value
	 */
	getParamValue : function(n) {
		return this.params.get(n);
	},

	/**
	 * Set parameter property value
	 */
	setParamValue : function(property, newValue, silent) {
		var ov = this.params.get(property);
		if (ov != newValue) {
			this.params.set(property, newValue);
			if ( silent !== true ) {
				this.fireEvent("parameterValueChanged", this, property, ov, newValue);
			}
		}
	},

	/**
	 * Returns the current record
	 * @return {Ext.data.Model} Current record or null 
	 */
	getRecord : function() {
		return this.record;
	},

	/**
	 * Template method to override with instance specific logic in case is
	 * necessary
	 */
	beforeSetRecord : function() {
		return true;
	},

	/**
	 * Set the given record as the current working record.
	 * If the selectIt parameter is true make it the selection.
	 * @param {Ext.data.Model} p Record to be set as current record
	 * @param {Boolean} selectIt Update the selected records
	 */
	setRecord : function(p, selectIt, eOpts) {
		
		p = (p != undefined) ? p : null;
		
		if (!this.multiEdit) {
			if (this.isDirty()) {
				throw (dnet.core.dc.DcExceptions.DIRTY_DATA_FOUND);
			}
		}
		
		if (this.beforeSetRecord() === false) {
			return false;
		}
		var rec, idx, changed = false, oldrec;
		if (p != null) {
			if (Ext.isNumber(p)) {
				idx = p;
				rec = this.store.getAt(p);
				if (rec && (this.record != rec)) {
					oldrec = this.record;
					this.record = rec;
					changed = true;
				}
			} else {

				rec = p;
				idx = this.store.indexOf(p);
				if (rec && (this.record != rec)) {
					oldrec = this.record;
					this.record = rec;
					changed = true;
				}
			}
		} else {
			rec = p;
			oldrec = this.record;
			this.record = rec;
			changed = (oldrec != null);

		}
		if (changed) {			
			this.fireEvent('recordChange', {
						dc : this,
						newRecord : rec,
						oldRecord : oldrec,
						newIdx : idx,
						status : this.getRecordStatus(),
						eOpts: eOpts
					});
			if (selectIt===true) {
				if (rec != null ) {
					this.setSelectedRecords([rec],eOpts);
				}else {
					this.setSelectedRecords([],eOpts);
				}
			}		
		}
	},

	/**
	 * Returns the selected records
	 */
	getSelectedRecords : function() {
		return this.selectedRecords;
	},

	/**
	 * Set the selected records
	 */
	setSelectedRecords : function(recArray,eOpts) {
		recArray = recArray || [];
		if (!Ext.isArray(recArray)) {
			recArray = [recArray];
		}

		if (this.selectedRecords !== recArray) {
			this.selectedRecords = recArray;
			this.fireEvent('selectionChange', {
						dc : this,
						eOpts: eOpts
					});
		}
	},

	/**
	 * Return parent data-control
	 */
	getParent : function() {
		return (this.dcContext) ? this.dcContext.parentDc : null;
	},

	/**
	 * Return children data-controls list
	 */
	getChildren : function() {
		return this.children;
	},

	
	/**
	 * Register a child data-control
	 */
	addChild : function(dc) {
		this.children[this.children.length] = dc;
	},

	isReadOnly: function() {
		return this.readOnly;
	},
	 
	setReadOnly: function(v, silent) {
		this.readOnly = v;
		if(silent !== true) {
			dnet.core.dc.DcActionsStateManager.applyStates(this);
			this.fireEvent("readOnlyChanged", this, v);
		}
	},
 

 	/**
	 * Creates a new record instance and initialize it
	 */
	newRecordInstance : function() {
		var o = Ext.create(this.recordModel, this.initNewRecordInstance({}));
		o.data["clientId"] = getApplication().getSession().client.id;
		return o;
	},

	initNewRecordInstance : function(o) {
		return o;
	},
	
	/**
	 * Creates a new filter instance and initialize it
	 */
	newFilterInstance : function() {
		var o = Ext.create(this.filterModel, {});
		o.data["clientId"] = getApplication().getSession().client.id;
		return o;
	},
	 
 
	/** ************************************************************************* */
	/** *********************** Internal API ****************************** */
	/** ************************************************************************* */
 

	/**
	 * Default proxy-exception handler
	 */
	proxyException : function(proxy, response, operation, eOpts) {
		//this.showAjaxErrors(response, eOpts);
		if(operation && operation.action == "destroy" && !this.multiEdit) {
			this.store.rejectChanges();
			//this.updateActionsState();
		}
		
	},

	/**
	 * Show errors to user. TODO: Externalize it as command.
	 */
	showAjaxErrors : function(response, options) {
		// Ext.MessageBox.hide();
		var msg, withDetails = false;
		if (response.responseText) {
			if (response.responseText.length > 2000) {
				msg = response.responseText.substr(0, 2000);
				withDetails = true;
			} else {
				msg = response.responseText;
			}
		} else {
			msg = "No response received from server.";
		}
		var alertCfg = {
			title : "Server message",
			msg : msg,
			scope : this,
			icon : Ext.MessageBox.ERROR,
			buttons : Ext.MessageBox.OK
		}
		if (withDetails) {
			alertCfg.buttons['cancel'] = 'Details';
			alertCfg['detailedMessage'] = response.responseText;
		}
		Ext.Msg.show(alertCfg);

	},

 
	
	
	// ************************************************
	// to be reviewd
	// ************************************************

	afterDoQuerySuccess : function() {
		this.fireEvent("afterDoQuerySuccess", this);
	},
	afterDoQueryFailure : function() {
		this.fireEvent("afterDoQueryFailure", this);
	}

	,
	beforeDoSave : function() {
		this.fireEvent("beforeDoSave", this);
	},
	afterDoSave : function() {
		this.fireEvent("afterDoSave", this);
	},
	afterDoSaveSuccess : function() {
		this.fireEvent("afterDoSaveSuccess", this);
	},
	afterDoSaveFailure : function() {
		this.fireEvent("afterDoSaveFailure", this);
	}

	,
	doEdit : function() {
		this.onEdit();
	},
	onEdit : function() {

		this.fireEvent("onEdit", this);
	}

	/**
	 * ************ SERVICE DATA *********************
	 */

	,
	doServiceUrl : function(serviceName, specs) {
		if (Ext.isEmpty(this.record)) {
			throw (dnet.core.dc.DcExceptions.NO_CURRENT_RECORD);
		}
		var s = specs || {};

		var p = {
			data : Ext.encode(this.record.data)
		};
		p[Dnet.requestParam.SERVICE_NAME_PARAM] = serviceName;
		p["rpcType"] = "data";
		if (s.modal) {
			Ext.Msg.wait('Working...');
		}
		return Dnet.dsAPI(this.dsName, "stream").service + "&"
				+ Ext.urlEncode(p);// + method:"POST", params: p
	}

	/**
	 * Default AJAX request failure handler.
	 */
	,
	onAjaxRequestSuccess : function(response, options) {
		Ext.MessageBox.hide();
		var o = options.options || {};
		if (o.action) {
			if (o.action == "doQuery") {
				this.afterDoQuerySuccess();
			}
			if (o.action == "doSave") {
				this.afterDoSaveSuccess();
			}
			if (o.action == "doService") {
				this.afterDoServiceSuccess(response, o.serviceName, o.specs);
			}
			if (o.action == "doServiceFilter") {
				this.afterDoServiceFilterSuccess(response, o.serviceName,
						o.specs);
			}
		}
	}

	/**
	 * Default AJAX request failure handler.
	 */
	,
	onAjaxRequestFailure : function(response, options) {

		var o = options.options || {};
		if (o.action) {
			if (o.action == "doQuery") {
				this.afterDoQueryFailure();
			}
			if (o.action == "doSave") {
				this.afterDoSaveFailure();
			}
			if (o.action == "doService") {
				this.afterDoServiceFailure(response, o.serviceName, o.specs);
			}
			if (o.action == "doServiceFilter") {
				this.afterDoServiceFilterFailure(response, o.serviceName,
						o.specs);
			}
		}
	}

	,
	setDcContext : function(dcCtx) {
		this.dcContext = dcCtx;
		this.mon(this.dcContext, "dataContextChanged",
				this.onDcContext_dataContextChanged, this);
	},

	onDcContext_dataContextChanged : function(dctx) {
		dnet.core.dc.DcActionsStateManager.applyStates(this);
		if (dctx.parentDc.getRecord() && dctx.parentDc.getRecord().phantom) {
			this.fireEvent("inContextOfNewRecord", this);
		} else {
			this.fireEvent("inContextOfEditRecord", this);
		}
	},

	isRecordChangeAllowed : function() {
		return (!(this.isAnyChildDirty() || ((!this.multiEdit) && this
				.isCurrentRecordDirty())));
	}

	,
	onCleanDc : function() {
		this.fireEvent('cleanDc', {
					dc : this
				});
		if (this.dcContext != null) {
			this.dcContext._onChildCleaned_();
		}
	},
	onChildCleaned : function() {
		if (!this.isStoreDirty() && !this.isAnyChildDirty()) {
			this.onCleanDc();
		}
	},

	destroy : function() {
		// console.log("AbstractDc.destroy");
		for (var p in this.actions) {
			delete this.actions[p].dc;
		}
		for (var p in this.commands) {
			delete this.commands[p].dc;
		} 
		
		this.store.clearListeners();
		this.store.destroyStore();
		//try {
			if (this.dcContext) {
				this.dcContext.destroy();
			}
		//} catch (e) {
		//}
		delete this.children;
		delete this.parent;
		delete this.dcContext;
		//this.callOverridden(arguments);
		// delete this.actions;
		// delete this.commands;
		// delete this.store;

		// Ext.destroy( this.dcContext);
	}
});
/**
 * Mixin which provides DC-view support.
 */
Ext.define("dnet.core.dc.AbstractDNetDcView", {

			// **************** Properties *****************

			/**
			 * 
			 * @type dnet.core.dc.AbstractDc
			 */
			_controller_ : null,

			// **************** Public API *****************

			/**
			 * @public Returns the controller of this view
			 * 
			 * @return {@type dnet.core.dc.AbstractDc}
			 */
			_getController_ : function() {
				return this._controller_;
			},

			/**
			 * @private Apply state rules to enable/disable or show/hide
			 *          components.
			 * 
			 * @param model
			 *            Is either the current record or current filter bound
			 *            to this form-view.
			 * 
			 */
			_applyStates_ : function(model) {
				if (this._beforeApplyStates_(model) !== false) {
					this._onApplyStates_(model);
				}
				this._afterApplyStates_(model);
			},

			/**
			 * @protected Template method checked before applying states.
			 * 
			 * @return {Boolean}
			 */
			_beforeApplyStates_ : function(model) {
				return true;
			},

			/**
			 * @protected Template method invoked after the state rules are
			 *            applied.
			 */
			_afterApplyStates_ : function(model) {
			},

			/**
			 * @protected Implement the state control logic in subclasses.
			 * 
			 */
			_onApplyStates_ : function(model) {

			},

			/**
			 * @protected Template method called after a new model is bound to
			 *            the form. Add custom logic in subclasses if necessary.
			 * @param {Ext.data.Model}
			 *            model Current record or current filter depending on
			 *            the DC view type
			 */
			_afterBind_ : function(model) {
			},

			/**
			 * @protected Template method called after a model is un-bound from
			 *            the form. Add custom logic in subclasses if necessary.
			 * @param {Ext.data.Model}
			 *            model Current record or current filter depending on
			 *            the DC view type
			 */
			_afterUnbind_ : function(model) {
			},

			// **************** Private methods *****************

			/**
			 * The filter model is not part of a store, so we have listen to
			 * changes made to the model through the `filterValueChanged` event
			 * raised by the data-control. So changes to the filter model should
			 * be done through the setFilterValue method of the data-control in
			 * order to be listened and picked-up to refresh the correcponding
			 * filter-form fields.
			 * 
			 * @param {dnet.core.dc.AbstractDc}
			 *            dc The controller
			 * @param {String}
			 *            property The filter property which has been changed
			 * @param {Object}
			 *            ov Old value
			 * @param {Object}
			 *            nv New value
			 */
			_onFilterValueChanged_ : function(dc, property, ov, nv) {
				var fld = this._elems_.findBy(function(item) {
							return (item.dataIndex == property);
						});
				if (fld) {
					fld = this._getElement_(fld.name);
					if (fld.getValue() != nv) {
						fld.suspendEvents();
						fld.setValue(nv);
						fld.resumeEvents();
					}
				}
			},
			/**
			 * @private The parameters model is not part of a store, so we have
			 *          listen to changes made to the model through the
			 *          `parameterValueChanged` event raised by the
			 *          data-control. Changes to the parameters model should be
			 *          done through the setParamValue method of the
			 *          data-control in order to be listened and picked-up to
			 *          refresh the correcponding form fields.
			 * 
			 * @param {dnet.core.dc.AbstractDc}
			 *            dc The controller
			 * @param {String}
			 *            property The parameters property which has been
			 *            changed
			 * @param {Object}
			 *            ov Old value
			 * @param {Object}
			 *            nv New value
			 */
			_onParameterValueChanged_ : function(dc, property, ov, nv) {
				var fld = this._elems_.findBy(function(item) {
							return (item.paramIndex == property);
						});
				if (fld) {
					fld = this._getElement_(fld.name);
					if (fld.getValue() != nv) {
						fld.suspendEvents();
						fld.setValue(nv);
						fld.resumeEvents();
					}
				}
			},

			/**
			 * Get the translation from the resource bundle for the specified
			 * key.
			 * 
			 * @param {String}
			 *            k Key to be translated
			 * @return {String} Translation of the key or the key itself if not
			 *         translation found.
			 */
			_getRBValue_ : function(k) {
				if (this._trl_ != null && this._trl_[k]) {
					return this._trl_[k];
				}
				if (this._controller_._trl_ != null
						&& this._controller_._trl_[k]) {
					return this._controller_._trl_[k];
				} else {
					return k;
				}
			},
			
			_beforeDestroyDNetDcView_ : function() {
				this._controller_ = null;
				if(this._builder_) {
					this._builder_.dcv = null;	
				} 		
			}

			
		});/**
 * Base grid used for data-control list views.
 */
Ext.define("dnet.core.dc.AbstractDNetDcGrid", {
	extend : "Ext.grid.Panel",

	mixins : {
		elemBuilder : "dnet.core.base.AbstractDNetView",
		dcViewSupport: "dnet.core.dc.AbstractDNetDcView"
	},

	// **************** Properties *****************
 
	/**
	 * Columns definition map
	 * 
	 * @type Ext.util.MixedCollection
	 */
	_columns_ : null,

	/**
	 * Flag to switch on/off advanced sort on multiple columns.
	 * 
	 * @type Boolean
	 */
	_noSort_ : false,

	/**
	 * Flag to switch on/off data export.
	 * 
	 * @type Boolean
	 */
	_noExport_ : false,

	/**
	 * Flag to switch on/off data import.
	 * 
	 * @type Boolean
	 */
	_noImport_ : true,

	/**
	 * Flag to switch on/off custom layout management.
	 * 
	 * @type Boolean
	 */
	_noLayoutCfg_ : false,

	/**
	 * Flag to switch on/off paging toolbar
	 * @type Boolean
	 */
	_noPaginator_: false,
	
	/**
	 * Data export window.
	 * 
	 * @type dnet.core.dc.DataExportWindow
	 */
	_exportWindow_ : null,

	/**
	 * Data import window.
	 * 
	 * @type dnet.core.dc.DataExportWindow
	 */
	_importWindow_ : null,

	/**
	 * Custom views management window.
	 * 
	 * @type dnet.core.dc.DataExportWindow
	 */
	_layoutWindow_ : null,


	// **************** Public API *****************

	_defineColumns_ : function() {
	},

	_beforeDefineColumns_ : function() {
		return true;
	},

	_afterDefineColumns_ : function() {
	},

	/**
	 * Open the data-import window
	 */
	_doImport_ : function() {
		if (this._importWindow_ == null) {
			this._importWindow_ = new dnet.core.dc.DataImportWindow({});
			this._importWindow_._grid_ = this;
		}
		this._importWindow_.show();
	},

	/**
	 * Open the data-export window
	 */
	_doExport_ : function() {
		if (this._exportWindow_ == null) {
			this._exportWindow_ = new dnet.core.dc.DataExportWindow({
						_grid_ : this
					});
		}
		this._exportWindow_.show();
	},

	/**
	 * Show the advanced sort window
	 */
	_doSort_ : function() {
		// if (this._sortWindow_ == null) {
		this._sortWindow_ = new dnet.core.dc.DataSortWindow({
					_grid_ : this
				});
		// }
		this._sortWindow_.show();
	},

	/**
	 * Show the custom views management window
	 */
	_doLayoutManager_ : function() {
		if (this._layoutWindow_ == null) {
			this._layoutWindow_ = new dnet.core.dc.GridLayoutManager({
						_grid_ : this
					});
		}
		this._layoutWindow_.show();
	},

	// **************** Defaults and overrides *****************
 
	buttonAlign : "left",
	forceFit : false,
	autoScroll : false,
	scroll : "both",
	 
	border : true,
	frame : true,
	deferRowRender : true,
	//enableLocking : true,
	loadMask: {
            msg: 'Loading...'
        },
	viewConfig : {
		loadMask: {
            msg: 'Loading...'
        },
		enableTextSelection: true,
		stripeRows : true,
		//loadingText : "Loading...",
		emptyText : Dnet.translate("msg", "grid_emptytext")
		
	},

	/**
	 * Redirect the default state management to our implementation.
	 * 
	 * @return {}
	 */
	getState : function() {
		return this._getViewState_();
	},

	/**
	 * Redirect the default state management to our implementation.
	 * 
	 * @return {}
	 */
	applyState : function(state) {
		return this._applyViewState_(state);
	},
	
	beforeDestroy : function() {
		// call the contributed helpers from mixins
		this._beforeDestroyDNetDcView_();
		this._beforeDestroyDNetView_();
		this.callParent(arguments);
	},

	// **************** Private methods *****************

	_initDcGrid_ : function() {

		// currently disabled until is finalized
		this._noImport_ = true;

		this._elems_ = new Ext.util.MixedCollection();
		this._columns_ = new Ext.util.MixedCollection();

		this._defineDefaultElements_();

		this._startDefine_();

		if (this._beforeDefineColumns_() !== false) {
			this._defineColumns_();
			this._afterDefineColumns_();
		}

		if (this._beforeDefineElements_() !== false) {
			this._defineElements_();
			this._afterDefineElements_();
		}

		this._columns_.each(this._postProcessColumn_, this);
		this._endDefine_();

		/*
		 * disable default selection handler in controller let it be triggered
		 * from here
		 */
		this._controller_.afterStoreLoadDoDefaultSelection = false;

	},

	/**
	 * Create the grid configuration object with the usual properties which are
	 * likely to be required by any subclass
	 * 
	 * @return {Object}
	 */
	_createDefaultGridConfig_ : function() {
		var cfg = {
			store : this._controller_.store,
			columns : this._columns_.getRange()
		};

		if (!this._noPaginator_) {
			cfg.bbar = {
				xtype : "pagingtoolbar",
				store : this._controller_.store,
				displayInfo : true
			}
			var bbitems = [];
			this._buildToolbox_(bbitems);

			if (bbitems.length > 0) {
				cfg["bbar"]["items"] = bbitems;
			}
		} else {
			this._noExport_ = true;
			this._noImport_ = true;
			this._noSort_ = true;
			this._noLayoutCfg_ = true;

		}
		return cfg;
	},

	/**
	 * Register event listeners
	 */
	_registerListeners_ : function() {
		this.mon(this._controller_.store, "load", this._onStore_load_, this);
		this.mon(this._controller_, "selectionChange",  // selectionChange
				this._onController_selectionChange, this );
	},

	/**
	 * Handler for the data-control selectionChange event.
	 * 
	 * @param {}
	 *            evnt
	 */
	_onController_selectionChange : function(evnt) {
		var s = evnt.dc.getSelectedRecords();
		// console.log("Abstractdcvgrid. onController_selectionChange sel.len =
		// " + s.length );
		if ( evnt.eOpts && evnt.eOpts.fromGrid === true && evnt.eOpts.grid === this) {
			return;			
		}
		if (s !== this.getSelectionModel().getSelection()) {
			this.getSelectionModel().suspendEvents();
			this.getSelectionModel().select(s, false);
			this.getSelectionModel().resumeEvents();
		}
	},

	/**
	 * Handler for the data-control's store load event.
	 * 
	 * @param {}
	 *            store
	 * @param {}
	 *            operation
	 * @param {}
	 *            eopts
	 */
	_onStore_load_ : function(store, operation, eopts) {
		if (!this._noExport_) {
			if (store.getCount() > 0) {
				this._get_("_btnExport_").enable();
			} else {
				this._get_("_btnExport_").disable();
			}
		}
		if (store.getCount() > 0) {			 
			if (this.selModel.getCount() == 0) {
				this.selModel.select(0);
			} else {
				this._controller_.setSelectedRecords(this.selModel
						.getSelection());
			}
		}
	},

	/**
	 * @private Build default tools
	 * @param {Array}
	 *            bbitems
	 */
	_buildToolbox_ : function(bbitems) {
		if (!this._noLayoutCfg_) {
			bbitems.push("-");
			bbitems.push(this._elems_.get("_btnLayout_"));
		}

		if (!this._noSort_) {
			bbitems.push("-");
			bbitems.push(this._elems_.get("_btnSort_"));
		}

		if (!this._noImport_) {
			bbitems.push("-");
			bbitems.push(this._elems_.get("_btnImport_"));
		}

		if (!this._noExport_) {
			bbitems.push("-");
			bbitems.push(this._elems_.get("_btnExport_"));
		}
	},

	/**
	 * Define defaults elements
	 */
	_defineDefaultElements_ : function() {
		this._elems_.add("_btnExport_", {
					xtype : "button",
					id : Ext.id(),
					disabled : true,
					// text : Dnet.translate("dcvgrid", "exp_btn"),
					tooltip : Dnet.translate("dcvgrid", "exp_title"),
					iconCls : 'icon-action-export',
					handler : this._doExport_,
					scope : this
				});
		this._elems_.add("_btnImport_", {
					xtype : "button",
					id : Ext.id(),
					// text : Dnet.translate("dcvgrid", "imp_btn"),
					tooltip : Dnet.translate("dcvgrid", "imp_title"),
					iconCls : 'icon-action-import',
					handler : this._doImport_,
					scope : this
				});
		this._elems_.add("_btnSort_", {
					xtype : "button",
					id : Ext.id(),
					// text : Dnet.translate("dcvgrid", "sort_btn"),
					tooltip : Dnet.translate("dcvgrid", "sort_title"),
					iconCls : 'icon-action-sort',
					handler : this._doSort_,
					scope : this
				});
		this._elems_.add("_btnLayout_", {
					xtype : "button",
					id : Ext.id(),
					// text : Dnet.translate("dcvgrid", "btn_perspective_txt"),
					tooltip : Dnet.translate("dcvgrid", "btn_perspective_tlp"),
					iconCls : 'icon-action-customlayout',
					handler : this._doLayoutManager_,
					scope : this
				});
	},

	/**
	 * Specific implementation to read the grid columns view-state to be stored
	 * as a custom view.
	 * 
	 * @return {}
	 */
	_getViewState_ : function() {
		var me = this, state = null, colStates = [], cm = this.headerCt, cols = cm.items.items;
		for (var i = 0, len = cols.length; i < len; i++) {
			var c = cols[i];
			colStates.push({
						n : c.name,
						h : c.hidden,
						w : c.width
					});
		}
		state = me.addPropertyToState(state, 'columns', colStates);
		return state;
	},

	/**
	 * Apply a view-state read by _getViewState_
	 * 
	 * @param {}
	 *            state
	 */
	_applyViewState_ : function(state) {
		if (!this.rendered) {
			this.on("afterrender", this._applyViewStateAfterRender_, this, {
						single : true,
						state : state
					});
			return;
		}
		var sCols = state.columns, cm = this.headerCt, cols = cm.items.items, col = null;

		for (var i = 0, slen = sCols.length; i < slen; i++) {
			var sCol = sCols[i];
			var colIndex = -1;

			for (var j = 0, len = cols.length; j < len; j++) {
				if (cols[j].name == sCol.n) { // && (!myCM.config[i]._aSel_)
					// ){
					colIndex = j;
					col = cols[j];
					break;
				}
			}

			if (colIndex >= 0) {
				if (sCol.h) {
					col.hide();
				} else {
					col.show();
				}
				col.setWidth(sCol.w);
				// cm.setColumnWidth(colIndex, s.width, true);
				// cm.setColumnHeader(colIndex, s.header, true);
				if (colIndex != i) {
					col.move(colIndex, i);
				}
			}
		}
	},

	_applyViewStateAfterRender_ : function(cmp, eOpts) {
		this._applyViewState_(eOpts.state);
	},

	_selectionHandler_: function(sm, selected, options) {
		var gridSel = this.getSelectionModel().getSelection();
		var dcSel = this._controller_.selectedRecords;
	 
		var ctrl = this._controller_;
		ctrl.setSelectedRecords(gridSel, {fromGrid: true, grid: this});
		if (gridSel.length <= 1) {
			if (gridSel.length == 1) {
				ctrl.setRecord(gridSel[0],{fromGrid: true, grid: this});
			} else {
				ctrl.setRecord(null,{fromGrid: true, grid: this});
			}
		}
		if (gridSel.length > 1) {
			if (ctrl.record == null || gridSel.indexOf(ctrl.record) < 0) {
				ctrl.setRecord(gridSel[0],{fromGrid: true, grid: this});
			}
		}
	},
	 
	/**
	 * Postprocessor run to inject framework specific settings into the columns.
	 * 
	 * @param {Object}
	 *            column Column's configuration object
	 * @param {Integer}
	 *            idx The index
	 * @param {Integer}
	 *            len Total length
	 */
	_postProcessColumn_ : function(column, idx, len) {
		if (column.header == undefined) {
			Dnet.translateColumn(this._trl_, this._controller_._trl_, column);
		}
	} 


});/**
 * Base panel used for data-control views. It serves as base class for
 * edit-forms and filter-forms.
 */
Ext.define("dnet.core.dc.AbstractDNetDcForm", {
	extend : "Ext.form.Panel",

	mixins: {
		elemBuilder: "dnet.core.base.AbstractDNetView",
		dcViewSupport: "dnet.core.dc.AbstractDNetDcView"
	},
	
	// **************** Properties *****************

	
	
	// **************** Public API *****************
	 
	
	/**
	 * @public Helper method to disable all fields.
	 */
	_disableAllFields_ : function() {
		this.getForm().getFields().each(function(item, index, length) {
					item.disable();
				});
	},

	/**
	 * @public Helper method to enable all fields.
	 */
	_enableAllFields_ : function() {
		this.getForm().getFields().each(function(item, index, length) {
					item.enable();
				});
	},
 
	/**
	 * There may be situations when a form should not validate. For example a
	 * data-control may have one form for insert and another one to edit an
	 * existing record. These forms may have different rules to validate (for
	 * example some fields are mandatory on insert but not on update others on
	 * update but not on insert). Override this function to implement the
	 * decision rules when to invoke the form validation.
	 * 
	 * @return {Boolean}
	 */
	_shouldValidate_ : function() {
		return true;
	},
	
 
	_setFieldsEnabledState_: function(names, model) {
		if(!model) return;
		for(var i=0,l=names.length; i<l; i++) {
			var n = names[i];
			this._getElement_(n).setDisabled( !this._canSetEnabled_(n, model ) );			 
		}
	},

	_setFieldsVisibleState_: function(names, model) {
		if(!model) return;
		var fields = this.getForm().getFields();
		for(var i=0,l=names.length; i<l; i++) {
			var n = names[i];
			this._getElement_(n).setVisible( this._canSetVisible_(n, model ) );	
		}
	},
	
	_canSetEnabled_ : function(name, model) {
		var fn = this._elems_.get(name)._enableFn_;
		if (fn) {
			return fn.call(this, this._controller_, model);
		} else {
			return true;
		}
	},

	_canSetVisible_ : function(name, model) {
		var fn = this._elems_.get(name)._visibleFn_;
		if (fn) {
			return fn.call(this, this._controller_, model);
		} else {
			return true;
		}
	},

 
	/**
	 * @public Helper method to disable the specified fields.
	 * 
	 * @param {Array}
	 *            fldNamesArray Array of field names
	 */
	_disableFields_ : function(fldNamesArray) {
		for (var i = 0, l = fldNamesArray.length; i < l; i++) {
			this._get_(fldNamesArray[i]).disable();
		}
	},

	/**
	 * Generic validation method which displays an message box for the user.
	 * 
	 * @return {Boolean}
	 */
	_isValid_ : function() {
		if (this.getForm().isValid()) {
			return true;
		} else {
			Ext.Msg.show({
				title : 'Invalid data',
				msg : 'Form contains invalid data.<br>Please correct values then try again. ',
				buttons : Ext.MessageBox.OK,
				scope : this,
				icon : Ext.MessageBox.ERROR
			});
			return false;
		}
	},
	
	
	// **************** Defaults and overrides *****************
	
	
	beforeDestroy : function() {
		// call the contributed helpers from mixins
		this._beforeDestroyDNetDcView_();
		this._beforeDestroyDNetView_();
		this.callParent(arguments);
	},
	
	// **************** Private methods *****************
	
	 
	/**
	 * Postprocessor run to inject framework specific settings into the
	 * elements.
	 * 
	 * @param {}
	 *            item
	 * @param {}
	 *            idx
	 * @param {}
	 *            len
	 * @return {Boolean}
	 */
	_postProcessElem_ : function(item, idx, len) {
		item["_dcView_"] = this;
		if (item.fieldLabel == undefined) {
			Dnet.translateField(this._trl_, this._controller_._trl_, item);
		}
		return true;
	}
	
 
 
	
	
});/**
 * Base class for data-control property grid based views. It serves as base class for
 * edit-form as property grids and filter-forms as property grids .
 */
Ext.define("dnet.core.dc.AbstractDNetDcPropGrid", {
	extend : "Ext.grid.property.Grid",

	mixins: {
		elemBuilder: "dnet.core.base.AbstractDNetView",
		dcViewSupport: "dnet.core.dc.AbstractDNetDcView"
	},
	
	// **************** Properties *****************
 
	// **************** Public API *****************
	 
 
	/**
	 * @public Helper method to disable all fields.
	 */
	_disableAllFields_ : function() {
		this.getForm().getFields().each(function(item, index, length) {
					item.disable();
				});
	},
 

	
	/**
	 * There may be situations when a form should not validate. For example a
	 * data-control may have one form for insert and another one to edit an
	 * existing record. These forms may have different rules to validate (for
	 * example some fields are mandatory on insert but not on update others on
	 * update but not on insert). Override this function to implement the
	 * decision rules when to invoke the form validation.
	 * 
	 * @return {Boolean}
	 */
	_shouldValidate_ : function() {
		return true;
	},
	


	_canSetEnabled_ : function(name, model) {
		var fn = this._elems_.get(name)._enableFn_;
		if (fn) {
			return fn.call(this, this._controller_, model);
		} else {
			return true;
		}
	},

	_canSetVisible_ : function(name, model) {
		var fn = this._elems_.get(name)._visibleFn_;
		if (fn) {
			return fn.call(this, this._controller_, model);
		} else {
			return true;
		}
	},

	 

	/**
	 * @public Helper method to disable the specified fields.
	 * 
	 * @param {Array}
	 *            fldNamesArray Array of field names
	 */
	_disableFields_ : function(fldNamesArray) {
		for (var i = 0, l = fldNamesArray.length; i < l; i++) {
			this._get_(fldNamesArray[i]).disable();
		}
	},

	/**
	 * Generic validation method which displays an message box for the user.
	 * 
	 * @return {Boolean}
	 */
	_isValid_ : function() {
		if (this.getForm().isValid()) {
			return true;
		} else {
			Ext.Msg.show({
				title : 'Invalid data',
				msg : 'Form contains invalid data.<br>Please correct values then try again. ',
				buttons : Ext.MessageBox.OK,
				scope : this,
				icon : Ext.MessageBox.ERROR
			});
			return false;
		}
	},
	
	
	// **************** Private methods *****************
 
	/**
	 * Postprocessor run to inject framework specific settings into the
	 * elements.
	 * 
	 * @param {}
	 *            item
	 * @param {}
	 *            idx
	 * @param {}
	 *            len
	 * @return {Boolean}
	 */
	_postProcessElem_ : function(item, idx, len) {
		item["_dcView_"] = this;
		if (item.fieldLabel == undefined) {
			Dnet.translateField(this._trl_, this._controller_._trl_, item);
		}
		return true;
	},
	
	beforeDestroy : function() {
		// call the contributed helpers from mixins
		this._beforeDestroyDNetDcView_();
		this._beforeDestroyDNetView_();
		this.callParent(arguments);
	},
 
	// **************** destroy component *****************

	// TODO: to be reviewed!!

	beforeDestroy : function() {
		this._controller_ = null;
		this.callParent();
		try {
			this._unlinkElemRefs_();
		} catch (e) {
		
		}
		this._elems_.each(this.unlinkElem, this);
		this._elems_.each(this.destroyElement, this);
	},

	unlinkElem : function(item, index, len) {
		item._dcView_ = null;
	},

	destroyElement : function(elemCfg) {
		try {
			var c = Ext.getCmp(elemCfg.id);
			if (c) {
				Ext.destroy(c);
			}
		} catch (e) {
			// alert(e);
		}
	}
	
	
});
Ext.define("dnet.core.dc.BulkEditWindow", {
	extend : "Ext.window.Window",

	/**
	 * 
	 * @type dnet.core.dc.AbstractDcvGrid
	 */
	_grid_ : null,
	_editorId_ : null,
	
	_editorConfig_ : null,
	_updatableFields_ : null,
	//TODO: optimize this
	
	initComponent : function(config) {
		this._editorId_ = Ext.id();
		this.setupBulkEditor();
		
		var btn = Ext.create('Ext.Button', {
			text : Dnet.translate("dcvgrid", "bulkedit_run"),
			//iconCls : 'icon-action-sort',
			scope: this,
			handler:this.executeTask
		});

		var avlCol = [];
		this._grid_._columns_.each(function(item, idx,len) {
			avlCol[avlCol.length] = [item.name, item.header, '',item.header+'' ]
		})
		 
	    
		var cfg = {
			title : Dnet.translate("dcvgrid", "bulkedit_title"),
			border : true,
			width : 350,
			height:180,
			resizable : true,
			closeAction : "hide",			 
			closable : true,
			constrain : true,
			buttonAlign : "center",
			modal : true,
			layout:"fit",
			items : [this._editorConfig_ ],
			buttons : [ btn ]
		};

		Ext.apply(cfg, config);
		Ext.apply(this, cfg);
		this.callParent(arguments);
	},
	
	
	setupBulkEditor: function() {
		this._updatableFields_ = [];
		var bef = this._grid_._bulkEditFields_;
		if (Ext.isEmpty(bef) || bef.length == 0 ) {
			return null;
		}
		
		var gc = this._grid_._columns_
		 	len = bef.length;
		 	
		 	
		for (var i=0; i<len; i++ ) {
			var name =bef[i];
			var col = gc.get(name);
			this._updatableFields_.push({"name": name, "title": col.header});
		}
		 
		var updFieldsStore = Ext.create('Ext.data.Store', {
		    fields: ['name', 'title'],
		    data : this._updatableFields_
		});
  
		 var be_source = {
		 		_ds_fieldName_ : "",
		 		_ds_fieldValue_ : ""
		 	},
		 	be_props = {
		 		_ds_fieldName_ : "Field",
		 		_ds_fieldValue_ : "New Value"
		 	},
		 	be_editors = {
		 		_ds_fieldName_ : Ext.create('Ext.form.field.ComboBox', {
		 					selectOnFocus:true, 
		 					store: updFieldsStore,
						    queryMode: 'local',
						    displayField: 'title',
						    valueField: 'title'
				})
		 	};
		
		for (var i=0; i<len; i++ ) {
			var name =bef[i];
			var col = gc.get(name);
			be_editors["_ds_fieldValue_"+name] = col.editor;
			//this._updatableFields_.push({"name": name, "title": col.header});
		}
				
		this._editorConfig_ = {			
			xtype: 'bulkeditor',	        
	        id : this._editorId_,
	       // preventHeader: true,
	        _grid_: this._grid_,
	        _updatableFields_ : this._updatableFields_, 
			customEditors: be_editors,		 
		    propertyNames: be_props,
		    source: be_source
		}
	},
	
	
	getEditor : function() {
		return Ext.getCmp(this._editorId_);
	},
	
	
	getField: function(title) {
		var f = this._updatableFields_,
			l = f.length;
		for(var i=0;i<l; i++) {
			if(f[i].title == title ) {
				return f[i].name
			}
		}
	},
	
	
	
	
	
	executeTask : function() {
		var ed = this.getEditor(),
			s = ed.getSource(), 
			dsfield = s._ds_fieldName_
			;
		if ( !Ext.isEmpty(dsfield)) {
			dsfield = this.getField(dsfield);
			var ctrl = this._grid_._controller_
				sel = ctrl.selectedRecords,
				store = ctrl.store,
				sellen = sel.length;
			store.suspendEvents(true);	
			for (var i=0;i<sellen;i++) {
				sel[i].set(dsfield, s._ds_fieldValue_);
				
				for(var p in s) {
					if (p != "_ds_fieldValue_" && p != "_ds_fieldName_") {
						sel[i].set(p, s[p]);
					}
				}
				
				
			}
			store.resumeEvents();
		} 
		this.close();
	}

});




Ext.define("dnet.core.dc.BulkEditor", {
	extend : "Ext.grid.property.Grid",	
	alias:"widget.bulkeditor",
	//title: 'Bulk edit',
    width: 300 ,
    _dcViewType_: "bulk-edit-field",
    _grid_ : null,
   // hideHeaders : true,
    _updatableFields_ : null,
     
    _getEditor_:function(propName) {
    	
    },
    _getField_: function(title) {
		var f = this._updatableFields_,
			l = f.length;
		for(var i=0;i<l; i++) {
			if(f[i].title == title ) {
				return f[i].name
			}
		}
	},
    getCellEditor : function(record, column) {
    	
    	var src = this.getSource();
    	if (record.data.name != "_ds_fieldName_") {    	
	    	if (Ext.isEmpty(src._ds_fieldName_)) {
	    		Ext.Msg.show({
					title : "No field selected",
					msg:"Select first the field to update, then try to set its value.",	
					icon : Ext.MessageBox.INFO,
					buttons : Ext.Msg.OK      
	    		});
	    		return null;
	    	}    		    		
    	}
    	
         var me = this,
            propName = record.get(me.nameField),
            val = record.get(me.valueField),
            editor = me.customEditors[propName]
            dsField = "";

        if (propName == "_ds_fieldName_") {
    		editor = me.customEditors[propName];
    	} else { 
    		dsField = this._getField_(src._ds_fieldName_);
    		editor = me.customEditors[propName+dsField];
    	}    
            
        // A custom editor was found. If not already wrapped with a CellEditor, wrap it, and stash it back
        // If it's not even a Field, just a config object, instantiate it before wrapping it.
        if (editor) {
            if (!(editor instanceof Ext.grid.CellEditor)) {
                if (!(editor instanceof Ext.form.field.Base)) {
                    editor = Ext.ComponentManager.create(editor, 'textfield');
                }
                editor = me.customEditors[propName] = Ext.create('Ext.grid.CellEditor', { field: editor });
            }
        } else if (Ext.isDate(val)) {
            editor = me.editors.date;
        } else if (Ext.isNumber(val)) {
            editor = me.editors.number;
        } else if (Ext.isBoolean(val)) {
            editor = me.editors['boolean'];
        } else {
            editor = me.editors.string;
        }

        // Give the editor a unique ID because the CellEditing plugin caches them
        if (propName == "_ds_fieldName_") {
    		editor.editorId = propName;
    	} else { 
    		editor.editorId = propName+dsField;    	 
    	}  
    	editor.field._dcView_ = this;
        return editor;
    }
});
Ext.define("dnet.core.dc.AbstractDcvGrid", {
	extend : "dnet.core.dc.AbstractDNetDcGrid",

  
	// **************** Properties *****************
	
	/**
	 * Component builder
	 * @type dnet.core.dc.DcvGridBuilder
	 */
	_builder_ : null,
	

	// **************** Public API *****************
	
	
	/**
	 * @public Returns the builder associated with this type of component. Each
	 *         predefined data-control view type has its own builder.
	 *         If it doesn't exist yet attempts to create it.
	 * 
	 * @return {dnet.core.dc.DcvGridBuilder}
	 */
	_getBuilder_ : function() {
		if (this._builder_ == null) {
			this._builder_ = new dnet.core.dc.DcvGridBuilder( {
				dcv : this
			});
		}
		return this._builder_;
	},
 
 	// **************** Private methods *****************
	
	initComponent : function(config) {

		this._initDcGrid_();
		var cfg = this._createDefaultGridConfig_();
//		this.plugins = [Ext.create('Ext.grid.plugin.CellEditing', {
//					clicksToEdit : 2
//					
//				})];
		Ext.apply(cfg, {			
			selModel : {
				mode : "MULTI",
				listeners : {
					"selectionchange" : {
						scope : this,
						fn : this._selectionHandler_ ,
						buffer: 200
						 
					},
					"beforedeselect" : {
						scope : this,
						fn : function(sm, record, index, eopts) {
							if (record == this._controller_.record
									&& !this._controller_
											.isRecordChangeAllowed()) {
								return false;
							}
						}
					}
				}
			},
			listeners : {
				"itemdblclick" : {
					scope : this,
					fn : function(view, model, item, idx, evnt, evntOpts) {
						this._controller_.onEdit();						 
					}
				}
			}
 
		});
//		Ext.apply(this.viewConfig, {
//			enableTextSelection: true
//		}); 
		Ext.apply(this, cfg);
		this.callParent(arguments);
		this._registerListeners_();
	}
});Ext.define("dnet.core.dc.AbstractDcvEditableGrid", {
	extend : "dnet.core.dc.AbstractDNetDcGrid",

	// **************** Properties *****************

	/**
	 * Component builder
	 * 
	 * @type dnet.core.dc.DcvEditableGridBuilder
	 */
	_builder_ : null,

	/**
	 * Flag to switch on/off bulk edit feature
	 * 
	 * @type Boolean
	 */
	_noBulkEdit_ : false,

	/**
	 * Array with field names which are allowed to be bulk-edited.
	 * 
	 * @type
	 */
	_bulkEditFields_ : null,

	/**
	 * Bulk-editor window.
	 * 
	 * @type
	 */
	_bulkEditWindow_ : null,

	/**
	 * Ensure the data-control is configured with multiEdit = true otherwise the
	 * grid cannot edit more than one record at a time.
	 * 
	 * @type Boolean
	 */
	_enforceMultiEditDc_ : true,

	// **************** Public API *****************

	/**
	 * @public Returns the builder associated with this type of component. Each
	 *         predefined data-control view type has its own builder. If it
	 *         doesn't exist yet attempts to create it.
	 * 
	 * @return {dnet.core.dc.DcvEditableGridBuilder}
	 */
	_getBuilder_ : function() {
		if (this._builder_ == null) {
			this._builder_ = new dnet.core.dc.DcvEditableGridBuilder({
						dcv : this
					});
		}
		return this._builder_;
	},

	/**
	 * Show the bulk-edit window
	 */
	_doBulkEdit_ : function() {
		if (this._controller_.selectedRecords.length == 0) {
			Ext.Msg.show({
				title : "No records selected",
				msg : "Select the records for which you want to apply the changes.",
				icon : Ext.MessageBox.INFO,
				buttons : Ext.Msg.OK
			});
			return;
		}
		if (this._bulkEditWindow_ == null) {
			this._bulkEditWindow_ = new dnet.core.dc.BulkEditWindow({
						_grid_ : this
					});
		}
		this._bulkEditWindow_.show();
	},

	// **************** Defaults and overrides *****************

	clicksToEdit : 1,

	/**
	 * 
	 * Override the parent method to add specific functions
	 * 
	 * @param {}
	 *            bbitems
	 */
	_buildToolbox_ : function(bbitems) {
		this.callParent(arguments);
		if (!this._noBulkEdit_ && this._bulkEditFields_ != null
				&& this._bulkEditFields_.length > 0) {
			bbitems.push("-");
			bbitems.push(this._elems_.get("_btnBulkEdit_"));
		}
	},

	/**
	 * Override parent to add specific elements.
	 */
	_defineDefaultElements_ : function() {
		this.callParent(arguments);
		this._elems_.add("_btnBulkEdit_", {
					xtype : "button",
					id : Ext.id(),
					tooltip : Dnet.translate("dcvgrid", "btn_bulkedit_tlp"),
					iconCls : 'icon-action-edit',
					handler : this._doBulkEdit_,
					scope : this
				});
	},

	beforeEdit : function(context) {
		if (this._controller_.readOnly) {
			return false;
		}
	},

	beforeDestroy : function() {
		this._columns_.each(function(item, idx, len) {
					delete item._dcView_;
					if (item.editor) {
						delete item.editor._dcView_;
					}
				}, this)
		this.callParent(arguments);
	},

	// **************** Private methods *****************

	initComponent : function(config) {
		if (this._enforceMultiEditDc_ && !this._controller_.multiEdit) {
			alert("Editable grids should be used with data-controls having multiEdit enabled.");
		}
		this._initDcGrid_();
		var cfg = this._createDefaultGridConfig_();

		this.plugins = [Ext.create('Ext.grid.plugin.CellEditing', {
					clicksToEdit : 1
				})];

		Ext.apply(cfg, {
					selModel : {
						mode : "MULTI",
						listeners : {
							"selectionchange" : {
								scope : this,
								fn : this._selectionHandler_,
								buffer : 200
							}
						},
						"beforedeselect" : {
							scope : this,
							fn : function(sm, record, index, eopts) {
								if (record == this._controller_.record
										&& !this._controller_
												.isRecordChangeAllowed()) {
									return false;
								}
							}
						}
					}
				});

		if (this._noPaginator_) {
			this._noBulkEdit_ = true;
		}
		Ext.apply(this, cfg);
		this.callParent(arguments);
		this._registerListeners_();

		this.on("afteredit", this._afterEdit_, this);

	},

	_afterEdit_ : function() {

	}

});
Ext.define("dnet.core.dc.AbstractDcvEditForm", {
	extend : "dnet.core.dc.AbstractDNetDcForm",

	// **************** Properties *****************

	/**
	 * Flag to automatically disable form fields if the data-control is marked as read-only. 
	 * @type Boolean
	 */
	_shouldDisableWhenDcIsReadOnly_ : true,
	
	/**
	 * Specify how to apply the form disable when shouldDisableWhenDcIsReadOnly property is true.
	 * Possible values are : 
	 * fields - call disable on all fields (default value)
	 * panel - disable the form panel
	 * elems - call disable on all elements
	 * 
	 * 
	 * @type String
	 */
	_disableModeWhenDcIsReadOnly_: "fields",
	
	/**
	 * 
	 * Component builder
	 * 
	 * @type dnet.core.dc.DcvEditFormBuilder
	 */
	_builder_ : null,

	/**
	 * Helper property to identify this dc-view type as edit form.
	 * 
	 * @type String
	 */
	_dcViewType_ : "edit-form",

	// **************** Public API *****************

	/**
	 * @public Returns the builder associated with this type of component. Each
	 *         predefined data-control view type has its own builder.
	 *         If it doesn't exist yet attempts to create it.
	 * 
	 * @return {dnet.core.dc.DcvEditFormBuilder}
	 */
	_getBuilder_ : function() {
		if (this._builder_ == null) {
			this._builder_ = new dnet.core.dc.DcvEditFormBuilder({
						dcv : this
					});
		}
		return this._builder_;
	},

	_setShouldDisableWhenDcIsReadOnly_: function(v, immediate) {
		this._shouldDisableWhenDcIsReadOnly_ = v;
		if (immediate && v) {
			this._doDisableWhenDcIsReadOnly_();
		}
	},
	
	
	// **************** Defaults and overrides *****************

	frame : true,
	border : false,
	bodyBorder : false,
	bodyPadding : '5 5 0 0',
	maskOnDisable : false,
	layout : "fit",
	buttonAlign : "left",
	bodyCls : 'dcv-edit-form',
	trackResetOnLoad : false,
	fieldDefaults : {
		labelAlign : "right",
		labelWidth : 100
	},

	defaults : {
		frame : false,
		border : false,
		bodyBorder : false,
		bodyStyle : " background:transparent "
	},

	initComponent : function() {
		this._runElementBuilder_();
		this.callParent(arguments);
		this._registerListeners_(); 
	},

	/**
	 * After the form is rendered invoke the record binding. This is necessary
	 * as the form may be rendered lazily(delayed) and the data-control may
	 * already have a current record set.
	 */
	afterRender : function() {
		this.callParent(arguments);
		if (this._controller_ && this._controller_.getRecord()) {
			this._onBind_(this._controller_.getRecord());
		} else {
			this._onUnbind_(null);
		}
	},
	
	beforeDestroy : function() {		
		this._elems_.each(function(item,idx,len) {
			delete item._dcView_;
		}, this)
		this.callParent(arguments);
	},
	// **************** Private API *****************

	/**
	 *  Register event listeners
	 */
	_registerListeners_ : function() {

		this.mon(this._controller_.store, "datachanged",
				this._onStore_datachanged_, this);
		this
				.mon(this._controller_.store, "update", this._onStore_update_,
						this);
		this.mon(this._controller_.store, "write", this._onStore_write_, this);

		this.mon(this._controller_, "recordChange",
				this._onController_recordChange_, this);

		this.mon(this._controller_, "readOnlyChanged",
				function() {this._applyStates_(this._controller_.getRecord());} , this); 
		// this.mon(this, "afterrender", this.on_afterrender, this);

		if (this._controller_.commands.doSave) {
			this._controller_.commands.doSave.beforeExecute = Ext.Function
					.createInterceptor(
							this._controller_.commands.doSave.beforeExecute,
							function() {
								if (this._shouldValidate_()
										&& !this.getForm().isValid()) {
									Ext.Msg.show({
										title : "Validation info",
										msg : "Form contains invalid data.<br> Please fix the errors then try again.",
										scope : this,
										icon : Ext.MessageBox.ERROR,
										buttons : Ext.MessageBox.OK
									});
									return false;
								} else {
									return true;
								}
							}, this, -1);
		}
	},

	/**
	 * @private Update the bound record when the store data is changed.
	 * @param {}
	 *            store
	 * @param {}
	 *            eopts
	 */
	_onStore_datachanged_ : function(store, eopts) {
		this._updateBound_(this._controller_.getRecord());
	},

	/**
	 * @private Update the bound record when the store data is updated.
	 * @param {}
	 *            store
	 * @param {}
	 *            rec
	 * @param {}
	 *            op
	 * @param {}
	 *            eopts
	 */
	_onStore_update_ : function(store, rec, op, eopts) {
		if (this.getForm().getRecord() === rec) {
			this._updateBound_(rec);
		}
	},

	/**
	 * @private Update the bound record after a succesful save.
	 * @param {}
	 *            store
	 * @param {}
	 *            operation
	 * @param {}
	 *            eopts
	 */
	_onStore_write_ : function(store, operation, eopts) {
		/**
		 * use the first record from the result list as reference see Store on
		 * write event handler defined in AbstractDc.
		 */
//		if (operation.action == "create") {
//			this._applyContextRules_(operation.resultSet.records[0]);
//		}
	},

	/**
	 * @private When the current record of the data-control is changed bind it
	 *          to the form.
	 * @param {}
	 *            evnt
	 */
	_onController_recordChange_ : function(evnt) {
		var newRecord = evnt.newRecord;
		var oldRecord = evnt.oldRecord;
		var newIdx = evnt.newIdx;
		if (newRecord != oldRecord) {
			this._onUnbind_(oldRecord);
			this._onBind_(newRecord);
		}
	},

	/**
	 * Bind the current record of the data-control to the form.
	 * 
	 * @param {Ext.data.Model}
	 *            record The record to bind
	 */
	_onBind_ : function(record) {
		if (record) {
			var fields = this.getForm().getFields();

			fields.each(function(field) {
						field.suspendEvents();
					});
			var trackResetOnLoad = this.getForm().trackResetOnLoad; 
			fields.each(function(field) {
				if(field.dataIndex) {
					field.setValue(record.get(field.dataIndex));
					if ( trackResetOnLoad) {
	                    field.resetOriginalValue();
	                }
				}					
			});
			this.getForm()._record = record;
			//this.getForm().loadRecord(record);
			fields.each(function(field) {
						field.resumeEvents();
					});
			this._applyStates_(record);
			this.getForm().isValid();
		}
		this._afterBind_(record);
	},

	/**
	 * Un-bind the record from the form.
	 * 
	 * @param {Ext.data.Model}
	 *            record
	 */
	_onUnbind_ : function(record) {
		this.getForm().getFields().each(function(field) {
					if (field.dataIndex) {
						field.setRawValue(null);
						field.clearInvalid();
					}
					field.disable();
				});
		this._afterUnbind_(record);
	},

	/**
	 * When the record has been changed in any way other than user interaction,
	 * update the fields of the form with the changed values from the model.
	 * Such change may happen by custom code snippets updating the model in a
	 * beginEdit-endEdit block, reload record from server, service methods which
	 * returns changed record data from server, etc.
	 * 
	 * @param {Ext.data.Model}
	 *            record
	 */
	_updateBound_ : function(record) {
		var msg = "null";
		if (record) {
			var fields = this.getForm().getFields();
			
			fields.each(function(field) {

						if (field.dataIndex) {
							var nv = record.data[field.dataIndex];
							if (field.getValue() != nv) {
								// field.suspendEvents();
								//this.suspendCheckChange++;
								if ( !(field.hasFocus && field.isDirty)) {
									field.setValue(nv);
								}
								//this.suspendCheckChange--;
								// field.resumeEvents();
							}
						}
					});
			
			
		}
	},

	/**
	 * @protected The edit-form specific state rules. The flow is: If the fields
	 *            are marked with noInsert, noUpdate or noEdit these rules are
	 *            applied and no other option checked If no such constraint, the
	 *            _canSetEnabled_ function is checked for each element.
	 * 
	 * 
	 * @param {Ext.data.Model}
	 *            record
	 */
	_onApplyStates_ : function(record) {
		// the form has been disabled by the (un)bind.
		// Nothing to change
		if (record == null || record == undefined) {
			return;
		}
		if(this._shouldDisableWhenDcIsReadOnly_ && this._controller_.isReadOnly() ) {
			this._doDisableWhenDcIsReadOnly_();
			return;
		}
		if (record.phantom) {
			this.getForm().getFields().each(function(item, index, length) {
						if (item.noEdit === true || item.noInsert === true) {
							item.disable();
						} else {
							item.setDisabled(!this._canSetEnabled_(item.name, record));							 
						}
						item.setVisible( this._canSetVisible_(item.name, record ) );	
					}, this);
		} else {
			this.getForm().getFields().each(function(item, index, length) {
						if (item.noEdit === true || item.noUpdate === true) {
							item.disable();
						} else {
							item.setDisabled(!this._canSetEnabled_(item.name, record));
						}
						item.setVisible( this._canSetVisible_(item.name, record ) );
					}, this);
		}

	},

	/**
	 * @deprecated Replaced by the _applyStates_ method. Kept only for backward
	 *             compatibility but will be removed. Please update your code.
	 * @param {Ext.data.Model}
	 *            record
	 */
	_applyContextRules_ : function(record) {
		this._applyStates_(record);
	},
	
	_doDisableWhenDcIsReadOnly_ : function() {
		if(this._shouldDisableWhenDcIsReadOnly_ && this._controller_.isReadOnly() ) {
			this["_doDisableWhenDcIsReadOnly_"+ this._disableModeWhenDcIsReadOnly_ + "_"]();
		}
	},
	
	_doDisableWhenDcIsReadOnly_fields_ : function() {
		this.getForm().getFields().each(this._disableElement_);
	},
	_doDisableWhenDcIsReadOnly_panel_ : function() {
		this.disable();
	},
	_doDisableWhenDcIsReadOnly_elems_ : function() {
		this._elems_.each(this._disableElement_);
	},
	_disableElement_:function(e) {
		e.disable();
	} 
	

});
/**
 * @deprecated
 * Use AbstractDcvEditForm instead of this 
 * 
 */
Ext.define("dnet.core.dc.AbstractDcvForm", {
	extend : "dnet.core.dc.AbstractDcvEditForm"
 
});Ext.define("dnet.core.dc.AbstractDcvEditPropGrid", {
			extend : "dnet.core.dc.AbstractDNetDcPropGrid",

			// **************** Properties *****************
			/**
			 * Component builder
			 * 
			 * @type dnet.core.dc.DcvEditPropGridBuilder
			 */
			_builder_ : null,

			/**
			 * Helper property to identify this dc-view type as filter property
			 * grid.
			 * 
			 * @type String
			 */
			_dcViewType_ : "edit-propgrid",

			// **************** Public API *****************

			/**
			 * @public Returns the builder associated with this type of
			 *         component. Each predefined data-control view type has its
			 *         own builder. If it doesn't exist yet attempts to create
			 *         it.
			 * 
			 * @return {dnet.core.dc.DcvEditPropGridBuilder}
			 */
			_getBuilder_ : function() {
				if (this._builder_ == null) {
					this._builder_ = new dnet.core.dc.DcvEditPropGridBuilder({
								dcv : this
							});
				}
				return this._builder_;
			},

			// **************** Defaults and overrides *****************

			initComponent : function(config) {
				this._runElementBuilder_();
				var sourceObj = {};
				var propertyNames = {};
				var customEditors = {};
				var customRenderers = {};

				var fnSourceObj = function(item, idx, len) {
					sourceObj[item.name] = item._default_;
				};
				var fnPropertyName = function(item, idx, len) {
					propertyNames[item.name] = item.fieldLabel;
				};
				var fnCustomEditors = function(item, idx, len) {
					if (item.editorInstance) {
						customEditors[item.name] = item.editorInstance;	
					}
				};
				var fnCustomRenderers = function(item, idx, len) {
					if ( item.renderer ) {
						customRenderers[item.name] = item.renderer;
					}
				};

				this._elems_.each(fnSourceObj, this);
				this._elems_.each(fnPropertyName, this);
				this._elems_.each(fnCustomEditors, this);
				this._elems_.each(fnCustomRenderers, this);

				var cfg = {
					autoScroll : true,
					source : sourceObj,
					propertyNames : propertyNames,
					customEditors : customEditors,
					customRenderers : customRenderers
				};
				Ext.apply(this, cfg);
				this.callParent(arguments);
				this._registerListeners_();
			},

			/**
			 * After the form is rendered invoke the record binding. This is
			 * necessary as the form may be rendered lazily(delayed) and the
			 * data-control may already have a current record set.
			 */
			afterRender : function() {
				this.callParent(arguments);
				if (this._controller_ && this._controller_.getRecord()) {
					this._onBind_(this._controller_.getRecord());
				} else {
					this._onUnbind_(null);
				}
			},

			// **************** Private API *****************

			/**
			 * Register event listeners
			 */
			_registerListeners_ : function() {

				this.mon(this._controller_.store, "datachanged",
						this._onStore_datachanged_, this);
				this.mon(this._controller_.store, "update",
						this._onStore_update_, this);
				this.mon(this._controller_.store, "write",
						this._onStore_write_, this);

				this.mon(this._controller_, "recordChange",
						this._onController_recordChange_, this);

				this.mon(this._controller_, "parameterValueChanged",
						this._onParameterValueChanged_, this);
 
				this.mon(this, "edit", function(editor, evnt, eOpts) { 
							 this._controller_.getRecord().set(evnt.record.data.name,
							 evnt.value);
							return true;
						}, this);
			},

			/**
			 * Bind the current filter model of the data-control to the form.
			 * 
			 * @param {Ext.data.Model}
			 *            record The record to bind
			 */
			_onBind_ : function(record) {
				this._updateBound_(record);
				this._applyStates_(record);
				this._afterBind_(record);
			},

			/**
			 * Un-bind the filter from the form.
			 * 
			 * @param {Ext.data.Model}
			 *            record
			 */
			_onUnbind_ : function(record) {
				this._updateBound_(record);
				this._afterUnbind_(record);
			},

			/**
			 * When the filter has been changed in any way other than user
			 * interaction, update the fields of the form with the changed
			 * values from the model. Such change may happen by custom code
			 * snippets updating the model in a beginEdit-endEdit block,
			 * filter-service methods which returns changed data from server,
			 * etc.
			 * 
			 * @param {Ext.data.Model}
			 *            filter
			 */
			_updateBound_ : function(record) {
				if (!record) {
					this.disable();
					// this.form.reset();
				} else {
					if (this.disabled) {
						this.enable();
					}
					var s = this.getSource();
					for (var p in s) {
						if (record.data.hasOwnProperty(p)) {
							this.setProperty(p, record.data[p], true);
						}
					}
				}
			},

			 
			/**
			 * @private The parameters model is not part of a store, so we have
			 *          listen to changes made to the model through the
			 *          `parameterValueChanged` event raised by the
			 *          data-control. Changes to the parameters model should be
			 *          done through the setParamValue method of the
			 *          data-control in order to be listened and picked-up to
			 *          refresh the correcponding form fields.
			 * 
			 * @param {dnet.core.dc.AbstractDc}
			 *            dc The controller
			 * @param {String}
			 *            property The parameters property which has been
			 *            changed
			 * @param {Object}
			 *            ov Old value
			 * @param {Object}
			 *            nv New value
			 */
			_onParameterValueChanged_ : function(dc, property, ov, nv) {
				var s = this.getSource();

				if (s.hasOwnProperty(property)) {
					this.setProperty(property, nv, false);
				}
			},

			/**
			 * @private Update the bound record when the store data is changed.
			 * @param {}
			 *            store
			 * @param {}
			 *            eopts
			 */
			_onStore_datachanged_ : function(store, eopts) {
				this._updateBound_(this._controller_.getRecord());
			},

			/**
			 * @private Update the bound record when the store data is updated.
			 * @param {}
			 *            store
			 * @param {}
			 *            rec
			 * @param {}
			 *            op
			 * @param {}
			 *            eopts
			 */
			_onStore_update_ : function(store, rec, op, eopts) {
				this._updateBound_(rec);
			},

			/**
			 * @private Update the bound record after a succesful save.
			 * @param {}
			 *            store
			 * @param {}
			 *            operation
			 * @param {}
			 *            eopts
			 */
			_onStore_write_ : function(store, operation, eopts) {
				/**
				 * use the first record from the result list as reference see
				 * Store on write event handler defined in AbstractDc.
				 */
				if (operation.action == "create") {
					this._applyContextRules_(operation.resultSet.records[0]);
				}
			},

			/**
			 * @private When the current record of the data-control is changed
			 *          bind it to the form.
			 * @param {}
			 *            evnt
			 */
			_onController_recordChange_ : function(evnt) {
				var newRecord = evnt.newRecord;
				var oldRecord = evnt.oldRecord;
				var newIdx = evnt.newIdx;
				if (newRecord != oldRecord) {
					this._onUnbind_(oldRecord);
					this._onBind_(newRecord);
				}
			}
		});
Ext.define("dnet.core.dc.AbstractDcvTree", {
	extend:  "Ext.tree.Panel" ,
  
	 _columns_: null
	,_elems_ : null
	,_controller_: null

	,initComponent: function(config) {
        this._elems_ =  new Ext.util.MixedCollection();
        this._columns_ =  new Ext.util.MixedCollection();
		this._startDefine_();
		/* define columns */
        if (this._beforeDefineColumns_()) {
		   this._defineColumns_();
           this._afterDefineColumns_();
		}
		this._defineDefaultElements_();
		this._endDefine_();

		var cfg = {
           rootVisible:true
	      ,lines:false
	      ,autoScroll:true
	      ,useArrows:true
	      ,enableDD: true
	      ,selModel : new  Ext.tree.MultiSelectionModel({})
	      ,root: new Ext.tree.AsyncTreeNode({
	           text:'Hierarchy: Default'
	          ,expanded:false
	        })
	      ,tbar: [{
                   iconCls: 'icon-action-refresh'
                  ,tooltip: this._TEXT_REFRESH
                  ,handler: this.handler_ActionRefresh
                  ,scope: this
              }]
	     ,loader: new Ext.tree.TreeLoader({
          	 preloadChildren: false
          	,nodeParameter: 'parentId'
          	,baseParams:{ hierarchyId : 3116 }
		    ,clearOnLoad: false
		    ,dataUrl: Dnet.dsAPI("OrganizationHierarchyItem","json").read

		    //,directFn: function() {
    	  	//	return dnet.menu.MbMainNavMenu().items;
    	  //	}
		    //,requestMethod: "GET"
   	    })
		}

		Ext.apply(cfg.loader, {
			processResponse : function(response, node, callback, scope){
		        var json = response.responseText;
		        try {
		            var o = response.responseData || Ext.decode(json);
		            o = o.data;
		            node.beginUpdate();
		            for(var i = 0, len = o.length; i < len; i++){
						o[i]["text"] = o[i]["organizationCode"];
	
		                var n = this.createNode(o[i]);
		                if(n){
		                    node.appendChild(n);
		                }
		            }
		            node.endUpdate();
		            this.runCallback(callback, scope || node, [node]);
		        }catch(e){
		            this.handleFailure(response);
		        }
	        }
	        ,requestData : function(node, callback, scope){
		        if(this.fireEvent("beforeload", this, node, callback) !== false){
		            if(this.directFn){
		                var args = this.getParams(node);
		                args.push(this.processDirectResponse.createDelegate(this, [{callback: callback, node: node, scope: scope}], true));
		                this.directFn.apply(window, args);
		            }else{
						var p = this.getParams(node);
						if (!Ext.isNumber (p[this.nodeParameter]) ) {
                            p[this.nodeParameter] = '';
						}
		                this.transId = Ext.Ajax.request({
		                    method:this.requestMethod,
		                    url: this.dataUrl||this.url,
		                    success: this.handleResponse,
		                    failure: this.handleFailure,
		                    scope: this,
		                    argument: {callback: callback, node: node, scope: scope},
		                    params: {data: Ext.encode( p ) }
		                });
		            }
		        }else{
		            // if the load is cancelled, make sure we notify
		            // the node that we are done
		            this.runCallback(callback, scope || node, []);
		        }
		    }
	    });

		cfg.loader.on('load' , this._onLoad_ , this);
		cfg.loader.on('beforeload' , this._beforeLoad_ , this);
		Ext.apply(cfg,config);
        Ext.apply(this,cfg);
		this.callParent(arguments);
 
	}
    , handler_ActionRefresh : function() {  //this.fireEvent("refreshCatalog", this );
	     var node = this.getRootNode() ;
		 while(node.firstChild){
                node.removeChild(node.firstChild);
            }
		 this.getLoader().load( this.getRootNode() );
	}


    ,_onLoad_: function(loader, node, response ) {
        response = response.data;
	}
    ,_beforeLoad_: function(loader, node, callback ) {
       // response = response.data;
	}

    ,_getElement_: function(name) {  return Ext.getCmp( this._elems_.get(name).id); }
    ,_getElementConfig_: function(name) {  return this._elems_.get(name); }

    ,_startDefine_: function () {}
    ,_endDefine_: function () {}

    ,_defineColumns_: function () {}
    	,_beforeDefineColumns_: function () {return true;}
    	,_afterDefineColumns_: function () {}

	,_defineElements_: function () {}
    	,_beforeDefineElements_: function () {return true;}
    	,_afterDefineElements_: function () {}

   	,_defineDefaultElements_: function () {
 	}



	,_onStoreLoad_: function(store,records,options) {
         if(!this._noExport_) {
             if( store.getCount()>0) {
	         	this._getElement_("_btnExport_").enable();
			 } else {
				this._getElement_("_btnExport_").disable();
			 }
		 }

		// this._getElement_("_btnSave_").disable();
	}
	,_afterEdit_: function(e) {
         //this._getElement_("_btnSave_").enable();
	}
    	/* get value from resource bundle for the specified key*/
	,_getRBValue_: function(k) {
		if (this._trl_ != null && this._trl_[k]) { return this._trl_[k]; }
		if (this._controller_._trl_ != null && this._controller_._trl_[k]) {
			return  this._controller_._trl_[k];
		} else {
			return k; 
		}
	}

});Ext.define("dnet.core.dc.AbstractDcvFilterForm", {
	extend : "dnet.core.dc.AbstractDNetDcForm",

	// **************** Properties *****************
	/**
	 * Component builder
	 * 
	 * @type dnet.core.dc.DcvFilterFormBuilder
	 */
	_builder_ : null,

	/**
	 * Helper property to identify this dc-view type as filter-form.
	 * 
	 * @type String
	 */
	_dcViewType_ : "filter-form",

	// **************** Public API *****************

	/**
	 * @public Returns the builder associated with this type of component. Each
	 *         predefined data-control view type has its own builder. If it
	 *         doesn't exist yet attempts to create it.
	 * 
	 * @return {dnet.core.dc.DcvFilterFormBuilder}
	 */
	_getBuilder_ : function() {
		if (this._builder_ == null) {
			this._builder_ = new dnet.core.dc.DcvFilterFormBuilder({
						dcv : this
					});
		}
		return this._builder_;
	},

	// **************** Defaults and overrides *****************

	frame : true,
	border : false,
	bodyBorder : false,
	maskDisabled : false,
	layout : "fit",
	buttonAlign : "left",
	bodyCls : 'dcv-filter-form',
	bodyPadding : '5 5 0 0',

	fieldDefaults : {
		labelAlign : "right",
		labelWidth : 100
	},

	defaults : {
		frame : false,
		border : false,
		bodyBorder : false,
		bodyStyle : " background:transparent "
	},

	initComponent : function(config) {
		this._runElementBuilder_();
		this.callParent(arguments);
		this._registerListeners_();
	},

	/**
	 * After the form is rendered invoke the record binding. This is necessary
	 * as the form may be rendered lazily(delayed) and the data-control may
	 * already have a current record set.
	 */
	afterRender : function() {
		this.callParent(arguments);
		if (this._controller_ && this._controller_.getFilter()) {
			this._onBind_(this._controller_.getFilter());
		} else {
			this._onUnbind_(null);
		}
	},

	beforeDestroy : function() {		
		this._elems_.each(function(item,idx,len) {
			delete item._dcView_;
		}, this)
		this.callParent(arguments);
	},
	
	// **************** Private API *****************

	/**
	 * Register event listeners
	 */
	_registerListeners_ : function() {
		// this.mon(this, "afterrender", this.on_afterrender, this);
		this.mon(this._controller_, "parameterValueChanged",
				this._onParameterValueChanged_, this);
		this.mon(this._controller_, "filterValueChanged",
				this._onFilterValueChanged_, this);

		if (this._controller_.commands.doQuery) {
			this._controller_.commands.doQuery.beforeExecute = Ext.Function
					.createInterceptor(
							this._controller_.commands.doQuery.beforeExecute,
							function() {
								if (this._shouldValidate_()
										&& !this.getForm().isValid()) {
									Ext.Msg.show({
										title : "Validation info",
										msg : "Filter contains invalid data.<br> Please fix the errors then try again.",
										scope : this,
										icon : Ext.MessageBox.ERROR,
										buttons : Ext.MessageBox.OK
									});
									return false;
								} else {
									return true;
								}
							}, this, -1);
		}
	},

	/**
	 * Bind the current filter model of the data-control to the form.
	 * 
	 * @param {Ext.data.Model}
	 *            record The record to bind
	 */
	_onBind_ : function(filter) {
		this._updateBound_(filter);
		this._applyStates_(filter);
		this._afterBind_(filter);
	},

	/**
	 * Un-bind the filter from the form.
	 * 
	 * @param {Ext.data.Model}
	 *            record
	 */
	_onUnbind_ : function(filter) {
		this._updateBound_(filter);
		this._afterUnbind_(filter);
	},

	/**
	 * When the filter has been changed in any way other than user interaction,
	 * update the fields of the form with the changed values from the model.
	 * Such change may happen by custom code snippets updating the model in a
	 * beginEdit-endEdit block, filter-service methods which returns changed
	 * data from server, etc.
	 * 
	 * @param {Ext.data.Model}
	 *            filter
	 */
	_updateBound_ : function(filter) {
		if (!filter) {
			this.disable();
			this.form.reset();
		} else {
			if (this.disabled) {
				this.enable();
			}
			this.form.loadRecord(filter);
		}
	}

 
});Ext.define("dnet.core.dc.AbstractDcvFilterPropGrid", {
			extend : "dnet.core.dc.AbstractDNetDcPropGrid",

			// **************** Properties *****************
			/**
			 * Component builder
			 * 
			 * @type dnet.core.dc.DcvFilterPropGridBuilder
			 */
			_builder_ : null,

			/**
			 * Helper property to identify this dc-view type as filter property
			 * grid.
			 * 
			 * @type String
			 */
			_dcViewType_ : "filter-propgrid",

			// **************** Public API *****************

			/**
			 * @public Returns the builder associated with this type of
			 *         component. Each predefined data-control view type has its
			 *         own builder. If it doesn't exist yet attempts to create
			 *         it.
			 * 
			 * @return {dnet.core.dc.DcvFilterPropGridBuilder}
			 */
			_getBuilder_ : function() {
				if (this._builder_ == null) {
					this._builder_ = new dnet.core.dc.DcvFilterPropGridBuilder(
							{
								dcv : this
							});
				}
				return this._builder_;
			},

			// **************** Defaults and overrides *****************

			initComponent : function(config) {
				this._runElementBuilder_();
				var sourceObj = {};
				var propertyNames = {};
				var customEditors = {};
				var customRenderers = {};

				var fnSourceObj = function(item, idx, len) {
					sourceObj[item.name] = item._default_;
				};
				var fnPropertyName = function(item, idx, len) {
					propertyNames[item.name] = item.fieldLabel;
				};
				var fnCustomEditors = function(item, idx, len) {
					if (item.editorInstance) {
						customEditors[item.name] = item.editorInstance;	
					}
				};
				var fnCustomRenderers = function(item, idx, len) {
					if ( item.renderer ) {
						customRenderers[item.name] = item.renderer;
					}
				};

				this._elems_.each(fnSourceObj, this);
				this._elems_.each(fnPropertyName, this);
				this._elems_.each(fnCustomEditors, this);
				this._elems_.each(fnCustomRenderers, this);

				var cfg = {
					autoScroll : true,
					source : sourceObj,
					propertyNames : propertyNames,
					customEditors : customEditors,
					customRenderers : customRenderers
				};
				Ext.apply(this, cfg);
				this.callParent(arguments);
				this._registerListeners_();
			},

			/**
			 * After the form is rendered invoke the record binding. This is
			 * necessary as the form may be rendered lazily(delayed) and the
			 * data-control may already have a current record set.
			 */
			afterRender : function() {
				this.callParent(arguments);
				if (this._controller_ && this._controller_.getFilter()) {
					this._onBind_(this._controller_.getFilter());
				} else {
					this._onUnbind_(null);
				}
			},

			// **************** Private API *****************

			/**
			 * Register event listeners
			 */
			_registerListeners_ : function() {
				this.mon(this._controller_, "parameterValueChanged",
						this._onParameterValueChanged_, this);
				this.mon(this._controller_, "filterValueChanged",
						this._onFilterValueChanged_, this);
				this.mon(this, "edit" ,function (editor, evnt, eOpts ) {// alert(22);
						if(this._getElementConfig_(evnt.record.data.name).paramIndex != undefined) {
							this._controller_.setParamValue(evnt.record.data.name, evnt.value);
						} else {
							this._controller_.setFilterValue(evnt.record.data.name, evnt.value);
						}
						return true; 
					} , this  ); 
			},

			/**
			 * Bind the current filter model of the data-control to the form.
			 * 
			 * @param {Ext.data.Model}
			 *            record The record to bind
			 */
			_onBind_ : function(filter) {
				this._updateBound_(filter);
				this._applyStates_(filter);
				this._afterBind_(filter);
			},

			/**
			 * Un-bind the filter from the form.
			 * 
			 * @param {Ext.data.Model}
			 *            record
			 */
			_onUnbind_ : function(filter) {
				this._updateBound_(filter);
				this._afterUnbind_(filter);
			},

			/**
			 * When the filter has been changed in any way other than user
			 * interaction, update the fields of the form with the changed
			 * values from the model. Such change may happen by custom code
			 * snippets updating the model in a beginEdit-endEdit block,
			 * filter-service methods which returns changed data from server,
			 * etc.
			 * 
			 * @param {Ext.data.Model}
			 *            filter
			 */
			_updateBound_ : function(filter) {
				if (!filter) {
					this.disable();
				} else {
					if (this.disabled) {
						this.enable();
					}
					var s = this.getSource();
					for (var p in s) {
						if (filter.data.hasOwnProperty(p) === true ) {
							this.setProperty(p, filter.data[p], true);
						}
					}
				}
			},

			/**
			 * The filter model is not part of a store, so we have listen to
			 * changes made to the model through the `filterValueChanged` event
			 * raised by the data-control. So changes to the filter model should
			 * be done through the setFilterValue method of the data-control in
			 * order to be listened and picked-up to refresh the correcponding
			 * filter-form fields.
			 * 
			 * @param {dnet.core.dc.AbstractDc}
			 *            dc The controller
			 * @param {String}
			 *            property The filter property which has been changed
			 * @param {Object}
			 *            ov Old value
			 * @param {Object}
			 *            nv New value
			 */
			_onFilterValueChanged_ : function(dc, property, ov, nv) {
	 
				var s = this.getSource();

				if (s.hasOwnProperty(property)) {
					this.setProperty(property, nv, false);
				}
			},
			/**
			 * @private The parameters model is not part of a store, so we have
			 *          listen to changes made to the model through the
			 *          `parameterValueChanged` event raised by the
			 *          data-control. Changes to the parameters model should be
			 *          done through the setParamValue method of the
			 *          data-control in order to be listened and picked-up to
			 *          refresh the correcponding form fields.
			 * 
			 * @param {dnet.core.dc.AbstractDc}
			 *            dc The controller
			 * @param {String}
			 *            property The parameters property which has been
			 *            changed
			 * @param {Object}
			 *            ov Old value
			 * @param {Object}
			 *            nv New value
			 */
			_onParameterValueChanged_ : function(dc, property, ov, nv) {
				var s = this.getSource();

				if (s.hasOwnProperty(property)) {
					this.setProperty(property, nv, false);
				}
			}
		});/**
 * Builder for filter-form views.
 */
Ext.define("dnet.core.dc.DcvFilterFormBuilder", {
	extend : "Ext.util.Observable",

	dcv : null,

	addTextField : function(config) {
		config.xtype = "textfield";		
		this.applyModelUpdater(config);
		this.applySharedConfig(config);		  
		return this;
	},

	addTextArea : function(config) {
		config.xtype = "textarea";
		this.applyModelUpdater(config);
		this.applySharedConfig(config);
		return this;
	},

	addCheckbox : function(config) {
		config.xtype = "checkbox";
		this.applyModelUpdater(config );
		this.applySharedConfig(config);
		return this;
	},

	addBooleanField : function(config) {
		Ext.applyIf(config, {
			forceSelection : false,
			width : 70
		});

		var yesNoStore = Ext.create('Ext.data.Store', {
			fields : [ "bv", "tv" ],
			data : [{
				"bv" : true,
				"tv" : Dnet.translate("msg", "bool_true")
			}, {
				"bv" : false,
				"tv" : Dnet.translate("msg", "bool_false")
			} ]
		});

		Ext.apply(config, {
			xtype : "combo",
			queryMode : "local",
			valueField : "bv",
			displayField : "tv",
			triggerAction : "all",
			store : yesNoStore
		});
		this.applyModelUpdater(config);
		this.applySharedConfig(config);
		return this;
	},
	addDateField : function(config) {
		config.xtype = "datefield";
		Ext.applyIf(config, {
			format : Dnet.DATE_FORMAT
		});
		this.applyModelUpdater(config);
		this.applySharedConfig(config);
		return this;
	},
	addNumberField : function(config) {
		config.xtype = "numberfield";
		this.applyModelUpdater(config);
		this.applySharedConfig(config);
		return this;
	},
	addLov : function(config) {
		//this.applyModelUpdater(config);
	 
		
		this.applySharedConfig(config);
		return this;
	},
	addCombo : function(config) {
		this.applyModelUpdater(config);
		this.applySharedConfig(config);
		return this;
	},

	addPanel : function(config) {		
		Ext.applyIf(config, this.dcv.defaults);
		if (config.defaults) {
			Ext.applyIf(config.defaults, this.dcv.defaults);
		} else {
			config.defaults = this.dcv.defaults;
		}
		Ext.applyIf(config, {			 
			xtype:"container",
			id: Ext.id()			
		});
		this.dcv._elems_.add(config.name, config);
		return this;
	},

	addFieldContainer : function(config) {
		Ext.applyIf(config, {
			xtype: 'fieldcontainer',
			layout: 'hbox',
			combineErrors: true,
			defaults: {
                    flex: 1,
                    hideLabel: true
                }
		});	
		this.dcv._elems_.add(config.name, config);
		return this;
	},
	
	addChildrenTo : function(c, list) {
		var items = this.dcv._elems_.get(c)["items"] || [];
		for ( var i = 0, len = list.length; i < len; i++) {
			items[items.length] = this.dcv._elems_.get(list[i]);
		}
		this.dcv._elems_.get(c)["items"] = items;
		return this;
	},
	addAuditFilter: function() {
	
	
	},	
	addAuditFilter: function() { 
			return this.createAuditFilter2();		
	},
 
 
	createAuditFilter2: function() {
		this
			.addDateField({ name:"createdAt_From", dataIndex:"createdAt_From" , emptyText:"From" })
			.addDateField({ name:"createdAt_To", dataIndex:"createdAt_To" , emptyText:"To"  })
			.addTextField({ name:"createdBy", dataIndex:"createdBy" ,  emptyText:"Created"  })
		
			.addDateField({ name:"modifiedAt_From", dataIndex:"modifiedAt_From" , emptyText:"From" })
			.addDateField({ name:"modifiedAt_To", dataIndex:"modifiedAt_To" , emptyText:"To"  })
			.addTextField({ name:"modifiedBy", dataIndex:"modifiedBy" ,  emptyText:"Modified"  })
			
			.addTextField({ name:"id", dataIndex:"id" , fieldLabel:"ID", emptyText:"ID"  })
			.addTextField({ name:"uuid", dataIndex:"uuid" , fieldLabel:"UUID",  emptyText:"UUID"  });
		this.add({
                xtype: 'fieldcontainer',
                fieldLabel: 'Created',
                name : 'created',
                combineErrors: true,
                msgTarget : 'side',
                layout: 'hbox',margin:0,padding:0, 
                defaults: {
                	flex: 1,padding:0,margin:0,
                    hideLabel: true
                } 
            })
            .add({
                xtype: 'fieldcontainer',
                fieldLabel: 'Modified',
                name : 'modified',
                combineErrors: true,
                msgTarget : 'side',
                layout: 'hbox',
                defaults: {
                	flex: 1,padding: 0, margin:0,
                    hideLabel: true
                } 
            }).add({
                xtype: 'fieldcontainer',
                fieldLabel: 'By',
                name : 'cre_mod_user',
                combineErrors: true,
                msgTarget : 'side',
                layout: 'hbox',
                defaults: {
                	flex: 1,padding: 0, margin:0,
                    hideLabel: true
                } 
            }).add({
                xtype: 'fieldcontainer',
                fieldLabel: 'ID/UUID',
                name : 'id_uuid',
                combineErrors: true,
                msgTarget : 'side',
                layout: 'hbox',
                defaults: {
                	flex: 1,padding: 0, margin:0,
                    hideLabel: true
                } 
            });	
		this
            .addPanel({ name:"colAudit", xtype:"fieldset",collapsed:true, defaults:{labelWidth:70 }, 
            	padding:'0 10 0 0',  margin:'0 0 0 5',   title:"Audit", border:true, collapsible: true, layout:"form"
            ,width:280})
            .addChildrenTo("colAudit",["created", "modified", "cre_mod_user","id_uuid" ])
            .addChildrenTo("created",["createdAt_From", "createdAt_To" ])
			.addChildrenTo("modified",["modifiedAt_From", "modifiedAt_To" ])
			.addChildrenTo("cre_mod_user",["createdBy", "modifiedBy" ])
			.addChildrenTo("id_uuid",["id", "uuid" ])
			.addChildrenTo("main",["colAudit"])
			return this;
	},
	
	add : function(config) {
		this.applySharedConfig(config);
		return this;
	},

	merge : function(name, config) {
		Ext.applyIf(this.dcv._elems_.get(name), config);
		return this;
	},

	change : function(name, config) {
		Ext.apply(this.dcv._elems_.get(name), config);
		return this;
	},

	remove : function(name) {
		this.dcv._elems_.remove(name);
		return this;
	},

	// private

	
	applyModelUpdater : function(config) {

		var en = "change";		
		var fn = null;
		
		if(config.xtype == "checkbox") {			 
			fn = this.createModelUpdaterCheckbox(config);
		} else {
			fn = this.createModelUpdaterField(config);		
		}
		
		if (!config.listeners) {
			config.listeners = {};
		}
		if (!config.listeners[en]) {
			config.listeners[en] = {};
		}
		config.listeners[en]['buffer'] = 250;
		if(fn!=null) {
			if (config.listeners[en].fn) {
				config.listeners[en].fn = Ext.Function.createInterceptor(config.listeners[en].fn, fn);
			} else {
				config.listeners[en]["fn"] = fn;
			}
		}
	},
	
	createModelUpdaterCheckbox: function(config) {
		var fn = null;
		if (config.paramIndex) {
			fn = function(f, nv, ov, eopts) {				 
				var r = f._dcView_._controller_.getParams();
				if (!r)
					return;
				var rv = !! r.get(f.paramIndex);				 
				if ( rv!==nv ) {
					r.set(f.paramIndex, nv);
				}				
			};
		} else if (config.dataIndex) {
			fn = function(f, nv, ov, eopts) {				 
				var r = f._dcView_._controller_.getFilter();
				if (!r)
					return;
				var rv = !!r.get(f.dataIndex);				 
				if ( rv!==nv ) {
					r.set(f.dataIndex, nv);
				}				 
			}
		}
		return fn;
	},
	createModelUpdaterField: function(config) {
		var fn = null;
		if (config.paramIndex) {
			fn = function(f, nv, ov, eopts) {
				if (!f.isValid()) {
					return;
				}
				var r = f._dcView_._controller_.getParams();
				if (!r)
					return;
				var rv = r.get(f.paramIndex);
				var ctrl = f._dcView_._controller_;
				if (Ext.isDate(rv)) {
					var rd = Ext.Date.parse(Ext.Date.format(rv, f.format),
							f.format);
					if (!r.isEqual(rd, nv)) {
						ctrl.setParamValue(f.paramIndex, nv);
					}
				} else {
					if (!r.isEqual(rv, nv)) {
						ctrl.setParamValue(f.paramIndex, nv);
					}
				}
			};
		} else if (config.dataIndex) {
			fn = function(f, nv, ov, eopts) {
				if (!f.isValid()) {
					return;
				}
				var r = f._dcView_._controller_.getFilter();
				if (!r)
					return;
				var rv = r.get(f.dataIndex);
				var ctrl = f._dcView_._controller_;
				if (Ext.isDate(rv)) {
					var rd = Ext.Date.parse(Ext.Date.format(rv, f.format),
							f.format);
					if (!r.isEqual(rd, nv)) {
						ctrl.setFilterValue(f.dataIndex, nv);
					}
				} else {
					if (!r.isEqual(rv, nv)) {
						ctrl.setFilterValue(f.dataIndex, nv);
					}
				}
			}
		}
		return fn;
	},
	
	 
	// ==============================================
	// ==============================================
	 
	applySharedConfig : function(config) {
		Ext.applyIf(config, {
			id : Ext.id(),
			selectOnFocus : true,
			_dcView_ : this.dcv
		});
		if (config.maxLength) {
			config.enforceMaxLength = true;
		}
		if (config.caseRestriction ) {
			config.fieldStyle = "text-transform:"+config.caseRestriction+";";
		}
		if (config.allowBlank === false) {
			config.labelSeparator = "*";
		}
		if (config._sharedLabel_) {
			config._rbkey_ = config.name;
		}
		this.dcv._elems_.add(config.name, config);
	}
});
/**
 * Builder for filter property grid views.
 */
Ext.define("dnet.core.dc.DcvFilterPropGridBuilder", {
	extend : "Ext.util.Observable",
	
	dcv : null,

	addTextField : function(config) {
		if(!config.editor) {
			config.editor = {};
		}		 
		var e = config.editor;
		 
		if (e.maxLength) {
			e.enforceMaxLength = true;
		}
		if (e.caseRestriction ) {
			e.fieldStyle += "text-transform:"+e.caseRestriction+";";
		}	
		config.editorInstance = Ext.create('Ext.form.field.Text', e);
	 
		config._default_ = "";
		this.applySharedConfig(config);		  
		return this;
	},

	addDateField : function(config) {
		if(!config.editor) {
			config.editor = {};
		}		 
		var e = config.editor;
		//config.xtype = "datefield";
		config.editorInstance = Ext.create('Ext.form.field.Date', Ext.applyIf(e, {
			format : Dnet.DATE_FORMAT,
			selectOnFocus: true
		}));
		config._default_ = "";
		config.renderer = config.renderer || Ext.util.Format.dateRenderer(Dnet.DATE_FORMAT);
		this.applySharedConfig(config);
		return this;
	},
	
	
	addNumberField : function(config) {
		if(!config.editor) {
			config.editor = {};
		}		 
		var e = config.editor;
		config.editorInstance = Ext.create('Ext.form.field.Number',  Ext.applyIf(e, {
			fieldStyle: "text-align:right;" ,
			selectOnFocus: true
		}));
		this.applySharedConfig(config);
		return this;
	},
	
	
	addLov : function(config) {  
		 
		try {
			config.editorInstance = Ext.create(config.editor._fqn_,  Ext.apply(config.editor , {
				 selectOnFocus: true
			}));
			config._default_ = "";
		}catch(e) {
			alert('Cannot create LOV editor from xtype `'+config.editor._fqn_+'`');
		}
		this.applySharedConfig(config);
		return this;
	},
	
	addCombo : function(config) {
		if(!config.editor) {
			config.editor = {};
		}		 
		var e = config.editor;
		 
		if (e.maxLength) {
			e.enforceMaxLength = true;
		}
		if (e.caseRestriction ) {
			e.fieldStyle += "text-transform:"+e.caseRestriction+";";
		}	
		config.editorInstance = Ext.create('Ext.form.field.ComboBox', e);
	 
		config._default_ = "";
		this.applySharedConfig(config);		  
		return this;
	},
	
	addBooleanField : function(config) {
		
		config.editor = Ext.applyIf(config.editor || {}, {
			forceSelection : false
		} );  
		Ext.applyIf(config, { 
			_default_ : null
		});
		var yesNoStore = Dnet.createBooleanStore();
		config.editorInstance = Ext.create('Ext.form.field.ComboBox', Ext.apply(config.editor||{},  {
			queryMode : "local",
			valueField : "bv",
			displayField : "tv",
			triggerAction : "all",
			store : yesNoStore
		}  ) );
		
		config.renderer = function(v) {  
			if (v==null) {
				return "";
			}
			return Dnet.translate("msg", "bool_"+ ((!!v)));
		}
		this.applySharedConfig(config);
		return this;  
	},
	
	
	// ==============================================
	
	applySharedConfig : function(config) {
		Ext.applyIf(config, {
			id : Ext.id()
		});
		config.editorInstance._dcView_ = this.dcv;
		if (config.allowBlank === false) {
			config.labelSeparator = "*";
		}
		if (config._sharedLabel_) {
			config._rbkey_ = config.name;
		}
		this.dcv._elems_.add(config.name, config);
	}
	
});/**
 * Builder for edit-form views.
 */

 
Ext.define("dnet.core.dc.DcvEditFormBuilder", {
	extend : "Ext.util.Observable",

	dcv : null,

	addTextField : function(config) {
		config.xtype = "textfield";
		this.applyModelUpdater(config);
		this.applySharedConfig(config);		  
		return this;
	},

	addTextArea : function(config) {
		config.xtype = "textarea";
		this.applyModelUpdater(config);
		this.applySharedConfig(config);
		return this;
	},

	addBooleanField : function(config) {
		return this.addCheckbox(config);
	},
	
	addCheckbox : function(config) {
		config.xtype = "checkbox";
		this.applyModelUpdater(config);
		this.applySharedConfig(config);
		return this;
	},

	addDateField : function(config) {
		config.xtype = "datefield";
		Ext.applyIf(config, {
			format : Dnet.DATE_FORMAT,
			checkChangeBuffer : 600
		});
		this.applyModelUpdater(config);
		this.applySharedConfig(config);
		return this;
	},

	addDateTimeField : function(config) {
		config.xtype = "datefield";
		Ext.applyIf(config, {
			format : Dnet.DATETIME_FORMAT
		});
		this.applyModelUpdater(config);
		this.applySharedConfig(config);
		return this;
	},

	addNumberField : function(config) {
		config.xtype = "numberfield";
		config.fieldStyle = (config.fieldStyle)? config.fieldStyle + ";text-align:right;": "text-align:right;";
			
		this.applyModelUpdater(config);
		this.applySharedConfig(config);
		return this;
	},

	
	addLov : function(config) {		
		Ext.applyIf(config, {			
			_validateListValue_ : true
		});		 		
		this.applySharedConfig(config);
		return this;
	},

	addCombo : function(config) {
		Ext.applyIf(config, {			
			xtype:"localcombo"
		});			 
		this.applyModelUpdater(config);
		this.applySharedConfig(config);
		return this;
	},
	
	
	addHiddenField : function(config) {
		config.xtype = "hiddenfield";
		Ext.applyIf(config, {
			maxLength:20		 
		});
		this.applySharedConfig(config);
		return this;
	},
	
	
	addDisplayFieldText : function(config) {
		config.xtype = "displayfieldtext";
		Ext.applyIf(config, {
			anchor:"-20" ,
			fieldCls: "displayfield"
			 
		});
		this.applySharedConfig(config);
		return this;
	},
	
	addDisplayFieldNumber : function(config) {
		config.xtype = "displayfieldnumber";		
		Ext.applyIf(config, {
			anchor:"-20" ,
			format: Dnet.getNumberFormat(config.decimals || 0 ),
			fieldCls: "displayfieldnumber"
		});
		this.applySharedConfig(config);
		return this;
	},
	addDisplayFieldDate : function(config) {
		config.xtype = "displayfielddate";
		Ext.applyIf(config, {
			anchor:"-20" ,
			fieldCls: "displayfield"
		});
		this.applySharedConfig(config);
		return this;
	},
	
	addDisplayFieldBoolean : function(config) {
		config.xtype = "displayfieldboolean";
		Ext.applyIf(config, {
			//anchor:"-20" ,
			fieldCls: "displayfield"
		});
		this.applySharedConfig(config);
		return this;
	},
	
	addPanel : function(config) {
		Ext.applyIf(config, this.dcv.defaults);
		Ext.applyIf(config, {
			defaults : this.dcv.defaults,
			xtype:"container",
			id: Ext.id()			
		});
		this.dcv._elems_.add(config.name, config);
		return this;
	},

	addChildrenTo : function(c, list) {
		var items = this.dcv._elems_.get(c)["items"] || [];
		for ( var i = 0, len = list.length; i < len; i++) {
			items[items.length] = this.dcv._elems_.get(list[i]);
		}
		this.dcv._elems_.get(c)["items"] = items;
		return this;
	},

	add : function(config) {
		this.applySharedConfig(config);
		return this;
	},

	merge : function(name, config) {
		Ext.applyIf(this.dcv._elems_.get(name), config);
		return this;
	},

	change : function(name, config) {
		Ext.apply(this.dcv._elems_.get(name), config);
		return this;
	},

	remove : function(name) {
		this.dcv._elems_.remove(name);
		return this;
	},

	// private

	applyModelUpdater : function(config) {

		var en = "change";		
		var fn = null;
		
		if(config.xtype == "checkbox") {			 
			fn = this.createModelUpdaterCheckbox(config);
		} else {
			fn = this.createModelUpdaterField(config);		
		}
		
		if (!config.listeners) {
			config.listeners = {};
		}
		if (!config.listeners[en]) {
			config.listeners[en] = {};
		}
		config.listeners[en]['buffer'] = 500;
		if(fn!=null) {
			if (config.listeners[en].fn) {
				config.listeners[en].fn = Ext.Function.createInterceptor(config.listeners[en].fn, fn);
			} else {
				config.listeners[en]["fn"] = fn;
			}
		}
	},

	createModelUpdaterCheckbox: function(config) {
		var fn = null;
		if (config.paramIndex) {
			fn = function(f, nv, ov, eopts) {				 
				var r = f._dcView_._controller_.getParams();
				if (!r)
					return;
				var rv = !! r.get(f.paramIndex);				 
				if ( rv!==nv ) {
					r.set(f.paramIndex, nv);
				}				
			};
		} else if (config.dataIndex) {
			fn = function(f, nv, ov, eopts) {				 
				var r = f._dcView_._controller_.getRecord();
				if (!r)
					return;
				var rv = !!r.get(f.dataIndex);				 
				if ( rv!==nv ) {
					r.set(f.dataIndex, nv);
				}				 
			}
		}
		return fn;
	},
	
	
	// move these functions out, onChange call them buffered , 
	// on blur call immediatly and cancel the buffered event 
	// -> fix for very fast typing
	  
	createModelUpdaterField: function(config) {
		var fn = null;
		if (config.paramIndex) {
			fn = function(f, nv, ov, eopts) {
				if (!f.isValid()) {
					return;
				}
				var r = f._dcView_._controller_.getParams();
				if (!r)
					return;
				var rv = r.get(f.paramIndex);
				if (Ext.isDate(rv)) {
					var rd = Ext.Date.parse(Ext.Date.format(rv, f.format),
							f.format);
					if (!r.isEqual(rd, nv)) {
						r.set(f.paramIndex, nv);
					}
				} else {
					if (!r.isEqual(rv, nv)) {
						r.set(f.paramIndex, nv);
					}
				}
			};
		} else if (config.dataIndex) {
			fn = function(f, nv, ov, eopts) {
				if (!f.isValid()) {
					return;
				}
				var r = f._dcView_._controller_.getRecord();
				if (!r)
					return;
				var rv = r.get(f.dataIndex);
				if (Ext.isDate(rv)) {
					var rd = Ext.Date.parse(Ext.Date.format(rv, f.format),
							f.format);
					if (!r.isEqual(rd, nv)) {
						r.set(f.dataIndex, nv);
					}
				} else {
					if (!r.isEqual(rv, nv)) {
						//r.beginEdit();
						r.set(f.dataIndex, nv);
						//r.endEdit();
					}
				}
			}
		}
		return fn;
	},
	
	
	
	applySharedConfig : function(config) {
		Ext.applyIf(config, {
			id : Ext.id(),
			itemId : config.name,
			selectOnFocus : true,
			//checkChangeBuffer : 700,
			_dcView_ : this.dcv
		});
		if (config.maxLength) {
			config.enforceMaxLength = true;
		}
		if (config.caseRestriction ) {
			config.fieldStyle = "text-transform:"+config.caseRestriction+";";
		}
		if (config.allowBlank === false) {
			config.labelSeparator = "*";
		}
		if (config._sharedLabel_) {
			config._rbkey_ = config.name;
		}
		this.dcv._elems_.add(config.name, config);
	}
});
/**
 * Builder for edit property grid views.
 */
Ext.define("dnet.core.dc.DcvEditPropGridBuilder", {
	extend : "Ext.util.Observable",
	
	dcv : null,

	addTextField : function(config) {
		if(!config.editor) {
			config.editor = {};
		}		 
		var e = config.editor;
		 
		if (e.maxLength) {
			e.enforceMaxLength = true;
		}
		if (e.caseRestriction ) {
			e.fieldStyle += "text-transform:"+e.caseRestriction+";";
		}	
		config.editorInstance = Ext.create('Ext.form.field.Text', e);
		config._default_ = "";
		this.applySharedConfig(config);		  
		return this;
	},

	addDateField : function(config) {
		config.xtype = "datefield";
		config.editorInstance = Ext.create('Ext.form.field.Date', Ext.apply(config.editor||{}, {
			format : Dnet.DATE_FORMAT 
		}));
		config.renderer = config.renderer || Ext.util.Format.dateRenderer(Dnet.DATE_FORMAT);
		this.applySharedConfig(config);
		return this;
	},
	
	
	addNumberField : function(config) {
		config.xtype = "numberfield";
		config.editorInstance = Ext.create('Ext.form.field.Number',  Ext.apply(config.editor||{}, {
			fieldStyle: "text-align:right;" 
		}));
		this.applySharedConfig(config);
		return this;
	},
	
	addLov : function(config) {	  
		if (config.editor && config.editor.xtype != "textfield") {
			config.editorInstance = Ext.create(config.editor._fqn_,  Ext.apply(config.editor , {
				selectOnFocus: true
			}));
		}
		this.applySharedConfig(config);
		return this;
	},
	
	addBooleanField : function(config) {
		config.editor = Ext.applyIf(config.editor || {}, {
			forceSelection : true
		} );  
		Ext.applyIf(config, { 
			_default_ : null
		});
		var yesNoStore = Dnet.createBooleanStore();
		config.editorInstance = Ext.create('Ext.form.field.ComboBox', Ext.apply(config.editor||{},  {
			queryMode : "local",
			valueField : "bv",
			displayField : "tv",
			triggerAction : "all",
			store : yesNoStore
		}  ) );
		
		config.renderer = function(v) {  
			if (v==null) {
				return "";
			}
			return Dnet.translate("msg", "bool_"+ ((!!v)));
		}
		this.applySharedConfig(config);
		return this;
	},
	
	
	// ==============================================
	
	applySharedConfig : function(config) {
		Ext.applyIf(config, {
			id : Ext.id()
		});
		config.editorInstance._dcView_ = this.dcv;
		if (config.allowBlank === false) {
			config.labelSeparator = "*";
		}
		if (config._sharedLabel_) {
			config._rbkey_ = config.name;
		}
		this.dcv._elems_.add(config.name, config);
	}
	
});/**
 * Builder for read-only grid views.
 */
Ext.define("dnet.core.dc.DcvGridBuilder", {
	extend : "Ext.util.Observable",

	/**
	 * 
	 * @type dnet.core.dc.AbstractDcvGrid 
	 */
	dcv : null,

	addTextColumn : function(config) {
		config.xtype = "gridcolumn";
		config.field = {
                xtype: 'textfield',
                readOnly: true
            };
		this.applySharedConfig(config);
		return this;
	},
	
	addBooleanColumn : function(config) {
		config.xtype = "booleancolumn";
		Ext.apply(config, {
			trueText : Dnet.translate("msg", "bool_true"),
			falseText : Dnet.translate("msg", "bool_false")
		});
		this.applySharedConfig(config);
		return this;
	},
	
	addDateColumn : function(config) {
		config.xtype = "datecolumn";
		Ext.applyIf(config, {
			format : Dnet.DATE_FORMAT
		});
		config.field = {
                xtype: 'datefield',
                hideTrigger:true,
                format : config.format,
                readOnly: true
            };
		
		this.applySharedConfig(config);
		return this;
	},
 
	addNumberColumn : function(config) {
		config.xtype = "numbercolumn";
		config.field = {
                xtype: 'textfield',
                readOnly: true
            };
		config.format = Dnet.getNumberFormat(config.decimals || 0 );
		Ext.applyIf(config, {
			align : "right"
		});
		this.applySharedConfig(config);
		return this;
	},
	
	addDefaults: function() {
		var r =  Ext.create(this.dcv._controller_.recordModel, {});
		r.fields.each(function(f,idx,len) {
			if(f.name=="id") {
				this.addNumberColumn({ name:"id", dataIndex:"id", hidden:true,format:"0",width:70 }) ;
			}
			if(f.name=="uuid") {
				this.addTextColumn({ name:"uuid", dataIndex:"uuid", hidden:true,width:100 })   	
			}
			if(f.name=="createdAt") {
				this.addDateColumn({ name:"createdAt", dataIndex:"createdAt", hidden:true,format:Dnet.DATETIMESEC_FORMAT})   
			}
			if(f.name=="modifiedAt") {
				this.addDateColumn({ name:"modifiedAt", dataIndex:"modifiedAt", hidden:true,format:Dnet.DATETIMESEC_FORMAT}) 
			}
			if(f.name=="createdBy") {
				this.addTextColumn({ name:"createdBy", dataIndex:"createdBy", hidden:true,width:100 })   	
			}
			if(f.name=="modifiedBy") {
				this.addTextColumn({ name:"modifiedBy", dataIndex:"modifiedBy", hidden:true,width:100 })  
			}
			
		}, this);
		return this;
	},
	
	
	
	add : function(config) {
		this.applySharedConfig(config);
		return this;
	},
	
	merge : function(name, config) {
		Ext.applyIf(this.dcv._columns_.get(name), config);
		return this;
	},
	
	change : function(name, config) {
		Ext.apply(this.dcv._columns_.get(name), config);
		return this;
	},
	
	remove : function(name) {
		this.dcv._columns_.remove(name);
		return this;
	},
	
	// private

	applySharedConfig : function(config) {
		Ext.applyIf(config, {
			id : Ext.id(),
			sortable : true,
			hidden : false
		});
		if (config._sharedLabel_) {
			config._rbkey_ = config.name;
		}
		this.dcv._columns_.add(config.name, config);
	},
	
	addAllFromDataSource : function() {

		var f = this.dcv._controller_.ds.recordFields;
		for ( var i = 0, len = f.length; i < len; i++) {
			var name = f[i]["name"];
			var type = f[i]["type"];
			var cfg = {
				name : name,
				dataIndex : name
			};

			// try to guess something
			if (name == "id" || name == "createdAt" || name == "createdBy"
					|| name == "version" || name == "clientId") {
				cfg.hidden = true;
			}

			if (type == "string") {
				this.addTextColumn(cfg);
			}
			if (type == "date") {
				this.addDateColumn(cfg);
			}
			if (type == "boolean") {
				this.addBooleanColumn(cfg);
			}
			if (type == "int") {
				this.addNumberColumn(cfg);
			}

		}
	}
});
/**
 * Builder for edit-grid views.
 */
Ext.define("dnet.core.dc.DcvEditableGridBuilder", {
	extend : "Ext.util.Observable",

	dcv : null,

	addTextColumn : function(config) {
		config.xtype = "gridcolumn";
		if (config.editor && config.editor.maxLength ) {
			config.editor.enforceMaxLength = true;
		}
		this.applySharedConfig(config);
		return this;
	},

	addBooleanColumn : function(config) {

		config.xtype = "booleancolumn";
		Ext.apply(config, {
			trueText : Dnet.translate("msg", "bool_true"),
			falseText : Dnet.translate("msg", "bool_false"),
			autoSelect: true
		});
		if (config.editor == undefined && !config._noEdit_ === true) {

			var yesNoStore = Ext.create('Ext.data.Store', {
				fields : [ "bv", "tv" ],
				data : [ {
					"bv" : true,
					"tv" : Dnet.translate("msg", "bool_true")
				}, {
					"bv" : false,
					"tv" : Dnet.translate("msg", "bool_false")
				} ]
			});

			config.editor = {
				xtype : 'combo',
				queryMode : 'local',
				selectOnFocus : true,
				valueField : 'bv',
				displayField : 'tv',
				store : yesNoStore,
				triggerAction : 'all',
				forceSelection : true
			};
		}

		this.applySharedConfig(config);
		return this;
	},

	addDateColumn : function(config) {
		config.xtype = "datecolumn";
		Ext.applyIf(config, {
			format : Dnet.DATE_FORMAT
		});
		this.applySharedConfig(config);
		return this;
	},

	
	addNumberColumn : function(config) {
		config.xtype = "numbercolumn";
		config.format = Dnet.getNumberFormat(config.decimals || 0 );
		Ext.applyIf(config, {
			align : "right"
		});
		this.applySharedConfig(config);
		return this;
	},

	addComboColumn : function(config) {
		config.xtype = "gridcolumn";
		this.applySharedConfig(config);
		return this;
	},

	addLov : function(config) {
		this.applySharedConfig(config);
		return this;
	},

	addDefaults: function() {
		var r =  Ext.create(this.dcv._controller_.recordModel, {});
		r.fields.each(function(f,idx,len) {
			if(f.name=="id") {
				this.addNumberColumn({ name:"id", dataIndex:"id", hidden:true,format:"0",width:70 }) ;
			}
			if(f.name=="uuid") {
				this.addTextColumn({ name:"uuid", dataIndex:"uuid", hidden:true,width:100 })   	
			}
			if(f.name=="createdAt") {
				this.addDateColumn({ name:"createdAt", dataIndex:"createdAt", hidden:true,format:Dnet.DATETIME_FORMAT})   
			}
			if(f.name=="modifiedAt") {
				this.addDateColumn({ name:"modifiedAt", dataIndex:"modifiedAt", hidden:true,format:Dnet.DATETIME_FORMAT}) 
			}
			if(f.name=="createdBy") {
				this.addTextColumn({ name:"createdBy", dataIndex:"createdBy", hidden:true,width:100 })   	
			}
			if(f.name=="modifiedBy") {
				this.addTextColumn({ name:"modifiedBy", dataIndex:"modifiedBy", hidden:true,width:100 })  
			}
			
		}, this);
		return this;
	},
	
	
	add : function(config) {
		this.applySharedConfig(config);
		return this;
	},

	merge : function(name, config) {
		Ext.applyIf(this.dcv._columns_.get(name), config);
		return this;
	},

	change : function(name, config) {
		Ext.apply(this.dcv._columns_.get(name), config);
		return this;
	},

	remove : function(name) {
		this.dcv._columns_.remove(name);
		return this;
	},

	// private

	applySharedConfig : function(config) {
		Ext.applyIf(config, {
			id : Ext.id(),
			headerId: config.name,
			selectOnFocus : true,
			sortable : true,
			//hidden : false,
			_dcView_ : this.dcv
		});
		if (config.editor) {
			Ext.applyIf(config.editor, {
				_dcView_ : this.dcv
			});
		}
		if (config._sharedLabel_) {
			config._rbkey_ = config.name;
		}
		this.dcv._columns_.add(config.name, config);
	}
});
/**
 * Factory class used to create the Ext.Action instances for the registered
 * action names. To create other actions register them here, create a command
 * executor and implement the state management rules in DcActionStateManager.
 */
dnet.core.dc.DcActionsFactory = {

	RUN_QUERY: "Query",
	CLEAR_QUERY: "ClearQuery",
	CREATE: "New",
	COPY: "Copy",
	SAVE: "Save",
	DELETE: "Delete",
	CANCEL: "Cancel",
	EDIT_IN: "EditIn",
	EDIT_OUT: "EditOut",
	PREV_REC: "PrevRec",
	NEXT_REC: "NextRec",
	RELOAD_REC: "ReloadRec",
	
	
	/**
	 * List with the names of the registered actions. See the appropriate create
	 * method for the meaning of each of the actions.
	 */
	actionNames : function() {
		return [ this.RUN_QUERY, this.CLEAR_QUERY, 
				this.CREATE, this.COPY, this.SAVE, this.DELETE, this.CANCEL, 
				this.EDIT_IN, this.EDIT_OUT, this.PREV_REC, this.NEXT_REC, this.RELOAD_REC ];
	},

	/**
	 * Create an object of actions for the given data-control and list of action
	 * names.
	 */
	createActions : function(dc, names) {
		var result = {};
		for ( var i = 0, l = names.length; i < l; i++) {
			var n = names[i];
			result["do" + n] = this["create" + n + "Action"](dc);
		}
		return result;
	},

	/**
	 * Create the action to execute a query.
	 */
	createQueryAction : function(dc) {
		return new Ext.Action( {
			name : "doQuery",
			iconCls : "icon-action-fetch",
			disabled : false,
			text : Dnet.translate("tlbitem", "load__lbl"),
			tooltip : Dnet.translate("tlbitem", "load__tlp"),
			scope : dc,
			handler : function() {
				try {
					dc.doQuery();
				} catch (e) {
					dnet.core.dc.DcExceptions.showMessage(e);
				}
			}
		});
	},

	/**
	 * Create the action to clear the filter values.
	 */
	createClearQueryAction : function(dc) {
		return new Ext.Action( {
			name : "doClearQuery",
			iconCls : "icon-action-fetch",
			disabled : false,
			text : Dnet.translate("tlbitem", "clearquery__lbl"),
			tooltip : Dnet.translate("tlbitem", "clearquery__tlp"),
			scope : dc,
			handler : function() {
				try {
					dc.doClearQuery();
				} catch (e) {
					dnet.core.dc.DcExceptions.showMessage(e);
				}
			}
		});
	},

	/**
	 * Create the action to create a new record.
	 */
	createNewAction : function(dc) {
		return new Ext.Action( {
			name : "doNew",
			iconCls : "icon-action-new",
			disabled : false,
			text : Dnet.translate("tlbitem", "new__lbl"),
			tooltip : Dnet.translate("tlbitem", "new__tlp"),
			scope : dc,
			handler : dc.doNew
		});
	},

	/**
	 * Create the action to copy a record in client.
	 */
	createCopyAction : function(dc) {
		return new Ext.Action( {
			name : "doCopy",
			iconCls : "icon-action-copy",
			disabled : true,
			text : Dnet.translate("tlbitem", "copy__lbl"),
			tooltip : Dnet.translate("tlbitem", "copy__tlp"),
			scope : dc,
			handler : dc.doCopy
		});
	},

	/**
	 * Create the action to save changes.
	 */
	createSaveAction : function(dc) {
		return new Ext.Action( {
			name : "doSave",
			iconCls : "icon-action-save",
			disabled : true,
			text : Dnet.translate("tlbitem", "save__lbl"),
			tooltip : Dnet.translate("tlbitem", "save__tlp"),
			scope : dc,
			handler : dc.doSave
		});
	},

	/**
	 * Create the action to delete records.
	 */
	createDeleteAction : function(dc) {
		return new Ext.Action( {
			name : "deleteSelected",
			iconCls : "icon-action-delete",
			disabled : true,
			text : Dnet.translate("tlbitem", "delete_selected__lbl"),
			tooltip : Dnet.translate("tlbitem", "delete_selected__tlp"),
			scope : dc,
			handler : function() {
				dc.doDelete();
			}
		});
	},

	/**
	 * Create the action to cancel changes made to the data-model.
	 */
	createCancelAction : function(dc) {
		return new Ext.Action( {
			name : "doCancel",
			iconCls : "icon-action-rollback",
			disabled : true,
			text : Dnet.translate("tlbitem", "cancel__lbl"),
			tooltip : Dnet.translate("tlbitem", "cancel__tlp"),
			scope : dc,
			handler : function() {
				dc.doCancel();
			}
		});
	},

	/**
	 * Create the action to enter edit-mode.
	 */
	createEditInAction : function(dc) {
		return new Ext.Action( {
			name : "doEdit",
			iconCls : "icon-action-edit",
			disabled : true,
			text : Dnet.translate("tlbitem", "edit__lbl"),
			tooltip : Dnet.translate("tlbitem", "edit__tlp"),
			scope : dc,
			handler : function() {
			}
		});
	},

	/**
	 * Create the action to exit edit-mode.
	 */
	createEditOutAction : function(dc) {
		return new Ext.Action( {
			name : "doLeaveEditor",
			iconCls : "icon-action-back",
			disabled : false,
			text : Dnet.translate("tlbitem", "back__lbl"),
			tooltip : Dnet.translate("tlbitem", "back__tlp"),
			scope : dc,
			handler : function() {
			}
		});
	},

	/**
	 * Create the action to load the previous available record in store as
	 * current record.
	 */
	createPrevRecAction : function(dc) {
		return new Ext.Action( {
			name : "doPrevRec",
			iconCls : "icon-action-previous",
			disabled : false,
			//text : Dnet.translate("tlbitem", "prev_rec__lbl"),
			tooltip : Dnet.translate("tlbitem", "prev_rec__tlp"),
			scope : dc,
			handler : function() {
				try {
					dc.setPreviousAsCurrent();
				} catch (e) {
					dnet.core.dc.DcExceptions.showMessage(e);
				}
			}
		});
	},

	/**
	 * Create the action to load the next available record in store as current
	 * record.
	 */
	createNextRecAction : function(dc) {
		return new Ext.Action( {
			name : "doNextRec",
			iconCls : "icon-action-next",
			disabled : false,
			//text : Dnet.translate("tlbitem", "next_rec__lbl"),
			tooltip : Dnet.translate("tlbitem", "next_rec__tlp"),
			scope : dc,
			handler : function() {
				try {
					dc.setNextAsCurrent();
				} catch (e) {
					dnet.core.dc.DcExceptions.showMessage(e);
				}
			}
		});
	},

	/**
	 * Create the action to reload the current record from server.
	 */
	createReloadRecAction : function(dc) {
		return new Ext.Action( {
			name : "doReloadRec",
			iconCls : "icon-action-refresh",
			disabled : false,
			text : Dnet.translate("tlbitem", "reload_rec__lbl"),
			tooltip : Dnet.translate("tlbitem", "reload_rec__tlp"),
			scope : dc,
			handler : function() {
				try {
					dc.reloadCurrentRecord();
				} catch (e) {
					dnet.core.dc.DcExceptions.showMessage(e);
				}
			}
		});
	}
};dnet.core.dc.DcCommandFactory = {

	/**
	 * List of command names which are not registered as actions either as they
	 * are internal DC commands or not managed as actions.
	 */
	commandNames : function() {
		return [ "RpcData", "RpcFilter" ];
	},

	/**
	 * Create an object of actions for the given data-control and list of action
	 * names.
	 */
	createCommands : function(dc, names) {

		var result = {};
		for ( var i = 0, l = names.length; i < l; i++) {
			var n = names[i];
			result["do" + n] = this["create" + n + "Command"](dc);
		}
		return result;
	},

	createQueryCommand : function(dc) {
		return new dnet.core.dc.command.DcQueryCommand( {
			dc : dc
		});
	},

	createClearQueryCommand : function(dc) {
		return new dnet.core.dc.command.DcClearQueryCommand( {
			dc : dc
		});
	},

	createNewCommand : function(dc) {
		return new dnet.core.dc.command.DcNewCommand( {
			dc : dc
		});
	},

	createCopyCommand : function(dc) {
		return new dnet.core.dc.command.DcCopyCommand( {
			dc : dc
		});
	},

	createSaveCommand : function(dc) {
		return new dnet.core.dc.command.DcSaveCommand( {
			dc : dc
		});
	},

	createCancelCommand : function(dc) {
		return new dnet.core.dc.command.DcCancelCommand( {
			dc : dc
		});
	},

	createDeleteCommand : function(dc) {
		return new dnet.core.dc.command.DcDeleteCommand( {
			dc : dc
		});
	}

	,
	createRpcDataCommand : function(dc) {
		return new dnet.core.dc.command.DcRpcDataCommand( {
			dc : dc
		});
	},

	createRpcFilterCommand : function(dc) {
		return new dnet.core.dc.command.DcRpcFilterCommand( {
			dc : dc
		});
	},

	createEditInCommand : function(dc) {
		return new dnet.core.dc.command.DcEditInCommand( {
			dc : dc
		});
	},

	createEditOutCommand : function(dc) {
		return new dnet.core.dc.command.DcEditOutCommand( {
			dc : dc
		});
	},

	createPrevRecCommand : function(dc) {
		return new dnet.core.dc.command.DcPrevRecCommand( {
			dc : dc
		});
	},

	createNextRecCommand : function(dc) {
		return new dnet.core.dc.command.DcNextRecCommand( {
			dc : dc
		});
	},

	createReloadRecCommand : function(dc) {
		return new dnet.core.dc.command.DcReloadRecCommand( {
			dc : dc
		});
	}
};
/**
 * Abstract base class for any kind of command. Do not directly subclass this
 * one but use one of the provided children depending on what type of command
 * you need. See the synchronous and asynchronous commands for details.
 */
Ext.define("dnet.core.dc.command.AbstractDcCommand", {
 
 	/**
 	 * Data-control on which this command is invoked.
 	 * @type dnet.core.dc.AbstractDc
 	 */
	dc : null,

	/**
	 * DC API method which delegates to this command
	 * @type String 
	 */
	dcApiMethod: null,
	
	/**
	 * Flag to set if this command needs explicit confirmation from the user in
	 * order to execute.
	 */
	confirmByUser : false,

	locked: false,
	
	/**
	 * User confirmation message title.
	 */
	confirmMessageTitle : "",

	/**
	 * User confirmation message body.
	 */
	confirmMessageBody : "Please confirm action",

	/**
	 * Constructor
	 */
	constructor : function(config) {
		config = config || {};
		Ext.apply(this, config);
		this.callParent(arguments);
	},

	/**
	 * Template method where subclasses or external contexts can provide
	 * additional logic. If it returns false the execution is stopped.
	 */
	beforeExecute : function() {
		return true;
	},

	/**
	 * Template method where subclasses or external contexts can provide
	 * additional logic. Called after the execution is finished. ATTENTION:
	 * Commands which initiate AJAX calls do not have the result of the remote
	 * call available here. Provide callbacks for such situations.
	 */
	afterExecute : function() {
		if(this.dcApiMethod != null) {
			var m = this.dc['afterDo'+this.dcApiMethod];
			if(m!= undefined && Ext.isFunction(m)) {
				m.call(this.dc);
			}			 
		}
	},

	/**
	 * Provide whatever extra-logic to check if the command can be executed.
	 */
	canExecute : function() {
		return true;
	},

	/**
	 * Default routine which ask confirmation from the user to proceed. Called
	 * if confirmByUser = true.
	 */
	confirmExecute : function(btn, options) {
		if (!btn) {
			Ext.Msg.confirm(this.confirmMessageTitle, this.confirmMessageBody,
					function(btn) {
						this.confirmExecute(btn, options);
					}, this);
		} else {
			if (btn == "yes" || btn == "ok") {
				options.confirmed = true;
				this.execute(options);
			}
		}
	},

	needsConfirm : function() {
		return this.confirmByUser;
	},

	/**
	 * Calls the appropriate method from the action-state manager.
	 */
	checkActionState : function(options) {
	},

	/**
	 * Command execution entry point.
	 */
	execute : function(options) {
		options = options || {};
		this.checkActionState();

		/* workaround to enable createInterceptor return false beeing handled correctly
		 * Seems that returnValue=false in Ext.Function.createInterceptor returns null not the specified false
		 * Used by AbstractDcvForm to inject its own form validation result.
		 * */
		if (this.beforeExecute() === false || this.beforeExecute() == -1 ) {
			return false; 
		}

		if (this.needsConfirm() && options.confirmed !== true) {
			this.confirmExecute(null, options);
			return;
		}

		this.onExecute(options);

		this.afterExecute(options);
	},

	/**
	 * Empty template method meant to be overriden by the subclasses.
	 * 
	 */
	onExecute : function(option) {
		throw new Exception("Unimplemented onExecute!");
	}

});
/**
 * Abstract base class for asynchronous commands. An asynchronous command is one
 * which involves an AJAX call so that the result is not available immediately.
 */
Ext.define("dnet.core.dc.command.AbstractDcAsyncCommand", {

	extend : "dnet.core.dc.command.AbstractDcCommand",

	onAjaxSuccess : function(response, options) {
		Ext.Msg.hide();
	},

	onAjaxFailure : function(response, options) {
		this.dc.showAjaxErrors(response, options);
	}

});/**
 * Abstract base class for synchronous commands. 
 */
Ext.define("dnet.core.dc.command.AbstractDcSyncCommand", {
	extend : "dnet.core.dc.command.AbstractDcCommand"

});Ext.define("dnet.core.dc.command.DcQueryCommand",{
	extend : "dnet.core.dc.command.AbstractDcAsyncCommand",

	onExecute : function() {
		var dc = this.dc;
		if (!dc.isFilterValid()) {
			Ext.Msg
					.show( {
						msg : "Filter contains invalid data. Fix the incorrect values then try again.",
						title : 'Invalid filter data',
						icon : Ext.MessageBox.ERROR,
						buttons :Ext.MessageBox.OK
					});
			return;
		}
		var request = dnet.core.base.RequestParamFactory
				.findRequest(dc.filter.data);
		for ( var p in request.data) {
			if (request.data[p] === "") {
				request.data[p] = null;
			}
		}
		var data = Ext.encode(request.data);
		request.data = data;
		request.params = Ext.encode(dc.params.data);
		dc.store.proxy.extraParams.params = request.params;
		dc.store.proxy.extraParams.data = request.data;
		dc.store.currentPage = 1;
		
		dc.store.load( {
			//params : request,
			scope : dc
		});
	},

	checkActionState : function() {
		if (dnet.core.dc.DcActionsStateManager
				.isQueryDisabled(this.dc)) {
			throw ("Query is not allowed.");
		}
	}
});
Ext.define("dnet.core.dc.command.DcNewCommand", {
	extend : "dnet.core.dc.command.AbstractDcSyncCommand",

	onExecute : function() {
		var dc = this.dc;
//				var ed = "";
//				dc.store.data.each( function(item,idx,len) { ed += item.dirty + "-" + item.phantom + ","});
//				console.log( "DcNewCommand start -> store.count =  " + dc.store.getCount() +
//				", store.dirty = "+dc.isStoreDirty() + ": " + ed);
		if (!dc.multiEdit && dc.isDirty()) {
			
			
			 throw ( dnet.core.dc.DcExceptions.DIRTY_DATA_FOUND);
		}
		
		var r = dc.newRecordInstance();	
		if (dc.dcContext) {
			dc.dcContext._applyContextData_(r);
		} 
		dc.setRecord(r, true);
		dc.store.add(r);
//		 		ed = "";
//				dc.store.data.each( function(item,idx,len) { ed += item.dirty + "-" + item.phantom + ","});
//				console.log( "DcNewCommand end -> store.count =  " + dc.store.getCount() +
//				", store.dirty = "+dc.isStoreDirty() + ": " + ed);
				
		dc.fireEvent("afterDoNew", {
			dc : dc
		});
	},

	checkActionState : function() {
		if (dnet.core.dc.DcActionsStateManager.isNewDisabled(this.dc)) {
			throw ("Creating new record is not allowed.");
		}
	}
});Ext.define("dnet.core.dc.command.DcCopyCommand", {
	extend : "dnet.core.dc.command.AbstractDcSyncCommand",

	dcApiMethod : dnet.core.dc.DcActionsFactory.COPY,
	
	onExecute : function() {
		var dc = this.dc;
 
		var source = dc.getRecord();
		if (!source) {
			return;
		} 
		var target = source.copy();		 
		target.data.id = null;
		if (target.data.code) {
			target.data.code = null;
		}
		if (target.data.name) {
			target.data.name = 'Copy of ' + target.data.name;
		}
		if (target.data.uuid) {
			target.data.uuid = null;
		}
		target.phantom = true;
		target.dirty = true;
		Ext.data.Model.id(target);
		if (dc.dcContext) {
			dc.dcContext._applyContextData_(target);
		}
		dc.setRecord(target, true);
		dc.store.add(target);
		
		dc.fireEvent("afterDoNew", {
			dc : dc
		});
 
	},

	checkActionState : function() {
		if (dnet.core.dc.DcActionsStateManager.isCopyDisabled(this.dc)) {
			throw ("Copying a record is not allowed.");
		}
	}

});


Ext.define("dnet.core.dc.command.DcSaveCommand", {
	extend: "dnet.core.dc.command.AbstractDcAsyncCommand" ,

	errorTpl: Ext.create('Ext.XTemplate', 
			['<ul><tpl for="."><li>',
			 '<span class="field-name">{field}</span>:',
			 '<span class="error">{message}</span>',
			 '</li></tpl></ul>']),
	 
 
	beforeExecute: function() {
		var dc = this.dc;
		if (!dc.multiEdit) {
			return this.isValid(dc.getRecord());			
		}else {			 
			if ( ! this.isValid(dc.store.getUpdatedRecords()) ) {
				return false;
			}
			if ( ! this.isValid(dc.store.getAllNewRecords()) ) {
				return false;
			} 			 
			return true;
		}		
	},
	
	
	onExecute: function() {	
		this.dc.store.proxy.extraParams.params=Ext.JSON.encode(this.dc.params.data);
		this.dc.store.sync(); 		 	
	},
	
	checkActionState: function() {
		if (dnet.core.dc.DcActionsStateManager.isSaveDisabled(this.dc)) {	
			throw("Save is not allowed.");
		}
	},
	
	isValid: function(recs) {
		if (!Ext.isArray(recs)) {
			recs = [recs];
		}
		var len = recs.length;
		var errors = null;
		for(var i=0;i<len;i++) {
			errors = recs[i].validate();
			if (!errors.isValid()) {
				Ext.Msg.show({
					   title: 'Invalid data',
					   msg: this.errorTpl.apply(errors.getRange()),
					   icon : Ext.MessageBox.ERROR,
					   buttons :Ext.MessageBox.OK
				   }); 
				return false;
			}
		}
		return true;
	}
});
 Ext.define("dnet.core.dc.command.DcCancelCommand", {
	extend : "dnet.core.dc.command.AbstractDcSyncCommand",

	onExecute : function() {
		var dc = this.dc;
 
		if (dc.store.getCount() == 0) {
			this.discardChanges();
		} else {
			this.discardChildrenChanges();
			this.discardChanges();
		}
 
	},

	discardChanges : function() {

		var dc = this.dc;
		var s = dc.store;
 
		if (dc.multiEdit) {
			s.rejectChanges();
		} else {
			s.rejectChanges();			
			if (dc.record.phantom) {
				/* workaround to avoid the dirty check in AbstractDc.setRecord */
				var cr = dc.record;
				cr.phantom = false;
				cr.dirty = false;
				dc.setRecord(null, true);
				cr.phantom = true;
				cr.dirty = true;
			} 
		}
 		
	},

	discardChildrenChanges : function() {
		var dc = this.dc;
		var dirty = false, l = dc.children.length;
		for ( var i = 0; i < l; i++) {
			if (dc.children[i].isDirty()) {
				dc.children[i].doCancel();
			}
		}
	}

});
Ext.define("dnet.core.dc.command.DcEditInCommand", {
	extend : "dnet.core.dc.command.AbstractDcSyncCommand",

	execute : function(dc) {

	}

});
Ext.define("dnet.core.dc.command.DcEditOutCommand", {
	extend : "dnet.core.dc.command.AbstractDcSyncCommand",

	execute : function(dc) {

	}

});
Ext.define("dnet.core.dc.command.DcClearQueryCommand", {
	extend : "dnet.core.dc.command.AbstractDcSyncCommand",

	onExecute : function() {
		var dc = this.dc;
		if (dnet.core.dc.DcActionsStateManager.isQueryDisabled(dc)) {
			throw ("Query is not allowed.");
		}

		if (this.beforeExecute(dc) === false) {
			return false;
		}

		if (this.needsConfirm(dc)) {
			this.confirmExecute(null, dc);
			return;
		}

		for ( var p in dc.filter.data) {
			dc.filter.set(p, null);
		}
		if (dc.dcContext) {
			dc.dcContext._updateChildFilter_();
		}
	}
});Ext.define("dnet.core.dc.command.DcDeleteCommand", {
	extend : "dnet.core.dc.command.AbstractDcSyncCommand",

	constructor : function(config) {		 
		this.callParent(arguments);
		this.confirmByUser = true;
		this.confirmMessageTitle = Dnet.translate("msg", "dc_confirm_action");
		this.confirmMessageBody = Dnet.translate("msg",
				"dc_confirm_delete_selection");
	},

	onExecute : function() {
		var dc = this.dc;
		dc.store.remove(dc.getSelectedRecords());
		if (!dc.multiEdit) {
			//dc.store.proxy.extraParams = Ext.encode(dc.params.data);
			dc.store.sync();
		}
		dc.doDefaultSelection();
	},

	checkActionState : function() {
		if (dnet.core.dc.DcActionsStateManager.isDeleteDisabled(this.dc)) {
			throw ("Delete is not allowed.");
		}
	}
});Ext.define("dnet.core.dc.command.DcReloadRecCommand", {
	extend : "dnet.core.dc.command.AbstractDcAsyncCommand",

	beforeExecute : function() {
		var r = this.dc.getRecord();
		if (!r || r.phantom || !r.data.id) {
			return false;
		}
		return true;
	},

	onExecute : function(dc) {
		var dc = this.dc;
		 
		Ext.Ajax.request({
		    url: dc.store.proxy.api.load,		    
		    params: {
				data : Ext.encode({
					id : dc.record.data.id
				})
			},			
		    success:this.onReload,
		    scope:this
		});
	 
	},

	onReload: function(response, opts) {
		var dc = this.dc;
		var rs = dc.store.proxy.reader.read(response);
		var r = dc.record;
		var nr = rs.records[0];
		var shouldCommit = !r.dirty;
		r.beginEdit();
		for ( var p in dc.record.data) {
			dc.record.set(p, nr.data[p]);
		}
		r.endEdit();
		if(shouldCommit) {
			r.commit();
		}		
	},
	
	checkActionState : function() {
		if (dnet.core.dc.DcActionsStateManager.isReloadRecDisabled(this.dc)) {
			throw ("Reload record is not allowed.");
		}
	}

});

Ext.define("dnet.core.dc.command.DcPrevRecCommand", {
	extend : "dnet.core.dc.command.AbstractDcSyncCommand",

	onExecute : function() {
		var dc = this.dc;
		if (dc.selectedRecords.length <= 1) {
			var crtIdx = dc.store.indexOf(dc.record);
			if (--crtIdx < 0) {
				throw (dnet.core.dc.DcExceptions.NAVIGATE_BEFORE_FIRST);
			} else {
				dc.setRecord(dc.store.getAt(crtIdx), true);
			}
		} else {
			var crtIdx = dc.selectedRecords.indexOf(dc.record);
			if (--crtIdx < 0) {
				throw (dnet.core.dc.DcExceptions.NAVIGATE_BEFORE_FIRST);
			} else {
				dc.setRecord(dc.selectedRecords[crtIdx]);
			}
		}
	}
});
Ext.define("dnet.core.dc.command.DcNextRecCommand", {
	extend : "dnet.core.dc.command.AbstractDcSyncCommand",

	onExecute : function() {
		var dc = this.dc;
		if (dc.selectedRecords.length <= 1) {
			var crtIdx = dc.store.indexOf(dc.record);
			if (++crtIdx >= dc.store.getCount()) {
				throw (dnet.core.dc.DcExceptions.NAVIGATE_AFTER_LAST);
			} else {
			 	dc.setRecord(dc.store.getAt(crtIdx), true);
			}
		} else {
			var crtIdx = dc.selectedRecords.indexOf(dc.record);
			if (++crtIdx >= dc.selectedRecords.length) {
				throw (dnet.core.dc.DcExceptions.NAVIGATE_AFTER_LAST);
			} else {			 
				dc.setRecord(dc.selectedRecords[crtIdx]);
			}
		}
	}
});

Ext.define("dnet.core.dc.command.DcRpcDataCommand", {
	extend : "dnet.core.dc.command.AbstractDcAsyncCommand",

	/**
	 * Call a service on the data-source.
	 * 
	 * @param serviceName:
	 *            The name of the data-source service to be executed.
	 * @param specs:
	 *            Specifications regarding the execution of this task.
	 *            Attributes of specs:
	 *            <li> modal: Boolean flag to show a progress bar during the
	 *            execution of the request to block user interaction.
	 *            <li> context: object with variables you may need in your
	 *            callbacks
	 *            <li> callbacks: Object specifying callback functions to be
	 *            invoked
	 *            <li> stream: It is a stream type call Attributes of callbacks :
	 *            <li>successFn: Callback to execute on successful execution</li>
	 *            <li>successScope: scope of the successFn</li>
	 *            <li>silentSuccess: do not fire the afterDoServiceSuccess
	 *            event</li>
	 *            <li>failureFn: callback to execute on failure</li>
	 *            <li>failureScope: scope of the failureFn</li>
	 *            <li>silentFailure: do not fire the
	 *            <code>afterDoServiceFailure</code> event</li>
	 *            The arguments passed to these functions are described in the
	 *            afterDoServiceSuccess() and afterDoServiceFailure() methods
	 *            which actually invoke them.
	 * 
	 */
	onExecute : function(options) {
		var dc = this.dc;
		var serviceName = options.name;
		//var s = options || {};
		var p = {
			data : Ext.encode(dc.record.data),
			params : Ext.encode(dc.params.data)
		};
		options.sourceRec = dc.record;
		p[Dnet.requestParam.SERVICE_NAME_PARAM] = serviceName;
		p["rpcType"] = "data";
		if (options.modal) {
			Ext.Msg.wait('Working...');
		}
		Ext.Ajax
				.request( {
					url : Dnet.dsAPI(dc.dsName,
							((options.stream) ? "stream" : "json")).service,
					method : "POST",
					params : p,
					success : this.onAjaxSuccess,
					failure : this.onAjaxFailure,
					scope : this,
					options : options
				});
	},

	/**
	 * Method called after a successful execution of the service. Successful
	 * means that server returns a 200 status code and the success attribute in
	 * the returning json is set to true. It first invokes the task specific
	 * callback then fires the associated event. Both type of callback methods (
	 * the one specified in callbacks and the handler of the fired event) will
	 * be passed the data-control instance (this) followed by the arguments of
	 * this method. If you need a certain callback to be executed each time,
	 * attach an event listener to the fired event.
	 * 
	 * @param response:
	 *            the server response object as received from the ajax request
	 * @param serviceName:
	 *            the name of service which has been executed.
	 * @param specs:
	 *            Specifications regarding the execution of this task.
	 * @See doService()
	 * 
	 */
	onAjaxSuccess : function(response, options) {
		if (options.options.modal) {		
			try {
				Ext.Msg.hide();
			} catch(e) {
				
			}
		}
		var o = options.options || {}, name = o.name, s = o || {};
		var dc = this.dc;
		var r = Ext.decode( response.responseText );
		 
		if(r.success) {
			var rec = dc.store.proxy.reader.readRecords([r.data]).records[0];
			var srcRec = options.options.sourceRec;
			var dirty = srcRec.dirty;
			srcRec.beginEdit();
			for(var p in rec.data) {
				srcRec.set(p, rec.data[p]);
			}
			srcRec.endEdit();
			if (!dirty) {
				srcRec.commit();
			}
		}
		 
		if (s.callbacks && s.callbacks.successFn) {
			s.callbacks.successFn.call(s.callbacks.successScope || dc, dc,
					response, name, options);
		}
		if (!(s.callbacks && s.callbacks.silentSuccess === true)) {
			dc.fireEvent("afterDoServiceSuccess", dc, response, name, options);
		}
	},

	/**
	 * Method called when execution of the service fails. Failure means that
	 * server returns anything except a 200 class status code or the success
	 * attribute in the returning json is set to false. It first invokes the
	 * task specific callback then fires the associated event. Both type of
	 * callback methods ( the one specified in callbacks and the handler of the
	 * fired event) will be passed the data-control instance (this) followed by
	 * the arguments of this method. If you need a certain callback to be
	 * executed each time, attach an event listener to the fired event.
	 * 
	 * @param response:
	 *            the server response object as received from the ajax request
	 * @param serviceName:
	 *            the name of service which has been executed.
	 * @param specs:
	 *            Specifications regarding the execution of this task.
	 * @See doService()
	 */
	onAjaxFailure : function(response, options) {
		if (options.options.modal) {		
			try {
				Ext.Msg.hide();
			} catch(e) {
				
			}
		}
		var o = options.options || {}, serviceName = o.name, s = o || {};
		var dc = this.dc;
		dc.showAjaxErrors(response,options);
		if (s.callbacks && s.callbacks.failureFn) {
			s.callbacks.failureFn.call(s.callbacks.failureScope || dc, dc,
					response, serviceName, options);
		} 
		if (!(s.callbacks && s.callbacks.silentFailure === true)) {
			dc.fireEvent("afterDoServiceFailure", dc, response, name, options);
		}
	},

	canExecute : function() {
		return true
	}

});
Ext.define("dnet.core.dc.command.DcRpcFilterCommand", {
	extend : "dnet.core.dc.command.AbstractDcAsyncCommand",

	/**
	 * Call a service on the data-source filter.
	 * 
	 * @param serviceName:
	 *            The name of the data-source service to be executed.
	 * @param options:
	 *            Specifications regarding the execution of this task. Command
	 *            specific attributes: Attributes of specs:
	 *            <li> namr: Name of the server-side RPC </li>
	 *            <li> modal: Boolean flag to show a progress bar during the
	 *            execution of the request to block user interaction.
	 *            <li> context: object with variables you may need in your
	 *            callbacks
	 *            <li> callbacks: Object specifying callback functions to be
	 *            invoked Attributes of callbacks :
	 *            <li>successFn: Callback to execute on successful execution</li>
	 *            <li>successScope: scope of the successFn</li>
	 *            <li>silentSuccess: do not fire the
	 *            afterDoServiceFilterSuccess event</li>
	 *            <li>failureFn: callback to execute on failure</li>
	 *            <li>failureScope: scope of the failureFn</li>
	 *            <li>silentFailure: do not fire the
	 *            <code>afterDoServiceFailure</code> event</li>
	 *            The arguments passed to these functions are described in the
	 *            afterDoServiceFilterSuccess() and
	 *            afterDoServiceFilterFailure() methods which actually invoke
	 *            them.
	 * 
	 */
	onExecute : function(options) {
		var dc = this.dc;
		var serviceName = options.name;
		var s = options || {};
		var p = {
			data : Ext.encode(dc.filter.data),
			params : Ext.encode(dc.params.data)
		};
		p[Dnet.requestParam.SERVICE_NAME_PARAM] = serviceName;
		p["rpcType"] = "filter";
		if (s.modal) {
			Ext.Msg.wait("Working...");
		}
		Ext.Ajax.request( {
			url : Dnet.dsAPI(dc.dsName, "json").service,
			method : "POST",
			params : p,
			success : this.onAjaxSuccess,
			failure : this.onAjaxFailure,
			scope : this,
			options : options
		});
	}

	/**
	 * Method called after a successful execution of the service. Successful
	 * means that server returns a 200 status code and the success attribute in
	 * the returning json is set to true. It first invokes the task specific
	 * callback then fires the associated event. Both type of callback methods (
	 * the one specified in callbacks and the handler of the fired event) will
	 * be passed the data-control instance (this) followed by the arguments of
	 * this method. If you need a certain callback to be executed each time,
	 * attach an event listener to the fired event.
	 * 
	 * @param response:
	 *            the server response object as received from the ajax request
	 * @param serviceName:
	 *            the name of service which has been executed.
	 * @param specs:
	 *            Specifications regarding the execution of this task.
	 * @See doService()
	 * 
	 */
	,
	onAjaxSuccess : function(response, options) {
		try {
			Ext.Msg.hide();
		} catch(e) {
			
		}
		var o = options.options || {}, name = o.name, s = o || {};
		var dc = this.dc;
		if (s.callbacks && s.callbacks.successFn) {
			s.callbacks.successFn.call(s.callbacks.successScope || dc, dc,
					response, name, options);
		}
		if (!(s.callbacks && s.callbacks.silentSuccess === true)) {
			dc.fireEvent("afterDoServiceFilterSuccess", dc, response, name,
					options);
		}
	}

	/**
	 * Method called when execution of the service fails. Failure means that
	 * server returns anything except a 200 class status code or the success
	 * attribute in the returning json is set to false. It first invokes the
	 * task specific callback then fires the associated event. Both type of
	 * callback methods ( the one specified in callbacks and the handler of the
	 * fired event) will be passed the data-control instance (this) followed by
	 * the arguments of this method. If you need a certain callback to be
	 * executed each time, attach an event listener to the fired event.
	 * 
	 * @param response:
	 *            the server response object as received from the ajax request
	 * @param serviceName:
	 *            the name of service which has been executed.
	 * @param specs:
	 *            Specifications regarding the execution of this task.
	 * @See doService()
	 */
	,
	onAjaxFailure : function(response, options) {
		try {
			Ext.Msg.hide();
		} catch(e) {
			
		}
		var o = options.options || {}, serviceName = o.name, s = o || {};
		var dc = this.dc;
		dc.showAjaxErrors(response,options);
		if (s.callbacks && s.callbacks.failureFn) {
			s.callbacks.failureFn.call(s.callbacks.failureScope || dc, dc,
					response, serviceName, options);
		}
		if (!(s.callbacks && s.callbacks.silentFailure === true)) {
			dc.fireEvent("afterDoServiceFilterFailure", dc, response, name,
					options);
		}
	}

});
Ext.define('dnet.core.ui.FrameBuilder$TocModel', {
	extend : 'Ext.data.Model',
	fields : [ {
		name : 'name',
		type : 'string'
	}, {
		name : 'title',
		type : 'string'
	}]
});
 
Ext.define("dnet.core.ui.FrameBuilder" , {
	
	/**
	 * 
	 * @type dnet.core.ui.AbstractUi
	 */
	frame : null,
	
	constructor : function(config) {
        config = config || {};
        Ext.apply(this, config);
        this.callParent(arguments);
    },

	addDc: function(name, obj) {
		this.frame._dcs_.add(name, obj);
		obj._instanceKey_ = name;
		return this;
	},
	
	linkDc: function(childName, parentName, relation) {
		Ext.applyIf(relation, {fetchMode:"manual", strict: true});
		var c = this.frame._dcs_.get(childName);
		var p = this.frame._dcs_.get(parentName);
		var ctx = new dnet.core.dc.DcContext({
			 childDc:c
			,parentDc:p
			,relation:relation});
		p.addChild(c);c.setDcContext(ctx);
		return this;
	},
	
	addDcFormView: function(dc, config) {	
		this.addDcView(dc, config);	
		var theDc = this.frame._dcs_.get(dc);
		var viewId = config.id;
		return this;
	},
	
	
	addDcFilterFormView: function(dc, config) {	
		var ttlKey = config.name + "__ttl";
		if (config.title && this.frame._trl_ && this.frame._trl_[ttlKey]) {
			config.title = this.frame._trl_[ttlKey];
		}
		this.addDcView(dc, config);	
		var theDc = this.frame._dcs_.get(dc);
		var viewId = config.id;
		return this;
	},
	
	addDcGridView: function(dc, config) {		 
		config.stateId = this.frame.$className + "-"+ config.name;
		if (getApplication().getSession().rememberViewState) {
			config.stateful = true;
		}
		return this.addDcView(dc, config);
	},
	
	addDcEditGridView: function(dc, config) {
		return this.addDcGridView(dc, config);
	},
	
	addDcView: function(dc, config) {	
		Ext.apply(config, {
			_controller_:this.frame._dcs_.get(dc) 			 
			//,listeners:{ activate:{scope:this,fn:function(p){p.doLayout(false,true);} } }
		});
		var ttlKey = config.name + "__ttl";
		if (config.title && this.frame._trl_ && this.frame._trl_[ttlKey]) {
			config.title = this.frame._trl_[ttlKey];
		}
		this.applyViewSharedConfig(config);				
		return this;
	},
	
	
	addPanel:function(config) {
		config.listeners = config.listeners || {};
		if (config.onActivateDoLayoutFor) {
			var onActivateDoLayoutFor = config.onActivateDoLayoutFor;
			delete config.onActivateDoLayoutFor;
			var activate = {
				scope:this.frame,
				fn:function(cmp,opt){ 
					for (var i=0;i<onActivateDoLayoutFor.length;i++) {  
						var e = this._getElement_(onActivateDoLayoutFor[i]);
						// workaround for the dissapearing scrollbars when returning from an editor after a new+ cancel combination  
						// hopefully not needed anymore
						
//						if (e instanceof dnet.core.dc.AbstractDcvGrid) {
//							//alert(onActivateDoLayoutFor[i] + " instanceof dnet.core.dc.AbstractDcvGrid");
//							//e.getView().refresh();
//							//e.determineScrollbars( );
//							e.getView().refresh();
//							e.invalidateScroller();
//						} else {
							e.doLayout(); 
//						}
						
					} 
				}
			}	
			if (!config.listeners.activate) {
				config.listeners.activate = activate;
			}
		}
		Ext.applyIf(config, {
			 
			id: Ext.id()			
		});
		var ttlKey = config.name + "__ttl";
		  
		if (config.title && this.frame._trl_ && this.frame._trl_[ttlKey]) {
			config.title = this.frame._trl_[ttlKey];
		}
		this.applyViewSharedConfig(config);	
		return this;
	},
	
	
	addChildrenTo: function(c,list,regions) {
		var isWrapped = this.frame._elems_.get(c)._wrapped_;
		var items = ((isWrapped)? this.frame._elems_.get(c)["items"]["items"]:this.frame._elems_.get(c)["items"] )|| [];
		for(var i=0, len=list.length; i<len; i++ ) {
			var cmp = this.frame._elems_.get(list[i]);
			items[items.length] = cmp;
			if (regions){
				cmp.region = regions[i];
			}
		}
		if(isWrapped) {
			this.frame._elems_.get(c)["items"]["items"] = items;
		} else {
			this.frame._elems_.get(c)["items"] = items;
		}
		
		return this;
	}
	
	,addToolbarTo: function(c,tlb) {
		this.frame._linkToolbar_(tlb, c); 
		return this;
	}
 
	// private

	,applyViewSharedConfig: function(config) {
		Ext.applyIf(config,{
			id:Ext.id(), itemId: config.name
		});				 
		this.frame._elems_.add(config.name, config);
	}
	
	,beginToolbar: function(name, config) {
		return new dnet.core.ui.ActionBuilder({name:name , frame:this.frame, dc: config.dc});
	}
	
	,addToc: function(canvases) {
		var data = [];
		for(var i=0;i<canvases.length;i++) {
			data[i] = { "name": canvases[i], "title":this.frame._elems_.get(canvases[i]).title } ;			
		}
		var store = Ext.create( 'Ext.data.Store', {
			model: 'dnet.core.ui.FrameBuilder$TocModel',
			data: data			 
		});
		
		var config = {
				name:"_toc_", collapsible: true, layout:"fit", id:Ext.id(),
				region:"west", title: 'Navigation', width: 200, frame:false
				, items:[{
					 name: "_toc_items_", xtype: 'gridpanel',id:Ext.id()
					,hideHeaders:true
					,autoScroll:true
					,viewConfig: {
						stripeRows: false
					}					 					 
					,singleSelect: true
					,forceFit: true
					,store: store			
					,columns: [ {
				        header: 'title',				        
				        dataIndex: 'title'
				    }]
				   , selModel :  {
						mode: "SINGLE",
						listeners : {
							 
							"selectionchange" : {
								scope : this.frame,
								fn : function(sm,selected, options) {
									if(this._getElement_("main").rendered) {
										this._showStackedViewElement_("main", selected[0].data.name);	
									} 									
								}
							}
						}
					},
				    listeners: {scope:this.frame, 
		            	 
						 afterrender: function() {
							this._showTocElement_(0);			 
		            	}   
		        	}
				}]
				
			} 
		this.frame._elems_.add(config.name, config);
		return this;
	} 
	
	
	,addButton: function(config) {		
		Ext.applyIf(config , {id:Ext.id(), xtype:"button"} );
		var ttlKey = config.name + "__ttl";
		var descKey = config.name + "__desc";
		
		if (config.text && this.frame._trl_ && this.frame._trl_[ttlKey]) {
			config.text = this.frame._trl_[ttlKey];
		}
		if (config.tooltip && this.frame._trl_ && this.frame._trl_[descKey]) {
			config.tooltip = this.frame._trl_[descKey];
		}		
		this.frame._elems_.add(config.name, config);
		if (config.stateManager ) {
			var options = {and:config.stateManager.and };
			if (options.and) {
				this.frame._buttonStateRules_[config.name] = options.and;
			}
			dnet.core.ui.FrameButtonStateManager.register(config.name, config.stateManager.name, config.stateManager.dc, this.frame, options );
		}
		return this;
	} 
	
	,addAsgn: function(config) {	 
		Ext.applyIf(config,{ id:Ext.id(), objectIdField: "id"  } );
		this.frame._elems_.add(config.name, config);
		return this;
	},  
	
	addWindow: function(config) {
		Ext.applyIf(config,{ id:Ext.id(),_window_:true} );
		var ttlKey = config.name + "__ttl";
		  
		if (config.title && this.frame._trl_ && this.frame._trl_[ttlKey]) {
			config.title = this.frame._trl_[ttlKey];
		}
		
		this.frame._elems_.add(config.name, config);
		return this;
	}
	,add: function(config) {
		Ext.applyIf(config,{ id:Ext.id()} );
		this.frame._elems_.add(config.name, config);
		return this;
	}
	,merge: function(name, config) {
		Ext.applyIf(this.frame._elems_.get(name) , config );
		return this;
	}
	,change: function(name, config) {
		Ext.apply(this.frame._elems_.get(name) , config );
		return this;
	}
	,remove: function(name) {
		this.frame._elems_.remove(name);
		return this;
	}
	
	
});
/*
Ext.override(Ext.layout.component.AbstractDock, {
	afterRemove : function(item) {
		this.callParent(arguments);
		if (this.itemCls) {
			item.el.removeCls(this.itemCls + '-' + item.dock);
		}
		var dom = item.el.dom;
		if (!item.destroying && dom) {
			if (dom.parentNode) {
				dom.parentNode.removeChild(dom);
			}			
		}
		this.childrenChanged = true;
	}

});
*/


Ext.define("dnet.core.ui.ActionBuilder", {
	
	/**
	 * 
	 * @type dnet.core.ui.AbstractUi
	 */
	frame : null,
	name : null,
	 
	dc : null,
	sepIdx : null,

	constructor : function(config) {
		config = config || {};
		Ext.apply(this, config);
		this.callParent(arguments);
	},

	setup : function() {
		// this.frame._tlbitms_ = new Ext.util.MixedCollection();
	},

	addLabel : function(config) {
		var cfg = config || {};
		Ext.applyIf(cfg, {
					dc : this.dc,
					xtype : "label",
					cls : "dnet-toolbar-label"
				});
		this.frame._tlbitms_.add(this.name + "__" + config.name, config);
		return this;
	},

	addTitle : function(config) {
		var cfg = config || {};		
		Ext.applyIf(cfg, {
					dc : this.dc,
					xtype : "label",
					name : "title",	
					height:20,
					cls : "dnet-toolbar-title"
				});
		if (cfg.text && this.frame._trl_ && this.frame._trl_[this.name + "__ttl"]) {
			cfg.text = this.frame._trl_[this.name + "__ttl"];
		}		
		this.frame._tlbitms_.add(this.name + "__" + cfg.name, cfg);
		if (this.frame._tlbtitles_ == null ) {
			this.frame._tlbtitles_ = new Ext.util.MixedCollection();
		}
		this.frame._tlbtitles_.add(this.name, cfg.text);
		 
		return this;
	},
	addQuery : function(config) {
		var cfg = config || {};
		Ext.applyIf(cfg, {
					dc : this.dc
				});
		var a = this.frame._getDc_(cfg.dc).actions.doQuery;
		this.frame._tlbitms_.add(this.name + "__" + a.initialConfig.name, a);
		return this;
	},
	addNew : function(config) {
		var cfg = config || {};
		Ext.applyIf(cfg, {
					dc : this.dc,
					tlb : this.name,
					autoEdit : true
				});
		var a = this.frame._getDc_(cfg.dc).actions.doNew;
		this.frame._tlbitms_.add(this.name + "__" + a.initialConfig.name, a);
		if (cfg.autoEdit != "false") {
			this.frame.mon(this.frame._getDc_(cfg.dc), "afterDoNew", function() {
						this._invokeTlbItem_("doEdit", cfg.tlb);
					}, this.frame);
		} else {
			if (!Ext.isEmpty(cfg.showView)) {
				var fn = function() {
					try {
						var ct = (cfg.inContainer) ? this
								._getElement_(cfg.inContainer) : this
								._getElement_("main");
						if (cfg.showView) {
							var cmp = this._get_(cfg.showView);
							if (cmp) {
								ct.getLayout().setActiveItem(cmp);
							} else {
								ct.getLayout().setActiveItem(this
										._getElementConfig_(cfg.showView).id);
							}
						} else {
							ct.getLayout().setActiveItem(1);
						}
					} catch (e) {
						dnet.core.dc.DcExceptions.showMessage(e);
					}
				};
				this.frame.mon(this.frame._getDc_(cfg.dc), "afterDoNew", fn, this.frame);
			}
		}
		return this;
	},

	
	addCopy : function(config) {
		var cfg = config || {};
		Ext.applyIf(cfg, {
					dc : this.dc,
					tlb : this.name,
					autoEdit : true
				});
		var a = this.frame._getDc_(cfg.dc).actions.doCopy;
		this.frame._tlbitms_.add(this.name + "__" + a.initialConfig.name, a);
		if (cfg.autoEdit != "false") {
			this.frame.mon(this.frame._getDc_(cfg.dc), "afterDoNew", function() {
						this._invokeTlbItem_("doEdit", cfg.tlb);
					}, this.frame);
		} else {
			if (!Ext.isEmpty(cfg.showView)) {
				var fn = function() {
					try {
						var ct = (cfg.inContainer) ? this
								._getElement_(cfg.inContainer) : this
								._getElement_("main");
						if (cfg.showView) {
							var cmp = this._get_(cfg.showView);
							if (cmp) {
								ct.getLayout().setActiveItem(cmp);
							} else {
								ct.getLayout().setActiveItem(this
										._getElementConfig_(cfg.showView).id);
							}
						} else {
							ct.getLayout().setActiveItem(1);
						}
					} catch (e) {
						dnet.core.dc.DcExceptions.showMessage(e);
					}
				};
				this.frame.mon(this.frame._getDc_(cfg.dc), "afterDoNew", fn, this.frame);
			}
		}
		return this;
	}
	
	,
	addCopy1 : function(config) {
		var cfg = config || {};
		Ext.applyIf(cfg, {
					dc : this.dc
				});
		var a = this.frame._getDc_(cfg.dc).actions.doCopy;
		this.frame._tlbitms_.add(this.name + "__" + a.initialConfig.name, a);
		return this;
	},
	addDeleteSelected : function(config) {
		var cfg = config || {};
		Ext.applyIf(cfg, {
					dc : this.dc
				});
		var a = this.frame._getDc_(cfg.dc).actions.doDelete;
		this.frame._tlbitms_.add(this.name + "__" + a.initialConfig.name, a);
		return this;
	},
	addEdit : function(config) {
		var cfg = config || {};
		Ext.applyIf(cfg, {
					dc : this.dc,
					tlb : this.name
				});
		var fn = function() {
			try {
				var ct = (cfg.inContainer)
						? this._getElement_(cfg.inContainer)
						: this._getElement_("main");
				if (cfg.showView) {
					var cmp = this._get_(cfg.showView);
					if (cmp) {
						ct.getLayout().setActiveItem(cmp);
					} else {
						ct.getLayout().setActiveItem(this
								._getElementConfig_(cfg.showView).id);
					}
				} else {
					ct.getLayout().setActiveItem(1);
				}
			} catch (e) {
				dnet.core.dc.DcExceptions.showMessage(e);
			}
		};
		var a = this.frame._getDc_(cfg.dc).actions.doEditIn;
		a.setHandler(fn, this.frame);
		this.frame._tlbitms_.add(this.name + "__" + a.initialConfig.name, a);

		this.frame.mon(this.frame._getDc_(cfg.dc), "onEdit", function() {
					this._invokeTlbItem_("doEdit", cfg.tlb);
				}, this.frame);
		return this;
	}

	,
	addCancel : function(config) {
		var cfg = config || {};
		Ext.applyIf(cfg, {
					dc : this.dc,
					tlb : this.name,
					autoBack : true
				});
		var a = this.frame._getDc_(cfg.dc).actions.doCancel;
		this.frame._tlbitms_.add(this.name + "__" + a.initialConfig.name, a);
		if (cfg.autoBack) {
			this.frame.mon(this.frame._getDc_(cfg.dc), 'recordChanged', function(event) {
						if (event.record == null) {
							this._invokeTlbItem_("doBack", cfg.tlb);
						}
					}, this.frame);
		}
		return this;
	},

	addPrevRec : function(config) {
		var cfg = config || {};
		Ext.applyIf(cfg, {
					dc : this.dc
				});
		var a = this.frame._getDc_(cfg.dc).actions.doPrevRec;
		this.frame._tlbitms_.add(this.name + "__" + a.initialConfig.name, a); // new
																				// Ext.Action(cfg)
		return this;
	},
	addNextRec : function(config) {
		var cfg = config || {};
		Ext.applyIf(cfg, {
					dc : this.dc
				});
		var a = this.frame._getDc_(cfg.dc).actions.doNextRec;
		this.frame._tlbitms_.add(this.name + "__" + a.initialConfig.name, a); // new
																				// Ext.Action(cfg)
		return this;
	}

	,
	addAutoLoad : function() {
		var cfg = {
			name : "autoLoad",
			disabled : false,
			enableToggle : true,
			text : "Auto",
			tooltip : "Toogle auto-load mode (Load data automatically when parent record is changed)",
			scope : dc,
			handler : function(btn, evnt) {
				try {
					dc.dcContext.relation.fetchMode = (btn.pressed)
							? "auto"
							: "manual";
				} catch (e) {
					dnet.core.dc.DcExceptions.showMessage(e);
				}
			},
			dc : this.dc
		};

		var dc = this.frame._getDc_(cfg.dc);
		if (dc.dcContext.relation.fetchMode == "auto"
				|| dc.dcContext.relation.fetchMode == undefined) {
			cfg.pressed = true;
		};
		var a = new Ext.Action(cfg);
		this.frame._tlbitms_.add(this.name + "__" + a.initialConfig.name, a); // new
																				// Ext.Action(cfg)
		return this;
	},
	addSave : function(config) {
		var cfg = config || {};
		Ext.applyIf(cfg, {
					dc : this.dc
				});
		var a = this.frame._getDc_(cfg.dc).actions.doSave;
		this.frame._tlbitms_.add(this.name + "__" + a.initialConfig.name, a);
		return this;
	}

	,
	addSeparator : function() {
		if (this.sepIdx == null) {
			this.sepIdx = 0;
		}
		this.frame._tlbitms_.add(this.name + "___S" + (this.sepIdx++) + "_",
				"-");
		return this;
	},
	
	
	addReports: function() {
		if (this.frame._reports_ != null) {
			var r = [],
				rc = this.frame._reports_;
			 	
			for (var i=0, l=rc.length ;i<l;i++) {
				if (rc[i].toolbar == this.name) {
					
					 
//					var title_ = rc[i].title,
//						report_ = rc[i].report,
//						url_ = rc[i].url +  ((rc[i].contextPath) ? '/' + rc[i].contextPath : '' ),						 
//						dcAlias_ = rc[i].dcAlias,
//						params_ = rc[i].params;
						
						 
					var fn = function(item, e) {
						 
						var dcReport = new dnet.core.dc.DcReport();
						var rec = this._getDc_(item.dcAlias).record;
						 
						
						if (!rec) {
							Ext.Msg.show({
								title : "No current record",
								msg:"There is no current record in execution context.<br> Cannot call report `"+item.title+"` without a current record which must provide values for the report parameters.",	
								icon : Ext.MessageBox.INFO,
								buttons : Ext.Msg.OK
							});
							return false;
						}
						dcReport.applyDsFieldValues(item.params, rec.data);
						dcReport.run({
							url: item.url,	
							contextPath: item.contextPath,
							params: item.params								
						});
					};
						
					var rcfg = Ext.apply({
						scope: this.frame, 
						text:rc[i].title, 
						handler:  fn
					}, rc[i]);
					
					r.push(rcfg);
				}
			}
			if (r.length > 0) {
				this.addSeparator();
				this.frame._tlbitms_.add(this.name + "___REPS_",
				{
					text: "Reports",
					menu: r
				});
				 
			}
		}
		return this;
	},
	
	end : function() {
		var n = this.name,
			t = this.frame._tlbitms_.filterBy(function(o, k) {
					return (k.indexOf(n + "__") != -1);
				}),
			tarray = t.getRange()
				;
				 
		this.frame._tlbs_.add(this.name, tarray );
		return this.frame._getBuilder_();
	}

	,
	addBack : function(config) {
		var cfg = config || {};
		var fn = function() {
			try {
				var ct = (cfg.inContainer)
						? this._getElement_(cfg.inContainer)
						: this._getElement_("main");
				if (cfg.showView) {
					var cmp = this._get_(cfg.showView);
					if (cmp) {
						ct.getLayout().setActiveItem(cmp);
					} else {
						ct.getLayout().setActiveItem(this
								._getElementConfig_(cfg.showView).id);
					}
				} else {
					ct.getLayout().setActiveItem(0);
				}

			} catch (e) {
				dnet.core.dc.DcExceptions.showMessage(e);
			}
		}
		Ext.applyIf(cfg, {
					dc : this.dc,
					handler : fn,
					scope : this.frame
				});
		var a = this.frame._getDc_(cfg.dc).actions.doEditOut;
		a.setHandler(cfg.handler, cfg.scope);
		this.frame._tlbitms_.add(this.name + "__" + a.initialConfig.name, a); // new
																				// Ext.Action(cfg)
		return this;
	},
	
	addButton: function(config) {
		var cfg = config || {};
		Ext.applyIf(config , {id:Ext.id(), xtype:"button"  } ); 
		this.frame._tlbitms_.add(this.name + "__" + config.name, config);
		return this;
	},
	
	addButtons: function(btns) {
		if (Ext.isArray(btns)) {
			for(var i=0;i<btns.length; i++) {
				this.addButton(btns[i]);
			}
		}
		return this;
	}
	
	
});









Ext.define("dnet.core.dc.DcReport", {
	run: function(config) {
		//var targetDc = config.dc;
		var params = config.params;
		var serverUrl = config.url;
		if (config.contextPath) {			 
			serverUrl += config.contextPath;
		}
		// config: params, url
		// param : code, name, type, lov, value, mandatory, noEdit
		
		var qs = "";
		for(var i=0,l=params.length; i<l;i++) {
			var p = params[i];
			if(qs != "") {
				qs += "&";
			}
			qs += p.code + "=" + p.value;
		}
		
		//alert(serverUrl + "?" + qs);
		window.open(serverUrl + "?" + qs, "Test-report","width=800,height=600,adressbar=true")
		.focus();
	 
	},
	
	
	
	applyDsFieldValues: function(params, data) {
		for(var i=0,l=params.length; i<l;i++) {
			var p = params[i];
			if (p.dsField) {
				p.value = data[p.dsField];
			}
		}
	},
	/**
	 * Validate the report parameters 
	 * @param {} params
	 */
	isValid: function(params) {
		for(var i=0,l=params.length; i<l;i++) {
			var p = params[i];
			if (!p.value && p.mandatory && !p.noEdit ) {
				return false;
			}
		}
	}
	
});







dnet.core.ui.FrameButtonStateManager =  {		
		register: function(btnName, state, dcName, frame, options) {	
			this[state](btnName, state, dcName, frame, options);
			return;			 
		}
	
		// record state based 
		,record_is_clean: function(btnName, state, dcName, frame, options) {
			var theDc = frame._getDc_(dcName);			
			this.registerForDcEvent(btnName, frame, theDc, "statusChange" );
			this.registerForDcEvent(btnName, frame, theDc, "recordChange" );
			this.registerForDcStoreEvent(btnName, frame, theDc, "update" ); 
		}
	
		,record_is_dirty: function(btnName, state, dcName, frame, options) {
			var theDc = frame._getDc_(dcName);			
			this.registerForDcEvent(btnName, frame, theDc, "statusChange" );
			this.registerForDcEvent(btnName, frame, theDc, "recordChange" );
			this.registerForDcStoreEvent(btnName, frame, theDc, "update" ); 
		}
		
		// record status based 	
		,record_status_is_new: function(btnName, state, dcName, frame, options) {
			var theDc = frame._getDc_(dcName);			
			this.registerForDcEvent(btnName, frame, theDc, "recordChange" );
		}
	
		,record_status_is_edit: function(btnName, state, dcName, frame, options) {
			var theDc = frame._getDc_(dcName);			
			this.registerForDcEvent(btnName, frame, theDc, "recordChange" );
		}
		 
		
		// selection based 
		,selected_zero: function(btnName, state, dcName, frame, options) {
			var theDc = frame._getDc_(dcName);			
			this.registerForDcEvent(btnName, frame, theDc, "selectionChange" );
			this.registerForDcEvent(btnName, frame, theDc, "recordChange" );
		}
		
		,selected_one: function(btnName, state, dcName, frame, options) {
			var theDc = frame._getDc_(dcName);			
			this.registerForDcEvent(btnName, frame, theDc, "selectionChange" );
		}
		
 		
		,selected_one_clean: function(btnName, state, dcName, frame, options) { 
			var theDc = frame._getDc_(dcName);			
			this.registerForDcEvent(btnName, frame, theDc, "selectionChange" );
			this.registerForDcEvent(btnName, frame, theDc, "recordChange" );
			this.registerForDcStoreEvent(btnName, frame, theDc, "update" );
		}
		
		,selected_one_dirty: function(btnName, state, dcName, frame, options) {
			var theDc = frame._getDc_(dcName);			
			this.registerForDcEvent(btnName, frame, theDc, "selectionChange" );
			this.registerForDcEvent(btnName, frame, theDc, "recordChange" );
			this.registerForDcStoreEvent(btnName, frame, theDc, "update" );
			 
		}
		
		
		 
		,selected_many: function(btnName, state, dcName, frame, options) {
			var theDc = frame._getDc_(dcName);			
			this.registerForDcEvent(btnName, frame, theDc, "selectionChange" );			 
		}
		
		
		
		,selected_not_zero: function(btnName, state, dcName, frame, options) {
			var theDc = frame._getDc_(dcName);			
			this.registerForDcEvent(btnName, frame, theDc, "selectionChange" ); 
		}
		
		
		
		 
		,registerForDcEvent: function(btnName, frame, dc, eventName) {
			dc.mon(dc, eventName, function(evnt) {
				this._applyStateButton_(btnName);
			 }, frame, { buffer:200 } );
		}
		
		,registerForDcStoreEvent: function(btnName, frame, dc, eventName) {			 
			dc.mon(dc.store, eventName, function(evnt) {
				this._applyStateButton_(btnName);
			 }, frame, { buffer:200 } );
		}
		
		// helper functions 
		
		
		,is_record_is_clean : function(dc) {
			return dc.getRecord() && !dc.isCurrentRecordDirty();
		}
		,is_record_is_dirty : function(dc) {
			return dc.getRecord() && dc.isCurrentRecordDirty() ;
		}
		
		,is_record_status_is_new : function(dc) {
			return dc.getRecordStatus()=='insert'
		}
		,is_record_status_is_edit : function(dc) {
			return dc.getRecordStatus()=='update'
		}
		
		,is_selected_zero : function(dc) {
			return dc.selectedRecords.length == 0;
		}
		
		,is_selected_one : function(dc) {
			return dc.selectedRecords.length == 1;
		}
		
		,is_selected_one_clean : function(dc) {
			return dc.selectedRecords.length >0;
		}
		
		,is_selected_one_dirty : function(dc) {
			return dc.selectedRecords.length >0;
		}		
		
		,is_selected_one_clean : function(dc) {
			return dc.selectedRecords.length == 1 && !dc.isCurrentRecordDirty();
		}
		
		,is_selected_one_dirty : function(dc) {
			return dc.selectedRecords.length == 1 && dc.isCurrentRecordDirty();
		}
				
		,is_selected_many : function(dc) {
			return dc.selectedRecords.length > 1;
		}
		
		,is_selected_not_zero : function(dc) {
			return dc.selectedRecords.length >0;
		}
		
	};Ext.define("dnet.core.ui.AbstractUi", {
	extend : "Ext.panel.Panel",

	mixins : {
		elemBuilder : "dnet.core.base.AbstractDNetView"
	},

	// **************** Properties *****************

	/**
	 * Component builder
	 * 
	 * @type dnet.core.ui.FrameBuilder
	 */
	_builder_ : null,

	/**
	 * Data-controls map
	 * 
	 * @type Ext.util.MixedCollection
	 */
	_dcs_ : null,

	/**
	 * Toolbar definitions map
	 * 
	 * @type Ext.util.MixedCollection
	 */
	_tlbs_ : null,

	/**
	 * Toolbar items definitions map
	 * 
	 * @type Ext.util.MixedCollection
	 */
	_tlbitms_ : null,

	/**
	 * Toolbar titles map. Used to retrieve the title element from a toolbar in
	 * a later stage, for example by the FrameInspector
	 * 
	 * @type Ext.util.MixedCollection
	 */
	_tlbtitles_ : null,

	/**
	 * Custom reports attached to this frame. It is an array of configuration
	 * objects with the following attributes: toolbar: the name of toolbar where
	 * the link to this report should be attached report: code of the report
	 * used to lookup the definition and rules to be invoked. title : title of
	 * the report to display in the menu
	 * 
	 * @type
	 */
	_reports_ : null,

	/**
	 * Buttons state rules. Map with functions executed in frame context to
	 * apply enabled/disabled visible/hidden state to a button. Usually the
	 * framework invokes this functions whenever a state change occurs in the
	 * frame like record change in a data-control, save, selection, etc but it
	 * can be called programmatically also whenever is necessary.
	 * 
	 * The function is called with a DC as single argument
	 * 
	 * 
	 * 
	 */
	_buttonStateRules_ : {},

	/**
	 * Frame status bar
	 * @type Ext.ux.StatusBar
	 */
	_statusBar_ : null,
	
	// ************** to be reviewed

	/**
	 * Injected configuration variables map
	 * 
	 * @type Ext.util.MixedCollection
	 */
	_configVars_ : null,

	/**
	 * Actions map
	 * 
	 * @type Ext.util.MixedCollection
	 */
	_actions_ : null,

	_name_ : null,
	_title_ : null,
	

	_header_ : null,

	// _trl_ : null,

	// **************** Public API *****************

	/**
	 * Get a data-control instance.
	 * 
	 * @param {String}
	 *            name
	 * @return {}
	 */
	_getDc_ : function(name) {
		return this._dcs_.get(name);
	},

	/**
	 * Get a toolbar instance.
	 * 
	 * @param {String}
	 *            name
	 * @return {}
	 */
	_getToolbar_ : function(name) {
		return Ext.getCmp(this._tlbs_.get(name).id);
	},

	/**
	 * Get a toolbar's configuration.
	 * 
	 * @param {String}
	 *            name
	 * @return {Object}
	 */
	_getToolbarConfig_ : function(name) {
		return this._tlbs_.get(name);
	},

	/**
	 * Get a toolbar item instance.
	 * 
	 * @param {String}
	 *            name
	 * @return {}
	 */
	_getToolbarItem_ : function(name) {
		return Ext.getCmp(this._tlbitms_.get(name).id);
	},

	/**
	 * Get a toolbar item's configuration object.
	 * 
	 * @param {String}
	 *            name
	 * @return {Object}
	 */
	_getToolbarItemConfig_ : function(name) {
		return this._tlbitms_.get(name);
	},

	/**
	 * @protected Factory method to create the data-control instances used in
	 *            this frame.
	 */
	_defineDcs_ : function() {
	},

	/**
	 * @protected Template method checked during elements definition flow.
	 *            Return false to skip the _defineDcs_ call.
	 * @return {Boolean}
	 */
	_beforeDefineDcs_ : function() {
		return true;
	},

	/**
	 * @protected Template method invoked after the elements definition flow.
	 *            Used mainly to add overrides to existing components.
	 */
	_afterDefineDcs_ : function() {
	},

	/**
	 * Factory method to create the toolbars
	 */
	_defineToolbars_ : function() {
	},

	/**
	 * Template method checked during toolbarss definition flow. Return false to
	 * skip the _defineToolbars_ call.
	 * 
	 * @return {Boolean}
	 */
	_beforeDefineToolbars_ : function() {
		return true;
	},

	/**
	 * @protected Template method invoked after the toolbar definition.
	 */
	_afterDefineToolbars_ : function() {
	},

	/**
	 * Link toolbars to view components.
	 * 
	 * @param {String}
	 *            tlbName Toolbar name
	 * @param {String}
	 *            viewName View name
	 */
	_linkToolbar_ : function(tlbName, viewName) {
		this._linkToolbarImpl_(tlbName, viewName)
	},
	/**
	 * @public Returns the builder associated with this type of component. If it
	 *         doesn't exist yet attempts to create it.
	 * @return {dnet.core.ui.FrameBuilder}
	 */
	_getBuilder_ : function() {
		if (this._builder_ == null) {
			this._builder_ = new dnet.core.ui.FrameBuilder({
						frame : this
					});
		}
		return this._builder_;
	},

	/**
	 * TOC stands for table of contents. Used when grouping multiple independent
	 * data-controls into one frame.
	 * Activates the canvas associated with the specified TOC element name.
	 * @param {String}
	 *            name TOC element name
	 */
	_showTocElement_ : function(name) {
		if (Ext.isNumber(name)) {
			var theToc = this._getElement_("_toc_").items.items[0];
			var r = theToc.store.getAt(name);
			theToc.getSelectionModel().select(r);
		} else {
			var theToc = this._getElement_("_toc_").items.items[0];
			var r = theToc.store.findRecord("name", name);
			theToc.getSelectionModel().select(r);
		}

	},

	/**
	 * Programmatically invoke a toolbar item action.
	 * @param {String} name Toolbar-item name
	 * @param {String} tlbName Toolbar name
	 */
	_invokeTlbItem_ : function(name, tlbName) {
		var b = null;
		if (!tlbName) {
			b = this._tlbitms_.get(name);
		} else {
			b = this._tlbitms_.get(tlbName + "__" + name);
		}
		if (b) {
			b.execute();
		}
	},
	
	/**
	 * Opens an assignment window component.
	 * @param {String} asgnWdwClass 
	 * @param {Object} cfg Extra configuration to apply
	 */
	_showAsgnWindow_: function(asgnWdwClass, cfg) {
		var objectId = this._dcs_.get(cfg.dc).record.data[cfg.objectIdField];
		var aw = Ext.create(asgnWdwClass, cfg);
		aw.show();
		aw._controller_.params.objectId = objectId;
		aw._controller_.initAssignement();
	}, 
	
	/**
	 * @deprecated Use _showAsgnWindow_ function. 
	 * @param {} asgnWdwClass
	 * @param {} cfg
	 */
	showAsgnWindow : function(asgnWdwClass, cfg) {
		this._showAsgnWindow_(asgnWdwClass, cfg);
	},
	
	// **************** Defaults and overrides *****************

	// **************** Private API *****************

	/**
	 * Show a generic message to user whenever an action cannot be executed because of outstanding changes.
	 * This will be moved to a specialized class with preconfigured messages. 
	 */
	_alert_dirty_ : function() {
		var msg = "Cannot execute the requested action as there are unsaved changes. <br> Save your changes or discard them then try again. ";
		Ext.Msg.show({
					title : "Action not allowed",
					msg : msg,
					scope : this,
					icon : Ext.MessageBox.INFO,
					buttons : Ext.MessageBox.OK
				});
		return;
	},
	
	
	
	_linkToolbarImpl_ : function(tlbName, viewName) {
		var tlb = this._tlbs_.get(tlbName);
		if (Ext.isEmpty(tlb)) {
			return;
		}
		var view = this._elems_.get(viewName);
		view["tbar"] = tlb;
//		var keys = [];
//		var n, kb = null;
//		for (var i = 0; i < tlb.length; i++) {
//			var ic = tlb[i]["initialConfig"];
//			if (ic) {
//				n = ic["_name_"];
//				kb = dnet.core.ui.DefaultKeyMap[n];
//				if (kb) {
//					keys[keys.length] = {
//						key : kb.key,
//						alt : kb.alt,
//						handler : this._createKeyBindingHandler_(n, tlbName),
//						scope : this,
//						stopEvent : true
//					}
//				}
//			}
//		}
//		view["keys"] = keys;
	},

	_onReady_ : function(p) {
		getApplication().setFrameTabTitle(this._name_, this._title_);
		// getApplication().registerFrameInstance(this._name_,this);
		getApplication().applyFrameCallback(this._name_, this);
		if (p == this) {
			this._onReady_();
		}
	},

	

	_config_ : function() {
	},

 
	_createKeyBindingHandler_ : function(itemName, tlbName) {
		return function() {
			this._invokeTlbItem_(itemName, tlbName);
		}
	},
 
	_applyStateButton_ : function(btnName) {  
		var btn = this._getElement_(btnName);
		if (btn) {
			var sm = btn.initialConfig.stateManager;
			if (sm) {
				var bsr = this._buttonStateRules_[btnName];
				var theDc = this._getDc_(sm.dc);
				var smstate = dnet.core.ui.FrameButtonStateManager["is_"
						+ sm.name](theDc);
				if (smstate && bsr) {
					smstate = smstate && bsr.call(this, theDc);
				}
				if (smstate) {
					btn.enable();
				} else {
					btn.disable();
				}
			}
		}
	},

	_applyStateButtons_ : function(buttonNames) {
		for (var i = 0, l = buttonNames.length; i < l; i++) {
			this._applyStateButton_(buttonNames[i]);
		}
	},

	_applyStateAllButtons_ : function() {
		this._elems_.filterBy(function(o, k) {
					return o.xtype == "button"
				}).eachKey(function(key, item) {
					this._applyStateButton_(key);
				}, this)
	},
 
 

	initComponent : function() {
		if (getApplication().getSession().rememberViewState) {
			Ext.state.Manager
					.setProvider(new Ext.state.LocalStorageProvider({}));
		}
		this._mainViewName_ = "main";
		try {
			this._trl_ = Ext.create(this.$className + "$Trl");
		} catch (e) {
			Ext.Msg.show({
						title : "Invalid language-pack",
						msg : "No translation file found for "
								+ this.$className
								+ "$Trl <br> Using the default system language.",
						icon : Ext.MessageBox.INFO,
						buttons : Ext.Msg.OK
					});
		}

		this._elems_ = new Ext.util.MixedCollection();
		this._configVars_ = new Ext.util.MixedCollection();
		this._tlbs_ = new Ext.util.MixedCollection();
		this._tlbitms_ = new Ext.util.MixedCollection();
		this._actions_ = new Ext.util.MixedCollection();
		this._dcs_ = new Ext.util.MixedCollection();
		this._buttonStateRules_ = {};

		this._statusBar_ = new Ext.ux.StatusBar({
					id : 'ui-status-bar',
					defaultText : 'Status bar. Watch for messages ... ',
					defaultIconCls : 'default-icon',
					text: 'Ready',
            		iconCls: 'x-status-valid',
					items : ['-', this._name_]
				});
		
				Ext.Ajax.on("beforerequest", function() {
					this._statusBar_.setText("Working ... ");					
				},this);
				
				Ext.Ajax.on("requestcomplete", function(conn, response, options, eOpts) {
					if(!Ext.Ajax.isLoading()){
						this._statusBar_.setText("Ready");
					}
				},this);
				
				Ext.Ajax.on("requestexception", function(conn, response, options, eOpts) {
					if(!Ext.Ajax.isLoading()){						 
						this._statusBar_.setText("Ready");
					}
					this.showAjaxErrors(response, options);
				},this);
				
				
		this._config_();
		this._startDefine_();

		/* define data-controls */
		if (this._beforeDefineDcs_() !== false) {
			this._defineDcs_();
			this._afterDefineDcs_();
		}
 

		/* define stand-alone user-interface elements */
		if (this._beforeDefineElements_() !== false) {
			this._defineElements_();
			this._afterDefineElements_();
		}

		/* define toolbars */
		if (this._beforeDefineToolbars_() !== false) {
			this._defineToolbars_();
			this._afterDefineToolbars_();
		}

		/* build the ui, linking elements */
		if (this._beforeLinkElements_() !== false) {
			this._linkElements_();
			this._afterLinkElements_();
		}
 
		this._endDefine_();

		Ext.apply(this, {
					layout : "fit",
					bbar : this._statusBar_,
					items : [this._elems_.get(this._mainViewName_)]
				});

		if (this._trl_ && this._trl_.title) {
			this._title_ = this._trl_.title;
		} else {
			this._title_ = Dnet.translate("ui", this._name_
							.substring(this._name_.lastIndexOf(".") + 1));
		}

		this.callParent(arguments);
		this.mon(this, "afterlayout", this._onReady_, this);
 
	},

	showAjaxErrors : function(response, options) {
		// Ext.MessageBox.hide();
		var msg, withDetails = false;
		if (response.responseText) {
			if (response.responseText.length > 2000) {
				msg = response.responseText.substr(0, 2000);
				withDetails = true;
			} else {
				msg = response.responseText;
			}
		} else {
			msg = "No response received from server.";
		}
		var alertCfg = {
			title : "Server message",
			msg : msg,
			scope : this,
			icon : Ext.MessageBox.ERROR,
			buttons : Ext.MessageBox.OK
		}
		if (withDetails) {
			alertCfg.buttons['cancel'] = 'Details';
			alertCfg['detailedMessage'] = response.responseText;
		}
		Ext.Msg.show(alertCfg);
	},
	
	beforeDestroy : function() { 
		this._beforeDestroyDNetView_();
		this._elems_.each(this.destroyElement, this);
		this._tlbitms_.each(function(item) {
					try {
						Ext.destroy(item);
					} catch (e) {
						alert(e);
					}
				});
		this._tlbs_.each(function(item) {
					try {
						Ext.destroy(item);
					} catch (e) {
						alert(e);
					}
				});
		this._dcs_.each(function(item) {
					try {
						Ext.destroy(item);
					} catch (e) {
						alert(e);
					}
				});
	},
	//	
	destroyElement : function(elemCfg) {
		try {
			var c = Ext.getCmp(elemCfg.id);
			if (c) {
				Ext.destroy(c);
			}
		} catch (e) {
			// alert(e);
		}
	}
});
/*
 * define the key as one of the following: 1) string: "l", "a" , ... 2) ascii
 * key code 65, 68, .... 3) Ext.EventObject.F2 , Ext.EventObject.ENTER , ....
 * are defined in Ext.EventManager
 */
//dnet.core.ui.DefaultKeyMap = Ext.apply({}, {
//			load : {
//				key : "l",
//				alt : true
//			},
//			save : {
//				key : "s",
//				alt : true
//			},
//			save_mr : {
//				key : "s",
//				alt : true
//			},
//			new_sr : {
//				key : "n",
//				alt : true
//			},
//			new_mr : {
//				key : "n",
//				alt : true
//			},
//			copy_sr : {
//				key : "c",
//				alt : true
//			},
//			copy_mr : {
//				key : "c",
//				alt : true
//			},
//			delete_sr : {
//				key : "d",
//				alt : true
//			},
//			delete_mr : {
//				key : "d",
//				alt : true
//			},
//			edit_sr : {
//				key : Ext.EventObject.ENTER,
//				alt : false
//			},
//			rollback_sr : {
//				key : "z",
//				alt : true
//			},
//			rollback_mr : {
//				key : "z",
//				alt : true
//			},
//			back_sr : {
//				key : "q",
//				alt : true
//			},
//			back_mr : {
//				key : "q",
//				alt : true
//			}
//
//		});
/**
 * Singleton class used to implement the state management rules for the
 * registered actions.
 */

dnet.core.dc.DcActionsStateManager = {

	applyStates : function(dc, ifNeeded) { 
		var flags = this._getFlags(dc);
		
		if (ifNeeded && !this.needsToRun(dc,flags ) ) {  
			return;
		}
		
		var names = dc.actionNames;
		if ((flags.hasParent && flags.parentIsNull)
				|| (flags.hasParent && flags.parentIsNew)) {
			this.disableAll(dc, names);
			return;
		}
		for ( var i = 0, l = names.length; i < l; i++) {
			var n = names[i];
			var an = "do" + n;
			if(dc.commands[an].locked === true) {
				dc.actions[an].setDisabled(true);
			} else {
				var b = this["_is" + n + "Disabled"](flags);
				dc.actions[an].setDisabled(b);
			}			
		}
		dc.lastStateManagerFlags =  flags;
	},

	needsToRun: function(dc, newFlags) {
		if (dc.lastStateManagerFlags == null) {
			return true;
		} else {
			var last = dc.lastStateManagerFlags;
			for(var f in newFlags) {
				//console.log(f+ 'new='+newFlags[f] + ' last='+last[f] );
				if (newFlags[f] != last[f] ) {
					return true;
				}
			}
		}
		
		return false;
	},
	
	
	disableAll : function(dc, names) {
		for ( var i = 0, l = names.length; i < l; i++) {
			var n = names[i];
			dc.actions["do" + n].setDisabled(true);
		}
	},

	/* public helpers */

	isSaveEnabled : function(dc) {
		return this._isSaveEnabled(this._getFlagsSave(dc));
	},
	
	isSaveDisabled : function(dc) {
		return !this.isSaveEnabled(dc);
	},

	isCancelEnabled : function(dc) {
		return this._isCancelEnabled(this._getFlagsCancel(dc));
	},
	
	isCancelDisabled : function(dc) {
		return !this.isCancelEnabled(dc);
	},

	isQueryDisabled : function(dc) {
		return this._isQueryDisabled(this._getFlagsQuery(dc));
	},
	
	isQueryEnabled : function(dc) {
		return !this.isQueryDisabled(dc);
	},

	isNewDisabled : function(dc) {
		return this._isNewDisabled(this._getFlagsNew(dc));
	},
	
	isNewEnabled : function(dc) {
		return !this.isNewDisabled(dc);
	},

	isCopyDisabled : function(dc) {
		return this._isCopyDisabled(this._getFlagsCopy(dc));
	},
	
	isCopyEnabled : function(dc) {
		return !this.isCopyDisabled(dc);
	},

	isDeleteDisabled : function(dc) {
		return this._isDeleteDisabled(this._getFlagsDelete(dc));
	},
	
	isDeleteEnabled : function(dc) {
		return !this.isDeleteDisabled(dc);
	},

	isPrevRecDisabled : function(dc) {
		return this._isPrevRecDisabled(this._getFlagsPrevRec(dc));
	},
	
	isPrevRecEnabled : function(dc) {
		return !this.isPrevRecDisabled(dc);
	},
	
	isNextRecDisabled : function(dc) {
		return this._isNextRecDisabled(this._getFlagsNextRec(dc));
	},
	
	isNextRecEnabled : function(dc) {
		return !this.isNextRecDisabled(dc);
	},
	
	isReloadRecDisabled : function(dc) {
		return this._isReloadRecDisabled(this._getFlagsReloadRec(dc));
	},
	
	isReloadRecEnabled : function(dc) {
		return !this.isReloadRecDisabled(dc);
	},

	isEditOutDisabled : function(dc) {
		return this._isEditOutDisabled(this._getFlagsEditOut(dc));
	},
	
	isEditOutEnabled : function(dc) {
		return !this.isEditOutDisabled(dc);
	},

	/* private helpers */

	_getFlags : function(dc) {
		// console.log("recordIsNull="+Ext.isEmpty(dc.record) );
		var flags = {
			isReadOnly : dc.isReadOnly(),
			isStoreDirty : dc.isStoreDirty(),
			isDirty : dc.isDirty(),
			isAnyChildDirty : dc.isAnyChildDirty(),
			recordIsNull : Ext.isEmpty(dc.record),
			isCurrentRecordDirty : dc.isCurrentRecordDirty(),
			multiEdit : dc.multiEdit,
			selectedRecordsLength : dc.getSelectedRecords().length,
			storeCount : dc.store.getCount()
		};
		this._getParentFlags(dc, flags);
		return flags;
	},

	_getParentFlags : function(dc, flags) {
		var hasParent = (dc.getParent() != null);
		var parentIsNull = (hasParent && !dc.getParent().getRecord()) ? true
				: false;
		var parentIsNew = (hasParent && !parentIsNull && dc.getParent().record.phantom) ? true
				: false;
				
		var parentisReadOnly = (hasParent && !parentIsNull && dc.getParent().isReadOnly()) ? true
				: false;		
		flags.hasParent = hasParent;
		flags.parentIsNull = parentIsNull;
		flags.parentIsNew = parentIsNew;
		flags.parentisReadOnly = parentisReadOnly;
	},

	/* decision makers implementation */

	_isQueryDisabled : function(flags) {
		return flags.isDirty || (flags.hasParent && flags.parentIsNull)
				|| (flags.hasParent && flags.parentIsNew);
	},

	_isQueryEnabled : function(flags) {
		return !this._isQueryDisabled(flags);
	},

	_getFlagsQuery : function(dc) {
		var flags = {
			isDirty : dc.isDirty()
		};
		this._getParentFlags(dc, flags);
		return flags;
	},

	_isClearQueryDisabled : function(flags) {
		return true;
	},

	_isClearQueryEnabled : function(flags) {
		return !this._isClearQueryDisabled(flags);
	},

	_isNewDisabled : function(flags) {
		return (flags.isReadOnly || flags.isAnyChildDirty || (flags.isCurrentRecordDirty && !flags.multiEdit));
	},

	_isNewEnabled : function(flags) {
		return !this._isNewDisabled(flags);
	},

	_getFlagsNew : function(dc) {
		return {
			isReadOnly : dc.isReadOnly(),
			isAnyChildDirty : dc.isAnyChildDirty(),
			isCurrentRecordDirty : dc.isCurrentRecordDirty(),
			multiEdit : dc.multiEdit
		};
	},

	_isCopyDisabled : function(flags) {
		return (flags.isReadOnly || flags.recordIsNull || flags.selectedRecordsLength != 1
				|| flags.storeCount == 0 || flags.isAnyChildDirty || (flags.isCurrentRecordDirty && !flags.multiEdit));
	},

	_isCopyEnabled : function(flags) {
		return !this._isCopyDisabled(flags);
	},

	_getFlagsCopy : function(dc) {
		return {
			isReadOnly : dc.isReadOnly(),
			isAnyChildDirty : dc.isAnyChildDirty(),
			isCurrentRecordDirty : dc.isCurrentRecordDirty(),
			multiEdit : dc.multiEdit,
			recordIsNull : Ext.isEmpty(dc.record),
			storeCount : dc.store.getCount(),
			selectedRecordsLength : dc.getSelectedRecords().length
		};
	},

	_isSaveEnabled : function(flags) {
		return (!flags.isReadOnly && (flags.isCurrentRecordDirty || flags.isStoreDirty) );
	},
	_isSaveDisabled : function(flags) {
		return !this._isSaveEnabled(flags);
	},
	_getFlagsSave : function(dc) {
		return {
			isReadOnly : dc.isReadOnly(),
			isCurrentRecordDirty : dc.isCurrentRecordDirty(),
			isStoreDirty : dc.isStoreDirty()
		};
	},

	_isCancelEnabled : function(flags) {
		return (flags.isDirty);
	},

	_isCancelDisabled : function(flags) {
		return !this._isCancelEnabled(flags);
	},

	_getFlagsCancel : function(dc) {
		return {
			isDirty : dc.isDirty()
		};
	},

	_isDeleteDisabled : function(flags) {		 
		return (flags.isReadOnly || flags.selectedRecordsLength == 0 || flags.isAnyChildDirty || flags.storeCount == 0|| (flags.isCurrentRecordDirty && !flags.multiEdit));
	},

	_isDeleteEnabled : function(flags) {
		return !this._isDeleteDisabled(flags);
	},

	_getFlagsDelete : function(dc) {
		return {
			isReadOnly : dc.isReadOnly(),
			isAnyChildDirty : dc.isAnyChildDirty(),
			selectedRecordsLength : dc.getSelectedRecords().length,
			isCurrentRecordDirty : dc.isCurrentRecordDirty(),
			multiEdit : dc.multiEdit,
			storeCount : dc.store.getCount()
		};
	},

	_isPrevRecDisabled : function(flags) {
		return (flags.storeCount == 0
				|| (flags.isCurrentRecordDirty && !flags.multiEdit) || flags.isAnyChildDirty);
	},

	_isPrevRecEnabled : function(flags) {
		return !this._isPrevRecDisabled(flags);
	},

	_getFlagsPrevRec : function(dc) {
		return {
			isAnyChildDirty : dc.isAnyChildDirty(),
			isCurrentRecordDirty : dc.isCurrentRecordDirty(),
			multiEdit : dc.multiEdit,
			storeCount : dc.store.getCount()
		};
	},

	_isNextRecDisabled : function(flags) {
		return (flags.storeCount == 0
				|| (flags.isCurrentRecordDirty && !flags.multiEdit) || flags.isAnyChildDirty); // and
	},

	_isNextRecEnabled : function(flags) {
		return !this._isNextRecDisabled(flags);
	},

	_getFlagsNextRec : function(dc) {
		return {
			isAnyChildDirty : dc.isAnyChildDirty(),
			isCurrentRecordDirty : dc.isCurrentRecordDirty(),
			multiEdit : dc.multiEdit,
			storeCount : dc.store.getCount()
		};
	},

	_isReloadRecDisabled : function(flags) {
		return (flags.storeCount == 0 || (flags.isCurrentRecordDirty));
	},

	_isReloadRecEnabled : function(flags) {
		return !this._isReloadRecDisabled(flags);
	},

	_getFlagsReloadRec : function(dc) {
		return {
			isCurrentRecordDirty : dc.isCurrentRecordDirty(),
			storeCount : dc.store.getCount()
		};
	},

	_isEditInEnabled : function(flags) {
		!this._isEditInDisabled(flags);
	},
	_isEditInDisabled : function(flags) {
		return flags.recordIsNull;
	},
	_getFlagsEditIn : function(dc) {
		return {
			recordIsNull : Ext.isEmpty(dc.record)
		};
	},

	_isEditOutDisabled : function(flags) {
		return (flags.isAnyChildDirty || (flags.isCurrentRecordDirty && !flags.multiEdit));
	},

	_isEditOutEnabled : function(flags) {
		return !this._isEditOutDisabled(flags);
	},

	_getFlagsEditOut : function(dc) {
		return {
			isAnyChildDirty : dc.isAnyChildDirty(),
			isCurrentRecordDirty : dc.isCurrentRecordDirty(),
			multiEdit : dc.multiEdit
		};
	}
};/**
 * Defines a parent-child relationship between data-controls in a frame.
 */
Ext.define("dnet.core.dc.DcContext", {
	mixins : {
		observable : 'Ext.util.Observable'
	},

	/**
	 * Reference to the parent data-control. Must be specified in initial
	 * configuration.
	 * @type dnet.core.dc.AbstractDc
	 */
	parentDc : null,

	/**
	 * Reference to the child data-control. Must be specified in initial
	 * configuration.
	 * @type dnet.core.dc.AbstractDc 
	 */
	childDc : null,
	/**
	 * Relation definition. Must be specified in initial configuration. Supports
	 * the following attributes:
	 * <li>fetchMode: 'auto' Automatically load the children data on parent
	 * record change. </li>
	 * <li>strict: true Enable /disable masterless operations, i.e. load
	 * children outside of the parent context</li>
	 * <li>fields: [ {childField:"field_name", parentField:"field_name"}, {...}
	 * ... ] Relation mapping fields. </li>
	 * 
	 */
	relation : null,

	doQueryTask : null,
	/**
	 * The values for the relation fields, the relation context data.
	 */
	ctxData : null,

	autoFetchDelay : 600,

	/**
	 * Flag used to automatically reload the data in the child DC after a new
	 * parent record has been saved. Useful if the creation of a parent
	 * generates child records in the business logic.
	 */
	reloadChildrenOnParentInsert : true,

	/**
	 * Flag used to automatically reload the data in the child DC after the
	 * parent record has been saved. Useful if the update of a parent alters
	 * data in the children or if the relation contains fields which are allowed
	 * to be modified by the user.
	 */
	reloadChildrenOnParentUpdate : false,

	constructor : function(config) {
		config = config || {};
		if (!config || !config.relation || !config.parentDc) {
			throw (dnet.core.dc.DcExceptions.DCCONTEXT_INVALID_SETUP);
		}

		Ext.apply(this, config);

		if (this.relation.strict == undefined) {
			this.relation["strict"] = true;
		}
		if (this.relation.fetchMode == undefined) {
			this.relation["fetchMode"] = "auto";
		}

		this.addEvents("dataContextChanged");

		this.mixins.observable.constructor.call(this);
		this._setup_();

	},

	_setup_ : function() {

		this._updateCtxData_();

		this.doQueryTask = new Ext.util.DelayedTask(function() {
			this.childDc.doQuery();
		}, this);

		this.parentDc.mon(this.parentDc, "recordChange", function() {
			this._updateCtxData_("recordChange");
		}, this);
		this.childDc.mon(this.childDc, "updateActionsState", function() {
			this.parentDc.updateActionsState();
		}, this);

		this.parentDc.mon(this.parentDc.store, "write", function(store, operation, eopts) {

			if (this.reloadChildrenOnParentInsert
					&& operation.action == "create") {
				this._updateCtxData_("parent_store_write");
			}
			if (this.reloadChildrenOnParentUpdate
					&& operation.action == "update") {
				this._updateCtxData_("parent_store_write");
			}
		}, this);

	},

	/**
	 * Update context data whenever context is changed.
	 */
	_updateCtxData_ : function(eventName) {

		this.ctxData = {};
		var f = this.relation.fields;
		var l = f.length;
		var r = this.parentDc.record;
		var changed = false;
		var nv = null;
		var ov = null;

		for ( var i = 0; i < l; i++) {
			ov = this.ctxData[f[i]["childField"]];
			nv = (r) ? r.get(f[i]["parentField"]) : null;
			this.ctxData[f[i]["childField"]] = nv;
			if (nv !== ov) {
				changed = true;
			}
		}

		if (!eventName) {
			return;
		}

		if (changed) {
			 
			this._updateChildFilter_();

			this.childDc.setRecord(null);

			this.childDc.store.loadData( [], false);

			this.fireEvent("dataContextChanged", this);
			
			if (this.relation.fetchMode == "auto" && this.parentDc.getRecord()
					&& !this.parentDc.getRecord().phantom) {

				this.doQueryTask.delay(this.autoFetchDelay);
			}
		}
	},

	/**
	 * Update the filter values in the child. Called usually after a context change.
	 */
	_updateChildFilter_ : function() {
		var f = this.childDc.filter;
		f.beginEdit();
		for ( var p in this.ctxData) {
			f.set(p, this.ctxData[p]);
		}
		this.childDc.filter.endEdit();
	},

	_applyContextData_ : function(record) {
		Ext.apply(record.data, this.ctxData);
	},

	destroy: function() {	
		this.clearListeners();
		delete this.childDc;
		delete this.parentDc;
	}
});
/**
 * @singleton
 */
Ext.ns("dnet.core.dc");
dnet.core.dc.DcExceptions = function(){

    return {
	     NAVIGATE_BEFORE_FIRST: "I:NAVIGATE_BEFORE_FIRST"
	    ,NAVIGATE_AFTER_LAST: "I:NAVIGATE_AFTER_LAST"

	    ,DCCONTEXT_INVALID_SETUP: "DcContext invalid setup."
	    ,CURRENT_RECORD_DIRTY: "Programmimg error! Error thrown CURRENT_RECORD_DIRTY is deprecated. Use DIRTY_DATA_FOUND instead "
	    ,PARENT_RECORD_NEW : "I:PARENT_RECORD_NEW"

	    //,NO_CURRENT_RECORD_TO_DELETE: "There is no current record selected. Nothing to delete."	
	    //,NO_SELECTED_RECORDS_TO_DELETE: "There are no selected records. Nothing to delete." 	
	    	
	    //,NO_CURRENT_RECORD_TO_EDIT: "Currently there is no record to edit."


	    ,NO_CURRENT_RECORD: "I:NO_CURRENT_RECORD"
	    ,NO_SELECTED_RECORDS: "I:NO_SELECTED_RECORDS"
	    ,DIRTY_DATA_FOUND: "I:DIRTY_DATA_FOUND"

	    ,showError: function(msg) {Ext.Msg.show({msg:msg,buttons:Ext.Msg.OK,scope:this,icon: Ext.MessageBox.ERROR});}
    	,showWarning: function(msg) {Ext.Msg.show({msg:msg,buttons:Ext.Msg.OK,scope:this,icon: Ext.MessageBox.WARNING});}
    	,showInfo: function(msg) {Ext.Msg.show({msg:msg,buttons:Ext.Msg.OK,scope:this,icon: Ext.MessageBox.INFO});}

		,showMessage: function(e) {
			if (Ext.isString(e)) {    //  alert("string: "  + e);
                var t = e.substr(0,1);
				var m = e.substr(2);
				if (!t)  this.showError(m);
				if (t=="I")  this.showInfo(Dnet.translate("exception",m));
				if (t=="E")  this.showError(Dnet.translate("exception",m));
	            if (t=="W")  this.showWarning(Dnet.translate("exception",m));
	            return;
			}
            if (e.message) {  // alert( "object: " + e);
				var m = e.name+": "+e.message + "<br>" + e.fileName + " line " + e.lineNumber;
				this.showError(Dnet.translate("exception",e));
			}

		}
	};
}();Ext.define("dnet.core.dc.DataExportForm", {
	extend : "Ext.form.Panel",

	actionButton : null,
	_elems_ : null,
	_formats_ : [ "csv", "html", "xml", "json", "pdf" ],
	_layouts_ : [ "portrait", "landscape" ],
	_grid_ : null,

	initComponent : function(config) {
		this._elems_ = new Ext.util.MixedCollection();

		this.actionButton = this.initialConfig.actionButton;
		this.actionButton.setHandler(this.executeTask, this);

		this._buildItems_();

		var cfg = {
			frame : true,
			
			fieldDefaults : {
				labelAlign : 'right',
				labelWidth : 100,  
				msgTarget : 'side'
			},
			defaults : {
				anchor : '-20'
				//anchor : '80%'
			},
			items :[ this._elems_.get("fld_format"), this._elems_.get("fld_layout"), 
				this._elems_.get("fld_columns"),this._elems_.get("fld_records")  ] 
		};
		Ext.apply(this, cfg);
		this.callParent(arguments);
		this.on("afterrender", this._on_format_changed_, this);
	},

	_getElement_ : function(name) {
		return Ext.getCmp(this._elems_.get(name).id);
	},

	_getElementConfig_ : function(name) {
		return this._elems_.get(name);
	},

	_on_format_changed_ : function(nv) { //return ;
		if (nv == "pdf") {
			this._getElement_("fld_columns_listvisible").setValue(true);
			this._getElement_("fld_columns_listall").disable();
			// this._getElement_("fld_columns_all").disable();
			this._getElement_("fld_layout").enable();
		} else {
			this._getElement_("fld_columns_listall").enable();
			// this._getElement_("fld_columns_all").enable();
			this._getElement_("fld_layout").disable();
		}
	},

	

	_buildItems_ : function() {

		this._elems_.add("fld_format", {
			fieldLabel : Dnet.translate("dcvgrid", "exp_format"),
			xtype : "combo",
			allowBlank : false,
			selectOnFocus : true,
			//width : 100,
			id : Ext.id(),
			triggerAction : "all",
			store : this._formats_,
			value : this._formats_[0],
			listeners : {
				"change" : {
					scope : this,
					fn : function(fld, nv, ov) {
						this._on_format_changed_(nv);
					}
				}
			}
		});
		this._elems_.add("fld_layout", {
			fieldLabel : Dnet.translate("dcvgrid", "exp_layout"),
			xtype : "combo",
			allowBlank : false,
			selectOnFocus : true,
			//width : 100,
			id : Ext.id(),
			triggerAction : "all",
			store : this._layouts_,
			value : this._layouts_[0]
		});

		this._elems_.add("fld_columns_listvisible", {
			boxLabel : Dnet.translate("dcvgrid", "exp_col_visible"),
			inputValue : 'v',
			name : "fld_columns",
			id : Ext.id(),
			checked : true
		});
		this._elems_.add("fld_columns_listall", {
			boxLabel : Dnet.translate("dcvgrid", "exp_col_all"),
			inputValue : 'l',
			name : "fld_columns",
			id : Ext.id()
		});

		this._elems_.add("fld_columns", {
			fieldLabel : Dnet.translate("dcvgrid", "exp_columns"),
			xtype : "radiogroup",
			//itemCls : "x-check-group-alt",
			style:"border-top:1px outset",
			columns : 1,
			id : Ext.id(),
			items : [ this._elems_.get("fld_columns_listvisible"),
					this._elems_.get("fld_columns_listall") ]
		});

		this._elems_.add("fld_records_selected", {
			boxLabel : Dnet.translate("dcvgrid", "exp_rec_sel"),
			inputValue : 's',
			name : "fld_records",
			id : Ext.id(),
			disabled : true
		});
		this._elems_.add("fld_records_currentpage", {
			boxLabel : Dnet.translate("dcvgrid", "exp_rec_pag"),
			inputValue : 'c',
			name : "fld_records",
			id : Ext.id(),
			disabled : true
		});
		this._elems_.add("fld_records_allpage", {
			boxLabel : Dnet.translate("dcvgrid", "exp_rec_all"),
			inputValue : 'a',
			name : "fld_records",
			id : Ext.id(),
			checked : true
		});
		this._elems_.add("fld_records", new Ext.form.RadioGroup( {
			fieldLabel : Dnet.translate("dcvgrid", "exp_records"),
			xtype : "radiogroup",
			//itemCls : "x-check-group-alt",
			style:"border-top:1px outset",
			columns : 1,
			id : Ext.id(),
			items : [ this._elems_.get("fld_records_selected"),
					this._elems_.get("fld_records_currentpage"),
					this._elems_.get("fld_records_allpage") ]
		}));

	},

	executeTask : function() {

		var ctrl = this._grid_._controller_;
		var url = Dnet.dsAPI(ctrl.dsName, this._getElement_("fld_format")
				.getValue());

		var filterData = ctrl.filter.data;
		var request = dnet.core.base.RequestParamFactory.findRequest(filterData);
		for ( var p in request.data) {
			if (request.data[p] == "") {
				request.data[p] = null;
			}
		}

		var data = Ext.encode(request.data);
		request.data = data;
		var params = {};
		var fcv = this._getElement_("fld_columns").getValue().fld_columns;

		if (this._getElement_("fld_records").getValue().fld_records == "c") {
			params["resultSize"] = 30;
			params["resultStart"] = 0;
		}
		
		var sortCols = "";
		var sortDirs = "";
		var first = true;
		ctrl.store.sorters.each(function(item, idx,len) {
			if(!first) {
				sortCols += ",";
				sortDirs += ",";
			}
			sortCols += item.property;
			sortDirs += item.direction || "ASC";
			first = false;
		},this); 
		  
		 params[Dnet.requestParam.SORT] = sortCols;
		 params[Dnet.requestParam.SENSE] = sortDirs;
		 
		if (fcv != "a") {
			var gridCm = this._grid_.columns;
			var cs = '';
			var cst = '';
			var csw = '';
			var cnt = 0;
			var len = gridCm.length;
			for ( var i = 0; i < len; i++) {
				if (fcv == "l" || !gridCm[i].hidden) {
					cs += (cnt > 0) ? "," : "";
					cs += gridCm[i].dataIndex;
					cst += (cnt > 0) ? "," : "";
					cst += gridCm[i].text.replace(",", " ");
					csw += (cnt > 0) ? "," : "";
					csw += gridCm[i].width;
					cnt++;
				}
			}
			params[Dnet.requestParam.EXPORT_COL_NAMES] = cs;
			params[Dnet.requestParam.EXPORT_COL_TITLES] = cst;
			params[Dnet.requestParam.EXPORT_COL_WIDTHS] = csw;

		}
		var opts = "adress=yes, width=710, height=450,"
				+ "scrollbars=yes, resizable=yes,menubar=yes";
		var v = window.open(url["exportdata"] + "&data=" + request.data + "&"
				+ Ext.urlEncode(params), 'Export', opts);
		v.focus();
	}
});

Ext.define("dnet.core.dc.DataExportWindow", {
	extend : "Ext.Window",

	_grid_ : null,

	initComponent : function(config) {

		var btn = Ext.create('Ext.Button', {
			text : Dnet.translate("dcvgrid", "exp_run"),
			iconCls : 'icon-action-run'
		});

		var cfg = {
			title : Dnet.translate("dcvgrid", "exp_title"),
			border : true,
			width : 350,
			resizable : false,
			closeAction : "hide",			 
			closable : true,
			constrain : true,
			buttonAlign : "center",
			modal : true,
			items : new dnet.core.dc.DataExportForm( {
				actionButton : btn,
				_grid_ : this._grid_
			}),
			buttons : [ btn ]
		};

		Ext.apply(cfg, config);
		Ext.apply(this, cfg);
		this.callParent(arguments);
	}

});

Ext.define("dnet.core.dc.DataImportWindow", {
	extend:  "Ext.Window" ,
  
 

    _elems_ : new Ext.util.MixedCollection()
   ,_formats_: ["csv", "json" ]

   ,_grid_:null

   ,_on_format_changed_: function(nv) {

   }

   ,initComponent:function() {

     Ext.apply(this, arguments);

     this._elems_.add("fld_format",{fieldLabel: Dnet.translate("dcvgrid", "imp_format"), xtype:"combo", allowBlank:false, selectOnFocus:true, width:100, id: Ext.id()
	 				, triggerAction:"all", store:this._formats_,value:this._formats_[0] , disabled: true
					,listeners: {
						"change" : {
							scope:this, fn: function(fld, nv,ov) {
								 this._on_format_changed_(nv);
							}
						}
					} });
      this._elems_.add("fld_file",{  buttonOnly: false, fieldLabel: Dnet.translate("dcvgrid", "imp_file"),
	   xtype:"fileuploadfield",   id: Ext.id() , width:300
 		 });
      this._elems_.add("fld_notes",{  buttonOnly: false, fieldLabel: Dnet.translate("dcvgrid", "imp_notes_lbl"),
	   xtype:"label",  text:Dnet.translate("dcvgrid", "imp_notes")
 		 });
    this._elems_.add("fld_strategy_pos",{boxLabel: Dnet.translate("dcvgrid", "imp_strgy_pos"), inputValue: 'p', name:"fld_strategy", id: Ext.id(),checked:true  });
    this._elems_.add("fld_strategy_bean",{boxLabel: Dnet.translate("dcvgrid", "imp_strgy_bean"), inputValue: 'b', name:"fld_strategy", id: Ext.id() });

	this._elems_.add("fld_strategy", {fieldLabel: Dnet.translate("dcvgrid", "imp_strgy"), xtype:"radiogroup", itemCls: "x-check-group-alt", columns: 1, id: Ext.id()
			    ,items: [ this._elems_.get("fld_strategy_pos"),this._elems_.get("fld_strategy_bean") ] });


     Ext.apply(this, {
           items:[this._elems_.get("fld_format"), this._elems_.get("fld_file"), this._elems_.get("fld_strategy"), this._elems_.get("fld_notes")  ]

          ,layout:"form"

          ,fileUpload: true
          ,closable: true
          ,closeAction: 'hide'
          ,bodyStyle:"padding:10 10 8 5;"
          ,title: Dnet.translate("dcvgrid", "imp_title")
          ,width: 650
          ,resizable:false
          ,autoHeight:true
          ,modal: true
          ,labelAlign:"right"
          ,buttonAlign:"center"
          ,labelWidth:100
          ,buttons:[
              {xtype:"button",scope:this, text:Dnet.translate("dcvgrid", "imp_run"), iconCls: 'icon-action-run', handler:function() {this.executeTask()} }
           //  ,{xtype:"button",scope:this, text:"Cancel", iconCls: 'icon-action-reset', handler:function() {this.hide()} }
          ]
       });
	 this.on("show",function() {this._on_format_changed_(this._getElement_("fld_format").getValue() ) ;} , this);
     dnet.core.dc.DataImportWindow.superclass.initComponent.apply(this, arguments);
   }
	,_getElement_: function(name) {  return Ext.getCmp( this._elems_.get(name).id); }
    ,_getElementConfig_: function(name) {  return this._elems_.get(name); }

    ,executeTask: function() {
		alert("Temporarily disabled");
		return ;
        var ctrl = this._grid_._controller_;
        var params = {};
		// read column layout
        if ( true ) {
			var gridCm = this._grid_.getColumnModel();
			var cs = '';  // visible columns
			var cst = '';  // visible columns title
			//var csw = ''; //visible columns width
			var cnt=0;
			for(var i=0; i<gridCm.getColumnCount(); i++) {
				if( /*this._getElement_("fld_columns").getValue().inputValue == "l" || */ ! gridCm.isHidden(i) ) {
					cs += (cnt>0)?",":"";
					cs += gridCm.getDataIndex(i);
			    	cst += (cnt>0)?",":"";
			        cst += gridCm.getColumnHeader(i).replace(","," ");
			        //csw += (cnt>0)?",":"";
			        //csw += gridCm.getColumnWidth(i);
			        cnt++;
				}
			}
			params[Dnet.requestParam.EXPORT_COL_NAMES] = cs;
			params[Dnet.requestParam.EXPORT_COL_TITLES] = cst;
			//params[Dnet.requestParam.EXPORT_COL_WIDTHS] = csw;

		}

        Ext.Ajax.request({
             method:"POST"
            ,params:params
            ,scope:this
            //,failure:this.doLoginFailure
		//	,success:this.doLoginSuccess
            ,url: Dnet.dsAPI(ctrl.dsName , "json" ) ["importdata"]
            ,timeout:600000
        });



   }
}); Ext.define('dnet.core.dc.SortItemSelector', {
    extend: 'Ext.ux.form.ItemSelector',
    alias: ['widget.sortitemselectorfield'],
    buttons: ['top', 'up', 'add', 'remove', 'down', 'bottom', 'desc'],

    buttonsText: {
        top: "Move to Top",
        up: "Move Up",
        add: "Add to Selected",
        remove: "Remove from Selected",
        down: "Move Down",
        bottom: "Move to Bottom",
        desc: "Change sort sense(ASC/DESC)"
    }, 
    
     
    onDescBtnClick : function() {
    	//alert();
        var list = this.toField.boundList,
            store = list.getStore(),
            selected = this.getSelections(list),
            i = 0,
            len = selected.length,
            max = store.getCount(),
            selection,
            index;
            
        store.suspendEvents();
        for (; i < len; ++i) {
            selection = selected[i];
            if (selection.data.sense == "DESC") {
            	selection.data.sense = "";
            	selection.data.text2 = selection.data.text;
            } else {
            	selection.data.sense = "DESC";
            	selection.data.text2 = selection.data.text+"-DESC";
            }
            //index = Math.min(max, store.indexOf(selection) + 1);
            //store.remove(selection);
            //store.insert(index, selection);
        }
        store.resumeEvents();
        list.refresh();
    },
    
    onRemoveBtnClick : function() {
        var me = this,i = 0,
            toList = me.toField.boundList,
            selected = this.getSelections(toList),
            len = selected.length;
        for (; i < len; ++i) {
         	selection = selected[i];
         	selection.data.sense = "";
         	selection.data.text2 = selection.data.text;
        }
        toList.getStore().remove(selected);
        this.fromField.boundList.getStore().add(selected);
    } 
    
    
 });
 
 
 
Ext.define("dnet.core.dc.DataSortWindow", {
	extend : "Ext.window.Window",

	/**
	 * 
	 * @type dnet.core.dc.AbstractDcvGrid
	 */
	_grid_ : null,
	_selectorId_ : null,
	
	
	//TODO: optimize this
	
	initComponent : function(config) {

		var btn = Ext.create('Ext.Button', {
			text : Dnet.translate("dcvgrid", "sort_run"),
			//iconCls : 'icon-action-sort',
			scope: this,
			handler:this.executeTask
		});

		var avlCol = [];
		this._grid_._columns_.each(function(item, idx,len) {
			avlCol[avlCol.length] = [item.name, item.header, '',item.header+'' ]
		})
		 
	    var selCol = [];
	    this._grid_.store.sorters.each(function(item, idx,len) {
			selCol[selCol.length] = [item.property]
			if (item.direction == "DESC") {
				for (var j=0;j<avlCol.length;j++) {
					var e = avlCol[j];
					if (e[0] == item.property) {
						e[2] = "DESC";
						e[3] = e[1] + "-DESC";
					}
				}
			}
		})
		var ds = Ext.create('Ext.data.ArrayStore', {
	        data: avlCol,
	        fields: ['value','text','sense','text2'],
	        sortInfo: {
	            field: 'text',
	            direction: 'ASC'
	        }
	    });
	    
	    ds.sort();
	    this._selectorId_ = Ext.id();
		var cfg = {
			title : Dnet.translate("dcvgrid", "sort_title"),
			border : true,
			width : 350,
			height:300,
			resizable : true,
			//closeAction : "hide",			 
			closable : true,
			constrain : true,
			buttonAlign : "center",
			modal : true,
			layout:"fit",
			items : [{
	            xtype: 'sortitemselectorfield',
	            //fieldLabel : Dnet.translate("dcvgrid", "exp_columns"),
	            id : this._selectorId_,
	           // width: 300,
	           // height: 300,
	            //imagePath: '../ux/images/',
	            store: ds,
	            displayField: 'text2',
	            valueField: 'value',
	            value: selCol,
	             
	            msgTarget: 'top'
	        }],
			buttons : [ btn ]
		};

		Ext.apply(cfg, config);
		Ext.apply(this, cfg);
		this.callParent(arguments);
	},
	
	getSelector : function() {
		return Ext.getCmp(this._selectorId_);
	},
	executeTask : function() {
		var s = this.getSelector(),
			sorts = s.toField.boundList.store.data.items
			len = sorts.length
			store = this._grid_.store 
			;
		var newsort = []; 
		store.sorters.clear();
		for(var i=0;i<len;i++) {
			newsort.push({
			    property : sorts[i].data.value,
			    direction: sorts[i].data.sense || "ASC"
			})			 
		}	
		this._grid_._controller_.store.sort(newsort);	 
		this.close();
	}

});
Ext.define("dnet.core.dc.GridLayoutManager$Model" ,{
	extend: 'Ext.data.Model',
	fields: [ 
		{name:"id", type:"int", useNull:true},
		{name:"owner", type:"string"},		 
		{name:"name", type:"string"},
		{name:"cmp", type:"string"},
		{name:"cmpType", type:"string"},
		{name:"stateValue", type:"string"}
	]
});

Ext.define("dnet.core.dc.GridLayoutManager", {
			extend : "Ext.Window",
			_selectedId_ : null,
			_elems_ : null,
			_grid_ : null,

			initComponent : function() {
				this._elems_ = new Ext.util.MixedCollection();

				this._buildItems_();

				Ext.apply(this, arguments);

				Ext.apply(this, {
							closable : true,
							closeAction : 'hide',
							bodyStyle : "padding:10 10 8 5;",
							title : Dnet.translate("dcvgrid", "layout_title"),
							width : 400,
							height : 200,
							modal : true,
							buttonAlign : "center",
							items : [this._elems_.get("panel_manage")],
							buttons : [this._elems_.get("btn_apply"),
									this._elems_.get("btn_cancel")],
							rbar : [this._elems_.get("btn_save"),
									this._elems_.get("btn_del"), "-",
									this._elems_.get("btn_share"), "->",
									this._elems_.get("btn_saveas") 
									
									]
						});

				this.callParent(arguments);
			},

			_getElement_ : function(name) {
				return Ext.getCmp(this._elems_.get(name).id);

			},
			_getElementConfig_ : function(name) {
				return this._elems_.get(name);
			},

			/* ============== handlers ============== */

			_doSave_ : function() {
				var s = this._getElement_("fld_views").store;
				var r = s.getById(  this._selectedId_  );
				r.set("stateValue", Ext.JSON.encode(this._grid_._getViewState_()) );
				//stateValue: Ext.JSON.encode(this._grid_.getState()),
				s.commitChanges();
				this.close();
			},
			_doSaveAs_ : function() {
		    	Ext.Msg.prompt( "Save view as...", "Name:", this._onDoSaveAs_, this );
		    	
			},
			_onDoSaveAs_ : function(btn, name) {
				if (btn != "ok") {
					return;
				}
				var n = name.trim();
				if (!n) {
					Ext.Msg.show({
 	    			   title: 'Invalid value',
 	    			   msg: 'Name must not be empty.',
 	    			   buttons: Ext.MessageBox.OK,
 	    			   icon: Ext.MessageBox.ERROR
 	    			});
 	    			return true;
				}
				
				Ext.Ajax.request({			
				    url: Dnet.dsAPI("UiViewStateDs", "json").create, 
				    method:"POST",
				    params: {
				    	data: Ext.JSON.encode({
				    		name: name,
				    		stateValue: Ext.JSON.encode(this._grid_._getViewState_()),
				    		cmp: this._grid_.stateId,
				    		cmpType: "frame-dcgrid"
				    	})
				    },
				    success: function(response, opts) {
				        this.close();
				    } ,
				    failure: function(response, opts) {
	        		 	Ext.Msg.show({
							title : "Error",
							msg:response.responseText,	
							icon : Ext.MessageBox.ERROR,
							buttons : Ext.Msg.OK
						});
				    } ,
				    scope: this
				});
		    	 
			},
			_doShare_ : function() {
				alert("Temporarily disabled");
			},
			_doDelete_ : function() {
				var s = this._getElement_("fld_views").store;
				s.remove(s.getById(  this._selectedId_  ) );
			},
			_doApply_ : function() {
				var r = this._getElement_("fld_views").store.getById(  this._selectedId_  ) ;
			 	this._grid_._applyViewState_(Ext.JSON.decode(r.get("stateValue")));
			 	this.close();
			},
			_doCancel_ : function() {
				this.close();
			},

			onBeforeLoadStates: function(store,op,eOpts) {
				var p ={};
				
				p.params = Ext.encode({
						hideMine: this._getElement_("fld_hide_mine").checked,
						hideOthers: this._getElement_("fld_hide_others").checked
					});
				p.data = Ext.encode({
						//name: (this._getElement_("fld_views").getRawValue() || "")  +"*"
						cmp:this._grid_.stateId
					});	
				store.proxy.extraParams.params = p.params;
				store.proxy.extraParams.data = p.data;
				//store.load();
			},
			
			_applyButtonStates_: function(record) {
				if (record) {
					this._getElement_("btn_apply").enable();
				} else {
					this._getElement_("btn_apply").disable();
				}
				if (record && record.data.owner == getApplication().getSession().user.code ) {
					this._getElement_("btn_save").enable();
					this._getElement_("btn_del").enable();
					this._getElement_("btn_share").enable();
					
				} else {
					this._getElement_("btn_save").disable();
					this._getElement_("btn_del").disable();
					this._getElement_("btn_share").disable();
					
				}
				
				
			},
			
			/* ============= build elements ================ */

			_buildItems_ : function() {

				this._buildFields_();
				this._buildButtons_();

				this._elems_.add("panel_manage", {

							xtype : "container",
							layout : {
								type : "form"
							},
							items : [this._elems_.get("fld_views"),
									this._elems_.get("fld_hide_mine"),
									this._elems_.get("fld_hide_others")

							],
							id : Ext.id()
						});

			},

			/**
			 * Build form buttons.
			 */
			_buildButtons_ : function() {
				this._elems_.add("btn_apply", {
							text : "Apply",
							xtype : "button",
							disabled:true,
							handler : this._doApply_,
							scope : this,
							id : Ext.id()
						});
				this._elems_.add("btn_cancel", {
							text : "Cancel",
							xtype : "button",
							handler : this._doCancel_,
							scope : this,
							id : Ext.id()
						});

				this._elems_.add("btn_saveas", {
							text : "Save as",
							xtype : "button",
							handler : this._doSaveAs_,
							scope : this,
							id : Ext.id()
						});
				this._elems_.add("btn_save", {
							text : "Save",
							xtype : "button",
							disabled:true,
							handler : this._doSave_,
							scope : this,
							id : Ext.id()
						});
				this._elems_.add("btn_del", {
							text : "Delete",
							xtype : "button",
							disabled:true,
							handler : this._doDelete_,
							scope : this,
							id : Ext.id()
						});
				this._elems_.add("btn_share", {
							text : "Share",
							xtype : "button",
							disabled:true,
							handler : this._doShare_,
							scope : this,
							id : Ext.id()
						});
			},

			/**
			 * Build form fields.
			 */
			_buildFields_ : function() {
				this._elems_.add("fld_views", {
							fieldLabel : "Layouts",
							xtype : "combo",
							 
							selectOnFocus : true,
							forceSelection: true,
							remoteSort : true,
							remoteFilter: true,
							autoSelect: true,
							autoScroll:true,
							allQuery : "*",
							//queryCaching: false,
							id : Ext.id(),
							displayField:"name",
							queryMode: 'remote' ,
							queryDelay : 100,
							triggerAction : "all",
							 
							listeners: {
								select: {
									scope:this,
									fn: function(combo,recs,eOpts) {
										this._selectedId_ = recs[0].data.id;
										//alert(this._selectedId_);
										this._applyButtonStates_(recs[0]);
									}
								}
							},
							listConfig: {		
								getInnerTpl: function() { 
									return '<span>{name} <br>  <b>By</b> <i>{owner}</i></span>';
								},
								width:250
							},
							 
							store : Ext.create('Ext.data.Store', {
							     model: 'dnet.core.dc.GridLayoutManager$Model',
							     autoSync : true,
							     listeners:{
							     	beforeload: {
										scope : this,
										fn : this.onBeforeLoadStates
									}
								 },  
							     proxy : {
									type : 'ajax',
									api : Dnet.dsAPI("UiViewStateRTLovDs", "json"),
									actionMethods : { 
										create : 'POST',
										read : 'POST',
										update : 'POST',
										destroy : 'POST' 
									},
									reader : {
										type : 'json',
										root : 'data',
										idProperty : 'id',
										totalProperty : 'totalCount' 
									},
									writer : {
										type : 'json',
										encode : true,
										root: "data",
										allowSingle : false,
										writeAllFields : true
									},
									listeners : {
										"exception" : {
											fn : this.proxyException,
											scope : this
										}
									},
									startParam : Dnet.requestParam.START,
									limitParam : Dnet.requestParam.SIZE,
									sortParam : Dnet.requestParam.SORT,
									directionParam : Dnet.requestParam.SENSE
								}
							      
							 })
							
						});

				this._elems_.add("fld_hide_mine", {
							fieldLabel : "Hide mine",
							xtype : "checkbox",
							listeners: {
								change: {
									scope: this,
									fn: function(field,nv,ov) {
										this._getElement_("fld_views").store.load(); 
									}
								}
							},
							id : Ext.id()
						});
				this._elems_.add("fld_hide_others", {
							fieldLabel : "Hide other's",
							xtype : "checkbox",
							listeners: {
								change: {
									scope: this,
									fn: function(field,nv,ov) {
										this._getElement_("fld_views").store.load();
									}
								}
							},
							id : Ext.id()
						});
				this._elems_.add("fld_name", {
							fieldLabel : "Name",
							xtype : "textfield",
							id : Ext.id()
						});
				this._elems_.add("fld_public", {
							fieldLabel : "Public",
							xtype : "checkbox",
							id : Ext.id()
						});
			}

		});

Ext.define('dnet.core.lov.AbstractRemoteCombo', {
    extend:'Ext.form.field.Picker',
    requires: ['Ext.util.DelayedTask', 
               'Ext.EventObject', 
               'Ext.view.BoundList', 
               'Ext.view.BoundListKeyNav', 
               'Ext.data.StoreManager'],
    alternateClassName: 'Ext.form.ComboBox',
    alias: ['widget.remotecombo' ],
 
    triggerCls: Ext.baseCSSPrefix + 'form-arrow-trigger',
   // multiSelect: false,
 
    delimiter: ', ',

    
    displayField: 'text',

    
    triggerAction: 'all',
 
    allQuery: '',

   
    queryParam: 'query',

    
    queryMode: 'remote',

    queryCaching: true,

   
    pageSize: 0,

    
    autoSelect: true,
 
    typeAhead: false,
 
    typeAheadDelay: 250,

   
    selectOnTab: true,

   
    forceSelection: false,

    defaultListConfig: {
        emptyText: '',
        loadingText: 'Loading...',
        loadingHeight: 70,
        minWidth: 70,
        maxHeight: 300,
        shadow: 'sides'
    },
 

    //private
    ignoreSelection: 0,

    initComponent: function() {
        var me = this,
            isDefined = Ext.isDefined,
            store = me.store,
            transform = me.transform,
            transformSelect, isLocalMode;

        //<debug>
        if (!store && !transform) {
            Ext.Error.raise('Either a valid store, or a HTML select to transform, must be configured on the combo.');
        }
//        if (me.typeAhead && me.multiSelect) {
//            Ext.Error.raise('typeAhead and multiSelect are mutually exclusive options -- please remove one of them.');
//        }
        if (me.typeAhead && !me.editable) {
            Ext.Error.raise('If typeAhead is enabled the combo must be editable: true -- please change one of those settings.');
        }
        if (me.selectOnFocus && !me.editable) {
            Ext.Error.raise('If selectOnFocus is enabled the combo must be editable: true -- please change one of those settings.');
        }
        //</debug>

        this.addEvents(
            // TODO need beforeselect?
 
            'beforequery',
 
            'select'
        );

        // Build store from 'transform' HTML select element's options
        if (!store && transform) {
            transformSelect = Ext.getDom(transform);
            if (transformSelect) {
                store = Ext.Array.map(Ext.Array.from(transformSelect.options), function(option) {
                    return [option.value, option.text];
                });
                if (!me.name) {
                    me.name = transformSelect.name;
                }
                if (!('value' in me)) {
                    me.value = transformSelect.value;
                }
            }
        }

        me.bindStore(store, true);
        store = me.store;
        if (store.autoCreated) {
            me.queryMode = 'local';
            me.valueField = me.displayField = 'field1';
            if (!store.expanded) {
                me.displayField = 'field2';
            }
        }


        if (!isDefined(me.valueField)) {
            me.valueField = me.displayField;
        }

        isLocalMode = me.queryMode === 'local';
        if (!isDefined(me.queryDelay)) {
            me.queryDelay = isLocalMode ? 10 : 500;
        }
        if (!isDefined(me.minChars)) {
            me.minChars = isLocalMode ? 0 : 4;
        }

        if (!me.displayTpl) {
            me.displayTpl = Ext.create('Ext.XTemplate',
                '<tpl for=".">' +
                    '{[typeof values === "string" ? values : values.' + me.displayField + ']}' +
                    '<tpl if="xindex < xcount">' + me.delimiter + '</tpl>' +
                '</tpl>'
            );
        } else if (Ext.isString(me.displayTpl)) {
            me.displayTpl = Ext.create('Ext.XTemplate', me.displayTpl);
        }

        me.callParent();

        me.doQueryTask = Ext.create('Ext.util.DelayedTask', me.doRawQuery, me);

        // store has already been loaded, setValue
        if (me.store.getCount() > 0) {
            me.setValue(me.value);
        }

        // render in place of 'transform' select
        if (transformSelect) {
            me.render(transformSelect.parentNode, transformSelect);
            Ext.removeNode(transformSelect);
            delete me.renderTo;
        }
    },

    beforeBlur: function() {
        var me = this;
        me.doQueryTask.cancel();
        me.assertValue();
//        if (me.forceSelection) {
//            me.assertValue();
//        } else {
//            me.collapse();
//        }
    },

    // private
    assertValue: function() {
        var me = this,
            value = me.getRawValue(),
            rec;

//        if (me.multiSelect) {
//            // For multiselect, check that the current displayed value matches the current
//            // selection, if it does not then revert to the most recent selection.
//            if (value !== me.getDisplayValue()) {
//                me.setValue(me.lastSelection);
//            }
//        } else {
            // For single-select, match the displayed value to a record and select it,
            // if it does not match a record then revert to the most recent selection.
            rec = me.findRecordByDisplay(value);
            if (rec) {
                me.select(rec);
            } else {
                me.setValue(me.lastSelection);
            }
        //}
        me.collapse();
    },

    onTypeAhead: function() {
        var me = this,
            displayField = me.displayField,
            record = me.store.findRecord(displayField, me.getRawValue()),
            boundList = me.getPicker(),
            newValue, len, selStart;

        if (record) {
            newValue = record.get(displayField);
            len = newValue.length;
            selStart = me.getRawValue().length;

            boundList.highlightItem(boundList.getNode(record));

            if (selStart !== 0 && selStart !== len) {
                me.setRawValue(newValue);
                me.selectText(selStart, newValue.length);
            }
        }
    },

    // invoked when a different store is bound to this combo
    // than the original
    resetToDefault: function() {

    },

    bindStore: function(store, initial) {
        var me = this,
            oldStore = me.store;

        // this code directly accesses this.picker, bc invoking getPicker
        // would create it when we may be preping to destroy it
        if (oldStore && !initial) {
            if (oldStore !== store && oldStore.autoDestroy) {
                oldStore.destroy();
            } else {
                oldStore.un({
                    scope: me,
                    load: me.onLoad,
                    exception: me.collapse
                });
            }
            if (!store) {
                me.store = null;
                if (me.picker) {
                    me.picker.bindStore(null);
                }
            }
        }
        if (store) {
            if (!initial) {
                me.resetToDefault();
            }

            me.store = Ext.data.StoreManager.lookup(store);
            me.store.on({
                scope: me,
                load: me.onLoad,
                exception: me.collapse
            });

            if (me.picker) {
                me.picker.bindStore(store);
            }
        }
    },

    onLoad: function() {
        var me = this,
            value = me.value;
		
        me.syncSelection();
        if (me.picker && !me.picker.getSelectionModel().hasSelection()) {
            me.doAutoSelect();
        }
    },

    /**
     * @private
     * Execute the query with the raw contents within the textfield.
     */
    doRawQuery: function() {
        this.doQuery(this.getRawValue());
    },

    /**
     * Executes a query to filter the dropdown list. Fires the {@link #beforequery} event prior to performing the
     * query allowing the query action to be canceled if needed.
     * @param {String} queryString The SQL query to execute
     * @param {Boolean} forceAll <code>true</code> to force the query to execute even if there are currently fewer
     * characters in the field than the minimum specified by the <code>{@link #minChars}</code> config option.  It
     * also clears any filter previously saved in the current store (defaults to <code>false</code>)
     * @return {Boolean} true if the query was permitted to run, false if it was cancelled by a {@link #beforequery} handler.
     */
    doQuery: function(queryString, forceAll) {
        queryString = queryString || '';

        // store in object and pass by reference in 'beforequery'
        // so that client code can modify values.
        var me = this,
            qe = {
                query: queryString,
                forceAll: forceAll,
                combo: me,
                cancel: false
            },
            store = me.store,
            isLocalMode = me.queryMode === 'local';

        if (me.fireEvent('beforequery', qe) === false || qe.cancel) {
            return false;
        }

        // get back out possibly modified values
        queryString = qe.query;
        forceAll = qe.forceAll;

        // query permitted to run
        if (forceAll || (queryString.length >= me.minChars)) {
            // expand before starting query so LoadMask can position itself correctly
            me.expand();

            // make sure they aren't querying the same thing
            if (!me.queryCaching || me.lastQuery !== queryString) {
                me.lastQuery = queryString;

                if (isLocalMode) {
                    // forceAll means no filtering - show whole dataset.
                    if (forceAll) {
                        store.clearFilter();
                    } else {
                        // Clear filter, but supress event so that the BoundList is not immediately updated.
                        store.clearFilter(true);
                        store.filter(me.displayField, queryString);
                    }
                } else {
                    // In queryMode: 'remote', we assume Store filters are added by the developer as remote filters,
                    // and these are automatically passed as params with every load call, so we do *not* call clearFilter.
                    store.load({
                        params: me.getParams(queryString)
                    });
                }
            }

            // Clear current selection if it does not match the current value in the field
            if (me.getRawValue() !== me.getDisplayValue()) {
                me.ignoreSelection++;
                me.picker.getSelectionModel().deselectAll();
                me.ignoreSelection--;
            }

            if (isLocalMode) {
                me.doAutoSelect();
            }
            if (me.typeAhead) {
                me.doTypeAhead();
            }
        }
        return true;
    },

    // private
    getParams: function(queryString) {
        var p = {},
            pageSize = this.pageSize,
            param = this.queryParam;

        if (param) {
            p[param] = queryString;
        }

        if (pageSize) {
            p.start = 0;
            p.limit = pageSize;
        }
        return p;
    },

    /**
     * @private
     * If the autoSelect config is true, and the picker is open, highlights the first item.
     */
    doAutoSelect: function() {
        var me = this,
            picker = me.picker,
            lastSelected, itemNode;
        if (picker && me.autoSelect && me.store.getCount() > 0) {
            // Highlight the last selected item and scroll it into view
            lastSelected = picker.getSelectionModel().lastSelected;
            itemNode = picker.getNode(lastSelected || 0);
            if (itemNode) {
                picker.highlightItem(itemNode);
                picker.listEl.scrollChildIntoView(itemNode, false);
            }
        }
    },

    doTypeAhead: function() {
        if (!this.typeAheadTask) {
            this.typeAheadTask = Ext.create('Ext.util.DelayedTask', this.onTypeAhead, this);
        }
        if (this.lastKey != Ext.EventObject.BACKSPACE && this.lastKey != Ext.EventObject.DELETE) {
            this.typeAheadTask.delay(this.typeAheadDelay);
        }
    },

    onTriggerClick: function() {
        var me = this;
        if (!me.readOnly && !me.disabled) {
            if (me.isExpanded) {
                me.collapse();
            } else {
                me.onFocus({});
                if (me.triggerAction === 'all') {
                    me.doQuery(me.allQuery, true);
                } else {
                    me.doQuery(me.getRawValue());
                }
            }
            me.inputEl.focus();
        }
    },


    // store the last key and doQuery if relevant
    onKeyUp: function(e, t) {
        var me = this,
            key = e.getKey();

        if (!me.readOnly && !me.disabled && me.editable) {
            me.lastKey = key;
            // we put this in a task so that we can cancel it if a user is
            // in and out before the queryDelay elapses

            // perform query w/ any normal key or backspace or delete
            if (!e.isSpecialKey() || key == e.BACKSPACE || key == e.DELETE) {
                me.doQueryTask.delay(me.queryDelay);
            }
        }

        if (me.enableKeyEvents) {
            me.callParent(arguments);
        }
    },

    initEvents: function() {
        var me = this;
        me.callParent();

        /*
         * Setup keyboard handling. If enableKeyEvents is true, we already have
         * a listener on the inputEl for keyup, so don't create a second.
         */
        if (!me.enableKeyEvents) {
            me.mon(me.inputEl, 'keyup', me.onKeyUp, me);
        }
    },

    createPicker: function() {
        var me = this,
            picker,
            menuCls = Ext.baseCSSPrefix + 'menu',
            opts = Ext.apply({
                selModel: {
                   // mode: me.multiSelect ? 'SIMPLE' : 'SINGLE'
                    mode:  'SINGLE'	
                },
                floating: true,
                hidden: true,
                ownerCt: me.ownerCt,
                cls: me.el.up('.' + menuCls) ? menuCls : '',
                store: me.store,
                displayField: me.displayField,
                focusOnToFront: false,
                pageSize: me.pageSize,
                tpl: me.tpl
            }, me.listConfig, me.defaultListConfig);

        picker = me.picker = Ext.create('Ext.view.BoundList', opts);

        me.mon(picker, {
            itemclick: me.onItemClick,
            refresh: me.onListRefresh,
            scope: me
        });

        me.mon(picker.getSelectionModel(), 'selectionchange', me.onListSelectionChange, me);

        return picker;
    },

    onListRefresh: function() {
        this.alignPicker();
        this.syncSelection();
    },

    onItemClick: function(picker, record){
        /*
         * If we're doing single selection, the selection change events won't fire when
         * clicking on the selected element. Detect it here.
         */
        var me = this,
            lastSelection = me.lastSelection,
            valueField = me.valueField,
            selected;

        //if (!me.multiSelect && lastSelection) {
        if ( lastSelection) {
            selected = lastSelection[0];
            if (selected && (record.get(valueField) === selected.get(valueField))) {
                me.collapse();
            }
        }
    },

    onListSelectionChange: function(list, selectedRecords) {
        var me = this,
            //isMulti = me.multiSelect,
            hasRecords = selectedRecords.length > 0;
        // Only react to selection if it is not called from setValue, and if our list is
        // expanded (ignores changes to the selection model triggered elsewhere)
        if (!me.ignoreSelection && me.isExpanded) {
             
                Ext.defer(me.collapse, 1, me);
            
            /*
             * Only set the value here if we're in multi selection mode or we have
             * a selection. Otherwise setValue will be called with an empty value
             * which will cause the change event to fire twice.
             */
//            if (isMulti || hasRecords) {
//                me.setValue(selectedRecords, false);
//            }
//            if (hasRecords) {
//                me.fireEvent('select', me, selectedRecords);
//            }
            if (hasRecords) {
                me.setValue(selectedRecords, false);
                this._mapReturnFields_(selectedRecords[0]);
                me.fireEvent('select', me, selectedRecords);
            }
             
            me.inputEl.focus();
        }
    },

    /**
     * @private
     * Enables the key nav for the BoundList when it is expanded.
     */
    onExpand: function() {
        var me = this,
            keyNav = me.listKeyNav,
            selectOnTab = me.selectOnTab,
            picker = me.getPicker();

        // Handle BoundList navigation from the input field. Insert a tab listener specially to enable selectOnTab.
        if (keyNav) {
            keyNav.enable();
        } else {
            keyNav = me.listKeyNav = Ext.create('Ext.view.BoundListKeyNav', this.inputEl, {
                boundList: picker,
                forceKeyDown: true,
                tab: function(e) {
                    if (selectOnTab) {
                        this.selectHighlighted(e);
                        me.triggerBlur();
                    }
                    // Tab key event is allowed to propagate to field
                    return true;
                }
            });
        }

        // While list is expanded, stop tab monitoring from Ext.form.field.Trigger so it doesn't short-circuit selectOnTab
        if (selectOnTab) {
            me.ignoreMonitorTab = true;
        }

        Ext.defer(keyNav.enable, 1, keyNav); //wait a bit so it doesn't react to the down arrow opening the picker
        me.inputEl.focus();
    },

    /**
     * @private
     * Disables the key nav for the BoundList when it is collapsed.
     */
    onCollapse: function() {
        var me = this,
            keyNav = me.listKeyNav;
        if (keyNav) {
            keyNav.disable();
            me.ignoreMonitorTab = false;
        }
    },

    /**
     * Selects an item by a {@link Ext.data.Model Model}, or by a key value.
     * @param r
     */
    select: function(r) {
        this.setValue(r, true);
    },

    /**
     * Find the record by searching for a specific field/value combination
     * Returns an Ext.data.Record or false
     * @private
     */
    findRecord: function(field, value) {
        var ds = this.store,
            idx = ds.findExact(field, value);
        return idx !== -1 ? ds.getAt(idx) : false;
    },
    findRecordByValue: function(value) {
        return this.findRecord(this.valueField, value);
    },
    findRecordByDisplay: function(value) {
        return this.findRecord(this.displayField, value);
    },

    /**
     * Sets the specified value(s) into the field. For each value, if a record is found in the {@link #store} that
     * matches based on the {@link #valueField}, then that record's {@link #displayField} will be displayed in the
     * field.  If no match is found, and the {@link #valueNotFoundText} config option is defined, then that will be
     * displayed as the default field text. Otherwise a blank value will be shown, although the value will still be set.
     * @param {String|Array} value The value(s) to be set. Can be either a single String or {@link Ext.data.Model},
     * or an Array of Strings or Models.
     * @return {Ext.form.field.Field} this
     */
    setValue: function(value, doSelect) {
        var me = this,
            valueNotFoundText = me.valueNotFoundText,
            inputEl = me.inputEl,
            i, len, record,
            models = [],
            displayTplData = [],
            processedValue = [];

        if (me.store.loading) {
            // Called while the Store is loading. Ensure it is processed by the onLoad method.
            me.value = value;
            return me;
        }

        // This method processes multi-values, so ensure value is an array.
        value = Ext.Array.from(value);

        // Loop through values
        for (i = 0, len = value.length; i < len; i++) {
            record = value[i];
            if (!record || !record.isModel) {
                record = me.findRecordByValue(record);
            }
            // record found, select it.
            if (record) {
                models.push(record);
                displayTplData.push(record.data);
                processedValue.push(record.get(me.valueField));
            }
            // record was not found, this could happen because
            // store is not loaded or they set a value not in the store
            else {
                // if valueNotFoundText is defined, display it, otherwise display nothing for this value
                if (me.isExpanded) {
                	if (Ext.isDefined(valueNotFoundText)) {
                        displayTplData.push(valueNotFoundText);
                    }
                } else {
                	displayTplData.push(value[i]);
                }
            	
                processedValue.push(value[i]);
            }
        }

        // Set the value of this field. If we are multiselecting, then that is an array.
        //me.value = me.multiSelect ? processedValue : processedValue[0];
        me.value =  processedValue[0];
        if (!Ext.isDefined(me.value)) {
            me.value = null;
        }
        me.displayTplData = displayTplData; //store for getDisplayValue method
        me.lastSelection = me.valueModels = models;

        if (inputEl && me.emptyText && !Ext.isEmpty(value)) {
            inputEl.removeCls(me.emptyCls);
        }

        // Calculate raw value from the collection of Model data
        me.setRawValue(me.getDisplayValue());
        me.checkChange();

        if (doSelect !== false) {
            me.syncSelection();
        }
        me.applyEmptyText();

        return me;
    },

    /**
     * @private Generate the string value to be displayed in the text field for the currently stored value
     */
    getDisplayValue: function() {
        return this.displayTpl.apply(this.displayTplData);
    },

    getValue: function() {
        // If the user has not changed the raw field value since a value was selected from the list,
        // then return the structured value from the selection. If the raw field value is different
        // than what would be displayed due to selection, return that raw value.
        var me = this,
            picker = me.picker,
            rawValue = me.getRawValue(), //current value of text field
            value = me.value; //stored value from last selection or setValue() call

        if (me.getDisplayValue() !== rawValue) {
            value = rawValue;
            me.value = me.displayTplData = me.valueModels = null;
            if (picker) {
                me.ignoreSelection++;
                picker.getSelectionModel().deselectAll();
                me.ignoreSelection--;
            }
        }

        return value;
    },

    getSubmitValue: function() {
        return this.getValue();
    },

    isEqual: function(v1, v2) {
        var fromArray = Ext.Array.from,
            i, len;

        v1 = fromArray(v1);
        v2 = fromArray(v2);
        len = v1.length;

        if (len !== v2.length) {
            return false;
        }

        for(i = 0; i < len; i++) {
            if (v2[i] !== v1[i]) {
                return false;
            }
        }

        return true;
    },

    /**
     * Clears any value currently set in the ComboBox.
     */
    clearValue: function() {
        this.setValue([]);
    },

    /**
     * @private Synchronizes the selection in the picker to match the current value of the combobox.
     */
    syncSelection: function() {
        var me = this,
            ExtArray = Ext.Array,
            picker = me.picker,
            selection, selModel;
        if (picker) {
            // From the value, find the Models that are in the store's current data
            selection = [];
            ExtArray.forEach(me.valueModels || [], function(value) {
                if (value && value.isModel && me.store.indexOf(value) >= 0) {
                    selection.push(value);
                }
            });

            // Update the selection to match
            me.ignoreSelection++;
            selModel = picker.getSelectionModel();
            selModel.deselectAll();
            if (selection.length) {
                selModel.select(selection);
            }
            me.ignoreSelection--;
        }
    }
});



Ext.define("dnet.core.lov.AbstractCombo", {
	extend:  "dnet.core.lov.AbstractRemoteCombo" ,
 	alias: "widget.xcombo",
 
 // DNet properties
 	
 	/**
	 * Example: ,filterFieldMapping: [{lovField:"...lovFieldName", dsField:
	 * "...dsFieldName"} ]
	 * 
	 * Or: ,filterFieldMapping: [{lovField:"...lovFieldName", value:
	 * "...static value"} ]
	 */
 	filterFieldMapping: null , 
	_dataProviderFields_  :null,
	_dataProviderName_ : null,
	_dummyValue_ : null,
	_editDialog_: null	 ,
	openDialog:null,
	retFieldMapping : null,	
	triggerAction :"query",
	matchFieldWidth : false,
	recordModel : null,
	pageSize : 20,
	autoSelect:true,
	minChars:0,	
	queryMode: "remote",
	  
	trigger1Cls: Ext.baseCSSPrefix + 'x-form-trigger',    
    trigger2Cls:  Ext.baseCSSPrefix + 'form-search-trigger',
     
	defaultListConfig: {
		emptyText: "", loadingText: "Loading...", 
		minWidth: 70, 
		width:200,  resizable:true,
		shadow: "sides", autoScroll:true
	},
	 
	autoScroll:true,
	
	initComponent : function(){		 	
		this._createStore_(); 
		if (this.retFieldMapping == null) {
			this.retFieldMapping = [];
		}
		 
		if (this.dataIndex) {
			this.retFieldMapping[this.retFieldMapping.length] = {
				lovField:this.displayField,
				dsField: this.dataIndex
			};
		} else if (this.paramIndex) {
			this.retFieldMapping[this.retFieldMapping.length] = {
				lovField:this.displayField,
				dsParam: this.paramIndex
			};
		}
		 			
	    this.callParent(arguments);	
	},	 
	
	onTrigger2Click : function(){
		getApplication().showFrame(this._editDialog_.name,  {
			url: Dnet.buildUiPath(this._editDialog_.bundle, this._editDialog_.name, this._editDialog_.custom ), 
			tocElement:this._editDialog_.tocElement} );		 
    },
	 
	_createStore_: function() {
		this.store = Ext.create("Ext.data.Store", {
			model : this.recordModel,
			remoteSort : true,
		  
			autoLoad : false,
			autoSync : false,
			clearOnPageLoad : true,
			pageSize : this.pageSize,
			proxy : {
				type : 'ajax',
				api : Dnet.dsAPI(this._dataProviderName_, "json"),
				actionMethods : {
					create : 'POST',
					read : 'POST',
					update : 'POST',
					destroy : 'POST'
				},
				reader : {
					type : 'json',
					root : 'data',
					idProperty : 'id',
					totalProperty : 'totalCount',
					messageProperty : 'message'
				},
				listeners : {
					"exception" : {
						fn : this.proxyException,
						scope : this
					}
				},
				startParam : Dnet.requestParam.START,
				limitParam : Dnet.requestParam.SIZE,
				sortParam : Dnet.requestParam.SORT,
				directionParam : Dnet.requestParam.SENSE
			}
		});
	 },
 
	 
 
    _mapReturnFields_: function(crec) {  
		var dcv = this._dcView_;
		var mrec = null;
		var targetIsFilter = false;
		if (this.inEditor && dcv._dcViewType_ != "edit-propgrid" 
				&& dcv._dcViewType_ != "filter-propgrid" ) {			
			if (dcv && dcv._dcViewType_ == "bulk-edit-field") {
            	/*is a bulk editor for one ds field in a property grid*/
            	mrec = dcv.getSource();
				this._mapReturnFieldsExecuteBulkEdit_(crec,mrec);
			} else {           	 
				mrec = this._targetRecord_;
				this._mapReturnFieldsExecute_(crec,mrec);
			}             
		} else {			
			if (dcv._dcViewType_ == "edit-form" || dcv._dcViewType_ == "edit-propgrid" ) {
	           mrec = dcv._controller_.getRecord();
			}
	        if (dcv._dcViewType_ == "filter-form" || dcv._dcViewType_ == "filter-propgrid" ) {
	           mrec = dcv._controller_.getFilter();
	           targetIsFilter = true;
			}			
			this._mapReturnFieldsExecute_(crec, mrec, dcv._controller_.getParams(), targetIsFilter );
		}
    },
    
    
    _mapReturnFieldsExecuteBulkEdit_: function(crec, recdata) {    
		if (!recdata) {return; }
        if (this.retFieldMapping != null) {
        	for(var i=this.retFieldMapping.length-1; i>=0; i-- ) {
        		
        		var retDataIndex = null;
        		var nv = null;
				isParam  =  !Ext.isEmpty(this.retFieldMapping[i]["dsParam"]);
				if (isParam) {				   		   
					retDataIndex = this.retFieldMapping[i]["dsParam"];
					ov = prec.get( retDataIndex );
				} else {
					retDataIndex = this.retFieldMapping[i]["dsField"];
					ov = recdata[retDataIndex];
				}
				
				 if (crec && crec.data) {
				 	nv = crec.data[ this.retFieldMapping[i]["lovField"] ];
				 	recdata[retDataIndex] = nv;
				 } else {
				 	
				 }
				
        	}
        }	 

	 } ,
    
    /**
     * Params: crec: combo selected record
     */
    _mapReturnFieldsExecute_: function(crec, mrec, prec, targetIsFilter) {    
		if (!mrec) {return; }
        if (this.retFieldMapping != null) {
				var nv,ov, isParam, rawv= this.getRawValue();
				 
                   for(var i=this.retFieldMapping.length-1; i>=0; i-- ) {
				  // for(var i=0,len = this.retFieldMapping.length; i<len; i++ ) {
                	   var retDataIndex = null;
				   	   isParam  =  !Ext.isEmpty(this.retFieldMapping[i]["dsParam"]);
				   	   if (isParam) {				   		   
					   		retDataIndex = this.retFieldMapping[i]["dsParam"];
					   		ov = prec.get( retDataIndex );
				   	   } else {
				   		   retDataIndex = this.retFieldMapping[i]["dsField"];
				   		   ov = mrec.get(retDataIndex);
				   	   }
				   	    
					   if (crec && crec.data) {
							nv = crec.data[ this.retFieldMapping[i]["lovField"] ];
					   		if (nv!= ov) {
								if(isParam) {
								   this._dcView_._controller_.setParamValue(retDataIndex, nv);
        						} else {
        							if(targetIsFilter) {
        								this._dcView_._controller_.setFilterValue(retDataIndex, nv);
        							} else {
        								mrec.set( retDataIndex ,nv   );
        							}
								}
							}
					   } else {	
						    
						   if (retDataIndex==this.dataIndex ) {							   
							   if (this._validateListValue_ && rawv != ov) {
								   rawv = null;
								   this.setRawValue(rawv);
							   }
							   if (rawv!= ov) {
								   if(isParam) {
									   this._dcView_._controller_.setParamValue(retDataIndex,rawv );
									} else {
										if(targetIsFilter) {
	        								this._dcView_._controller_.setFilterValue(retDataIndex, rawv);
	        							} else {
	        								mrec.set(retDataIndex, rawv );
	        							}
									}
							   }
								
							} else {
								if ((ov != null && ov!= "" && Ext.isEmpty(rawv))) {
									if(isParam) {
									   this._dcView_._controller_.setParamValue(retDataIndex, null);
	        						} else {
										 //mrec.set(retDataIndex ,null);
										 if(targetIsFilter) {
	        								this._dcView_._controller_.setFilterValue(retDataIndex, null);
	        							} else {
	        								mrec.set(retDataIndex, null );
	        							}
									}
								}
							}
						    
					   }
				   }
			}

	 } ,
	 
	 
	 _mapFilterFields_: function(bp) {
	 	
	 	var mrec = null;
	 	var dcv = this._dcView_;
	 	var prec = dcv._controller_.getParams();
	 	
		if (this.inEditor) {
	       mrec = this._targetRecord_;
           this._mapFilterFieldsExecute_(bp,mrec, prec);
		} else {			
			if (dcv._dcViewType_ == "edit-form") {
	           mrec = dcv._controller_.getRecord();
			}
	        if (dcv._dcViewType_ == "filter-form") {
	           mrec = dcv._controller_.getFilter();
			}
           this._mapFilterFieldsExecute_(bp, mrec, prec);
		}
    },
    /**
     * Parameters:
     * bp: base params for the store
     */
    _mapFilterFieldsExecute_: function(bp, mrec, prec) {  
		if (!mrec) {return; }
        if (this.filterFieldMapping != null) {
        	for(var i=0, len=this.filterFieldMapping.length; i<len; i++ ) {
			   if (this.filterFieldMapping[i]["value"]) {
			   		bp[this.filterFieldMapping[i]["lovField"]] =  this.filterFieldMapping[i]["value"];
			   	} else {
			   		var isParam  =  !Ext.isEmpty(this.filterFieldMapping[i]["dsParam"]);
			   		if (isParam) {
			   			bp[this.filterFieldMapping[i]["lovField"]] =  prec.get(this.filterFieldMapping[i]["dsParam"]);
			   		} else {
			   			bp[this.filterFieldMapping[i]["lovField"]] =   mrec.get( this.filterFieldMapping[i]["dsField"]);	
			   		}        			
				}
			}
		 }
	 } ,
	  
    /**
	 * Default proxy-exception handler
	 */
	proxyException : function(proxy, response, operation, eOpts) {
		this.showAjaxErrors(response, eOpts);
	},
 
	
    /**
	 * Show errors to user. TODO: Externalize it as command.
	 */
	showAjaxErrors : function(response, options) {
		// Ext.MessageBox.hide();
		var msg, withDetails = false;
		if (response.responseText) {
			if (response.responseText.length > 2000) {
				msg = response.responseText.substr(0, 2000);
				withDetails = true;
			} else {
				msg = response.responseText;
			}
		} else {
			msg = "No response received from server.";
		}
		var alertCfg = {
			msg : msg,
			scope : this,
			//fn: this.focus,
			icon : Ext.MessageBox.ERROR,
			buttons : Ext.MessageBox.OK			
		}
		if (withDetails) {
			alertCfg.buttons['cancel'] = 'Details';
			alertCfg['detailedMessage'] = response.responseText;
		}
		Ext.Msg.show(alertCfg);

	},
    
	
	// ****************************************************************
	// *********************** OVERRIDES ******************************
	// ****************************************************************
	
	
    doQuery: function(queryString, forceAll, rawQuery) {		 
        var bp = {}
        bp[this.displayField] = queryString+"*";
        bp["clientId"] = getApplication().getSession().client.id;			 
			if (this.filterFieldMapping != null) {
			   this._mapFilterFields_(bp);
			   /*TODO: check the filter mapping change instead of this brut force stuff*/
			   this.queryCaching = false;
			}
        this.store.proxy.extraParams["data"] =  Ext.encode(bp)
        return this.callParent(arguments);
	 },
	 
 
    
    assertValue: function() {
        var me = this,
            val = me.getRawValue(),
            rec;
        
        rec = this.findRecord(this.displayField, val);
        
        if(!rec && this.forceSelection){
			this.setRawValue("");
			if(val.length > 0 && val != this.emptyText){
				 
                this.applyEmptyText();
            }else{
                this.clearValue();
            }

            this._mapReturnFields_(null);

        }else{
            if(rec){
                
                this._mapReturnFields_(rec);
                if (val == rec.get(this.displayField) && this.value == rec.get(this.valueField)){
				    return;
                }
                val = rec.get(this.valueField || this.displayField);
            } else {
            	if (val != this.value ) {
            		this._mapReturnFields_(null);
            	}			   
			}
			if (this.getValue() != val) {
               this.setValue(val);
			}

        }
        
        me.collapse();
    }

});





Ext.define('dnet.core.lov.LocalCombo', {
	extend: "Ext.form.field.ComboBox",
	alias:"widget.localcombo",
	
	queryMode: "local", 
	 selectOnFocus:true,
	 triggerAction:"query", 
	 forceSelection:true,
	 
	 
	 setValue: function(value, doSelect) {
	    var me = this,
	        valueNotFoundText = me.valueNotFoundText,
	        inputEl = me.inputEl,
	        i, len, record,
	        models = [],
	        displayTplData = [],
	        processedValue = [];
	
	    if (me.store.loading) {
	        // Called while the Store is loading. Ensure it is processed by the onLoad method.
	        me.value = value;
	        return me;
	    }
	
	    // This method processes multi-values, so ensure value is an array.
	    value = Ext.Array.from(value);
	
	    // Loop through values
	    for (i = 0, len = value.length; i < len; i++) {
	        record = value[i];
	        if (!record || !record.isModel) {
	            record = me.findRecordByValue(record);
	        }
	        // record found, select it.
	        if (record) {
	            models.push(record);
	            displayTplData.push(record.data);
	            processedValue.push(record.get(me.valueField));
	        }
	        // record was not found, this could happen because
	        // store is not loaded or they set a value not in the store
	        else {
	            // if valueNotFoundText is defined, display it, otherwise display nothing for this value
	            if (Ext.isDefined(valueNotFoundText)) {
	                displayTplData.push(valueNotFoundText);
	            }
	            processedValue.push(value[i]);
	        }
	    }
	
	    // Set the value of this field. If we are multiselecting, then that is an array.
	    me.value = me.multiSelect ? processedValue : processedValue[0];
	    if (!Ext.isDefined(me.value)) {
	        me.value = null;
	    }
	    me.displayTplData = displayTplData; //store for getDisplayValue method
	    me.lastSelection = me.valueModels = models;
	
	    if (inputEl && me.emptyText && !Ext.isEmpty(value)) {
	        inputEl.removeCls(me.emptyCls);
	    }
	
	    // Calculate raw value from the collection of Model data
	    me.setRawValue(me.getDisplayValue());
	    me.checkChange();
	
	    me._processedValue_ = processedValue;
	    if(processedValue.length > 0) {
	    	var r = this._dcView_._controller_.getRecord();
			if (r) {
				var rv = r.get(me.dataIndex);				  
				if (!r.isEqual(rv, me.value)) {
					r.set(me.dataIndex, me.value);
				}
			} 
			 
	    }
	    if (doSelect !== false) {
	        me.syncSelection();
	    }
	    me.applyEmptyText();
	
	    return me;
	}
});



 
Ext.define("dnet.core.lov.AbstractLov", {
	extend:  "Ext.form.TriggerField" ,
 	alias: "xlovfield",
 	 
	 defaultAutoCreate : {tag: "input", type: "text", size: "16",style:"cursor:default;", autocomplete: "off"}
    ,triggerClass: 'x-form-search-trigger'

    ,lovName: null
    ,fieldMapping:null
	,viewColumns: null
	,windowConfig:null
    ,_view_: null
    ,_window_:null
	,resultPageSize:50
	,_filterField_: null
    ,_dataProviderFields_ : null
	,_dataProviderName_: null

    , initComponent: function(){
        Ext.apply(this,arguments);
        //this._dummyDefaults();
        dnet.core.lov.AbstractLov.superclass.initComponent.call(this);
    } 

    ,initEvents: function() {
     dnet.core.lov.AbstractLov.superclass.initEvents.call(this);

     this.keyNav = new Ext.KeyNav(this.el, {
            "down" : function(e){
				e.stopEvent();
                this.onTriggerClick();
            }
            ,scope : this
            //,forceKeyDown : true
            ,defaultEventAction: 'stopEvent'
        });

    //this._view_.on('keypress', this.onViewKeypress , this);
    //this._view_.on('dblclick', this.onDoSelect, this);
    //this._view_.getStore().proxy.on('loadexception', this.onLoadException, this);

  }

	, onTriggerClick: function() {
    	if (!this.disabled) this._showLov_();
	}

	,_showLov_: function() {
    	if ( this._window_ == null) {
           this._createLov_();
		}
		this._window_.show();

	}
	,_defaultFocusElement_:function() {
        this._filterField_.el.focus();
	}
	,_hideLov_: function() {
	   this._window_.hide();
	   //this.el.focus();
	}
	,_createLov_: function() {
		this._createFilter_();
        this._createView_();
		this._createWindow_();
	}

    ,_createWindow_: function() {
		if (this._window_ != null ) return;
		if (this.windowConfig == null) {
          this.windowConfig = {
                title:"List of Values ("+this.lovName+")"
               ,width:300
               ,height:300
		  }
		}
        this.windowConfig = Ext.apply(this.windowConfig, {
           //  modal: true
             autoScroll: true
            ,closable: true
            ,closeAction: 'hide'
            ,layout: 'fit'
            ,items: this._view_
		});
        this._window_ = new Ext.Window(this.windowConfig);
        this._window_.on("hide", this._hideLov_, this);
        this._window_.on("show", this._defaultFocusElement_, this);

	}
	,_createFilter_:function() {
		this._filterField_ = new Ext.form.TextField({
			selectOnFocus:true
		   ,width:100
		});
	}
    ,_createView_: function() {
		var store =  new Ext.data.Store({
			        remoteSort:true
			       ,proxy: new Ext.data.HttpProxy({
					        api: getProtocolAPI(this._dataProviderName_,"json")
					    })
           			,reader: new Ext.data.JsonReader(
					   		 {totalProperty: 'totalCount',idProperty: 'id',root: 'data',messageProperty: 'message'}
							,Ext.data.Record.create(this._dataProviderFields_))
			       , listeners: { "exception":{ fn:  this.proxyException, scope:this }}
			    });
		this._view_ = new Ext.grid.GridPanel({
			 store:store
             //,columns: [new Ext.grid.RowNumberer(),{id:"code",header:"Code",width:100,dataIndex:'code',sortable:true,hidden:false},{id:"name",header:"Name",width:200,dataIndex:'name',sortable:true,hidden:false} ]
            ,columns: this.viewColumns
			,forceFit: true
		    ,loadMask:true
		    ,bbar:{xtype:"paging", store: store, displayInfo:true, pageSize:this.resultPageSize }
		    ,tbar:[ this._filterField_
                ,new Ext.Action({ id:Ext.id(), text: "Select", tooltip: 'Apply selection (Double click record in list)', iconCls: "icon-action-confirm", scope:this
					, handler: this.onDoSelect   })
				, new Ext.Action({ id:Ext.id(), text: "Cancel", tooltip: 'Close window', iconCls: "icon-action-reset", scope:this
					, handler: this.onDoCancel   })
            ]
           })
	}
   , proxyException: function(dataProxy, type, action , options , response , arg ) {
        if(type=="response") {
          this.afterAjaxFailure(response , options);
        } else {
           alert(response.message.substr(0,1500));
        }
	  }
});

 